{ KEYWORDS: VERSION }
{ VERSION: 1.8 }
{ 1.8 / 8.10.2005
  - opraveno: funkce TCfgReader.ExpandFileName() nerespektovala
    pripadne predefinovany TCfgReader.RootPath
  1.7 / 26.11.2004
  - TCfgArrayReader.Empty vrati True kdyz nema zadne soubory
  - TCfgReader.RootPath vraci adresar s konfiguraci
  1.6 / 19.10.2004
  - Writer bez Add() zachova puvodni velikost nosice
  1.5 / 10.1.2004
  - zverejnena metoda TExeCfgArrayReader.GetStream()
  1.4 / 13.6.2003
  - TCfgReader je soucastka pro cteni vestavene konfigurace s overridem ve slozce
    jmenoExe.dat
  1.3 / 12.6.2003
  - TCfgArrayWriter.Add() projede rekurzivne i podadresare
  1.2 / 11.3.2003
  - pouziva kompresi (viz prepinac COMPRESS)
  1.1 / 27.7.2002
  - jmeno souboru do TCfgArrayWriter.Add() muze obsahovat souborovou masku,
    napr. Add('korkoran\*.bobek')
  - TCfgArray zachovava datum-cas souboru
}
unit cfgArray;

interface
uses Windows,fileSys,Classes,SysUtils;

{$IFNDEF VER130}
  {$DEFINE DEFVALUES}
{$ENDIF}

 type
  TCfgArrayWriter=class
  private
   BasePosition:Cardinal;
  protected
   Stream:TStream;
  public
   constructor Create(AStream:TStream);
   destructor Destroy;override;
   procedure Add(const fileName:string);
  end;

  TOverride=(oAlways,oNever,oNewer);
  TCfgArrayReader=class
  private
   FEmpty:Boolean;
  protected
   FPath:string;
  public
   constructor Create(AStream:TStream);
   destructor Destroy;override;
   function ExpandFileName(const fileName:string; allowOverride:TOverride{$IFDEF DEFVALUES}=oNewer{$ENDIF}):string;virtual;
   function RootPath:string;virtual;
   property Empty: Boolean read FEmpty;
  end;

  TExeCfgArrayReader=class(TCfgArrayReader)
  public
   class function GetStream(const exeFile:string; Mode:integer):TStream;
   constructor Create(const exeFile:string);
  end;

  TCfgReader=class(TExeCfgArrayReader)
  public
   constructor Create;
   function ExpandFileName(const fileName:string; allowOverride:TOverride{$IFDEF DEFVALUES}=oNewer{$ENDIF}):string;override;
   function RootPath:string;override;
  end;

implementation
{ TUNE: COMPRESS zapina podporu pro kompresi }
{$DEFINE COMPRESS}
uses fileUtils,ExeHdr{$IFDEF COMPRESS},Lzrw1{$ENDIF};

 procedure PutString(stream:TStream; const value:string);
 var Len:byte;
 begin
  Len:=Length(value);
  stream.WriteBuffer(Len,sizeof(Len));
  if Len<>0 then
   stream.WriteBuffer(value[1],Len)
 end;

 function GetString(stream:TStream):string;
 var Len:byte;
 begin
  if stream.Read(Len,sizeof(Len))<>sizeof(Len) then Len:=0;
  SetLength(Result,Len);
  if Len<>0 then
   stream.ReadBuffer(Result[1],Len)
 end;

 function GetInt(stream:TStream):integer;
 begin
  stream.ReadBuffer(Result,sizeof(Result))
 end;

 procedure PutInt(stream:TStream; value:integer);
 begin
  stream.WriteBuffer(value,sizeof(value))
 end;

 constructor TCfgArrayWriter.Create;
 begin
  inherited Create;
  Stream:=AStream;
  BasePosition:=Stream.Position
 end;

 destructor TCfgArrayWriter.Destroy;
 begin
  if Stream.Position>BasePosition then
   { ulozime ukoncovac. v druhem pripade jde o odriznuti predchoziho obsahu }
   PutString(Stream,'');
  Stream.Size:=Stream.Position;
  Stream.Free;
  inherited
 end;

const CompressFlag=$80000000;

 procedure TCfgArrayWriter.Add;

  procedure DoAdd(const subFolder:string; const fileName:string);
  var src:TStream;
  {$IFDEF COMPRESS}
  var comp:TLzrw1;
  var buf:TStream;
  {$ENDIF}
  begin
   src:=TFileStream.Create(fileName,fmOpenRead or fmShareDenyNone);
   try
    PutString(Stream,subFolder+ExtractFileName(fileName));
    {$IFDEF COMPRESS}
    comp:=TLzrw1.Create(nil);
    buf:=TMemoryStream.Create;
    try
     comp.CompressMode:=Auto;
     comp.InputStream:=src;
     comp.OutputStream:=buf;
     if comp.Compress<src.Size then begin
      PutInt(Stream,buf.Size or compressFlag);
      PutInt(Stream,FileAge(fileName));
      buf.Position:=0;
      if buf.Size>0 then
       Stream.CopyFrom(buf,0)
     end else begin
      PutInt(Stream,src.Size);
      PutInt(Stream,FileAge(fileName));
      src.Position:=0;
      if src.Size>0 then
       Stream.CopyFrom(src,0)
     end
    finally
     comp.Free;
     buf.Free
    end
    {$ELSE}
    PutInt(Stream,src.Size);
    PutInt(Stream,FileAge(fileName));
    if src.Size>0 then
     Stream.CopyFrom(src,0)
    {$ENDIF}
   finally
    src.Free
   end
  end;

  procedure AddFolder(const folder:string; const subFolder:string; const mask:string);
  var search:TSearchRec;
  var found:integer;
  begin
   found:=FindFirst(folder+mask,faAnyFile,search);
   try
    while found=0 do begin
     if (search.Attr and faDirectory)=0 then
      DoAdd(subFolder,folder+search.Name);
     found:=FindNext(search) 
    end
   finally
    FindClose(search)
   end;
   found:=FindFirst(folder+'*.*',faAnyFile,search);
   try
    while found=0 do begin
     if ((search.Attr and faDirectory)<>0) and (search.Name[1]<>'.') then
      AddFolder(folder+search.Name+'\',subFolder+search.Name+'\',mask);
     found:=FindNext(search)
    end
   finally
    FindClose(search)
   end
  end;

 begin
  AddFolder(ExtractFilePath(fileName),'',ExtractFileName(fileName))
 end;

 constructor TCfgArrayReader.Create;
 var fname:string;
 var fsize:integer;
 var fage:integer;
 var dst:TStream;
 var fh:integer;
 {$IFDEF COMPRESS}
 var comp:TLzrw1;
 var buf:TStream;
 {$ENDIF}
 begin
  inherited Create;
  FEmpty:=True;
  if Assigned(AStream) then
  try
   FPath:=OSFileSystem.GetTempFileName('\');
   assert(FPath<>'');
   FPath:=FPath+'\';
   while true do begin
    fname:=GetString(AStream);
    if fname='' then break;
    FEmpty:=False;
    fsize:=GetInt(AStream);
    fage:=GetInt(AStream);
    ForceDirectories(OSFileSystem,FPath+ExtractFileDir(fname));
    dst:=TFileStream.Create(FPath+fname,fmCreate);
    try
     {$IFDEF COMPRESS}
     if (fsize and compressFlag)<>0 then begin
      fsize:=fsize and (not compressFlag);
      if fsize>0 then begin
       comp:=TLzrw1.Create(nil);
       buf:=TMemoryStream.Create;
       try
        buf.CopyFrom(AStream,fsize);
        buf.Position:=0;
        comp.InputStream:=buf;
        comp.OutputStream:=dst;
        comp.Decompress
       finally
        comp.Free;
        buf.Free
       end
      end
     end else begin
      if fsize>0 then
       dst.CopyFrom(AStream,fsize)
     end
     {$ELSE}
     if fsize>0 then
      dst.CopyFrom(AStream,fsize)
     {$ENDIF}
    finally
     dst.Free
    end;
    fh:=FileOpen(FPath+fname,fmOpenReadWrite);
    if fh>0 then begin
     FileSetDate(fh,fage);
     CloseHandle(fh)
    end
   end
  finally
   AStream.Free
  end
 end;

 function TCfgArrayReader.ExpandFileName;
 begin
  Result:=FPath+fileName
 end;

 function TCfgArrayReader.RootPath:string;
 begin
  Result:=FPath
 end;

 destructor TCfgArrayReader.Destroy;
 begin
  if FPath<>'' then
   EraseFolder(OSFileSystem,FPath);
  inherited
 end;

 class function TExeCfgArrayReader.GetStream;
 var dosHdr:TImageDosHeader;
 var peHdr:TImageNTHeaders;
 var peOfs:integer;
 var i:integer;
 var sec:TImageSectionHeader;
 begin
  Result:=TFileStream.Create(exeFile,Mode);
  Result.ReadBuffer(dosHdr,sizeof(dosHdr));
  peOfs:=dosHdr.e_lfanew;
  Result.Position:=peOfs;
  Result.ReadBuffer(peHdr,sizeof(peHdr));

  for i:=peHdr.FileHeader.NumberOfSections-1 downto 0 do begin
   Result.Position:=peOfs+sizeof(TImageNTHeaders)+i*sizeof(TImageSectionHeader);
   Result.ReadBuffer(sec,sizeof(sec));

   if (sec.PointerToRawData<>0) and (sec.SizeOfRawData<>0) then begin
    Result.Position:=sec.PointerToRawData+sec.SizeOfRawData;
    exit
   end
  end;
  Result.Free;
  Result:=nil    // TODO:
 end;

 constructor TExeCfgArrayReader.Create;
 begin
  inherited Create(GetStream(exeFile,fmOpenRead or fmShareDenyWrite))
 end;

 function TCfgReader.RootPath:string;
 begin
  if Empty then
   Result:=ChangeFileExt(ParamStr(0),'.dat' { TUNE: })+'\'
  else
   Result:=inherited RootPath
 end;

 // TODO: funkce FileExists a FileAge nezaberou na adresar
 function TCfgReader.ExpandFileName;
 var fileDate:integer;
 var cfgDate:integer;
 var fname:string;
 begin
  if ExtractFileDrive(fileName)<>'' then
   Result:=fileName
  else begin
// NEW: 1.8   Result:=ChangeFileExt(ParamStr(0),'.dat' { TUNE: })+'\'+fileName;
   Result:=RootPath+fileName;
   fname:=inherited ExpandFileName(fileName,oNever);
   { Result je predefinovana varianta
     fname je vestavena varianta }
   case allowOverride of
   oNever: Result:=fname;
   oAlways: if not FileExists(Result) then Result:=fname;
   oNewer: if (not FileExists(Result)) or
              (FileAge(fname)>=FileAge(Result)) then Result:=fname;
   else assert(false)
   end;
  end
 end;

 constructor TCfgReader.Create;
 begin
  inherited Create(ParamStr(0))
 end;

end.
