{ KEYWORDS: VERSION }
{ VERSION: 1.0 }
{

 obecny predek ini filu je TCustomIniFile

}
{ 0.9 / 3.12.2004
  - TMemIniFile a TFileSysIniFile maji indikator Modified: Boolean a TFileSysIniFile
    neuklada kdyz nebyla zmena
  0.8 / 25.11.2004
  - nova pomocna fce TEnum.CommaText (NEvirtualni)
  0.7 / 3.4.2002
  - nova pomocna fce WriteEnum a ReadEnum
  0.6 / 8.11.2001
  - TCustomIniFile.ReadEnum,TCustomIniFile.WriteEnum a TEnum
  0.5 / 26.2.2001
  - nova funkce IniFileCopySection()
  0.4
  - prepinac USEFILESYS
  0.3
  - TUplnePametovyIni - da se akorat zkopirovat fci IniFileCopy()

  0.2
  - podpurna funkce
    procedure IniFileCopy(source,destination:TCustomIniFile);

  0.1
  - pridana podpora pro fileSys ve forme TFileSysIniFile
}
{*******************************************************}
{                                                       }
{       Borland Delphi Visual Component Library         }
{                                                       }
{       Copyright (c) 1995,99 Inprise Corporation       }
{                                                       }
{*******************************************************}

unit IniFilesD5;

{$R-,T-,H+,X+}

interface

{$DEFINE USEFILESYS}
uses Windows, SysUtils, Classes{$IFDEF USEFILESYS}, fileSys{$ENDIF};

type
  TEnum=class(TStringList)
  public
   { do konstruktoru jde neco jako
     A=0,B=2,C=4
     -> A=0,B=2,C=4
     nebo
     A,B,C=3
     -> A=0,B=1,C=3
   }
   constructor Create(const values:string);
   function NameToID(const name:string; Default:integer):integer;
   function IDToName(id:integer; const Default:string):string;
   { vrati rozepsany normalizovany parametr konstruktoru }
   function CommaText:string;
  end;

  TCustomIniFile = class(TObject)
  private
    FFileName: string;
  public
    constructor Create(const FileName: string);
    function SectionExists(const Section: string): Boolean;
    function ReadString(const Section, Ident, Default: string): string; virtual; abstract;
    procedure WriteString(const Section, Ident, Value: String); virtual; abstract;
    function ReadInteger(const Section, Ident: string; Default: Longint): Longint; virtual;
    procedure WriteInteger(const Section, Ident: string; Value: Longint); virtual;
    function ReadBool(const Section, Ident: string; Default: Boolean): Boolean; virtual;
    procedure WriteBool(const Section, Ident: string; Value: Boolean); virtual;
    function ReadDate(const Section, Name: string; Default: TDateTime): TDateTime; virtual;
    function ReadDateTime(const Section, Name: string; Default: TDateTime): TDateTime; virtual;
    function ReadFloat(const Section, Name: string; Default: Double): Double; virtual;
    function ReadTime(const Section, Name: string; Default: TDateTime): TDateTime; virtual;
    procedure WriteDate(const Section, Name: string; Value: TDateTime); virtual;
    procedure WriteDateTime(const Section, Name: string; Value: TDateTime); virtual;
    procedure WriteFloat(const Section, Name: string; Value: Double); virtual;
    procedure WriteTime(const Section, Name: string; Value: TDateTime); virtual;
    procedure ReadSection(const Section: string; Strings: TStrings); virtual; abstract;
    procedure ReadSections(Strings: TStrings); virtual; abstract;
    procedure ReadSectionValues(const Section: string; Strings: TStrings); virtual; abstract;
    procedure EraseSection(const Section: string); virtual; abstract;
    procedure DeleteKey(const Section, Ident: String); virtual; abstract;
    procedure UpdateFile; virtual; abstract;
    function ValueExists(const Section, Ident: string): Boolean;
    property FileName: string read FFileName;
  public
    function ReadEnum(const Section,Ident:string; Default: Longint; values:TEnum):Longint;
    procedure WriteEnum(const Section,Ident:string; Value: Longint; values:TEnum);
  end;

  { TIniFile - Encapsulates the Windows INI file interface
    (Get/SetPrivateProfileXXX functions) }

  TIniFile = class(TCustomIniFile)
  public
    function ReadString(const Section, Ident, Default: string): string; override;
    procedure WriteString(const Section, Ident, Value: String); override;
    procedure ReadSection(const Section: string; Strings: TStrings); override;
    procedure ReadSections(Strings: TStrings); override;
    procedure ReadSectionValues(const Section: string; Strings: TStrings); override;
    procedure EraseSection(const Section: string); override;
    procedure DeleteKey(const Section, Ident: String); override;
    procedure UpdateFile; override;
  end;

  { TMemIniFile - loads and entire ini file into memory and allows all
    operations to be performed on the memory image.  The image can then
    be written out to the disk file }

  TMemIniFile = class(TCustomIniFile)
  private
    FModified: Boolean;
    FSections: TStringList;
    function AddSection(const Section: string): TStrings;
  protected
    procedure LoadValues;
  public
    constructor Create(const FileName: string);
    destructor Destroy; override;
    procedure Clear;
    procedure DeleteKey(const Section, Ident: String); override;
    procedure EraseSection(const Section: string); override;
    procedure GetStrings(List: TStrings);
    procedure ReadSection(const Section: string; Strings: TStrings); override;
    procedure ReadSections(Strings: TStrings); override;
    procedure ReadSectionValues(const Section: string; Strings: TStrings); override;
    function ReadString(const Section, Ident, Default: string): string; override;
    procedure Rename(const FileName: string; Reload: Boolean);
    procedure SetStrings(List: TStrings);
    procedure UpdateFile; override;
    procedure WriteString(const Section, Ident, Value: String); override;
  public
    property Modified: Boolean read FModified write FModified;
  end;

  TUplnePametovyIni=class(TMemIniFile)
  public
   constructor Create;
   procedure UpdateFile;override;
  end;

  {$IFDEF USEFILESYS}
  TFileSysIniFile=class(TMemIniFile)
  protected
   FFileSystem:TFileSystem;
   procedure LoadValues;
  public
   destructor Destroy;override;
   procedure UpdateFile;override;
  public
   constructor Create(AFileSystem:TFileSystem; AFileName:string);
   property FileSystem:TFileSystem read FFileSystem;
  end;
  {$ENDIF}

  procedure IniFileCopy(source,destination:TCustomIniFile);
  procedure IniFileCopySection(source,destination:TCustomIniFile; const section:string);

  function ReadEnum(ini:TCustomIniFile; const Section,Ident:string; Default: Longint; const values:string):Longint;
  procedure WriteEnum(ini:TCustomIniFile; const Section,Ident:string; Value: Longint; const values:string);

implementation

uses Consts;

{ TCustomIniFile }

constructor TCustomIniFile.Create(const FileName: string);
begin
  FFileName := FileName;
end;

function TCustomIniFile.SectionExists(const Section: string): Boolean;
var
  S: TStrings;
begin
  S := TStringList.Create;
  try
    ReadSection(Section, S);
    Result := S.Count > 0;
  finally
    S.Free;
  end;
end;

function TCustomIniFile.ReadInteger(const Section, Ident: string;
  Default: Longint): Longint;
var
  IntStr: string;
begin
  IntStr := ReadString(Section, Ident, '');
  if (Length(IntStr) > 2) and (IntStr[1] = '0') and
    ((IntStr[2] = 'X') or (IntStr[2] = 'x')) then
    IntStr := '$' + Copy(IntStr, 3, Maxint);
  Result := StrToIntDef(IntStr, Default);
end;

procedure TCustomIniFile.WriteInteger(const Section, Ident: string; Value: Longint);
begin
  WriteString(Section, Ident, IntToStr(Value));
end;

function TCustomIniFile.ReadBool(const Section, Ident: string;
  Default: Boolean): Boolean;
begin
  Result := ReadInteger(Section, Ident, Ord(Default)) <> 0;
end;

function TCustomIniFile.ReadDate(const Section, Name: string; Default: TDateTime): TDateTime;
var
  DateStr: string;
begin
  DateStr := ReadString(Section, Name, '');
  Result := Default;
  if DateStr <> '' then
  try
    Result := StrToDate(DateStr);
  except
    on EConvertError do
    else raise;
  end;
end;

function TCustomIniFile.ReadDateTime(const Section, Name: string; Default: TDateTime): TDateTime;
var
  DateStr: string;
begin
  DateStr := ReadString(Section, Name, '');
  Result := Default;
  if DateStr <> '' then
  try
    Result := StrToDateTime(DateStr);
  except
    on EConvertError do
    else raise;
  end;
end;

function TCustomIniFile.ReadFloat(const Section, Name: string; Default: Double): Double;
var FloatStr: string;
var OldSeparator:Char;
var i:Integer;
begin
  FloatStr := ReadString(Section, Name, '');
  Result := Default;
  if FloatStr <> '' then
  try
   i:=Pos(',',FloatStr);
   if i<>0 then FloatStr[i]:='.';
   OldSeparator:=DecimalSeparator;
   DecimalSeparator:='.';
   try
    Result := StrToFloat(FloatStr);
   finally
    DecimalSeparator:=OldSeparator;
   end;
  except
    on EConvertError do
    else raise;
  end;
end;

function TCustomIniFile.ReadTime(const Section, Name: string; Default: TDateTime): TDateTime;
var
  TimeStr: string;
begin
  TimeStr := ReadString(Section, Name, '');
  Result := Default;
  if TimeStr <> '' then
  try
    Result := StrToTime(TimeStr);
  except
    on EConvertError do
    else raise;
  end;
end;

procedure TCustomIniFile.WriteDate(const Section, Name: string; Value: TDateTime);
begin
  WriteString(Section, Name, DateToStr(Value));
end;

procedure TCustomIniFile.WriteDateTime(const Section, Name: string; Value: TDateTime);
begin
  WriteString(Section, Name, DateTimeToStr(Value));
end;

procedure TCustomIniFile.WriteFloat(const Section, Name: string; Value: Double);
var OldSeparator:Char;
begin
 OldSeparator:=DecimalSeparator;
 DecimalSeparator:='.';
 try
  WriteString(Section, Name, FloatToStr(Value));
 finally
  DecimalSeparator:=OldSeparator;
 end;
end;

procedure TCustomIniFile.WriteTime(const Section, Name: string; Value: TDateTime);
begin
  WriteString(Section, Name, TimeToStr(Value));
end;

procedure TCustomIniFile.WriteBool(const Section, Ident: string; Value: Boolean);
const
  Values: array[Boolean] of string = ('0', '1');
begin
  WriteString(Section, Ident, Values[Value]);
end;

function TCustomIniFile.ValueExists(const Section, Ident: string): Boolean;
var
  S: TStrings;
begin
  S := TStringList.Create;
  try
    ReadSection(Section, S);
    Result := S.IndexOf(Ident) > -1;
  finally
    S.Free;
  end;
end;

{ TIniFile }

function TIniFile.ReadString(const Section, Ident, Default: string): string;
var
  Buffer: array[0..2047] of Char;
begin
  SetString(Result, Buffer, GetPrivateProfileString(PChar(Section),
    PChar(Ident), PChar(Default), Buffer, SizeOf(Buffer), PChar(FFileName)));
end;

procedure TIniFile.WriteString(const Section, Ident, Value: string);
begin
  if not WritePrivateProfileString(PChar(Section), PChar(Ident),
    PChar(Value), PChar(FFileName)) then
    raise Exception.CreateResFmt(integer(@SIniFileWriteError), [FileName]);
end;

procedure TIniFile.ReadSections(Strings: TStrings);
const
  BufSize = 16384;
var
  Buffer, P: PChar;
begin
  GetMem(Buffer, BufSize);
  try
    Strings.BeginUpdate;
    try
      Strings.Clear;
      if GetPrivateProfileString(nil, nil, nil, Buffer, BufSize,
        PChar(FFileName)) <> 0 then
      begin
        P := Buffer;
        while P^ <> #0 do
        begin
          Strings.Add(P);
          Inc(P, StrLen(P) + 1);
        end;
      end;
    finally
      Strings.EndUpdate;
    end;
  finally
    FreeMem(Buffer, BufSize);
  end;
end;

procedure TIniFile.ReadSection(const Section: string; Strings: TStrings);
const
  BufSize = 16384;
var
  Buffer, P: PChar;
begin
  GetMem(Buffer, BufSize);
  try
    Strings.BeginUpdate;
    try
      Strings.Clear;
      if GetPrivateProfileString(PChar(Section), nil, nil, Buffer, BufSize,
        PChar(FFileName)) <> 0 then
      begin
        P := Buffer;
        while P^ <> #0 do
        begin
          Strings.Add(P);
          Inc(P, StrLen(P) + 1);
        end;
      end;
    finally
      Strings.EndUpdate;
    end;
  finally
    FreeMem(Buffer, BufSize);
  end;
end;

procedure TIniFile.ReadSectionValues(const Section: string; Strings: TStrings);
var
  KeyList: TStringList;
  I: Integer;
begin
  KeyList := TStringList.Create;
  try
    ReadSection(Section, KeyList);
    Strings.BeginUpdate;
    try
      for I := 0 to KeyList.Count - 1 do
        Strings.Values[KeyList[I]] := ReadString(Section, KeyList[I], '');
    finally
      Strings.EndUpdate;
    end;
  finally
    KeyList.Free;
  end;
end;

procedure TIniFile.EraseSection(const Section: string);
begin
  if not WritePrivateProfileString(PChar(Section), nil, nil,
    PChar(FFileName)) then
    raise Exception.CreateResFmt(integer(@SIniFileWriteError), [FileName]);
end;

procedure TIniFile.DeleteKey(const Section, Ident: String);
begin
  WritePrivateProfileString(PChar(Section), PChar(Ident), nil,
     PChar(FFileName));
end;

procedure TIniFile.UpdateFile;
begin
  WritePrivateProfileString(nil, nil, nil, PChar(FFileName));
end;

{ TMemIniFile }

constructor TMemIniFile.Create(const FileName: string);
begin
  inherited Create(FileName);
  FSections := TStringList.Create;
  LoadValues;
  FModified := False
end;

destructor TMemIniFile.Destroy;
begin
  if FSections <> nil then Clear;
  FSections.Free;
  inherited;
end;

function TMemIniFile.AddSection(const Section: string): TStrings;
begin
  Result := TStringList.Create;
  try
    FSections.AddObject(Section, Result);
  except
    Result.Free;
  end;
end;

procedure TMemIniFile.Clear;
var
  I: Integer;
begin
  for I := 0 to FSections.Count - 1 do
    TStrings(FSections.Objects[I]).Free;
  FSections.Clear;
  FModified := True
end;

procedure TMemIniFile.DeleteKey(const Section, Ident: String);
var
  I, J: Integer;
  Strings: TStrings;
begin
  I := FSections.IndexOf(Section);
  if I >= 0 then
  begin
    Strings := TStrings(FSections.Objects[I]);
    J := Strings.IndexOfName(Ident);
    if J >= 0 then begin
      Strings.Delete(J);
      FModified := True
    end
  end;
end;

procedure TMemIniFile.EraseSection(const Section: string);
var
  I: Integer;
begin
  I := FSections.IndexOf(Section);
  if I >= 0 then
  begin
    TStrings(FSections.Objects[I]).Free;
    FSections.Delete(I);
    FModified := True
  end;
end;

procedure TMemIniFile.GetStrings(List: TStrings);
var
  I, J: Integer;
  Strings: TStrings;
begin
  List.BeginUpdate;
  try
    for I := 0 to FSections.Count - 1 do
    begin
      List.Add('[' + FSections[I] + ']');
      Strings := TStrings(FSections.Objects[I]);
      for J := 0 to Strings.Count - 1 do List.Add(Strings[J]);
      List.Add('');
    end;
  finally
    List.EndUpdate;
  end;
end;

procedure TMemIniFile.LoadValues;
var
  List: TStringList;
begin
  if (FileName <> '') and FileExists(FileName) then
  begin
    List := TStringList.Create;
    try
      List.LoadFromFile(FileName);
      SetStrings(List);
    finally
      List.Free;
    end;
  end else Clear;
end;

procedure TMemIniFile.ReadSection(const Section: string;
  Strings: TStrings);
var
  I, J: Integer;
  SectionStrings: TStrings;
begin
  Strings.BeginUpdate;
  try
    Strings.Clear;
    I := FSections.IndexOf(Section);
    if I >= 0 then
    begin
      SectionStrings := TStrings(FSections.Objects[I]);
      for J := 0 to SectionStrings.Count - 1 do
        Strings.Add(SectionStrings.Names[J]);
    end;
  finally
    Strings.EndUpdate;
  end;
end;

procedure TMemIniFile.ReadSections(Strings: TStrings);
begin
  Strings.Assign(FSections);
end;

procedure TMemIniFile.ReadSectionValues(const Section: string;
  Strings: TStrings);
var
  I: Integer;
begin
  Strings.BeginUpdate;
  try
    Strings.Clear;
    I := FSections.IndexOf(Section);
    if I >= 0 then Strings.Assign(TStrings(FSections.Objects[I]));
  finally
    Strings.EndUpdate;
  end;
end;

function TMemIniFile.ReadString(const Section, Ident,
  Default: string): string;
var
  I: Integer;
  Strings: TStrings;
begin
  I := FSections.IndexOf(Section);
  if I >= 0 then
  begin
    Strings := TStrings(FSections.Objects[I]);
    I := Strings.IndexOfName(Ident);
    if I >= 0 then
    begin
      Result := Copy(Strings[I], Length(Ident) + 2, Maxint);
      Exit;
    end;
  end;
  Result := Default;
end;

procedure TMemIniFile.Rename(const FileName: string; Reload: Boolean);
begin
  FFileName := FileName;
  if Reload then LoadValues;
end;

procedure TMemIniFile.SetStrings(List: TStrings);
var
  I: Integer;
  S: string;
  Strings: TStrings;
begin
  Clear;
  Strings := nil;
  for I := 0 to List.Count - 1 do
  begin
    S := List[I];
    if (S <> '') and (S[1] <> ';') then begin
      FModified := True;
      if (S[1] = '[') and (S[Length(S)] = ']') then
        Strings := AddSection(Copy(S, 2, Length(S) - 2))
      else
        if Strings <> nil then Strings.Add(S);
    end
  end;
end;

procedure TMemIniFile.UpdateFile;
var
  List: TStringList;
begin
  List := TStringList.Create;
  try
    GetStrings(List);
    List.SaveToFile(FFileName);
  finally
    List.Free;
  end;
end;

procedure TMemIniFile.WriteString(const Section, Ident, Value: String);
var
  I: Integer;
  S: string;
  Strings: TStrings;
begin
  I := FSections.IndexOf(Section);
  if I >= 0 then
    Strings := TStrings(FSections.Objects[I]) else
    Strings := AddSection(Section);
  S := Ident + '=' + Value;
  I := Strings.IndexOfName(Ident);
  if I >= 0 then Strings[I] := S else Strings.Add(S);
  FModified := True
end;

{-------------------------------------------------}
{$IFDEF USEFILESYS}
constructor TFileSysIniFile.Create;
begin
 inherited Create('');
 FFileName:=AFileName;
 FFileSystem:=AFileSystem;
 FileSystem.Use;
 LoadValues;
 FModified := False
end;

destructor TFileSysIniFile.Destroy;
begin
 try
  UpdateFile
 finally
  FileSystem.Release
 end;
 inherited
end;

procedure TFileSysIniFile.UpdateFile;
var
  List: TStringList;
  stream:TStream;
begin
  if Modified then begin
    List := TStringList.Create;
    try
      stream:=FileSystem.GetFileStream(FileName,fmCreate);
      try
       GetStrings(List);
       List.SaveToStream(stream)
      finally
       stream.Free
      end
    finally
      List.Free;
    end;
  end
end;

procedure TFileSysIniFile.LoadValues;
var
  List: TStringList;
  stream:TStream;
begin
  if FileSystem.FileExists(FileName) then
   try
    stream:=FileSystem.GetFileStream(FileName,fmOpenReadWrite);
    try
     List:=TStringList.Create;
     try
      List.LoadFromStream(stream);
      SetStrings(List)
     finally
      List.Free
     end
    finally
     stream.Free
    end;
   except
    Clear
   end
  else
   Clear;
  Modified:=False  
end;
{$ENDIF}

procedure IniFileCopy;
var sections:TStrings;
var keys:TStrings;
var i:integer;

 procedure CopySection(section:string);
 var i:integer;
 begin
  keys.Clear;
  source.ReadSection(section,keys);
  with keys do
   for i:=Count-1 downto 0 do
    destination.WriteString(section,keys[i],source.ReadString(section,keys[i],''))
 end;

begin
 sections:=TStringList.Create;
 keys:=TStringList.Create;

 Destination.ReadSections(sections);
 for i:=sections.Count-1 downto 0 do Destination.EraseSection(sections[i]);

 sections.Clear;
 source.ReadSections(sections);
 for i:=sections.Count-1 downto 0 do CopySection(sections[i]);

 sections.Free;
 keys.Free
end;

procedure IniFileCopySection;

 procedure CopySection(section:string);
 var keys:TStrings;
 var i:integer;
 begin
  keys:=TStringList.Create;
  source.ReadSection(section,keys);
  with keys do begin
   for i:=Count-1 downto 0 do
    destination.WriteString(section,keys[i],source.ReadString(section,keys[i],''));
   Free
  end
 end;

begin
 if source=destination then exit;

 Destination.EraseSection(section);
 CopySection(section);
end;

constructor TUplnePametovyIni.Create;
begin
 inherited Create('')
end;

procedure TUplnePametovyIni.UpdateFile;
begin
end;

function TEnum.NameToID;
var i:integer;
begin
 i:=IndexOf(name);
 if i<0 then
  Result:=Default
 else
  Result:=integer(Objects[i])
end;

function TEnum.IDToName;
var i:integer;
begin
 i:=IndexOfObject(TObject(id));
 if i<0 then
  Result:=Default
 else
  Result:=Strings[i]
end;

function TEnum.CommaText:string;
var i:integer;
begin
  Result:='';
  for i:=0 to Count-1 do begin
    if i>0 then
      Result:=Result+',';
    Result:=Result+Format('%s=%d',[Strings[i],integer(Objects[i])])
  end
end;

constructor TEnum.Create;
var i:integer;
var id:integer;
var item:string;
var list:string;
begin
 list:=values;
 id:=-1;
 while length(list)>0 do begin
  i:=Pos(',',list);
  if i=0 then begin
   item:=list;
   list:=''
  end else begin
   item:=Copy(list,1,i-1);
   System.Delete(list,1,i)
  end;
  i:=Pos('=',item);
  if i=0 then
   inc(id)
  else begin
   id:=StrToInt(Copy(item,i+1,maxint));
   SetLength(item,i-1)
  end;
  AddObject(item,TObject(id))
 end;
 Sorted:=true
end;

const noDefault=#1;

function TCustomIniFile.ReadEnum(const Section,Ident:string; Default: Longint; values:TEnum):Longint;
var tmp:string;
begin
 tmp:=ReadString(Section,Ident,noDefault);
 Result:=Default;
 if tmp=noDefault then exit;
 Result:=values.NameToID(tmp,Default)
end;

procedure TCustomIniFile.WriteEnum(const Section,Ident:string; Value: Longint; values:TEnum);
begin
 WriteString(Section,Ident,values.IDToName(Value,''))
end;

function ReadEnum(ini:TCustomIniFile; const Section,Ident:string; Default: Longint; const values:string):Longint;
var enum:TEnum;
begin
 enum:=TEnum.Create(values);
 try
  Result:=ini.ReadEnum(Section,Ident,Default,enum)
 finally
  enum.Free
 end
end;

procedure WriteEnum(ini:TCustomIniFile; const Section,Ident:string; Value: Longint; const values:string);
var enum:TEnum;
begin
 enum:=TEnum.Create(values);
 try
  ini.WriteEnum(Section,Ident,Value,enum)
 finally
  enum.Free
 end
end;


end.
