{ KEYWORDS: VERSION,TUNE,TODO }
{ VERSION: 1.3
 This file implements extensions around Borland's Registry.pas.

  Features:
  - TRegistry redirectable to TRegAPI database
  - TVBReg provides simple interface typically used by VB programmers

 1.3 / 13.12.2004
 - compatible with Delphi6
 1.2 / 3.1.2004
 - even more Delphi7 compatible - implements TRegistryIniFile
 1.1 / 6.6.2003
 - minor bug fixes for Delphi7
 1.0 / 9.5.2003
 - Delphi7 compatible
 0.9 / 23.8.2002
 - fixed incorrect behaviour in TRegistry.MoveKey. Default values were always
   coppied even if they were not set
 0.8 / 22.8.2002
 - fixed bug in TRegistry.DeleteKey. Sometimes some subkeys were not deleted
 0.7
 - TRegistry.OpenKey() left as it was (compatibility trick) and extended
   with TRegistry.OpenKeyEx() 
 0.6
 - RegReadString
 0.5
 - default property TVBReg.Values[]
 0.4
 - RegRead,RegWrite,RegDelete,RegExists wrappers around TVBReg
 0.3
 - TVBReg simplifier using Variants
 - SetCurrentKey,GetCurrentKey navigator functions for use with TRegistry.
   used key string includes both CurrentPath and RootKey.
 0.2
 - TRegistry.GetFileStream()
 - enhanced functions ( RegistryConnect, RestoreKey, .. ) are supported for
   OSRegApi database redirector.
 0.1
 - extended D3 original with TRegistry.OpenKeyEx, and D5 compatible
   TRegistry.OpenKeyReadOnly
 - internal key openings respect readonly access
}

{ TUNE: if defined, registry workers will be automatically redirected to
  RegApi.defaultRegApi or explicitly to other TRegApi database using constructor
  CreateUsing() }
{ $DEFINE USEREGAPI}

{$IFDEF VER140}
 {$DEFINE VER150}
{$ENDIF}

{$IFDEF VER150}
 {$DEFINE REGISTRYINIFILE}
{$ELSE}
 {$UNDEF REGISTRYINIFILE}
{$ENDIF}

unit Registry;

{$R-}

interface

uses Windows, Classes, SysUtils
     {$IFDEF USEREGAPI},RegAPI{$ENDIF}
     {$IFDEF REGISTRYINIFILE},IniFiles{$ENDIF};

type
  ERegistryException = class(Exception);

  TRegKeyInfo = record
    NumSubKeys: Integer;
    MaxSubKeyLen: Integer;
    NumValues: Integer;
    MaxValueLen: Integer;
    MaxDataLen: Integer;
    FileTime: TFileTime;
  end;

  TRegDataType = (rdUnknown, rdString, rdExpandString, rdInteger, rdBinary);

  TRegDataInfo = record
    RegData: TRegDataType;
    DataSize: Integer;
  end;

{$IFDEF USEREGAPI}
  TRegDB=TRegAPI;
{$ENDIF}
  TRegistry = class(TObject)
  private
    FCurrentKey: HKEY;
    FRootKey: HKEY;
    FLazyWrite: Boolean;
    FCurrentPath: string;
    FCloseRootKey: Boolean;
    procedure SetRootKey(Value: HKEY);
  protected
    procedure ChangeKey(Value: HKey; const Path: string);
    function GetBaseKey(Relative: Boolean): HKey;
    function GetData(const Name: string; Buffer: Pointer;
      BufSize: Integer; var RegData: TRegDataType): Integer;
    function GetKeyEx(const Key: string; readonly:boolean): HKEY;
    function GetKey(const Key: string): HKEY;
    procedure PutData(const Name: string; Buffer: Pointer; BufSize: Integer; RegData: TRegDataType);
    procedure SetCurrentKey(Value: HKEY);
  public
    constructor Create;
    {$IFDEF USEREGAPI}
    constructor CreateUsing(ADB:TRegDB);
    {$ENDIF}
    destructor Destroy; override;
    procedure CloseKey;
    function CreateKey(const Key: string): Boolean;
    function DeleteKey(const Key: string): Boolean;
    function DeleteValue(const Name: string): Boolean;
    function GetDataInfo(const ValueName: string; var Value: TRegDataInfo): Boolean;
    function GetDataSize(const ValueName: string): Integer;
    function GetDataType(const ValueName: string): TRegDataType;
    function GetKeyInfo(var Value: TRegKeyInfo): Boolean;
    procedure GetKeyNames(Strings: TStrings);
    procedure GetValueNames(Strings: TStrings);
    function HasSubKeys: Boolean;
    function KeyExists(const Key: string): Boolean;
    function LoadKey(const Key, FileName: string): Boolean;
    procedure MoveKey(const OldName, NewName: string; Delete: Boolean);
    function OpenKey(const Key: string; CanCreate: Boolean): Boolean;
    function OpenKeyEx(const Key: string; CanCreate: Boolean; readonly:boolean): Boolean;
    function OpenKeyReadOnly(const Key:string):boolean;
    function ReadCurrency(const Name: string): Currency;
    function ReadBinaryData(const Name: string; var Buffer; BufSize: Integer): Integer;
    function ReadBool(const Name: string): Boolean;
    function ReadDate(const Name: string): TDateTime;
    function ReadDateTime(const Name: string): TDateTime;
    function ReadFloat(const Name: string): Double;
    function ReadInteger(const Name: string): Integer;
    function ReadString(const Name: string): string;
    function ReadTime(const Name: string): TDateTime;
    function RegistryConnect(const UNCName: string): Boolean;
    procedure RenameValue(const OldName, NewName: string);
    function ReplaceKey(const Key, FileName, BackUpFileName: string): Boolean;
    function RestoreKey(const Key, FileName: string): Boolean;
    function SaveKey(const Key, FileName: string): Boolean;
    function UnLoadKey(const Key: string): Boolean;
    function ValueExists(const Name: string): Boolean;
    procedure WriteCurrency(const Name: string; Value: Currency);
    procedure WriteBinaryData(const Name: string; var Buffer; BufSize: Integer);
    procedure WriteBool(const Name: string; Value: Boolean);
    procedure WriteDate(const Name: string; Value: TDateTime);
    procedure WriteDateTime(const Name: string; Value: TDateTime);
    procedure WriteFloat(const Name: string; Value: Double);
    procedure WriteInteger(const Name: string; Value: Integer);
    procedure WriteString(const Name, Value: string);
    procedure WriteExpandString(const Name, Value: string);
    procedure WriteTime(const Name: string; Value: TDateTime);
    property CurrentKey: HKEY read FCurrentKey;
    property CurrentPath: string read FCurrentPath;
    property LazyWrite: Boolean read FLazyWrite write FLazyWrite;
    property RootKey: HKEY read FRootKey write SetRootKey;
  public
    function GetFileStream(const Name:string; Mode:integer):TStream;
  {$IFDEF USEREGAPI}
  protected
    FDB:TRegDB;
    procedure UseDB(ADB:TRegDB);
    function OSLink:boolean;
  public
    property DB:TRegDB read FDB write UseDB;
  {$ENDIF}
  end;

  TRegIniFile = class(TRegistry)
  private
    FFileName: string;
  public
    constructor Create(const FileName: string);
    {$IFDEF USEREGAPI}
    constructor CreateUsing(const FileName:string; ADB:TRegDB);
    {$ENDIF}
    function ReadString(const Section, Ident, Default: string): string;
    procedure WriteString(const Section, Ident, Value: String);
    function ReadInteger(const Section, Ident: string;
      Default: Longint): Longint;
    procedure WriteInteger(const Section, Ident: string; Value: Longint);
    function ReadBool(const Section, Ident: string; Default: Boolean): Boolean;
    procedure WriteBool(const Section, Ident: string; Value: Boolean);
    procedure ReadSection(const Section: string; Strings: TStrings);
    procedure ReadSections(Strings: TStrings);
    procedure ReadSectionValues(const Section: string; Strings: TStrings);
    procedure EraseSection(const Section: string);
    procedure DeleteKey(const Section, Ident: String);
    property FileName: string read FFileName;
  end;

  {$IFDEF REGISTRYINIFILE}
  TRegistryIniFile = class(TCustomIniFile)
  private
    FRegIniFile: TRegIniFile;
  public
    constructor Create(const FileName: string); overload;
    constructor Create(const FileName: string; AAccess: LongWord); overload;
    {$IFDEF USEREGAPI}
    constructor CreateUsing(const FileName:string; ADB:TRegDB);
    {$ENDIF}
    destructor Destroy; override;
    function ReadDate(const Section, Name: string; Default: TDateTime): TDateTime; override;
    function ReadDateTime(const Section, Name: string; Default: TDateTime): TDateTime; override;
    function ReadInteger(const Section, Ident: string; Default: Longint): Longint; override;
    function ReadFloat(const Section, Name: string; Default: Double): Double; override;
    function ReadString(const Section, Ident, Default: string): string; override;
    function ReadTime(const Section, Name: string; Default: TDateTime): TDateTime; override;
    function ReadBinaryStream(const Section, Name: string; Value: TStream): Integer; override;
//    procedure ReadKeys(const Section: string; Sections: TStrings);
    procedure WriteDate(const Section, Name: string; Value: TDateTime); override;
    procedure WriteDateTime(const Section, Name: string; Value: TDateTime); override;
    procedure WriteFloat(const Section, Name: string; Value: Double); override;
    procedure WriteInteger(const Section, Ident: string; Value: Longint); override;
    procedure WriteString(const Section, Ident, Value: String); override;
    procedure WriteTime(const Section, Name: string; Value: TDateTime); override;
    procedure WriteBinaryStream(const Section, Name: string; Value: TStream); override;
    procedure ReadSection(const Section: string; Strings: TStrings); override;
    procedure ReadSections(Strings: TStrings); override;
    procedure ReadSectionValues(const Section: string; Strings: TStrings); override;
    procedure EraseSection(const Section: string); override;
    procedure DeleteKey(const Section, Ident: String); override;
    procedure UpdateFile; override;

    property RegIniFile: TRegIniFile read FRegIniFile;
  end;
  {$ENDIF}

  TVBReg=class
  protected
   {$IFDEF USEREGAPI}
   DB:TRegDB;
   {$ENDIF}
   function CreateHandler(const path:string):TObject;
   procedure DoWrite(const path:string; const value:OleVariant);
  public
   destructor Destroy;override;
  public
   constructor Create;
   {$IFDEF USEREGAPI}
   constructor CreateUsing(ADB:TRegDB);
   {$ENDIF}
  public
   function Read(const path:string):OleVariant;
   function Write(const path:string; const value:OleVariant):boolean;
   function Delete(const path:string):boolean;
   function Exists(const path:string):boolean;
   property Values[const path:string]:OleVariant read Read write DoWrite;default;
  end;

 { TVBReg shortcuts }
 function RegRead(const path:string):OleVariant;
 function RegWrite(const path:string; const value:OleVariant):boolean;
 function RegDelete(const path:string):boolean;
 function RegExists(const path:string):boolean;

 function RegReadString(const path:string):string;

 { registry path helpers }
 function SetCurrentKey(reg:TRegistry; const path:string; Mode:Integer):boolean;
 function GetCurrentKey(reg:TRegistry):string;

{ examples using VBReg

 // write to default value
 RegWrite('HKCU\Software\First Information Systems\',3)

 // create keys and write text value
 RegWrite('HKLM\Software\Microsoft\Windows\CurrentVersion\Uninstall\ActiveChart\DisplayName','hallo 3D')

 // remove associations for files with KUK extension. (both ext key and its shell
 // definition key)
 if RegExists('HKCR\.kuk\') then begin
  RegDelete('HKCR\'+RegRead('HKCR\.kuk\')+'\')
  RegDelete('HKCR\.kuk\')
 end

 // delete value
 RegDelete('HKCU\Software\First Information Systems\ActiveChart\OpenFilePath')
 // delete value by assignment
 RegWrite('HKCU\Software\First Information Systems\ActiveChart\OpenFilePath',Unassigned)

 var reg1,reg2:TRegistry;

 // navigate reg1 worker to given position, create key if necessary
 SetCurrentKey(reg1,'HKCU\Software\First Information Systems\',fmOpenReadWrite)

 // navigate reg2 worker at the same place where worker reg1 currently stands and
 // request read-only access
 SetCurrentKey(reg2,GetCurrentKey(reg1),fmOpenRead)

}

implementation

uses Consts{$IFDEF VER150},RTLConsts,Variants{$ENDIF};

procedure ReadError(const Name: string);
begin
  raise ERegistryException.CreateFmt(SInvalidRegType, [Name]);
end;

function IsRelative(const Value: string): Boolean;
begin
  Result := not ((Value <> '') and (Value[1] = '\'));
end;

function RegDataToDataType(Value: TRegDataType): Integer;
begin
  case Value of
    rdString: Result := REG_SZ;
    rdExpandString: Result := REG_EXPAND_SZ;
    rdInteger: Result := REG_DWORD;
    rdBinary: Result := REG_BINARY;
  else
    Result := REG_NONE;
  end;
end;

function DataTypeToRegData(Value: Integer): TRegDataType;
begin
  if Value = REG_SZ then Result := rdString
  else if Value = REG_EXPAND_SZ then Result := rdExpandString
  else if Value = REG_DWORD then Result := rdInteger
  else if Value = REG_BINARY then Result := rdBinary
  else Result := rdUnknown;
end;

constructor TRegistry.Create;
begin
  {$IFDEF USEREGAPI}
  FDB:=defaultRegApi;
  FDB.Use;
  {$ENDIF}
  RootKey := HKEY_CURRENT_USER;
  LazyWrite := True;
end;

{$IFDEF USEREGAPI}
constructor TRegistry.CreateUsing;
begin
  FDB:=ADB;
  if not Assigned(FDB) then FDB:=defaultRegApi;
  FDB.Use;
  RootKey:=HKEY_CURRENT_USER;
  LazyWrite:=true
end;

function TRegistry.OSLink;
begin
 Result:=DB=OSRegApi
end;

procedure TRegistry.UseDB;
begin
  if ADB=FDB then exit;

  CloseKey;

  FDB.Release;
  FDB:=ADB;
  if not Assigned(FDB) then FDB:=defaultRegApi;
  FDB.Use;

  // TODO: reopen the key 
end;
{$ENDIF}

destructor TRegistry.Destroy;
begin
  CloseKey;
  {$IFDEF USEREGAPI}
  FDB.Release;
  {$ENDIF}
  inherited;
end;

procedure TRegistry.CloseKey;
begin
  if CurrentKey <> 0 then
  begin
    if LazyWrite then
      {$IFDEF USEREGAPI}
      DB.CloseKey(CurrentKey) else
      DB.FlushKey(CurrentKey);
      {$ELSE}
      RegCloseKey(CurrentKey) else
      RegFlushKey(CurrentKey);
      {$ENDIF}
    FCurrentKey := 0;
    FCurrentPath := '';
  end;
end;

procedure TRegistry.SetRootKey(Value: HKEY);
begin
  if RootKey <> Value then
  begin
    if FCloseRootKey then
    begin
      {$IFDEF USEREGAPI}
      DB.CloseKey(RootKey);
      {$ELSE}
      RegCloseKey(RootKey);
      {$ENDIF}
      FCloseRootKey := False;
    end;
    FRootKey := Value;
    CloseKey;
  end;
end;

procedure TRegistry.ChangeKey(Value: HKey; const Path: string);
begin
  CloseKey;
  FCurrentKey := Value;
  FCurrentPath := Path;
end;

function TRegistry.GetBaseKey(Relative: Boolean): HKey;
begin
  if (CurrentKey = 0) or not Relative then
    Result := RootKey else
    Result := CurrentKey;
end;

procedure TRegistry.SetCurrentKey(Value: HKEY);
begin
  FCurrentKey := Value;
end;

function TRegistry.CreateKey(const Key: string): Boolean;
var
  TempKey: HKey;
  S: string;
  Disposition: Integer;
  Relative: Boolean;
begin
  TempKey := 0;
  S := Key;
  Relative := IsRelative(S);
  if not Relative then Delete(S, 1, 1);
  {$IFDEF USEREGAPI}
  Result := DB.CreateKey(GetBaseKey(Relative), S, '',
    REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, nil, TempKey, @Disposition) = ERROR_SUCCESS;
  if Result then DB.CloseKey(TempKey)
  {$ELSE}
  Result := RegCreateKeyEx(GetBaseKey(Relative), PChar(S), 0, nil,
    REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, nil, TempKey, @Disposition) = ERROR_SUCCESS;
  if Result then RegCloseKey(TempKey)
  {$ENDIF}
  else raise ERegistryException.CreateFmt(SRegCreateFailed, [Key]);
end;

function TRegistry.OpenKeyReadOnly(const Key:string):boolean;
begin
 Result:=OpenKeyEx(Key,false,true)
end;

function TRegistry.OpenKey(const Key:string; CanCreate:boolean):boolean;
begin
 Result:=OpenKeyEx(Key,CanCreate,false)
end;

function TRegistry.OpenKeyEx(const Key: string; CanCreate: Boolean; ReadOnly:Boolean): Boolean;
var
  TempKey: HKey;
  S: string;
  Disposition: Integer;
  Relative: Boolean;
  sam:REGSAM;
begin
  if ReadOnly then sam:=KEY_READ else sam:=KEY_ALL_ACCESS;
  S := Key;
  Relative := IsRelative(S);
  if not Relative then Delete(S, 1, 1);
  TempKey := 0;
  if not CanCreate or (S = '') then
  begin
    {$IFDEF USEREGAPI}
    Result := DB.OpenKey(GetBaseKey(Relative),S, 0,
      sam, TempKey) = ERROR_SUCCESS;
    {$ELSE}
    Result := RegOpenKeyEx(GetBaseKey(Relative), PChar(S), 0,
      sam, TempKey) = ERROR_SUCCESS;
    {$ENDIF}
  end else
    {$IFDEF USEREGAPI}
    Result := DB.CreateKey(GetBaseKey(Relative), S, '',
      REG_OPTION_NON_VOLATILE, sam, nil, TempKey, @Disposition) = ERROR_SUCCESS;
    {$ELSE}
    Result := RegCreateKeyEx(GetBaseKey(Relative), PChar(S), 0, nil,
      REG_OPTION_NON_VOLATILE, sam, nil, TempKey, @Disposition) = ERROR_SUCCESS;
    {$ENDIF}
  if Result then
  begin
    if (CurrentKey <> 0) and Relative then S := CurrentPath + '\' + S;
    ChangeKey(TempKey, S);
  end;
end;

function TRegistry.DeleteKey(const Key: string): Boolean;
var
  Len: {$IFDEF VER150}Cardinal{$ELSE}Integer{$ENDIF};
  Relative: Boolean;
  S, KeyName: string;
  OldKey, DeleteKey: HKEY;
  Info: TRegKeyInfo;
begin
  S := Key;
  Relative := IsRelative(S);
  if not Relative then Delete(S, 1, 1);
  OldKey := CurrentKey;
  DeleteKey := GetKeyEx(Key,false);
  if DeleteKey <> 0 then
  try
    SetCurrentKey(DeleteKey);
    if GetKeyInfo(Info) then
    begin
      SetString(KeyName, nil, Info.MaxSubKeyLen + 1);
      while true do begin
       Len:=Info.MaxSubKeyLen+1;
       case
        {$IFDEF USEREGAPI}DB.EnumKey(DeleteKey, 0, PChar(KeyName), @Len, nil, nil, nil)
        {$ELSE}RegEnumKeyEx(DeleteKey, 0, PChar(KeyName), Len, nil, nil, nil, nil){$ENDIF}
       of
       ERROR_SUCCESS: Self.DeleteKey(PChar(KeyName));
       ERROR_MORE_DATA: if GetKeyInfo(Info) then SetString(KeyName,nil,Info.MaxSubKeyLen+1);
       else break
       end
      end;
      (*
      for I := 0 to Info.NumSubKeys - 1 do
      begin
        Len := Info.MaxSubKeyLen + 1;
        {$IFDEF USEREGAPI}
        if DB.EnumKey(DeleteKey, I, PChar(KeyName), @Len, nil, nil, nil) = ERROR_SUCCESS then
          Self.DeleteKey(PChar(KeyName));
        {$ELSE}
        if RegEnumKeyEx(DeleteKey, I, PChar(KeyName), Len, nil, nil, nil, nil) = ERROR_SUCCESS then
          Self.DeleteKey(PChar(KeyName));
        {$ENDIF}
      end;
      *)
    end;
  finally
    SetCurrentKey(OldKey);
    {$IFDEF USEREGAPI}
    DB.CloseKey(DeleteKey);
    {$ELSE}
    RegCloseKey(DeleteKey);
    {$ENDIF}
  end;
  {$IFDEF USEREGAPI}
  Result := DB.DeleteKey(GetBaseKey(Relative), S) = ERROR_SUCCESS;
  {$ELSE}
  Result := RegDeleteKey(GetBaseKey(Relative), PChar(S)) = ERROR_SUCCESS;
  {$ENDIF}
end;

function TRegistry.DeleteValue(const Name: string): Boolean;
begin
  {$IFDEF USEREGAPI}
  Result := DB.DeleteValue(CurrentKey, Name) = ERROR_SUCCESS;
  {$ELSE}
  Result := RegDeleteValue(CurrentKey, PChar(Name)) = ERROR_SUCCESS;
  {$ENDIF}
end;

function TRegistry.GetKeyInfo(var Value: TRegKeyInfo): Boolean;
begin
  FillChar(Value, SizeOf(TRegKeyInfo), 0);
  {$IFDEF USEREGAPI}
  Result := DB.QueryInfo(CurrentKey, nil, nil, @Value.NumSubKeys,
    @Value.MaxSubKeyLen, nil, @Value.NumValues, @Value.MaxValueLen,
    @Value.MaxDataLen, nil, @Value.FileTime) = ERROR_SUCCESS;
  {$ELSE}
  Result := RegQueryInfoKey(CurrentKey, nil, nil, nil, @Value.NumSubKeys,
    @Value.MaxSubKeyLen, nil, @Value.NumValues, @Value.MaxValueLen,
    @Value.MaxDataLen, nil, @Value.FileTime) = ERROR_SUCCESS;
  {$ENDIF}
end;

procedure TRegistry.GetKeyNames(Strings: TStrings);
var
  I:integer;
  Len: {$IFDEF VER150}Cardinal{$ELSE}Integer{$ENDIF};
  Info: TRegKeyInfo;
  S: string;
begin
  Strings.Clear;
  if GetKeyInfo(Info) then
  begin
    SetString(S, nil, Info.MaxSubKeyLen + 1);
    for I := 0 to Info.NumSubKeys - 1 do
    begin
      Len := Info.MaxSubKeyLen + 1;
      {$IFDEF USEREGAPI}
      DB.EnumKey(CurrentKey, I, PChar(S), @Len, nil, nil, nil);
      {$ELSE}
      RegEnumKeyEx(CurrentKey, I, PChar(S), Len, nil, nil, nil, nil);
      {$ENDIF}
      Strings.Add(PChar(S));
    end;
  end;
end;

procedure TRegistry.GetValueNames(Strings: TStrings);
var
  I:integer;
  Len: {$IFDEF VER150}Cardinal{$ELSE}Integer{$ENDIF};
  Info: TRegKeyInfo;
  S: string;
begin
  Strings.Clear;
  if GetKeyInfo(Info) then
  begin
    SetString(S, nil, Info.MaxValueLen + 1);
    for I := 0 to Info.NumValues - 1 do
    begin
      Len := Info.MaxValueLen + 1;
      {$IFDEF USEREGAPI}
      DB.EnumValue(CurrentKey, I, PChar(S), @Len, nil, nil, nil);
      {$ELSE}
      RegEnumValue(CurrentKey, I, PChar(S), Len, nil, nil, nil, nil);
      {$ENDIF}
      Strings.Add(PChar(S));
    end;
  end;
end;

function TRegistry.GetDataInfo(const ValueName: string; var Value: TRegDataInfo): Boolean;
var
  DataType: Integer;
begin
  FillChar(Value, SizeOf(TRegDataInfo), 0);
  {$IFDEF USEREGAPI}
  Result := DB.QueryValue(CurrentKey, ValueName, @DataType, nil,
    @Value.DataSize) = ERROR_SUCCESS;
  {$ELSE}
  Result := RegQueryValueEx(CurrentKey, PChar(ValueName), nil, @DataType, nil,
    @Value.DataSize) = ERROR_SUCCESS;
  {$ENDIF}
  Value.RegData := DataTypeToRegData(DataType);
end;

function TRegistry.GetDataSize(const ValueName: string): Integer;
var
  Info: TRegDataInfo;
begin
  if GetDataInfo(ValueName, Info) then
    Result := Info.DataSize else
    Result := -1;
end;

function TRegistry.GetDataType(const ValueName: string): TRegDataType;
var
  Info: TRegDataInfo;
begin
  if GetDataInfo(ValueName, Info) then
    Result := Info.RegData else
    Result := rdUnknown;
end;

procedure TRegistry.WriteString(const Name, Value: string);
begin
  PutData(Name, PChar(Value), Length(Value), rdString);
end;

procedure TRegistry.WriteExpandString(const Name, Value: string);
begin
  PutData(Name, PChar(Value), Length(Value), rdExpandString);
end;

function TRegistry.ReadString(const Name: string): string;
var
  Len: Integer;
  RegData: TRegDataType;
begin
  Len := GetDataSize(Name);
  if Len > 0 then
  begin
    SetString(Result, nil, Len);
    GetData(Name, PChar(Result), Len, RegData);
    if (RegData = rdString) or (RegData = rdExpandString) then
      SetLength(Result, StrLen(PChar(Result)))
    else ReadError(Name);
  end
  else Result := '';
end;

procedure TRegistry.WriteInteger(const Name: string; Value: Integer);
begin
  PutData(Name, @Value, SizeOf(Integer), rdInteger);
end;

function TRegistry.ReadInteger(const Name: string): Integer;
var
  RegData: TRegDataType;
begin
  GetData(Name, @Result, SizeOf(Integer), RegData);
  if RegData <> rdInteger then ReadError(Name);
end;

procedure TRegistry.WriteBool(const Name: string; Value: Boolean);
begin
  WriteInteger(Name, Ord(Value));
end;

function TRegistry.ReadBool(const Name: string): Boolean;
begin
  Result := ReadInteger(Name) <> 0;
end;

procedure TRegistry.WriteFloat(const Name: string; Value: Double);
begin
  PutData(Name, @Value, SizeOf(Double), rdBinary);
end;

function TRegistry.ReadFloat(const Name: string): Double;
var
  Len: Integer;
  RegData: TRegDataType;
begin
  Len := GetData(Name, @Result, SizeOf(Double), RegData);
  if (RegData <> rdBinary) or (Len <> SizeOf(Double)) then
    ReadError(Name);
end;

procedure TRegistry.WriteCurrency(const Name: string; Value: Currency);
begin
  PutData(Name, @Value, SizeOf(Currency), rdBinary);
end;

function TRegistry.ReadCurrency(const Name: string): Currency;
var
  Len: Integer;
  RegData: TRegDataType;
begin
  Len := GetData(Name, @Result, SizeOf(Currency), RegData);
  if (RegData <> rdBinary) or (Len <> SizeOf(Currency)) then
    ReadError(Name);
end;

procedure TRegistry.WriteDateTime(const Name: string; Value: TDateTime);
begin
  PutData(Name, @Value, SizeOf(TDateTime), rdBinary);
end;

function TRegistry.ReadDateTime(const Name: string): TDateTime;
var
  Len: Integer;
  RegData: TRegDataType;
begin
  Len := GetData(Name, @Result, SizeOf(TDateTime), RegData);
  if (RegData <> rdBinary) or (Len <> SizeOf(TDateTime)) then
    ReadError(Name);
end;

procedure TRegistry.WriteDate(const Name: string; Value: TDateTime);
begin
  WriteDateTime(Name, Value);
end;

function TRegistry.ReadDate(const Name: string): TDateTime;
begin
  Result := ReadDateTime(Name);
end;

procedure TRegistry.WriteTime(const Name: string; Value: TDateTime);
begin
  WriteDateTime(Name, Value);
end;

function TRegistry.ReadTime(const Name: string): TDateTime;
begin
  Result := ReadDateTime(Name);
end;

procedure TRegistry.WriteBinaryData(const Name: string; var Buffer; BufSize: Integer);
begin
  PutData(Name, @Buffer, BufSize, rdBinary);
end;

function TRegistry.ReadBinaryData(const Name: string; var Buffer; BufSize: Integer): Integer;
var
  RegData: TRegDataType;
  Info: TRegDataInfo;
begin
  if GetDataInfo(Name, Info) then
  begin
    Result := Info.DataSize;
    RegData := Info.RegData;
    if (RegData = rdBinary) and (Result <= BufSize) then
      GetData(Name, @Buffer, Result, RegData)
    else ReadError(Name);
  end else
    Result := 0;
end;

procedure TRegistry.PutData(const Name: string; Buffer: Pointer;
  BufSize: Integer; RegData: TRegDataType);
var
  DataType: Integer;
begin
  DataType := RegDataToDataType(RegData);
  {$IFDEF USEREGAPI}
  if DB.SetValue(CurrentKey, Name, DataType, Buffer,
    BufSize) <> ERROR_SUCCESS then
  {$ELSE}
  if RegSetValueEx(CurrentKey, PChar(Name), 0, DataType, Buffer,
    BufSize) <> ERROR_SUCCESS then
  {$ENDIF}
    raise ERegistryException.CreateFmt(SRegSetDataFailed, [Name]);
end;

function TRegistry.GetData(const Name: string; Buffer: Pointer;
  BufSize: Integer; var RegData: TRegDataType): Integer;
var
  DataType: Integer;
begin
  DataType := REG_NONE;
  {$IFDEF USEREGAPI}
  if DB.QueryValue(CurrentKey, Name, @DataType, Buffer,
    @BufSize) <> ERROR_SUCCESS then
  {$ELSE}
  if RegQueryValueEx(CurrentKey, PChar(Name), nil, @DataType, PByte(Buffer),
    @BufSize) <> ERROR_SUCCESS then
  {$ENDIF}
    raise ERegistryException.CreateFmt(SRegGetDataFailed, [Name]);
  Result := BufSize;
  RegData := DataTypeToRegData(DataType);
end;

function TRegistry.HasSubKeys: Boolean;
var
  Info: TRegKeyInfo;
begin
  Result := GetKeyInfo(Info) and (Info.NumSubKeys > 0);
end;

function TRegistry.ValueExists(const Name: string): Boolean;
var
  Info: TRegDataInfo;
begin
  Result := GetDataInfo(Name, Info);
end;

function TRegistry.GetKey;
begin
 Result:=GetKeyEx(Key,false)
end;

function TRegistry.GetKeyEx(const Key: string; readonly:boolean): HKEY;
var
  S: string;
  Relative: Boolean;
  sam:REGSAM;
begin
  if ReadOnly then sam:=KEY_READ else sam:=KEY_ALL_ACCESS;
  S := Key;
  Relative := IsRelative(S);
  if not Relative then Delete(S, 1, 1);
  Result := 0;
  {$IFDEF USEREGAPI}
  DB.OpenKey(GetBaseKey(Relative), S, 0,
    sam, Result);
  {$ELSE}
  RegOpenKeyEx(GetBaseKey(Relative), PChar(S), 0,
    sam, Result);
  {$ENDIF}
end;

function TRegistry.RegistryConnect(const UNCName: string): Boolean;
var
  TempKey: HKEY;
begin
  {$IFDEF USEREGAPI}
  if not OSLink then begin Result:=false; exit end;
  {$ENDIF}
  Result := RegConnectRegistry(PChar(UNCname), RootKey, TempKey) = ERROR_SUCCESS;
  if Result then
  begin
    RootKey := TempKey;
    FCloseRootKey := True;
  end;
end;

function TRegistry.LoadKey(const Key, FileName: string): Boolean;
var
  S: string;
begin
  {$IFDEF USEREGAPI}
  if not OSLink then begin Result:=false; exit end;
  {$ENDIF}
  S := Key;
  if not IsRelative(S) then Delete(S, 1, 1);
  Result := RegLoadKey(RootKey, PChar(S), PChar(FileName)) = ERROR_SUCCESS;
end;

function TRegistry.UnLoadKey(const Key: string): Boolean;
var
  S: string;
begin
  {$IFDEF USEREGAPI}
  if not OSLink then begin Result:=false; exit end;
  {$ENDIF}
  S := Key;
  if not IsRelative(S) then Delete(S, 1, 1);
  Result := RegUnLoadKey(RootKey, PChar(S)) = ERROR_SUCCESS;
end;

function TRegistry.RestoreKey(const Key, FileName: string): Boolean;
var
  RestoreKey: HKEY;
begin
  Result := False;
  {$IFDEF USEREGAPI}
  if not OSLink then exit;
  {$ENDIF}
  RestoreKey := GetKeyEx(Key,false);
  if RestoreKey <> 0 then
  try
    Result := RegRestoreKey(RestoreKey, PChar(FileName), 0) = ERROR_SUCCESS;
  finally
    RegCloseKey(RestoreKey);
  end;
end;

function TRegistry.ReplaceKey(const Key, FileName, BackUpFileName: string): Boolean;
var
  S: string;
  Relative: Boolean;
begin
  {$IFDEF USEREGAPI}
  if not OSLink then begin Result:=false; exit end;
  {$ENDIF}
  S := Key;
  Relative := IsRelative(S);
  if not Relative then Delete(S, 1, 1);
  Result := RegReplaceKey(GetBaseKey(Relative), PChar(S),
    PChar(FileName), PChar(BackUpFileName)) = ERROR_SUCCESS;
end;

function TRegistry.SaveKey(const Key, FileName: string): Boolean;
var
  SaveKey: HKEY;
begin
  Result := False;
  {$IFDEF USEREGAPI}
  if not OSLink then exit;
  {$ENDIF}
  SaveKey := GetKeyEx(Key,true);
  if SaveKey <> 0 then
  try
    Result := RegSaveKey(SaveKey, PChar(FileName), nil) = ERROR_SUCCESS;
  finally
    RegCloseKey(SaveKey);
  end;
end;

function TRegistry.KeyExists(const Key: string): Boolean;
var
  TempKey: HKEY;
begin
  TempKey := GetKeyEx(Key,true);
  if TempKey <> 0 then
   {$IFDEF USEREGAPI}
   DB.CloseKey(TempKey);
   {$ELSE}
   RegCloseKey(TempKey);
   {$ENDIF}
  Result := TempKey <> 0;
end;

procedure TRegistry.RenameValue(const OldName, NewName: string);
var
  Len: Integer;
  RegData: TRegDataType;
  Buffer: PChar;
begin
  if ValueExists(OldName) and not ValueExists(NewName) then
  begin
    Len := GetDataSize(OldName);
    if Len > 0 then
    begin
      Buffer := AllocMem(Len);
      try
        Len := GetData(OldName, Buffer, Len, RegData);
        DeleteValue(OldName);
        PutData(NewName, Buffer, Len, RegData);
      finally
        FreeMem(Buffer);
      end;
    end;
  end;
end;

procedure TRegistry.MoveKey(const OldName, NewName: string; Delete: Boolean);
var
  SrcKey, DestKey: HKEY;

  procedure MoveValue(SrcKey, DestKey: HKEY; const Name: string);
  var
    Len: Integer;
    OldKey, PrevKey: HKEY;
    Buffer: PChar;
    RegData: TRegDataType;
  begin
    OldKey := CurrentKey;
    SetCurrentKey(SrcKey);
    try
      Len := GetDataSize(Name);
      if Len > 0 then
      begin
        Buffer := AllocMem(Len);
        try
          Len := GetData(Name, Buffer, Len, RegData);
          PrevKey := CurrentKey;
          SetCurrentKey(DestKey);
          try
            PutData(Name, Buffer, Len, RegData);
          finally
            SetCurrentKey(PrevKey);
          end;
        finally
          FreeMem(Buffer);
        end;
      end;
    finally
      SetCurrentKey(OldKey);
    end;
  end;

  procedure CopyValues(SrcKey, DestKey: HKEY);
  var
    I:integer;
    Len: {$IFDEF VER150}Cardinal{$ELSE}Integer{$ENDIF};
    KeyInfo: TRegKeyInfo;
    S: string;
    OldKey: HKEY;
  begin
    OldKey := CurrentKey;
    SetCurrentKey(SrcKey);
    try
      if GetKeyInfo(KeyInfo) then
      begin
//        MoveValue(SrcKey, DestKey, '');
        SetString(S, nil, KeyInfo.MaxValueLen + 1);
        for I := 0 to KeyInfo.NumValues - 1 do
        begin
          Len := KeyInfo.MaxValueLen + 1;
          {$IFDEF USEREGAPI}
          if DB.EnumValue(SrcKey, I, PChar(S), @Len, nil, nil, nil) = ERROR_SUCCESS then
          {$ELSE}
          if RegEnumValue(SrcKey, I, PChar(S), Len, nil, nil, nil, nil) = ERROR_SUCCESS then
          {$ENDIF}
            MoveValue(SrcKey, DestKey, PChar(S));
        end;
      end;
    finally
      SetCurrentKey(OldKey);
    end;
  end;

  procedure CopyKeys(SrcKey, DestKey: HKEY);
  var
    I:integer;
    Len: {$IFDEF VER150}Cardinal{$ELSE}Integer{$ENDIF};
    Info: TRegKeyInfo;
    S: string;
    OldKey, PrevKey, NewSrc, NewDest: HKEY;
  begin
    OldKey := CurrentKey;
    SetCurrentKey(SrcKey);
    try
      if GetKeyInfo(Info) then
      begin
        SetString(S, nil, Info.MaxSubKeyLen + 1);
        for I := 0 to Info.NumSubKeys - 1 do
        begin
          Len := Info.MaxSubKeyLen + 1;
          {$IFDEF USEREGAPI}
          if DB.EnumKey(SrcKey, I, PChar(S), @Len, nil, nil, nil) = ERROR_SUCCESS then
          {$ELSE}
          if RegEnumKeyEx(SrcKey, I, PChar(S), Len, nil, nil, nil, nil) = ERROR_SUCCESS then
          {$ENDIF}
          begin
            NewSrc := GetKeyEx(PChar(S),true);
            if NewSrc <> 0 then
            try
              PrevKey := CurrentKey;
              SetCurrentKey(DestKey);
              try
                CreateKey(PChar(S));
                NewDest := GetKeyEx(PChar(S),false);
                try
                  CopyValues(NewSrc, NewDest);
                  CopyKeys(NewSrc, NewDest);
                finally
                  {$IFDEF USEREGAPI}
                  DB.CloseKey(NewDest);
                  {$ELSE}
                  RegCloseKey(NewDest);
                  {$ENDIF}
                end;
              finally
                SetCurrentKey(PrevKey);
              end;
            finally
              {$IFDEF USEREGAPI}
              DB.CloseKey(NewSrc);
              {$ELSE}
              RegCloseKey(NewSrc);
              {$ENDIF}
            end;
          end;
        end;
      end;
    finally
      SetCurrentKey(OldKey);
    end;
  end;

begin
  if KeyExists(OldName) and not KeyExists(NewName) then
  begin
    SrcKey := GetKeyEx(OldName,not Delete);
    if SrcKey <> 0 then
    try
      CreateKey(NewName);
      DestKey := GetKeyEx(NewName,false);
      if DestKey <> 0 then
      try
        CopyValues(SrcKey, DestKey);
        CopyKeys(SrcKey, DestKey);
        if Delete then DeleteKey(OldName);
      finally
        {$IFDEF USEREGAPI}
        DB.CloseKey(DestKey);
        {$ELSE}
        RegCloseKey(DestKey);
        {$ENDIF}
      end;
    finally
      {$IFDEF USEREGAPI}
      DB.CloseKey(SrcKey);
      {$ELSE}
      RegCloseKey(SrcKey);
      {$ENDIF}
    end;
  end;
end;

//---------------------------------------------------------------------------

type
 TRegStream=class(TMemoryStream)
 protected
  DB:TRegistry;
  Name:string;
  ReadOnly:boolean;
 public
  constructor Create(ARegistry:TRegistry; const AName:string; AReadOnly:boolean; load:boolean);
  destructor Destroy;override;
 end;

 constructor TRegStream.Create;
 begin
  inherited Create;
  DB:=ARegistry;
  Name:=AName;
  ReadOnly:=AReadOnly;

  if load then begin
   Size:=DB.GetDataSize(Name);
   if Size<>0 then
    try
     DB.ReadBinaryData(Name,Memory^,Size)
    except
     ReadOnly:=true;       // prevent write back
     raise
    end
  end
 end;

 destructor TRegStream.Destroy;
 begin
  if not ReadOnly then
   try
    DB.WriteBinaryData(Name,Memory^,Size)
   except
   end;

  inherited
 end;

function TRegistry.GetFileStream;

 procedure Make(readonly:boolean; load:boolean);
 begin
  Result:=TRegStream.Create(Self,Name,readonly,load);
 end;

begin
 if Mode=fmCreate then
  Make(false,false)
 else case Mode and $F { ignore fmShare.. flags } of
  fmOpenRead:
   Make(true,true);

  fmOpenWrite:
   Make(false,false);

  fmOpenReadWrite:
   Make(false,ValueExists(Name))
 else
  assert(false)  
 end;
end;

//---------------------------------------------------------------------------

constructor TRegIniFile.Create(const FileName: string);
begin
  inherited Create;
  FFileName := FileName;
  OpenKey(FileName, True);
end;

{$IFDEF USEREGAPI}
constructor TRegIniFile.CreateUsing;
begin
 inherited CreateUsing(ADB);
 FFileName:=FileName;
 OpenKey(FileName,true)
end;
{$ENDIF}

function TRegIniFile.ReadString(const Section, Ident, Default: string): string;
var
  Key, OldKey: HKEY;
begin
  Key := GetKeyEx(Section,true);
  if Key <> 0 then
  try
    OldKey := CurrentKey;
    SetCurrentKey(Key);
    try
      if ValueExists(Ident) then
        Result := inherited ReadString(Ident) else
        Result := Default;
    finally
      SetCurrentKey(OldKey);
    end;
  finally
    {$IFDEF USEREGAPI}
    DB.CloseKey(Key);
    {$ELSE}
    RegCloseKey(Key);
    {$ENDIF}
  end
  else Result := Default;
end;

procedure TRegIniFile.WriteString(const Section, Ident, Value: String);
var
  Key, OldKey: HKEY;
begin
  CreateKey(Section);
  Key := GetKeyEx(Section,false);
  if Key <> 0 then
  try
    OldKey := CurrentKey;
    SetCurrentKey(Key);
    try
      inherited WriteString(Ident, Value);
    finally
      SetCurrentKey(OldKey);
    end;
  finally
    {$IFDEF USEREGAPI}
    DB.CloseKey(Key);
    {$ELSE}
    RegCloseKey(Key);
    {$ENDIF}
  end;
end;

function TRegIniFile.ReadInteger(const Section, Ident: string; Default: LongInt): LongInt;
var
  Key, OldKey: HKEY;
  S: string;
begin
  Key := GetKeyEx(Section,true);
  if Key <> 0 then
  try
    OldKey := CurrentKey;
    SetCurrentKey(Key);
    try
      if ValueExists(Ident) then
      begin
        S := inherited ReadString(Ident);
        if (Length(S) > 2) and (S[1] = '0') and (UpCase(S[2]) = 'X') then
          S := '$' + Copy(S, 3, Maxint);
        Result := StrToIntDef(S, Default);
      end else
        Result := Default;
    finally
      SetCurrentKey(OldKey);
    end;
  finally
    {$IFDEF USEREGAPI}
    DB.CloseKey(Key);
    {$ELSE}
    RegCloseKey(Key);
    {$ENDIF}
  end
  else Result := Default;
end;

procedure TRegIniFile.WriteInteger(const Section, Ident: string; Value: LongInt);
var
  Key, OldKey: HKEY;
begin
  CreateKey(Section);
  Key := GetKeyEx(Section,false);
  if Key <> 0 then
  try
    OldKey := CurrentKey;
    SetCurrentKey(Key);
    try
      inherited WriteString(Ident, IntToStr(Value));
    finally
      SetCurrentKey(OldKey);
    end;
  finally
    {$IFDEF USEREGAPI}
    DB.CloseKey(Key);
    {$ELSE}
    RegCloseKey(Key);
    {$ENDIF}
  end;
end;

function TRegIniFile.ReadBool(const Section, Ident: string; Default: Boolean): Boolean;
begin
  Result := ReadInteger(Section, Ident, Ord(Default)) <> 0;
end;

procedure TRegIniFile.WriteBool(const Section, Ident: string; Value: Boolean);
const
  Values: array[Boolean] of string = ('0', '1');
var
  Key, OldKey: HKEY;
begin
  CreateKey(Section);
  Key := GetKeyEx(Section,false);
  if Key <> 0 then
  try
    OldKey := CurrentKey;
    SetCurrentKey(Key);
    try
      inherited WriteString(Ident, Values[Value]);
    finally
      SetCurrentKey(OldKey);
    end;
  finally
    {$IFDEF USEREGAPI}
    DB.CloseKey(Key);
    {$ELSE}
    RegCloseKey(Key);
    {$ENDIF}
  end;
end;

procedure TRegIniFile.ReadSection(const Section: string; Strings: TStrings);
var
  Key, OldKey: HKEY;
begin
  Key := GetKeyEx(Section,true);
  if Key <> 0 then
  try
    OldKey := CurrentKey;
    SetCurrentKey(Key);
    try
      inherited GetValueNames(Strings);
    finally
      SetCurrentKey(OldKey);
    end;
  finally
    {$IFDEF USEREGAPI}
    DB.CloseKey(Key);
    {$ELSE}
    RegCloseKey(Key);
    {$ENDIF}
  end;
end;

procedure TRegIniFile.ReadSections(Strings: TStrings);
begin
  GetKeyNames(Strings);
end;

procedure TRegIniFile.ReadSectionValues(const Section: string; Strings: TStrings);
var
  KeyList: TStringList;
  I: Integer;
begin
  KeyList := TStringList.Create;
  try
    ReadSection(Section, KeyList);
    Strings.BeginUpdate;
    try
      for I := 0 to KeyList.Count - 1 do
        Strings.Values[KeyList[I]] := ReadString(Section, KeyList[I], '');
    finally
      Strings.EndUpdate;
    end;
  finally
    KeyList.Free;
  end;
end;

procedure TRegIniFile.EraseSection(const Section: string);
begin
  inherited DeleteKey(Section);
end;

procedure TRegIniFile.DeleteKey(const Section, Ident: String);
var
  Key, OldKey: HKEY;
begin
  Key := GetKeyEx(Section,false);
  if Key <> 0 then
  try
    OldKey := CurrentKey;
    SetCurrentKey(Key);
    try
      inherited DeleteValue(Ident);
    finally
      SetCurrentKey(OldKey);
    end;
  finally
    {$IFDEF USEREGAPI}
    DB.CloseKey(Key);
    {$ELSE}
    RegCloseKey(Key);
    {$ENDIF}
  end;
end;

{$IFDEF REGISTRYINIFILE}
//---------------------------------------------------------------------------

{ TRegistryIniFile }

constructor TRegistryIniFile.Create(const FileName: string);
begin
  Create(FileName, KEY_ALL_ACCESS);
end;

constructor TRegistryIniFile.Create(const FileName: string; AAccess: LongWord);
begin
  inherited Create(FileName);
  { TODO: original version of this file was from Delphi 5 mixed with Delphi 3.
    other access modes must yet be verified, I mean it has to be checked out
    if ignoring AAccess argument won't break functionality by rising unexpected
    exceptions. Until then, following assertion should guard such cases }
  assert(AAccess=KEY_ALL_ACCESS);
  FRegIniFile := TRegIniFile.Create(FileName{ TODO: , AAccess });
end;

{$IFDEF USEREGAPI}
constructor TRegistryIniFile.CreateUsing(const FileName:string; ADB:TRegDB);
begin
 inherited Create(FileName);
 FRegIniFile := TRegIniFile.CreateUsing(FileName, ADB)
end;
{$ENDIF}

destructor TRegistryIniFile.Destroy;
begin
  FRegIniFile.Free;
  inherited Destroy;
end;

function TRegistryIniFile.ReadString(const Section, Ident, Default: string): string;
begin
  Result := FRegIniFile.ReadString(Section, Ident, Default);
end;

function TRegistryIniFile.ReadDate(const Section, Name: string; Default: TDateTime): TDateTime;
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        if ValueExists(Name) then
          Result := ReadDate(Name)
        else Result := Default;
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end else Result := Default;
  end;
end;

function TRegistryIniFile.ReadDateTime(const Section, Name: string; Default: TDateTime): TDateTime;
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        if ValueExists(Name) then
          Result := ReadDateTime(Name)
        else Result := Default;
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end else Result := Default;
  end;
end;

function TRegistryIniFile.ReadFloat(const Section, Name: string; Default: Double): Double;
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        if ValueExists(Name) then
          Result := ReadFloat(Name)
        else Result := Default;
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end else Result := Default;
  end;
end;

function TRegistryIniFile.ReadInteger(const Section, Ident: string; Default: LongInt): LongInt;
var
  Key, OldKey: HKEY;
begin
  with TRegistry(FRegIniFile) do
  begin
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        Result := Default;
        if ValueExists(Ident) then
          if GetDataType(Ident) = rdString then
            Result := StrToIntDef(ReadString(Ident), Default)
          else Result := ReadInteger(Ident);
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end
    else Result := Default;
  end;
end;

function TRegistryIniFile.ReadTime(const Section, Name: string; Default: TDateTime): TDateTime;
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        if ValueExists(Name) then
          Result := ReadTime(Name)
        else Result := Default;
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end else Result := Default;
  end;
end;

function TRegistryIniFile.ReadBinaryStream(const Section, Name: string; Value: TStream): Integer;
var
  RegData: TRegDataType;
  Info: TRegDataInfo;
  Key, OldKey: HKEY;
  Stream: TMemoryStream;
begin
  Result := 0;
  with RegIniFile do
  begin
    Key := TRegistry(FRegIniFile).GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      TRegistry(FRegIniFile).SetCurrentKey(Key);
      try
        if ValueExists(Name) then
        begin
          if GetDataInfo(Name, Info) then
          begin
            Result := Info.DataSize;
            RegData := Info.RegData;
            if Value is TMemoryStream then
              Stream := TMemoryStream(Value)
            else Stream := TMemoryStream.Create;
            try
              if (RegData = rdBinary) or (RegData = rdUnknown) then
              begin
                Stream.Size := Stream.Position + Info.DataSize;
                Result := ReadBinaryData(Name,
                  Pointer(Integer(Stream.Memory) + Stream.Position)^, Stream.Size);
                if Stream <> Value then Value.CopyFrom(Stream, Stream.Size - Stream.Position);
              end;
            finally
              if Stream <> Value then Stream.Free;
            end;
          end;
        end;
      finally
        TRegistry(FRegIniFile).SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

procedure TRegistryIniFile.WriteDate(const Section, Name: string; Value: TDateTime);
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    CreateKey(Section);
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        WriteDate(Name, Value);
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

procedure TRegistryIniFile.WriteDateTime(const Section, Name: string; Value: TDateTime);
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    CreateKey(Section);
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        WriteDateTime(Name, Value);
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

procedure TRegistryIniFile.WriteFloat(const Section, Name: string; Value: Double);
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    CreateKey(Section);
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        WriteFloat(Name, Value);
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

procedure TRegistryIniFile.WriteInteger(const Section, Ident: string; Value: LongInt);
var
  Key, OldKey: HKEY;
begin
  with TRegistry(FRegIniFile) do
  begin
    CreateKey(Section);
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        if ValueExists(Ident) and (GetDataType(Ident) = rdString) then
          WriteString(Ident, IntToStr(Value))
        else WriteInteger(Ident, Value);
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

procedure TRegistryIniFile.WriteTime(const Section, Name: string; Value: TDateTime);
var
  Key, OldKey: HKEY;
begin
  with FRegIniFile do
  begin
    CreateKey(Section);
    Key := GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      SetCurrentKey(Key);
      try
        WriteTime(Name, Value);
      finally
        SetCurrentKey(OldKey);
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

procedure TRegistryIniFile.WriteBinaryStream(const Section, Name: string;
  Value: TStream);
var
  Key, OldKey: HKEY;
  Stream: TMemoryStream;
begin
  with RegIniFile do
  begin
    CreateKey(Section);
    Key := TRegistry(FRegIniFile).GetKey(Section);
    if Key <> 0 then
    try
      OldKey := CurrentKey;
      if Value is TMemoryStream then
        Stream := TMemoryStream(Value)
      else Stream := TMemoryStream.Create;
      try
        if Stream <> Value then
        begin
          Stream.CopyFrom(Value, Value.Size - Value.Position);
          Stream.Position := 0;
        end;
        TRegistry(FRegIniFile).SetCurrentKey(Key);
        try
          WriteBinaryData(Name, Pointer(Integer(Stream.Memory) + Stream.Position)^,
            Stream.Size - Stream.Position);
        finally
          TRegistry(FRegIniFile).SetCurrentKey(OldKey);
        end;
      finally
        if Value <> Stream then Stream.Free;
      end;
    finally
      RegCloseKey(Key);
    end;
  end;
end;

procedure TRegistryIniFile.WriteString(const Section, Ident, Value: String);
begin
  FRegIniFile.WriteString(Section, Ident, Value);
end;

procedure TRegistryIniFile.ReadSection(const Section: string; Strings: TStrings);
begin
  FRegIniFile.ReadSection(Section, Strings);
end;

procedure TRegistryIniFile.ReadSections(Strings: TStrings);
begin
  FRegIniFile.ReadSections(Strings);
end;

procedure TRegistryIniFile.ReadSectionValues(const Section: string; Strings: TStrings);
begin
  FRegIniFile.ReadSectionValues(Section, Strings);
end;

procedure TRegistryIniFile.EraseSection(const Section: string);
begin
  FRegIniFile.EraseSection(Section);
end;

procedure TRegistryIniFile.DeleteKey(const Section, Ident: String);
begin
  FRegIniFile.DeleteKey(Section, Ident);
end;

procedure TRegistryIniFile.UpdateFile;
begin
  { Do nothing }
end;

//---------------------------------------------------------------------------
{$ENDIF}

constructor TVBReg.Create;
begin
 {$IFDEF USEREGAPI}
 DB:=defaultRegApi;
 DB.Use
 {$ENDIF}
end;

{$IFDEF USEREGAPI}
constructor TVBReg.CreateUsing;
begin
 DB:=ADB;
 if not Assigned(DB) then DB:=defaultRegApi;
 DB.Use
end;
{$ENDIF}

destructor TVBReg.Destroy;
begin
 {$IFDEF USEREGAPI}
 DB.Release;
 {$ENDIF}
 inherited
end;

type
 { this is a VB helper class. after instantiation just one of declared functions
   may be run. }
 TReg=class
 private
  r:TRegistry;
  path:string;
  name:string;
  key:HKEY;
  keyName:string;
 public
  constructor Create(APath:string{$IFDEF USEREGAPI};DB:TRegDB{$ENDIF});
  destructor Destroy;override;

  function Exists:boolean;
  function Remove:boolean;
  function GetValue:OleVariant;
  function SetValue(const value:OleVariant):boolean;
 end;

 procedure Parse(const APath:string; var path:string; var key:HKEY; var keyName:string; var name:string);
 var p:integer;
 begin
  path:=APath;
  p:=Pos('\',path);
  key:=0;
  keyName:='';
  if p<>0 then begin
   name:=UpperCase(Copy(path,1,p-1));
   delete(path,1,p);

   keyName:=name+'\';
   if name='HKCR' then
    key:=HKEY_CLASSES_ROOT
   else if name='HKCU' then
    key:=HKEY_CURRENT_USER
   else if name='HKLM' then
    key:=HKEY_LOCAL_MACHINE
   else if name='HKU' then
    key:=HKEY_USERS
  end;

  name:='';
  p:=Pos('\\',path);
  if p=0 then begin
   for p:=length(path) downto 1 do
    if path[p]='\' then begin
     name:=Copy(path,p+1,maxint);
     SetLength(path,p-1);
     break
    end
  end else begin
   name:=Copy(path,p+2,maxint);
   SetLength(path,p-1)
  end
 end;

 constructor TReg.Create;
 begin
  {$IFDEF USEREGAPI}
  r:=TRegistry.CreateUsing(DB);
  {$ELSE}
  r:=TRegistry.Create;
  {$ENDIF}
  Parse(APath,path,key,keyName,name)
 end;

 destructor TReg.Destroy;
 begin
  r.Free;
  inherited
 end;

 function TReg.Exists;
 begin
  Result:=false;
  if key=0 then exit;
  r.RootKey:=key;

  if not r.OpenKeyReadOnly(path) then exit;
  Result:=(name='') or r.ValueExists(name)
 end;

 function TReg.GetValue;
 begin
  if key=0 then exit;
  r.RootKey:=key;

  if path<>'' then if not r.OpenKeyReadOnly(path) then exit;
  if not r.ValueExists(name) then exit;

  case r.GetDataType(name) of
   rdString:  Result:=r.ReadString(name);
   rdInteger: Result:=r.ReadInteger(name);
   // TODO: read back other value types
  end;
 end;

 function TReg.Remove;
 begin
  Result:=true;
  if key=0 then exit;
  r.RootKey:=key;

  if name<>'' then begin
   if not r.OpenKey(path,false) then exit;
   r.DeleteValue(name)
  end else begin
   if r.KeyExists(path) then
    Result:=r.DeleteKey(path)
  end
 end;

 function TReg.SetValue;
 begin
  Result:=false;
  if key=0 then exit;
  r.RootKey:=key;

  if path<>'' then if not r.OpenKey(path,true) then exit;

  case VarType(value) of
   varEmpty,
   varNull:
    begin
     r.DeleteValue(name);
     Result:=true
    end;

   varSmallint,
   varInteger,
   varByte:
    begin
     r.WriteInteger(name,value);
     Result:=true
    end;

   varOleStr,
   varString:
    begin
     r.WriteString(name,value);
     Result:=true
    end;

   varSingle,
   varDouble,
   varCurrency:
    begin
     r.WriteFloat(name,value);
     Result:=true
    end;

   varBoolean:
    begin
     r.WriteBool(name,value);
     Result:=true
    end;

   varDate:
    begin
     r.WriteDate(name,value);
     Result:=true
    end;

{   varDispatch = $0009;
   varError    = $000A;
   varVariant  = $000C;
   varUnknown  = $000D;
   varTypeMask = $0FFF;
   varArray    = $2000;
   varByRef    = $4000; }
  end
 end;

function TVBReg.CreateHandler;
begin
 Result:=TReg.Create(path{$IFDEF USEREGAPI},DB{$ENDIF})
end;

function TVBReg.Read(const path:string):OleVariant;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.GetValue
  finally
   r.Free
  end
 except
  Result:=Unassigned
 end
end;

function TVBReg.Write(const path:string; const value:OleVariant):boolean;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.SetValue(value)
  finally
   r.Free
  end
 except
  Result:=false
 end
end;

procedure TVBReg.DoWrite;
begin
 if not Write(path,value) then { TODO: raise exception ? }
end;

function TVBReg.Delete(const path:string):boolean;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.Remove
  finally
   r.Free
  end
 except
  Result:=false
 end
end;

function TVBReg.Exists(const path:string):boolean;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.Exists
  finally
   r.Free
  end
 except
  Result:=false
 end
end;

function RegRead(const path:string):OleVariant;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Read(path);
 r.Free
end;

function RegReadString(const path:string):string;
begin
 try
  Result:=string(RegRead(path))
 except
  Result:=''
 end
end;

function RegWrite(const path:string; const value:OleVariant):boolean;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Write(path,value);
 r.Free
end;

function RegDelete(const path:string):boolean;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Delete(path);
 r.Free
end;

function RegExists(const path:string):boolean;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Exists(path);
 r.Free
end;

function SetCurrentKey;
var keyPath,keyName,name:string;
var key:HKEY;
var flags:set of (fCanCreate,fReadOnly);
begin
 Parse(path,keyPath,key,keyName,name);
 Result:=false;
 if (length(name)>0) or (key=0) then exit;
 try
  reg.RootKey:=key;
  if Mode=fmCreate then
   flags:=[fCanCreate]
  else case Mode and $F of
   fmOpenRead: flags:=[fReadOnly];
   fmOpenReadWrite: flags:=[fCanCreate];
   fmOpenWrite: flags:=[]
   else assert(false)
  end;
  Result:=reg.OpenKeyEx(keyPath,fCanCreate in flags,fReadOnly in flags)
 except
 end
end;

function GetCurrentKey;
begin
 case reg.RootKey of
 HKEY_CURRENT_USER: Result:='HKCU';
 HKEY_USERS: Result:='HKU';
 HKEY_LOCAL_MACHINE: Result:='HKLM';
 HKEY_CLASSES_ROOT: Result:='HKCR'
 else
  Result:=''
 end;
 if length(Result)>0 then Result:=Result+'\'+reg.CurrentPath+'\'
end;

end.


