{ VERSION: 1.8 }
{ 1.8 / 11.10.2005
  - nova funkce ValidateLogon() pro overeni uzivatelskeho jmena a hesla
  1.7 / 15.9.2005
  - nova funkce GetDisplayName() pro prevod jmena souboru na zobrazovaci jmeno
  1.6 / 7.6.2005
  - nova funkce AppRunning() pro kontrolu spusteni "single instance"
  1.5 / 11.5.2005
  - nova funkce GetProcessFileName(ProcessId)
  1.4 / 2.4.2005
  - funkce Exec() akceptuje formatovaci znak %1 v parametru Command
  1.3 / 8.2.2005
  - funkce Timer..() se odstehovaly do SimpleTimers.pas
  1.2 / 7.2.2005
  - odstranena jedna zavislost na Win2000 up (viz. GetLastInputInfo)
  1.1 / 5.2.2005
  - Exec() zobrazuje chybove okno jenom kdyz je zadan ef_Interactive
  - funkce Timer..() pro mereni s milisekundovou presnosti. funkce jsou
    odolne na preteceni systemoveho pocitadla GetTickCount()
}
unit SystemUtils;

interface
uses
  Windows, Classes;

const
  SE_CREATE_TOKEN_NAME = 'SeCreateTokenPrivilege';
  SE_ASSIGNPRIMARYTOKEN_NAME = 'SeAssignPrimaryTokenPrivilege';
  SE_LOCK_MEMORY_NAME = 'SeLockMemoryPrivilege';
  SE_INCREASE_QUOTA_NAME = 'SeIncreaseQuotaPrivilege';
  SE_UNSOLICITED_INPUT_NAME = 'SeUnsolicitedInputPrivilege';
  SE_MACHINE_ACCOUNT_NAME = 'SeMachineAccountPrivilege';
  SE_TCB_NAME = 'SeTcbPrivilege';
  SE_SECURITY_NAME = 'SeSecurityPrivilege';
  SE_TAKE_OWNERSHIP_NAME = 'SeTakeOwnershipPrivilege';
  SE_LOAD_DRIVER_NAME = 'SeLoadDriverPrivilege';
  SE_SYSTEM_PROFILE_NAME = 'SeSystemProfilePrivilege';
  SE_SYSTEMTIME_NAME = 'SeSystemtimePrivilege';
  SE_PROF_SINGLE_PROCESS_NAME = 'SeProfileSingleProcessPrivilege';
  SE_INC_BASE_PRIORITY_NAME = 'SeIncreaseBasePriorityPrivilege';
  SE_CREATE_PAGEFILE_NAME = 'SeCreatePagefilePrivilege';
  SE_CREATE_PERMANENT_NAME = 'SeCreatePermanentPrivilege';
  SE_BACKUP_NAME = 'SeBackupPrivilege';
  SE_RESTORE_NAME = 'SeRestorePrivilege';
  SE_SHUTDOWN_NAME = 'SeShutdownPrivilege';
  SE_DEBUG_NAME = 'SeDebugPrivilege';
  SE_AUDIT_NAME = 'SeAuditPrivilege';
  SE_SYSTEM_ENVIRONMENT_NAME = 'SeSystemEnvironmentPrivilege';
  SE_CHANGE_NOTIFY_NAME = 'SeChangeNotifyPrivilege';
  SE_REMOTE_SHUTDOWN_NAME = 'SeRemoteShutdownPrivilege';
  SE_UNDOCK_NAME = 'SeUndockPrivilege';
  SE_SYNC_AGENT_NAME = 'SeSyncAgentPrivilege';
  SE_ENABLE_DELEGATION_NAME = 'SeEnableDelegationPrivilege';
  SE_MANAGE_VOLUME_NAME = 'SeManageVolumePrivilege';

// zapne nebo vypne prislusne opravneni
function SetPrivilege(const Privilege: string; bEnabled: Boolean): Boolean;

// zjisti jestli mame prava administratora
function IsAdmin: Boolean;

// vrati cas ( v milisekundach ) po ktery uzivatel nic nedelal.
function GetSystemIdleTime: Cardinal;

type
  TShutDown = set of (
    sdLogOff,                // odhlasit se
    sdRestart,               // restartovat
    sdKillApps,              // zabit nereagujici programy
    sdPowerOff               // vypnout pocitac
  );

// provede nejaky druh restartu pocitace, pripadne pocka TimeOut milisekund a zobrazi pritom zpravu
function ShutDown(Flags: TShutDown = []; TimeOut: Cardinal = 0; const Msg: string = ''): Boolean;
// pokusi se zastavit probihajici ShutDown
function CancelShutDown: Boolean;

function IsAppResponding(Wnd: HWND): Boolean;

type
  TSystemVersion = (
    sysUnknown,                 // fakt nevime co to je

    sysWin9x,                   // nejake devadesatky
    sysWin95,
    sysWin98,
    sysWinME,
    sysWinNT,                   // nejake normalni NTcko
    sysWinNT3,
    sysWinNT4,
    sysWin2000,
    sysWinXP,

    sysWinUltra                 // nejaky super ultra moderni system
  );

// vrati verzi operacniho systemu
function GetOSVersion: TSystemVersion;
// jestli je operacni system kompatibilni ( tj. stejna nebo vyssi verze )
function OSCompatibleWith(Version: TSystemVersion): Boolean;
// zkratka za OSCompatibleWith(sysWinNT)
function IsWinNT: Boolean;

type
  TExecFlags = set of (
    ef_Wait,           // pockej dokud nedojede
    ef_Interactive,    // kdyz to nepude, tak zobraz chybove okno

    ef_Hidden,         // skryte okno
    ef_Maximized,
    ef_Minimized
  );

// TODO: predelej exec() at vraci ThreadId i ProcessId ( misto ShellExecute() pouzij
// CreateProcess() )
// spusti program
function Exec(const Command: string; const Parameters: string = ''; const Directory: string = ''; Flags: TExecFlags = []; PProcess: PHandle = nil): Boolean;

function GetProcessNameFromWnd(Wnd: HWND): string;
function GetProcessID(Process: THandle): Cardinal;
function RunningProcessesList(const List: TStrings; FullPath: Boolean): Boolean;
function KillProcess(Process: THandle; ProcessId: Cardinal = 0; TimeOut: Cardinal = 0): Boolean;
function IsProcessActive(Process: THandle): Boolean;
function ProcessWindowsList(List: TList; ProcessId: Cardinal; OnlyVisible: Boolean = False): Boolean;
function GetProcessFileName(ProcessId: Cardinal; FullPath: Boolean): string;

// AppID je jednoznacny identifikator v ramci pocitace identifikujici aplikaci
function AppRunning(const AppID: string; CheckOnly: Boolean = False): Boolean;

// vraci jmeno souboru pro zobrazeni
function GetDisplayName(const FileName: string): string;

function ValidateLogon(const UserName: string; const Password: string; const Domain: string = '.'): Cardinal;

implementation
uses
  Messages, ShellApi, SysUtils, PsAPI, TlHelp32, SimpleTimers;

function IsWinNT: Boolean;
begin
  Result := Win32Platform = VER_PLATFORM_WIN32_NT
end;

var
  OSVersion: TSystemVersion = sysUnknown;

function GetOSVersion: TSystemVersion;
begin
  if OSVersion <> sysUnknown then begin
    Result := OSVersion;
    exit
  end;

  // TODO: a co ty dalsi cisla ?
  if Win32Platform = VER_PLATFORM_WIN32_WINDOWS then begin
    Result := sysWin9x;
    if Win32MajorVersion = 4 then begin
      case Win32MinorVersion of
      0:
        Result := sysWin95;
      10:
        Result := sysWin98;
      90:
        Result := sysWinME
      end
    end
  end else
  if Win32Platform = VER_PLATFORM_WIN32_NT then begin
    Result := sysWinNT;
    case Win32MajorVersion of
    3:
      Result := sysWinNT3;
    4:
      Result := sysWinNT4;
    5:
      case Win32MinorVersion of
      0:
        Result := sysWin2000;
      1:
        Result := sysWinXP
      else
        Result := sysWinUltra
      end;
    else
      if Win32MajorVersion > 5 then
        Result := sysWinUltra
    end
  end else
    Result := sysUnknown;

  OSVersion := Result
end;

function OSCompatibleWith(Version: TSystemVersion): Boolean;
begin
  Result := GetOSVersion >= Version
end;

//--------------------------------------------------------------------------

{
  For some functions you need to get the right privileges
  on a Windows NT machine.
  (e.g: To shut down or restart windows with ExitWindowsEx or
  to change the system time)
  The following code provides a procedure to adjust the privileges.
  The AdjustTokenPrivileges() function enables or disables privileges
  in the specified access token.
}

{
  Um einige Funktionen unter Windows NT erfolgreich ausfhren zu knnen,
  muss der ausfhrende Prozess die Privilegien dazu haben.
  (z.B um das System herunterzufahren mit ExitWindowsEx oder die
  Systemzeit zu ndern) AdjustTokenPrivileges() passt die Privilegien an.
}


// Enables or disables privileges debending on the bEnabled
// Aktiviert oder deaktiviert Privilegien, abhngig von bEnabled

function SetPrivilege(const Privilege: string; bEnabled: Boolean): Boolean;
var
  hToken: THandle;
  TokenPriv: TOKEN_PRIVILEGES;
  PrevTokenPriv: TOKEN_PRIVILEGES;
  ReturnLength: Cardinal;
begin
  Result := True;
  // Only for Windows NT/2000/XP and later.
  if not IsWinNT then Exit;
  Result := False;

  // obtain the processes token
  if OpenProcessToken(GetCurrentProcess(),
    TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, hToken) then
  begin
    try
      // Get the locally unique identifier (LUID) .
      if LookupPrivilegeValue(nil, PChar(Privilege),
        TokenPriv.Privileges[0].Luid) then
      begin
        TokenPriv.PrivilegeCount := 1; // one privilege to set

        case bEnabled of
          True: TokenPriv.Privileges[0].Attributes  := SE_PRIVILEGE_ENABLED;
          False: TokenPriv.Privileges[0].Attributes := 0;
        end;

        ReturnLength := 0; // replaces a var parameter
        PrevTokenPriv := TokenPriv;

        // enable or disable the privilege

        AdjustTokenPrivileges(hToken, False, TokenPriv, SizeOf(PrevTokenPriv),
          PrevTokenPriv, ReturnLength);
        Result := GetLastError = ERROR_SUCCESS;
      end;
    finally
      CloseHandle(hToken);
    end;
  end;
end;

//--------------------------------------------------------------------------

function ShutDown(Flags: TShutDown; TimeOut: Cardinal; const Msg: string): Boolean;
var
  Flgs: Cardinal;
begin
  TimeOut := TimeOut div 1000; // MsecToSec

  Result := SetPrivilege(SE_SHUTDOWN_NAME, True);
  if not Result then exit;
  if (TimeOut = 0) or (not IsWinNT) then begin
    Flgs := 0;
    if sdKillApps in Flags then
      Flgs := Flgs or EWX_FORCE;
    if sdRestart in Flags then
      Flgs := Flgs or EWX_REBOOT
    else
      Flgs := Flgs or EWX_SHUTDOWN;
    if [sdPowerOff, sdRestart] * Flags = [sdPowerOff] then
      Flgs := Flgs or EWX_POWEROFF;
    if sdLogOff in Flags then
      Flgs := (Flgs and (not (EWX_REBOOT or EWX_SHUTDOWN or EWX_POWEROFF))) or EWX_LOGOFF;
    Result := ExitWindowsEx(Flgs, 0)
  end else
    // TODO: a jak se ridi POWEROFF nebo LOGOFF ?
    Result := InitiateSystemShutdown(nil, PChar(Msg), TimeOut, sdKillApps in Flags, sdRestart in Flags)
end;

function CancelShutDown: Boolean;
begin
  // TODO: a co zastaveni ExitWindowsEx()
  Result := AbortSystemShutdown(nil)
end;

//--------------------------------------------------------------------------

var
  FIdleInitialized: Boolean = False;
  FIdleTimer: TTimer;
  FIdleMousePos: TPoint;

function GuessSystemIdleTime: Cardinal;
var
  MousePos: TPoint;

  // TODO: na NT to bude vracet 0 kdyz neni okno aktivni ?
  function KeyStateChanged: Boolean;
  var
    I: Integer;
  begin
    Result := False;
    for I := 0 to 255 do
      Result := Result or ((GetAsyncKeyState(I) and 1) <> 0)
  end;

begin
  if not GetCursorPos(MousePos) then begin
    MousePos.X := 0;
    MousePos.Y := 0
  end;
  if not FIdleInitialized then begin
    FIdleInitialized := True;
    FIdleMousePos := MousePos;
    TimerReset(FIdleTimer);
    TimerStart(FIdleTimer);
    KeyStateChanged;
    Result := 0
  end else begin
    if (MousePos.X <> FIdleMousePos.X) or (MousePos.Y <> FIdleMousePos.Y) then begin
      FIdleMousePos := MousePos;
      TimerReStart(FIdleTimer);
    end;
    if KeyStateChanged then
      TimerReStart(FIdleTimer);

    Result := TimerValue(FIdleTimer)
  end
end;

function GetSystemIdleTime: Cardinal;
var
  Lib: THandle;
  GetInput: function(var plii: TLastInputInfo): BOOL; stdcall;
  LInput: TLastInputInfo;
  Timer: TTimer;
begin
  if OSCompatibleWith(sysWin2000) then begin
    LInput.cbSize := SizeOf(LInput);
    Lib := GetModuleHandle('user32.dll');
    @GetInput := GetProcAddress(Lib, 'GetLastInputInfo');
    if (@GetInput <> nil) and GetInput(LInput) then begin
      TimerReStart(Timer, LInput.dwTime);
      Result := TimerValue(Timer)
    end else
      Result := GuessSystemIdleTime
  end else
    Result := GuessSystemIdleTime
end;

//--------------------------------------------------------------------------

(* 1. The Documented way
function IsAppResponding(Wnd: HWND): Boolean;
const
  { Specifies the duration, in milliseconds, of the time-out period }
  TIMEOUT = 50;
var
  Res: DWORD;
begin
  Result := SendMessageTimeOut(H,
    WM_NULL,
    0,
    0,
    SMTO_NORMAL or SMTO_ABORTIFHUNG,
    TIMEOUT,
    Res) <> 0
end;
*)

// 2. The Undocumented way

{
  // Translated form C to Delphi by Thomas Stutz
  // Original Code:
  // (c)1999 Ashot Oganesyan K, SmartLine, Inc
  // mailto:ashot@aha.ru, http://www.protect-me.com, http://www.codepile.com

 The code doesn't use the Win32 API SendMessageTimout function to
 determine if the target application is responding but calls
 undocumented functions from the User32.dll.

 --> For NT/2000/XP the IsHungAppWindow() API:

 The function IsHungAppWindow retrieves the status (running or not responding)
 of the specified application

 IsHungAppWindow(Wnd: HWND): // handle to main app's window
 BOOL;

 --> For Windows 95/98/ME we call the IsHungThread() API

 The function IsHungThread retrieves the status (running or not responding) of
 the specified thread

 IsHungThread(DWORD dwThreadId): // The thread's identifier of the main app's window
 BOOL;

 Unfortunately, Microsoft doesn't provide us with the exports symbols in the
 User32.lib for these functions, so we should load them dynamically using the
 GetModuleHandle and GetProcAddress functions:
}

// For Win9X/ME
function IsAppResponding9X(dwThreadId: DWORD): Boolean;
type
  TIsHungThread = function(dwThreadId: DWORD): BOOL; stdcall;
var
  hUser32: THandle;
  IsHungThread: TIsHungThread;
begin
  Result := True;
  hUser32 := GetModuleHandle('user32.dll');
  if (hUser32 > 0) then
  begin
    @IsHungThread := GetProcAddress(hUser32, 'IsHungThread');
    if Assigned(IsHungThread) then
    begin
      Result := not IsHungThread(dwThreadId);
    end;
  end;
end;

// For Win NT/2000/XP
function IsAppRespondingNT(wnd: HWND): Boolean;
type
  TIsHungAppWindow = function(wnd:hWnd): BOOL; stdcall;
var
  hUser32: THandle;
  IsHungAppWindow: TIsHungAppWindow;
begin
  Result := True;
  hUser32 := GetModuleHandle('user32.dll');
  if (hUser32 > 0) then
  begin
    @IsHungAppWindow := GetProcAddress(hUser32, 'IsHungAppWindow');
    if Assigned(IsHungAppWindow) then
    begin
      Result := not IsHungAppWindow(wnd);
    end;
  end;
end;

function IsAppResponding(Wnd: HWND): Boolean;
begin
 { ?
 if not IsWindow(Wnd) then
 begin
   ShowMessage('Incorrect window handle!');
   Exit;
 end;
 }
 if IsWinNT then
   Result := IsAppRespondingNT(Wnd)
 else
   Result := IsAppResponding9X(GetWindowThreadProcessId(Wnd,nil));
end;

//--------------------------------------------------------------------------

// portions by Project Jedi www.delphi-jedi.org/
const
  RsSystemIdleProcess = 'System Idle Process';
  RsSystemProcess = 'System Process';

function GetProcessFileName(ProcessID: Cardinal; FullPath: Boolean): string;
var
  Handle: THandle;
begin
  Result := '';
  Handle := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, False, ProcessID);
  if Handle <> 0 then
    try
      SetLength(Result, MAX_PATH);
      if FullPath then
      begin
        if GetModuleFileNameEx(Handle, 0, PChar(Result), MAX_PATH) > 0 then
          SetLength(Result, StrLen(PChar(Result)))
        else
          Result := '';
      end
      else
      begin
        if GetModuleBaseNameA(Handle, 0, PChar(Result), MAX_PATH) > 0 then
          SetLength(Result, StrLen(PChar(Result)))
        else
          Result := '';
      end;
    finally
      CloseHandle(Handle);
    end;
end;

function RunningProcessesList(const List: TStrings; FullPath: Boolean): Boolean;

  function BuildListTH: Boolean;
  var
    SnapProcHandle: THandle;
    ProcEntry: TProcessEntry32;
    NextProc: Boolean;
    FileName: string;
  begin
    SnapProcHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    Result := (SnapProcHandle <> INVALID_HANDLE_VALUE);
    if Result then
      try
        ProcEntry.dwSize := SizeOf(ProcEntry);
        NextProc := Process32First(SnapProcHandle, ProcEntry);
        while NextProc do
        begin
          if ProcEntry.th32ProcessID = 0 then
          begin
            // PID 0 is always the "System Idle Process" but this name cannot be
            // retrieved from the system and has to be fabricated.
            FileName := RsSystemIdleProcess;
          end
          else
          begin
            if OSCompatibleWith(sysWin2000) then
            begin
              FileName := GetProcessFileName(ProcEntry.th32ProcessID, FullPath);
              if FileName = '' then
                FileName := ProcEntry.szExeFile;
            end
            else
            begin
              FileName := ProcEntry.szExeFile;
              if not FullPath then
                FileName := ExtractFileName(FileName);
            end;
          end;
          List.AddObject(FileName, Pointer(ProcEntry.th32ProcessID));
          NextProc := Process32Next(SnapProcHandle, ProcEntry);
        end;
      finally
        CloseHandle(SnapProcHandle);
      end;
  end;

  function BuildListPS: Boolean;
  var
    PIDs: array [0..1024] of DWORD;
    Needed: DWORD;
    I: Integer;
    FileName: string;
  begin
    Result := EnumProcesses(@PIDs, SizeOf(PIDs), Needed);
    if Result then
    begin
      for I := 0 to (Needed div SizeOf(DWORD)) - 1 do
      begin
        case PIDs[I] of
          0:
            // PID 0 is always the "System Idle Process" but this name cannot be
            // retrieved from the system and has to be fabricated.
            FileName := RsSystemIdleProcess;

          2:
            // On NT 4 PID 2 is the "System Process" but this name cannot be
            // retrieved from the system and has to be fabricated.
            if GetOSVersion = sysWinNT4 then
              FileName := RsSystemProcess
            else
              FileName := GetProcessFileName(PIDs[I], FullPath);

          8:
          // On Win2K PID 8 is the "System Process" but this name cannot be
          // retrieved from the system and has to be fabricated.
          if OSCompatibleWith(sysWin2000) then
            FileName := RsSystemProcess
          else
            FileName := GetProcessFileName(PIDs[I], FullPath);

        else
          FileName := GetProcessFileName(PIDs[I], FullPath);
        end;
        if FileName <> '' then
          List.AddObject(FileName, Pointer(PIDs[I]));
      end;
    end;
  end;
begin
  if GetOSVersion in [sysWinNT3, sysWinNT4] then
    Result := BuildListPS
  else
    Result := BuildListTH;
end;

function GetProcessNameFromWnd(Wnd: HWND): string;
var
  PID: DWORD;
{
  List: TStringList;
  I: Integer;
}
begin
  Result := '';
  if IsWindow(Wnd) then
  begin
    PID := INVALID_HANDLE_VALUE;
    GetWindowThreadProcessId(Wnd, @PID);
    Result := GetProcessFileName(PID, True)
    {
    List := TStringList.Create;
    try
      if RunningProcessesList(List, True) then
      begin
        I := List.IndexOfObject(Pointer(PID));
        if I > -1 then
          Result := List[I];
      end;
    finally
      List.Free;
    end;
    }
  end;
end;

//--------------------------------------------------------------------------

const
  SECURITY_NT_AUTHORITY: TSIDIdentifierAuthority =
    (Value: (0, 0, 0, 0, 0, 5));
  SECURITY_BUILTIN_DOMAIN_RID = $00000020;
  DOMAIN_ALIAS_RID_ADMINS = $00000220;

function IsAdmin: Boolean;
var
  hAccessToken: THandle;
  ptgGroups: PTokenGroups;
  dwInfoBufferSize: DWORD;
  psidAdministrators: PSID;
  x: Integer;
  bSuccess: BOOL;
begin
  if not IsWinNT then begin
    Result := True;
    exit
  end;
  Result   := False;
  bSuccess := OpenThreadToken(GetCurrentThread, TOKEN_QUERY, True,
    hAccessToken);
  if not bSuccess then
  begin
    if GetLastError = ERROR_NO_TOKEN then
      bSuccess := OpenProcessToken(GetCurrentProcess, TOKEN_QUERY,
        hAccessToken);
  end;
  if bSuccess then
  begin
    GetMem(ptgGroups, 1024);
    bSuccess := GetTokenInformation(hAccessToken, TokenGroups,
      ptgGroups, 1024, dwInfoBufferSize);
    CloseHandle(hAccessToken);
    if bSuccess then
    begin
      AllocateAndInitializeSid(SECURITY_NT_AUTHORITY, 2,
        SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS,
        0, 0, 0, 0, 0, 0, psidAdministrators);
      {$R-}
      for x := 0 to ptgGroups.GroupCount - 1 do
        if EqualSid(psidAdministrators, ptgGroups.Groups[x].Sid) then
        begin
          Result := True;
          Break;
        end;
      {$R+}
      FreeSid(psidAdministrators);
    end;
    FreeMem(ptgGroups);
  end;
end;

//--------------------------------------------------------------------------

// TODO: a co status ?

function Exec(const Command: string; const Parameters: string; const Directory: string; Flags: TExecFlags; PProcess: PHandle): Boolean;
var
  Info: TShellExecuteInfo;
  // TODO: Status: Cardinal;
  ShowCmd: Integer;
  _Command: string;
  _Parameters: string;
  I: Integer;
begin
  if PProcess <> nil then
    PProcess^ := 0;

  if ef_Hidden in Flags then
    ShowCmd := SW_HIDE
  else
  if ef_Maximized in Flags then
    ShowCmd := SW_SHOWMAXIMIZED
  else
  if ef_Minimized in Flags then
    ShowCmd := SW_SHOWMINIMIZED
  else
    ShowCmd := SW_SHOWNORMAL;

  I := Pos('%1', Command);
  if I = 0 then begin
    _Command := Trim(Command);
    _Parameters := Parameters
  end else begin
    _Command := Trim(Command);
    _Command[I + 1] := 's';
    try
      _Command := Format(_Command, [Parameters]);
      _Parameters := ''
    except
      // TODO: a co interactive ?
      Result := False;
      exit
    end
  end;

  if _Command <> '' then begin
    // oddelime pripadne parametry
    if _Command[1] = '"' then begin
      // format: "prikaz" parametry
      I := Pos('"', Copy(_Command, 2, MaxInt)) + 1;
      if I > 1 then begin
        _Parameters := Copy(_Command, I + 1, MaxInt) + ' ' + _Parameters;
        SetLength(_Command, I)
      end
    end else begin
      I := Pos(' ', _Command);
      if I <> 0 then begin
        _Parameters := Copy(_Command, I + 1, MaxInt) + ' ' + _Parameters;
        SetLength(_Command, I - 1)
      end
    end
  end;

  FillChar(Info, SizeOf(Info), 0);
  Info.cbSize := SizeOf(Info);
  Info.lpFile := PChar(_Command);
  Info.lpParameters := PChar(_Parameters);
  Info.lpDirectory := PChar(Directory);
  Info.nShow := ShowCmd;
  Info.fMask := SEE_MASK_NOCLOSEPROCESS;

  if not (ef_Interactive in Flags) then
    Info.fMask := Info.fMask or SEE_MASK_FLAG_NO_UI;

  if ShellExecuteEx(@Info) then begin
    Result := True;
    if PProcess <> nil then
      PProcess^ := Info.hProcess;
    if ef_Wait in Flags then begin
      WaitForSingleObject(Info.hProcess, INFINITE);
      if PProcess <> nil then
        PProcess^ := 0;
      CloseHandle(Info.hProcess)
    end else
    if PProcess = nil then
      CloseHandle(Info.hProcess)
  end else
    Result := False;

{  end else
    Result := ShellExecute(0, nil, PChar(Command), PChar(Parameters), PChar(Directory), ShowCmd) > 32
}
end;

//--------------------------------------------------------------------------

function IsProcessActive(Process: THandle): Boolean;
var
  ExitCode: Cardinal;
begin
  Result := GetExitCodeProcess(Process, ExitCode) and (ExitCode = STILL_ACTIVE)
end;

//--------------------------------------------------------------------------

function GetProcessID(Process: THandle): Cardinal;
var
  Lib: THandle;
  GetID: function (Process: THandle): Cardinal; stdcall;
begin
  if OSCompatibleWith(sysWinXP) then begin
    Lib := GetModuleHandle('kernel32.dll');
    @GetID := GetProcAddress(Lib, 'GetProcessId');
    if @GetID = nil then
      Result := 0
    else
      Result := GetID(Process);
  end else begin
    // TODO: ? a jak se to zjisti normalne ?
    Result := 0
  end
end;

//--------------------------------------------------------------------------

type
  PKillData = ^TKillData;
  TKillData = record
    ProcessId: Cardinal;
    Succeeded: Boolean;
  end;

function BorlandKillEnumerator(Handle: HWND; Data: Pointer): BOOL; stdcall;

  function GetWindowClass( WinHandle : HWND ) : string;
  begin
    SetLength( Result, MAX_PATH );
    SetLength( Result, GetClassName( WinHandle, pchar( Result ), length(Result)));
  end;

var
  ProcessId: Cardinal;
begin
  Result := True;
  ProcessId := 0;
  GetWindowThreadProcessId(Handle, ProcessId);
  if ProcessId = PKillData(Data)^.ProcessId then
    if GetWindowClass(Handle) = 'TApplication' then begin
      PostMessage(Handle, WM_QUIT, 0, 0);
      PKillData(Data)^.Succeeded := True;
      Result := False
    end
end;

function GenericKillEnumerator(Handle: HWND; Data: Pointer): BOOL; stdcall;

  function GetWindowClass( WinHandle : HWND ) : string;
  begin
    SetLength( Result, MAX_PATH );
    SetLength( Result, GetClassName( WinHandle, pchar( Result ), length(Result)));
  end;

var
  ProcessId: Cardinal;
begin
  // posleme WM_CLOSE vsem Top-level ? oknum hlavniho ? vlakna at to funguje i na ne-Delphi aplikace
  Result := True;
  ProcessId := 0;
  GetWindowThreadProcessId(Handle, ProcessId);
  if ProcessId = PKillData(Data)^.ProcessId then
    if IsWindowVisible(Handle) and IsWindowEnabled(Handle) then begin
      PKillData(Data)^.Succeeded := True;
      PostMessage(Handle, WM_CLOSE, 0, 0);
    end
end;

function KillProcess(Process: THandle; ProcessId: Cardinal; TimeOut: Cardinal): Boolean;
type
  TKillMode = (kmClose, kmDestroy, kmKill);

  function Kill(Mode: TKillMode): Boolean;
  var
    Data: TKillData;
  begin
    if Mode = kmKill then
      Result := TerminateProcess(Process, 0)
    else begin
      if ProcessId <> 0 then
        Data.ProcessId := ProcessId
      else
        Data.ProcessId := GetProcessID(Process);
      Data.Succeeded := False;

      if Data.ProcessId <> 0 then begin
        if Mode = kmDestroy then
          EnumWindows(@BorlandKillEnumerator ,Integer(@Data));

        if Mode = kmClose then
          EnumWindows(@GenericKillEnumerator, Integer(@Data));

        Result := Data.Succeeded
      end else
        Result := TerminateProcess(Process, 0)
    end
  end;

begin
  if TimeOut <> 0 then begin
    // nejdriv podobrem
    if Kill(kmClose) then begin
      WaitForSingleObject(Process, TimeOut);
      if not IsProcessActive(Process) then begin
        Result := True;
        exit
      end
    end;

    if Kill(kmDestroy) then begin
      WaitForSingleObject(Process, TimeOut);
      if not IsProcessActive(Process) then begin
        Result := True;
        exit
      end
    end;
  end;
  Kill(kmKill);          // TODO: nemelo by se tu chvili pockat ?
  Result := not IsProcessActive(Process);
end;

//--------------------------------------------------------------------------

type
  PGetWindowData = ^TGetWindowData;
  TGetWindowData = record
    ProcessId: Cardinal;
    OnlyVisible: Boolean;
    List: TList;
  end;

function ProcessWindowsEnumerator(Handle: HWND; Data: Pointer): BOOL; stdcall;
var
  ProcessId: Cardinal;
begin
  Result := True;
  ProcessId := 0;
  GetWindowThreadProcessId(Handle, ProcessId);
  if ProcessId = PGetWindowData(Data)^.ProcessId then
    if (not PGetWindowData(Data)^.OnlyVisible) or IsWindowVisible(Handle) then
      PGetWindowData(Data)^.List.Add(Pointer(Handle))
end;

function ProcessWindowsList(List: TList; ProcessId: Cardinal; OnlyVisible: Boolean): Boolean;
var
  Data: TGetWindowData;
begin
  Result := ProcessId <> 0;
  if Result then begin
    Data.ProcessId := ProcessId;
    Data.List := List;
    Data.OnlyVisible := OnlyVisible;
    EnumWindows(@ProcessWindowsEnumerator, Integer(@Data))
  end
end;

//-------------------------------------------------------------------------

function AppRunning(const AppID: string; CheckOnly: Boolean): Boolean;
var
  Mutex: THandle;
begin
  Result := False;
  Mutex := CreateMutex(nil, True, PChar(AppID));
  if Mutex <> 0 then begin
    if GetLastError = ERROR_ALREADY_EXISTS then
      // We leave the handle open so that others might detect us.
      // upon ExitProcess() Windows will close the handle
      Result := True
  end else
    // TODO: CreateMutex failed
    ;
  if CheckOnly then
    CloseHandle(Mutex)
end;

// --------------------------------------------------------------------------

function GetDisplayName(const FileName: string): string;
var
  Info: TSHFileInfo;
begin
  if SHGetFileInfo(PChar(ExpandFileName(FileName)),
    0,
    Info,
    SizeOf(Info),
    SHGFI_DISPLAYNAME) <> 0 then begin
    Result := StrPas(Info.szDisplayName)
  end else
    Result := FileName
end;

// --------------------------------------------------------------------------

function _ValidateLogon(const UserName: string; const Password: string; const Domain: string): Cardinal;
var
  Retvar : Boolean;
  LHandle : THandle;
begin
  Retvar := LogonUser(PChar(UserName), PChar(Domain), PChar(PassWord),
                      LOGON32_LOGON_NETWORK, LOGON32_PROVIDER_DEFAULT,
                      LHandle);
  if Retvar then begin
    Result := ERROR_SUCCESS;
    CloseHandle(LHandle);
  end else begin
    Result := GetLastError;
    if Result = ERROR_SUCCESS then
      Result := ERROR_INVALID_FUNCTION
  end;
end;

function ValidateLogon(const UserName: string; const Password: string; const Domain: string): Cardinal;
begin
  SetPrivilege(SE_TCB_NAME, True);
  Result := _ValidateLogon(UserName, Password, Domain);
  if Result = ERROR_SUCCESS then begin
    // TODO: check valid user name
    if _ValidateLogon(UserName, Password + 'x', Domain) = 0 then
      // if user has two valid passwords then its a ghost
      Result := ERROR_BAD_USERNAME
  end;
end;

end.
