{ KEYWORDS: VERSION,TODO }
{ VERSION: 1.4 }
{ 1.4 / 4.6.2006
  - nov funkce FileSize()
  1.3 / 18.11.2004
  - alternativy bez parametru TFileSystem
  1.2
  - procedure CopyFiles() -> function CopyFiles()
  1.1 / 15.9.2003
  - CreateFileName() zachova znaky * a ?
  1.0
  - pred komunikaci s OSFileSystem (z konzolove aplikace) udelej
    af:boolean=SwitchToAnsi
    a potom
    RestoreDefault(af)
  0.9
  - Synchronize()
  0.8
  - umi kopirovat i readonly soubory (zmena poradi SetAttr a SetDate)
  0.7
  - CreateFileName
  0.6
  - TFileNameGenerator
  0.5
  - TCachedStream made more cache based
  0.4
  - TCachedStream
  0.3
  - GetStatistics()
}
{ TODO: matchesMask() }
{ TODO: operace ve stylu SHFileOperation }
unit fileUtils;

interface
uses Windows, Classes, SysUtils, fileSys;

procedure Synchronize(source: TFileSystem;
  sourceDir: string;
  target: TFileSystem;
  targetDir: string;
  masks: TStrings;
  identity: boolean);

function CopyFiles(source: TFileSystem;
  sourceDir: string;
  target: TFileSystem;
  targetDir: string): boolean; overload;
function CopyFiles(sourceDir: string;
  targetDir: string): boolean; overload;

function CopyFile(source: TFileSystem; sourceName: string;
  target: TFileSystem; targetName: string): boolean; overload;
function CopyFile(sourceName: string;
  targetName: string): boolean; overload;

function EraseFolder(source: TFileSystem;
  sourceDir: string): boolean; overload;
function EraseFolder(sourceDir: string): boolean; overload;

function DeleteFiles(source: TFileSystem;
  sourceDir: string): boolean;

function ForceDirectories(fs: TFileSystem; Dir: string): boolean; overload;
function ForceDirectories(Dir: string): boolean; overload;

procedure FileAgeToFileTime(Age: integer; var FileTime: TFileTime);
function FileTimeToFileAge(var FileTime: TFileTime): integer;
function FileAgeNow: integer;
function IsAbsoluteFileName(const fileName: string): boolean;
function CreateFileName(fs: TFileSystem; const root: string; const fileName: string; isFolder: boolean): string;

function FileSize(fs: TFileSystem; const FileName: string): Integer; overload;
function FileSize(const FileName: string): Integer; overload;

type
  TFileNameGenerator = class(TComponent)
  private
    FID: integer;
    FIDs: string;
    function IDCount: integer;
    function ID(index: integer): integer;
  protected
    fs: TFileSystem;
    rootFolder: string;
    ext: string;
    folders: boolean;
  public
    constructor Create(AnOwner: TComponent; AFileSystem: TFileSystem; AFolder: string; AnExtension: string);
    destructor Destroy; override;
  protected
    function AllocID: integer;
    procedure RemoveID(AnID: integer);
  public
    function GetIndexFileName(AnID: integer): string;
    function GetIndexFromFileName(const fileName: string): integer;
    function CreateFile: string;
    procedure DeleteFile(fileName: string);
  end;

type
  TStatInfo = packed record
    files: packed record
      count: integer;
      bytes: integer;
    end;
    folders: packed record
      count: integer;
    end
  end;

procedure GetStatistics(source: TFileSystem; sourceDir: string; var stat: TStatInfo);

type
  TCachedStream = class(TStream)
  private
    FNoRead, FNoWrite: boolean;
    FOwnStream: boolean;
    FSize: integer; { logical size }
    FStreamSize: integer; { physical size }
    FStreamPosition: integer; { physical position }
    { logical and physical sizes may go out of sync temporarily in order to optimize
      access when resizing }
    FPosition: integer; { logical position }
    FCacheOffset: integer; { FPosition's offset within current page }
    FPageSize: integer;
    cache: record
      pageNo: integer;
      buffer: pointer;
      dirty: boolean;
      size: integer;
    end;
    procedure SetPosition(value: integer);
    procedure SetPageNo(value: integer);
  protected
    FStream: TStream;
  public
    constructor Create(AStream: TStream; releaseStreamOnTerminate: boolean; APageSize: integer; AMode: word);
    destructor Destroy; override;
    procedure SetSize(NewSize: Longint); override;
    function Read(var Buffer; Count: Longint): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
  end;

procedure AddSlash(var dir: string);
procedure RemoveSlash(var dir: string);

function SwitchToAnsi: boolean;
procedure RestoreDefault(flag: boolean);

implementation

function SwitchToAnsi: boolean;
begin
  if AreFileApisANSI then
    Result := false
  else begin
    Result := true;
    SetFileApisToANSI
  end
end;

procedure RestoreDefault(flag: boolean);
begin
  if flag then
    SetFileApisToOEM
end;

procedure AddSlash(var dir: string);
begin
  if length(dir) > 0 then
    if dir[length(dir)] <> '\' then
      dir := dir + '\'
end;

procedure RemoveSlash(var dir: string);
begin
  if length(dir) > 0 then
    if dir[length(dir)] = '\' then
      SetLength(dir, length(dir) - 1)
end;

function IsAbsoluteFileName;
begin
  Result := (Copy(fileName, 1, 2) = '\\') or (Copy(fileName, 2, 2) = ':\')
end;

function CreateFileName;

  function WildSafe(const fullName: string; const maskSource: string): string;
  var
    fname: string;
  begin
    fname := ExtractFileName(maskSource);
    if (Pos('?', fname) <> 0) or (Pos('*', fname) <> 0) then
      Result := ExtractFilePath(fullName) + fname
    else
      Result := fullName
  end;

begin
  if IsAbsoluteFileName(fileName) then
    Result := fileName
  else begin
    Result := root;
    AddSlash(Result);
    Result := Result + fileName;
    if Assigned(fs) then
      Result := WildSafe(fs.ExpandFileName(Result), Result)
  end;
  if isFolder then
    AddSlash(Result)
  else
    RemoveSlash(Result)
end;

procedure FileAgeToFileTime(Age: integer; var FileTime: TFileTime);
var
  LocalFileTime: TFileTime;
begin
  if DosDateTimeToFileTime(LongRec(Age).Hi, LongRec(Age).Lo, LocalFileTime) and
    LocalFileTimeToFileTime(LocalFileTime, FileTime) then
    exit;
  FileTime.dwLowDateTime := 0;
  FileTime.dwHighDateTime := 0
end;

function FileTimeToFileAge;
var
  LocalFileTime: TFileTime;
begin
  if FileTimeToLocalFileTime(FileTime, LocalFileTime) and
    FileTimeToDosDateTime(LocalFileTime, LongRec(Result).Hi,
    LongRec(Result).Lo) then
    Exit;
  Result := -1;
end;

function FileAgeNow;
var
  stamp: TFileTime;
begin
  GetSystemTimeAsFileTime(stamp);
  Result := FileTimeToFileAge(stamp)
end;

function EraseFolder(source: TFileSystem;
  sourceDir: string): boolean;
begin
  Result := DeleteFiles(source, sourceDir);
  RemoveSlash(sourceDir);
  Result := source.RemoveDir(sourceDir) and Result
end;

function EraseFolder(sourceDir: string): boolean;
begin
  Result := EraseFolder(defaultFileSystem, sourceDir)
end;

function DeleteFiles;
var
  search: TSearchRec;
var
  found: integer;
begin
  Result := true;
  AddSlash(sourceDir);
  found := source.FindFirst(sourceDir + '*.*', faAnyFile, search);
  while found = 0 do begin
    if (search.Attr and faDirectory) <> 0 then begin
      if search.Name[1] <> '.' then begin
        Result := DeleteFiles(source, sourceDir + search.Name) and Result;
        Result := source.RemoveDir(sourceDir + search.Name) and Result;
      end
    end
    else
      Result := source.DeleteFile(sourceDir + search.Name) and Result;
    found := source.FindNext(search)
  end;
  source.FindClose(search);
end;

function CopyFile(sourceName: string;
  targetName: string): boolean;
begin
  Result := CopyFile(defaultFileSystem, sourceName, defaultFileSystem, targetName)
end;

function CopyFile(source: TFileSystem; sourceName: string;
  target: TFileSystem; targetName: string): boolean;
var
  sourceStream, targetStream: TStream;
var
  search: TSearchRec;
var
  found: integer;
begin
  found := source.FindFirst(sourceName, faAnyFile, search);
  if found = 0 then try
    sourceStream := source.GetFileStream(sourceName, fmOpenRead or fmShareDenyNone);
    try
      targetStream := target.GetFileStream(targetName, fmCreate);
      try
        targetStream.CopyFrom(sourceStream, 0);
      finally
        targetStream.Free;
        target.FileSetDate(targetName, search.Time);
        target.FileSetAttr(targetName, search.Attr);
      end
    finally
      sourceStream.Free
    end;
    Result := true
  except
    Result := false
  end
  else
    Result := false;
  source.FindClose(search)
end;

procedure GetFolders(source: TFileSystem; sourceDir: string; folderList: TStrings);

  procedure InternalGetFolders(const folder: string);
  var
    found: integer;
  var
    search: TSearchRec;
  var
    folderName: string;
  begin
    found := source.FindFirst(sourceDir + folder + '*.*', faDirectory, search);
    while found = 0 do begin
      if ((search.Attr and faDirectory) = faDirectory) and
        (search.Name[1] <> '.') then begin
        folderName := folder + search.Name;
        if folderList.IndexOf(folderName) < 0 then begin
          folderList.Add(folderName);
          InternalGetFolders(folderName + '\')
        end
      end;
      found := source.FindNext(search)
    end;
    source.FindClose(search)
  end;

begin
  AddSlash(sourceDir);
  InternalGetFolders('')
end;

procedure GetFileStamps(source: TFileSystem; sourceDir: string; fileList: TStrings; masks: TStrings);

  procedure InternalGetStamps(const folder: string);

    procedure Load(const mask: string);
    var
      found: integer;
    var
      search: TSearchRec;
    var
      fileName: string;
    begin
      found := source.FindFirst(sourceDir + folder + mask, faAnyFile, search);
      while found = 0 do begin
        if (search.Attr and faDirectory) = 0 then begin
          fileName := folder + search.Name;
          if fileList.IndexOf(fileName) < 0 then
            fileList.AddObject(fileName, TObject(pointer(search.Time)))
        end;
        found := source.FindNext(search)
      end;
      source.FindClose(search)
    end;

  var
    i: integer;
  var
    found: integer;
  var
    search: TSearchRec;
  begin
    if Assigned(masks) then begin
      for i := masks.Count - 1 downto 0 do
        Load(masks[i])
    end
    else
      Load('*.*');

    found := source.FindFirst(sourceDir + folder + '*.*', faDirectory, search);
    while found = 0 do begin
      if ((search.Attr and faDirectory) = faDirectory) and
        (search.Name[1] <> '.') then
        InternalGetStamps(folder + search.Name + '\');
      found := source.FindNext(search)
    end;
    source.FindClose(search)
  end;

begin
  AddSlash(sourceDir);
  InternalGetStamps('')
end;

procedure Synchronize;
var
  sourceFiles: TStrings;
var
  targetFiles: TStrings;
var
  ti, si: integer;
begin
  AddSlash(sourceDir);
  AddSlash(targetDir);
  sourceFiles := TStringList.Create;
  TStringList(sourceFiles).Sorted := true;
  targetFiles := TStringList.Create;
  TStringList(targetFiles).Sorted := true;
  try
    GetFileStamps(source, sourceDir, sourceFiles, masks);
    if identity then
      GetFileStamps(target, targetDir, targetFiles, nil)
    else
      GetFileStamps(target, targetDir, targetFiles, masks);

    { delete files in target that don't appear in source }
    for ti := targetFiles.Count - 1 downto 0 do
      if sourceFiles.IndexOf(targetFiles[ti]) < 0 then
        target.DeleteFile(targetDir + targetFiles[ti]);

    ForceDirectories(target, targetDir);
    { copy files in source to target if they don't exist or are of different age }
    for si := sourceFiles.Count - 1 downto 0 do begin
      ti := targetFiles.IndexOf(sourceFiles[si]);
      if (ti < 0) or (sourceFiles.Objects[si] <> targetFiles.Objects[ti]) then
        CopyFile(source, sourceDir + sourceFiles[si], target, targetDir + sourceFiles[si]);
    end;

    if identity then begin
      sourceFiles.Clear;
      targetFiles.Clear;
      GetFolders(source, sourceDir, sourceFiles);
      GetFolders(target, targetDir, targetFiles);
      { delete folders in target that don't appear in source }
      for ti := targetFiles.Count - 1 downto 0 do
        if sourceFiles.IndexOf(targetFiles[ti]) < 0 then
          EraseFolder(target, targetDir + targetFiles[ti])
    end
  finally
    sourceFiles.Free;
    targetFiles.Free
  end
end;

function CopyFiles(source: TFileSystem;
  sourceDir: string;
  target: TFileSystem;
  targetDir: string): boolean;

  function InternalCopyFiles(sourceDir, targetDir: string): boolean;
  var
    found: integer;
  var
    search: TSearchRec;
  begin
    Result := false;
    AddSlash(sourceDir);
    AddSlash(targetDir);
    found := source.FindFirst(sourceDir + '*.*', faAnyFile, search);
    try
      while found = 0 do begin
        if (search.Attr and faDirectory) = faDirectory then begin
          if search.Name[1] <> '.' then begin
            target.CreateDir(targetDir + search.Name);
            if not InternalCopyFiles(sourceDir + search.Name, targetDir + search.Name) then
              exit
          end
        end
        else begin
          if not CopyFile(source, sourceDir + search.Name, target, targetDir + search.Name) then
            exit
        end;
        found := source.FindNext(search)
      end
    finally
      source.FindClose(search)
    end;
    Result := true
  end;

begin
  ForceDirectories(target, targetDir);
  Result := InternalCopyFiles(sourceDir, targetDir)
end;

function CopyFiles(sourceDir: string;
  targetDir: string): boolean;
begin
  Result := CopyFiles(defaultFileSystem, sourceDir, defaultFileSystem, targetDir)
end;

procedure GetStatistics;

  procedure InternalGet(folder: string);
  var
    search: TSearchRec;
  var
    found: integer;
  begin
    AddSlash(folder);
    found := source.FindFirst(folder + '*.*', faAnyFile, search);
    while found = 0 do begin
      if (search.Attr and faDirectory) = faDirectory then begin
        if search.Name[1] <> '.' then begin
          inc(stat.folders.count);
          InternalGet(folder + search.Name)
        end
      end
      else begin
        inc(stat.files.Count);
        inc(stat.files.bytes, search.Size)
      end;
      found := source.FindNext(search)
    end;
    source.FindClose(search)
  end;

begin
  FillChar(stat, sizeof(stat), 0);
  InternalGet(sourceDir)
end;

function ForceDirectories(fs: TFileSystem; Dir: string): boolean;

  function InternalForceDirectories(Dir: string): boolean;
  begin
    Result := true;
    if dir = '' then
      exit;
    if dir[length(dir)] = '\' then
      SetLength(dir, length(dir) - 1);
    if (Length(Dir) < 3) or fs.DirectoryExists(Dir)
      or (ExtractFilePath(Dir) = Dir) then
      Exit; { avoid 'xyz:\' problem. }
    if InternalForceDirectories(ExtractFilePath(Dir)) then
      Result := fs.CreateDir(Dir);
  end;

begin
  Result := InternalForceDirectories(fs.ExpandFileName(Dir))
end;

function ForceDirectories(Dir: string): boolean;
begin
  Result := ForceDirectories(defaultFileSystem, Dir)
end;

function FileSize(fs: TFileSystem; const FileName: string): Integer;
var
  Search: TSearchRec;
begin
  if FindFirst(FileName, faAnyFile, Search) = 0 then begin
    Result := Search.Size
  end
  else
    Result := -1;
  FindClose(Search)
end;

function FileSize(const FileName: string): Integer;
begin
  Result := FileSize(defaultFileSystem, FileName)
end;

{-------------------------------------------------------------------------}
const
  NoNo = -1;

constructor TCachedStream.Create;
begin
  inherited Create;
  FStream := AStream;
  FOwnStream := releaseStreamOnTerminate;
  cache.pageNo := NoNo;

  FPageSize := APageSize;
  GetMem(cache.buffer, FPageSize);

  case AMode and $F of
    fmOpenRead: FNoWrite := true;
    fmOpenWrite: FNoRead := true
  end;

  { use mimikrii }
  if not FNoRead then begin
    FSize := FStream.Size;
    FStreamSize := FSize;
  end;
  FPosition := FStream.Position;
  FStreamPosition := FPosition
end;

destructor TCachedStream.Destroy;
begin
  SetPageNo(NoNo);
  if cache.buffer <> nil then
    FreeMem(cache.buffer);
  try
    if FStreamSize <> FSize then
      FStream.Size := FSize;
  finally
    if FOwnStream then
      FStream.Free;
    inherited
  end
end;

procedure TCachedStream.SetPageNo;
var
  pos: integer;
var
  siz: integer;
begin
  if value = cache.pageNo then
    exit;

  if cache.dirty then begin
    pos := cache.pageNo * FPageSize;
    if pos <> FStreamPosition then
      FStream.Position := pos;

    FStream.WriteBuffer(cache.buffer^, cache.size);
    inc(pos, cache.size);
    if pos > FStreamSize then
      FStreamSize := pos;
    FStreamPosition := pos;
    cache.dirty := false
  end;

  cache.pageNo := value;
  if value = NoNo then
    exit;

  pos := cache.pageNo * FPageSize;

  if pos >= FStreamSize then begin
    { this is virtual page access. virtual page has no real content yet }
    siz := FSize - pos;
    if siz > FPageSize then
      cache.size := FPageSize
    else
      cache.size := siz
  end
  else begin
    { this is a real page access }
    if pos <> FStreamPosition then
      FStream.Position := pos;
    cache.size := FStream.Read(cache.buffer^, FPageSize);
    inc(FStreamPosition, cache.size)
  end
end;

procedure TCachedStream.SetPosition;
begin
  SetPageNo(value div FPageSize);
  FCacheOffset := value mod FPageSize
end;

procedure TCachedStream.SetSize(NewSize: Longint);
var
  cacheFirst, cacheLast: integer;

  { synchronize underlying stream with user's desire }

  procedure DoSetSize;
  begin
    try
      FStream.Size := NewSize;
      FSize := NewSize;
      FStreamSize := NewSize
    except
      FSize := FStream.Size;
      FStreamSize := FSize;
      raise
    end
  end;

var
  newCacheSize: integer;
begin
  if newSize < 0 then
    newSize := 0;
  if FSize = newSize then
    exit;

  if cache.pageNo = NoNo then begin
    if newSize > FSize then
      { its enlarging. now we'll just delay the resize }
      FSize := newSize
    else
      { apply shrinker now }
      DoSetSize
  end
  else begin
    { now check if cache must be reloaded after resize. case study:

     <
     B: xxxxCCCCxxxx
     A: xxxxCCCCxx

     <
     B: xxxxCCCCxxxx
     A: xxxxCC

     <
     B: xxxxCCCCxxxx
     A: xxx

     >
     B: xxxxCCCCxxxx
     A: xxxxCCCCxxxxxx

     >
     B: xxxxCC
     A: xxxxCCCCxx

    }
    cacheFirst := cache.pageNo * FPageSize;

    if newSize < FSize then begin
      { shrinking }

      cacheLast := cacheFirst + cache.size - 1;
      { currently cache buffer covers cacheFirst..cacheStart }

      if newSize > cacheLast then
        {
         <
         B: xxxxCCCCxxxx
         A: xxxxCCCCxx
        }
      else if newSize <= cacheFirst then begin
        {
         <
         B: xxxxCCCCxxxx
         A: xxx
        }
        cache.dirty := false;
        SetPageNo(NoNo);
      end
      else begin
        {
         <
         B: xxxxCCCCxxxx
         A: xxxxCC
        }
        cache.size := newSize - cacheFirst
      end;

      DoSetSize
    end
    else begin
      { enlarging }
      FSize := newSize;
      if cache.size < FPageSize then begin
        {
         >
         B: xxxxCC
         A: xxxxCCCCxx
        }
        newCacheSize := newSize - cacheFirst;
        if newCacheSize > FPageSize then
          cache.size := FPageSize
        else
          cache.size := newCacheSize;
      end;
      {else
       >
       B: xxxxCCCCxxxx
       A: xxxxCCCCxxxxxx
      }
    end;
  end;

end;

function TCachedStream.Read(var Buffer; Count: Longint): Longint;
var
  offset: integer;
var
  chunkSize: integer;
var
  sizeToDo: integer;
var
  buf: PChar;
begin
  if (FPosition >= 0) and (Count >= 0) then begin
    Result := FSize - FPosition;
    if Result > 0 then begin
      if Result > Count then
        Result := Count;

      { now load from the cache }
      offset := FPosition;
      sizeToDo := Result;
      buf := @Buffer;

      while true do begin
        SetPosition(offset);
        chunkSize := cache.size - FCacheOffset;
        if chunkSize > sizeToDo then
          chunkSize := sizeToDo;

        Move(PChar(cache.buffer)[FCacheOffset], buf[offset - FPosition], chunkSize);
        dec(sizeToDo, chunkSize);

        if sizeToDo = 0 then
          break;

        inc(offset, chunkSize);
      end;

      Inc(FPosition, Result);
      Exit;
    end;
  end;
  Result := 0;
end;

function TCachedStream.Write(const Buffer; Count: Longint): Longint;
var
  Pos: Longint;
var
  offset: integer;
var
  chunkSize: integer;
var
  sizeToDo: integer;
var
  buf: PChar;
begin
  if (FPosition >= 0) and (Count >= 0) then begin
    Pos := FPosition + Count;
    if Pos > 0 then begin
      if Pos > FSize then begin
        SetSize(Pos);
        FPosition := Pos - Count
      end;

      Result := Count;

      { now save to the cache }
      offset := FPosition;
      sizeToDo := Result;
      buf := @Buffer;

      while true do begin
        SetPosition(offset);
        chunkSize := cache.size - FCacheOffset;
        if chunkSize > sizeToDo then
          chunkSize := sizeToDo;

        Move(buf[offset - FPosition], PChar(cache.buffer)[FCacheOffset], chunkSize);
        cache.dirty := true;

        dec(sizeToDo, chunkSize);

        if sizeToDo = 0 then
          break;

        inc(offset, chunkSize);
      end;

      FPosition := Pos;
      Exit;
    end;
  end;
  Result := 0;
end;

function TCachedStream.Seek(Offset: Longint; Origin: Word): Longint;
begin
  case Origin of
    0: FPosition := Offset;
    1: Inc(FPosition, Offset);
    2: FPosition := FSize + Offset;
  end;
  Result := FPosition;
end;

//------------------------------------------------------------------------

 { TUNE: }
const
  FIDSFile = 'ids.dat';

constructor TFileNameGenerator.Create;
var
  stream: TStream;
begin
  fs := AFileSystem;
  if fs = nil then
    fs := DefaultFileSystem;
  fs.Use;
  rootFolder := AFolder;
  RemoveSlash(rootFolder);
  rootFolder := fs.ExpandFileName(rootFolder);
  fs.CreateDir(rootFolder);
  AddSlash(rootFolder);
  ext := AnExtension;
  folders := ext = '\';
  if folders then
    ext := ''
  else if length(ext) > 0 then
    ext := '.' + ext;
  inherited Create(AnOwner);

  try
    stream := fs.GetFileStream(rootFolder + FIDSFile, fmOpenRead);
    try
      SetLength(FIDS, stream.Size);
      if length(FIDS) > 0 then
        stream.ReadBuffer(FIDS[1], length(FIDS))
    finally
      stream.Free
    end;
  except
  end;
end;

destructor TFileNameGenerator.Destroy;
var
  stream: TStream;
begin
  try
    stream := fs.GetFileStream(rootFolder + FIDSFile, fmCreate);
    try
      if length(FIDS) > 0 then
        stream.WriteBuffer(FIDS[1], length(FIDS))
    finally
      stream.Free
    end
  except
  end;

  fs.Release;
  inherited
end;

function TFileNameGenerator.IDCount;
begin
  Result := length(FIDs) div sizeof(integer)
end;

function TFileNameGenerator.ID;
begin
  Move(FIDS[1 + index * sizeof(integer)], Result, sizeof(Result))
end;

procedure TFileNameGenerator.RemoveID;
var
  i: integer;
begin
  for i := IDCount - 1 downto 0 do
    if ID(i) = AnID then begin
      Delete(FIDS, 1 + i * sizeof(integer), sizeof(integer));
      break
    end
end;

function TFileNameGenerator.AllocID;
var
  i: integer;

  function n2s(n: integer): string;
  begin
    SetLength(Result, sizeof(integer));
    Move(n, Result[1], sizeof(integer))
  end;

label
  loop;
begin
  loop:
  inc(FID);
  i := IDCount - 1;
  while i >= 0 do
    if FID = ID(i) then
      goto loop
    else
      dec(i);

  Result := FID;
  FIDS := FIDS + n2s(Result)
end;

function TFileNameGenerator.GetIndexFileName;
begin
  if AnID = 0 then
    Result := ''
  else
    Result := Format('%s%d%s', [rootFolder, AnID, ext]);
end;

function TFileNameGenerator.GetIndexFromFileName;
begin
  try
    if fileName = '' then
      Result := 0
    else
      Result := StrToInt(ChangeFileExt(ExtractFileName(fileName), ''));
  except
    Result := 0
  end
end;

procedure TFileNameGenerator.DeleteFile;
var
  id: integer;
begin
  RemoveSlash(fileName);
  id := GetIndexFromFileName(fileName);
  fileName := GetIndexFileName(id);
  if folders then
    EraseFolder(fs, fileName)
  else
    fs.DeleteFile(fileName);
  RemoveID(id)
end;

function TFileNameGenerator.CreateFile;
var
  id: integer;
begin
  id := AllocID;
  Result := GetIndexFileName(id);
  if folders then
    fs.CreateDir(Result)
  else
    fs.GetFileStream(Result, fmCreate).Free
end;

end.

function MatchesMask(What, Mask: string): boolean;

  function upper(const s: string): string;
  var
    i: longint;
  begin
    for i := 1 to length(s) do
      if s[i] in ['a'..'z'] then
        upper[i] := char(byte(s[i]) - 32)
      else
        upper[i] := s[i];
    upper[0] := s[0];
  end;

  function CmpStr(const hstr1, hstr2: string): boolean;
  var
    found: boolean;
    i1, i2: longint;
  begin
    i1 := 0;
    i2 := 0;
    found := true;
    while found and (i1 < length(hstr1)) and (i2 < length(hstr2)) do begin
      inc(i2);
      inc(i1);
      case hstr1[i1] of
        '?':
          found := true;
        '*': begin
            found := true;
            if (i1 = length(hstr1)) then
              i2 := length(hstr2)
            else if (i1 < length(hstr1)) and (hstr1[i1 + 1] <> hstr2[i2]) then begin
              if i2 < length(hstr2) then
                dec(i1)
            end
            else if i2 > 1 then
              dec(i2);
          end;
      else
        found := (hstr1[i1] = hstr2[i2]) or (hstr2[i2] = '?');
      end;
    end;
    if found then begin
      { allow 'p*' matching 'p' }
      if (i1 < length(hstr1)) and (hstr1[i1 + 1] = '*') then
        inc(i1);
      found := (i1 >= length(hstr1)) and (i2 >= length(hstr2));
    end;
    CmpStr := found;
  end;

var
  D1, D2: DirStr;
  N1, N2: NameStr;
  E1, E2: Extstr;
begin
{$IFDEF linux}
  FSplit(What, D1, N1, E1);
  FSplit(Mask, D2, N2, E2);
{$ELSE}
  FSplit(Upper(What), D1, N1, E1);
  FSplit(Upper(Mask), D2, N2, E2);
{$ENDIF}
  MatchesMask := CmpStr(N2, N1) and CmpStr(E2, E1);
end;

