{ VERSION: 1.5 }
{ 1.5 / 12.7.2005
  - podporuje so_Closed
  1.4 / 17.5.2005
  - funguje ovlada IB_DRIVER_IBX. pro ten je mnohem rychlej ne
    IB_DRIVER_ZEOS, ale pro zpis je pomalej
  1.3 / 9.5.2005
  - pokus o opravu problmu s FireBird Embedded: pi uzaven databze po
    zpisech dojde v isc_detach_database() k 'Access Violation'.
    viz. FIX_EMBEDDED
  1.2 / 3.4.2005
  - Zeos verze dekoduje parametr 'host' z parametru 'db' takze se da pak
    zjistit cilovy pocitac podle DbcConnection..HostName
  1.1 / 10.3.2005
  - pro TMyskaRegion pouziva IB mechanismus POST_EVENT
}
unit uMyskaBaseIBStore;

// TUNE:
{$DEFINE IB_DRIVER_ZEOS}
{ $DEFINE IB_DRIVER_IBX}
{ $DEFINE IB_DRIVER_BDE}

interface
uses
  uMyskaDB, uMyskaSQL, Classes, DB
  {$IFDEF IB_DRIVER_ZEOS}
  ,uMyskaZeosDB, ZConnection, ZDbcInterbase6Notify
  {$ENDIF}
  {$IFDEF IB_DRIVER_IBX}
  ,IBDatabase
  {$ENDIF}
  {$IFDEF IB_DRIVER_BDE}
  ,DBTables
  {$ENDIF}
  ;

type
  {$IFDEF IB_DRIVER_ZEOS}
  TMyskaBaseIBStore = class(TZeosStore)
  private
    FEventNotify: TZInterbase6Notify;
    FEvents: TList;
    procedure RemoveEvent(Sender: TObject);
    procedure EventSignaled(Sender: TObject; EventName: string; EventCount: longint; var CancelAlerts: boolean);
  protected
    procedure DoDisconnect; override;
    procedure DoPrepareRegions; override;
    procedure ConfigureConnection(Params: TStrings); override;
    procedure GetConnectedUsers(List: TStrings);
    function ConnectRegionEvent(ID: Integer; Handler: TEventHandler): TObject; override;
    procedure SignalRegionEvent(ID: Integer); override;
  end;
  {$ENDIF}
  {$IFDEF IB_DRIVER_IBX}
  TMyskaBaseIBStore = class(TMyskaSQLStore)
  private
    FDatabase: TIBDatabase;
    FTransaction: TIBTransaction;
    FAutoCommit: Boolean;
    FOldAutoCommit: Boolean;
    procedure Commit;
  protected
    function DoExec(const Query: string; Options: TSelectOptions): TDataSet; override;
    procedure DoConnect; override;
    procedure DoDisconnect; override;
    procedure DoBeginUpdate; override;
    procedure DoEndUpdate; override;
  protected
    procedure GetConnectedUsers(List: TStrings);
  public
    property Handle: TIBDatabase read FDatabase;
  end;
  {$ENDIF}
  {$IFDEF IB_DRIVER_BDE}
  TMyskaBaseIBStore = class(TMyskaSQLStore)
  private
    FDatabase: TDatabase;
  protected
    function DoExec(const Query: string; Options: TSelectOptions): TDataSet; override;
    procedure DoConnect; override;
    procedure DoDisconnect; override;
  protected
    procedure GetConnectedUsers(List: TStrings);
  public
    property Handle: TDatabase read FDatabase;
  end;
  {$ENDIF}

implementation
{ TODO: BDE nefunguje kvuli dialektu a IB neumi dobre LiveQuery a je zbytecne pomaly na zapis }
uses
  SysUtils, MiscUtils
  {$IFDEF PROFILER_ON}
  ,Profiler
  {$ENDIF}
  {$IFDEF IB_DRIVER_ZEOS}
  ,ZDbcIntfs, ZPlainInterbaseDriver, ZDbcInterbase6, ZDbcInterbase6Utils
  {$ENDIF}
  {$IFDEF IB_DRIVER_IBX}
  ,IBQuery, IBCustomDataSet, IBUpdateSQL, IBDatabaseInfo, IBTable, IBSQLMonitor, DBCommon, uPropSet
  {$ENDIF}
  {$IFDEF IB_DRIVER_BDE}
  ,Windows, uPropSet, RegUtils
  {$ENDIF}
  ;

{$IFDEF IB_DRIVER_ZEOS}
procedure TMyskaBaseIBStore.ConfigureConnection(Params: TStrings);

  function Param(const ID: string; const DefaultValue: string = ''): string;
  begin
    Result := Params.Values[ID];
    if Result = '' then
      Result := DefaultValue
  end;

var
  I: Integer;  
begin
  FDBase.Protocol := Param('protocol', 'firebird-1.0');
  FDBase.HostName := Param('host');
  FDBase.Database := Param('db');

  if FDBase.HostName = '' then begin
    I := Pos(':', FDBase.Database);
    if (I <> 0) and
       (
        (Copy(FDBase.Database, I + 1, 2) = '\\') or
        (Copy(FDBase.Database, I + 2, 2) = ':\')
       ) then begin
      FDBase.HostName := Copy(FDBase.Database, 1, I - 1);
      FDBase.Database := Copy(FDBase.Database, I + 1, MaxInt)
    end
  end;

  FDBase.User := Param('user', 'SYSDBA');
  FDBase.Password := Param('password', 'masterkey');
  FDBase.Properties.Values['dialect'] := '3';
  FDBase.Properties.Values['codepage'] := 'WIN1250';
end;

procedure TMyskaBaseIBStore.GetConnectedUsers(List: TStrings);
var
  DatabaseInfoCommand1: Char;
  StatusVector: TARRAY_ISC_STATUS;
  Buffer: array[0..IBBigLocalBufferLength - 1] of Char;
  Conn: IZInterbase6Connection;
  I: Integer;
  Item: Integer;
  ItemLength: Integer;

  function ItemAsString(Ptr: Pointer; Size: Integer): string;
  begin
    SetLength(Result, Size);
    if Result <> '' then
      Move(Ptr^, Result[1], Size)
  end;

begin
  DatabaseInfoCommand1 := Char(isc_info_user_names);
  Conn := Handle.DbcConnection as IZInterbase6Connection;
  Conn.GetPlainDriver.isc_database_info(
    @StatusVector, Conn.GetDBHandle, 1, @DatabaseInfoCommand1,
    IBLocalBufferLength, Buffer);
  CheckInterbase6Error(Conn.GetPlainDriver, StatusVector);

  I := 0;
  while Buffer[I] <> Char(isc_info_end) do begin
    Item := Integer(Buffer[I]); Inc(I);
    ItemLength := Conn.GetPlainDriver.isc_vax_integer(@Buffer[I], 2); Inc(I, 2);
    case Item of
    isc_info_user_names:
      List.Add(ItemAsString(@Buffer[I + 1], Ord(Buffer[I])));
    end;
    Inc(I, ItemLength)
  end;
end;

type
  TIBEventHandler = class
  public
    FStore: TMyskaBaseIBStore;
    EventName: string;
    Handler: TEventHandler;
    destructor Destroy; override;
  end;

procedure TMyskaBaseIBStore.DoDisconnect;
var
  I: Integer;
  Ev: TIBEventHandler;
begin
  FreeAndNil(FEventNotify);
  if Assigned(FEvents) then begin
    for I := 0 to FEvents.Count - 1 do begin
      Ev := TIBEventHandler(FEvents[I]);
      Ev.FStore := nil
    end;
    FreeAndNil(FEvents);
  end;
  inherited
end;

destructor TIBEventHandler.Destroy;
begin
  if Assigned(FStore) then
    FStore.RemoveEvent(Self);
  inherited
end;

procedure TMyskaBaseIBStore.RemoveEvent(Sender: TObject);
var
  Ev: TIBEventHandler absolute Sender;
  Handlers: TStringList;
  I: Integer;
begin
  FEvents.Remove(Sender);
  Handlers := TStringList.Create;
  Handlers.Duplicates := dupIgnore;
  Handlers.Sorted := True;
  try
    for I := FEvents.Count - 1 downto 0 do
      Handlers.Add(TIBEventHandler(FEvents[I]).EventName);
    if Handlers.IndexOf(Ev.EventName) < 0 then begin
      // tahle udalost uz nikoho nezajima
      I := FEventNotify.Events.IndexOf(Ev.EventName);
      if I >= 0 then
        FEventNotify.Events.Delete(I);
    end
  finally
    Handlers.Free
  end
end;

procedure TMyskaBaseIBStore.EventSignaled(Sender: TObject; EventName: string; EventCount: longint; var CancelAlerts: boolean);
var
  I: Integer;
  Ev: TIBEventHandler;
begin
  I := 0;
  while I < FEvents.Count do begin
    Ev := TIBEventHandler(FEvents[I]);
    if SameText(EventName, Ev.EventName) then
      try
        if Assigned(Ev.Handler) then
          Ev.Handler(False { TODO: ? True })
      except
      end;
    Inc(I)
  end
end;

function TMyskaBaseIBStore.ConnectRegionEvent(ID: Integer; Handler: TEventHandler): TObject;
var
  I: Integer;
  S: string;
  Ev: TIBEventHandler;
begin
  if not Assigned(FEventNotify) then begin
    FEventNotify := TZInterbase6Notify.Create(Self);
    FEventNotify.Connection := Handle;
    FEventNotify.OnEventAlert := EventSignaled;
    FEvents := TList.Create;
  end;
  S := Format('REGION%d', [ID]);
  if FEventNotify.Events.IndexOf(S) < 0 then begin
    FEventNotify.Events.Add(S);
    FEventNotify.Registered := True
  end;
  Ev := TIBEventHandler.Create;
  Ev.FStore := Self;
  Ev.Handler := Handler;
  Ev.EventName := S;
  FEvents.Add(Ev);
  Result := Ev
end;

procedure TMyskaBaseIBStore.DoPrepareRegions;
begin
  inherited;
  if not Exists('SELECT RDB$PROCEDURE_NAME FROM RDB$PROCEDURES WHERE RDB$PROCEDURE_NAME=''MYS$SIGNAL''') then
    try
      Exec('SET TERM ##;' +
           'CREATE PROCEDURE MYS$SIGNAL(EVENT_NAME VARCHAR(50)) AS '+
           'BEGIN '+
           'POST_EVENT EVENT_NAME; '+
           'END', [so_Command])
    except
    end
end;

procedure TMyskaBaseIBStore.SignalRegionEvent(ID: Integer);
begin
  Exec('EXECUTE PROCEDURE MYS$SIGNAL(''REGION%d'')', [so_Command], [ID])
end;
{$ENDIF}

{$IFDEF IB_DRIVER_IBX}
// TODO: implantuj notify

procedure TMyskaBaseIBStore.GetConnectedUsers(List: TStrings);
var
  Info: TIBDatabaseInfo;
begin
  Info := TIBDatabaseInfo.Create(nil);
  try
    Info.Database := FDatabase;
    List.Assign(Info.UserNames);
  finally
    Info.Free
  end
end;

function GetFilterFromQuery(const Query: string): string;
const
  CmdWhere = 'WHERE ';
var
  I: Integer;
begin
  Result := SqlNormalize(Query);
  I := SqlPos(CmdWhere, UpperCase(Result));
  if I = 0 then
    Result := ''
  else
    Result := TrimLeft(Copy(Result, I + Length(CmdWhere), MaxInt))
end;

type
  TIBDataSetAccess = class(TIBCustomDataSet)
  end;

  TTable = class(TIBTable)
  protected
    procedure InternalRefreshRow; override;
  end;

procedure TTable.InternalRefreshRow;
begin
  // TODO: pro nektere pripady si vystacime s interni cachi
  if False then
    inherited InternalRefreshRow
end;

{ TODO:
  ohledne transakci se musi autocommitovat vechny DDL pkazy a sgrupovat ty ostatni
  asi pres monitorhook
}

function TMyskaBaseIBStore.DoExec(const Query: string; Options: TSelectOptions): TDataSet;
var
  Exec: TIBQuery;    // TODO: pouzij viceprikazovy interpret
  RWQuery: TIBTable;
  ROQuery: TIBQuery;
  Filter: string;
begin
  Result := nil;
  if so_Command in Options then begin
    Exec := TIBQuery.Create(nil);
    try
      Exec.ParamCheck := False;
      Exec.SQL.Text := Query;
      Exec.Database := FDatabase;
      Exec.ExecSQL
    finally
      Exec.Free;
      if FAutoCommit then
        Commit
    end
  end else begin
    if [so_ReadOnly, so_ReadOnce] * Options = [so_ReadOnly, so_ReadOnce] then begin
      ROQuery := TIBQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.ParamCheck := False;
      ROQuery.Database := FDatabase;
      ROQuery.UniDirectional := True;
      ROQuery.SQL.Text := Query
      // TODO: readonly ?
    end else
    if so_ReadOnly in Options then begin
      ROQuery := TIBQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.ParamCheck := False;
      ROQuery.Database := FDatabase;
      ROQuery.UniDirectional := False;
      ROQuery.SQL.Text := Query
      // TODO: readonly ?
    end else begin
      // TODO: update funguje jenom pro jednu tabulku
      RWQuery := TTable.Create(Self { TODO: });
      Result := RWQuery;
      TIBDataSetAccess(RWQuery).ParamCheck := False;
      Filter := GetFilterFromQuery(Query);
      if Filter <> '' then begin
        RWQuery.Filter := Filter;
        RWQuery.Filtered := True;
      end;
      RWQuery.TableName := { !! }UpperCase(DBCommon.GetTableNameFromQuery(Query));
      RWQuery.DefaultIndex := False;
      RWQuery.ReadOnly := False;
      RWQuery.Database := FDatabase;
      // TODO: read/write ? vygeneruj updateobject podle TField.Origin-u
      // TODO: kdyz je AutoCommit tak odesilej zmeny z internich updatu
    end;
    try
      if not (so_Closed in Options) then
        Result.Active := True
    except
      Result.Free;
      raise
    end
  end
end;

procedure TMyskaBaseIBStore.DoConnect;
var
  Conn: TStrings;

  function Param(const ID: string; const DefaultValue: string = ''): string;
  begin
    Result := Conn.Values[ID];
    if Result = '' then
      Result := DefaultValue
  end;

begin
  FDatabase := TIBDatabase.Create(nil);
  FTransaction := TIBTransaction.Create(nil);
  try
    Conn := TStringList.Create;
    try
      Conn.Text := StringReplace(Connection, ';' , #13#10, [rfReplaceAll]);

      if (Conn.Count = 1) and (Pos('=', Conn.Text) = 0) then
        Conn.Text := 'db=' + Conn.Text;

      // ConfigureConnection
//    FDBase.TransactIsolationLevel := tiReadCommitted;
      FTransaction.Params.Add('read_committed');
//      FTransaction.Params.Add('autocommit');

      FDatabase.DatabaseName := Param('db');
      FDatabase.Params.Values['USER_NAME'] := Param('user', 'SYSDBA');
      FDatabase.Params.Values['PASSWORD'] := Param('password', 'masterkey');
      FDatabase.SQLDialect := 3;
    finally
      Conn.Free
    end;

    FTransaction.DefaultDatabase := FDatabase;
    FDatabase.DefaultTransaction := FTransaction;
    FDatabase.LoginPrompt := False;

//    FDBase.AutoCommit := True;
    FAutoCommit := True;

    FDatabase.Connected := True;

    FDBQueries := TPropSet.Create(FDatabase);
    ConfigureQueries;
    Params.Add(FDBQueries);
  except
    FreeAndNil(FDatabase);
    FreeAndNil(FTransaction);
    raise
  end
end;

procedure TMyskaBaseIBStore.DoDisconnect;
begin
  // TODO: nebo v opacnem poradi ?
  FreeAndNil(FDatabase);
  FreeAndNil(FTransaction);
end;

procedure TMyskaBaseIBStore.DoBeginUpdate;
begin
  FOldAutoCommit := FAutoCommit;
  FAutoCommit := False
end;

procedure TMyskaBaseIBStore.DoEndUpdate;
begin
  FAutoCommit := FOldAutoCommit;
  if FAutoCommit then
    Commit
end;

procedure TMyskaBaseIBStore.Commit;
begin
  { TODO: ? a co FTransaction.Params('autocommit') }
  if Assigned(FTransaction) then
    if FTransaction.Active then
      FTransaction.Commit//Retaining
end;

{$ENDIF}

{$IFDEF IB_DRIVER_BDE}
procedure TMyskaBaseIBStore.GetConnectedUsers(List: TStrings);
begin
  // TODO:
end;

type
  TROQuery = class(TQuery)
  protected
    procedure InternalRefresh; override;
  end;
  TRWQuery = class(TROQuery)
  end;

procedure TROQuery.InternalRefresh;
begin
  // TODO: vrat se na puvodni pozici
  Close;
  Open
end;

function TMyskaBaseIBStore.DoExec(const Query: string; Options: TSelectOptions): TDataSet;
var
  Exec: TQuery;    // TODO: pouzij viceprikazovy interpret
  RWQuery: TQuery;
  ROQuery: TQuery;
begin
  Result := nil;
  if so_Command in Options then begin
    Exec := TQuery.Create(nil);
    try
      Exec.ParamCheck := False;
      Exec.SQL.Text := Query;
      Exec.DatabaseName := FDatabase.DatabaseName;
      Exec.ExecSQL
    finally
      Exec.Free
    end
  end else begin
    if [so_ReadOnly, so_ReadOnce] * Options = [so_ReadOnly, so_ReadOnce] then begin
      ROQuery := TROQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.ParamCheck := False;
      ROQuery.DatabaseName := FDatabase.DatabaseName;
      ROQuery.UniDirectional := True;
      ROQuery.SQL.Text := Query;
      ROQuery.RequestLive := False
    end else
    if so_ReadOnly in Options then begin
      ROQuery := TROQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.ParamCheck := False;
      ROQuery.DatabaseName := FDatabase.DatabaseName;
      ROQuery.UniDirectional := False;
      ROQuery.SQL.Text := Query;
      ROQuery.RequestLive := False
    end else begin
      RWQuery := TRWQuery.Create(Self { TODO: });
      Result := RWQuery;
      RWQuery.ParamCheck := False;
      RWQuery.DatabaseName := FDatabase.DatabaseName;
      RWQuery.UniDirectional := False;
      RWQuery.SQL.Text := Query;
      RWQuery.RequestLive := True;
    end;
    try
      if not (so_Closed in Options) then
        Result.Active := True
    except
      Result.Free;
      raise
    end
  end
end;

procedure TMyskaBaseIBStore.DoConnect;
var
  Conn: TStrings;

  function Param(const ID: string; const DefaultValue: string = ''): string;
  begin
    Result := Conn.Values[ID];
    if Result = '' then
      Result := DefaultValue
  end;

  function DBName(const UID: string): string;
  var
    I: Integer;
  begin
    Result := 'D';
    for I := 1 to Length(UID) do
      if UID[I] in ['0'..'9', 'A'..'F'] then
        Result := Result + UID[I]
  end;

var
  S: string;
  UID: TGUID;
begin
  RegWrite('HKLM\SOFTWARE\Borland\Database Engine\Settings\DRIVERS\INTRBASE\DB OPEN\SQLDIALECT', '3');

  FDatabase := TDatabase.Create(nil);
  try
    Conn := TStringList.Create;
    try
      Conn.Text := StringReplace(Connection, ';' , #13#10, [rfReplaceAll]);

      if (Conn.Count = 1) and (Pos('=', Conn.Text) = 0) then
        Conn.Text := 'db=' + Conn.Text;

      // ConfigureConnection
      {
      CreateGUID(UID);
      S := DBName(UpperCase(GUIDToString(UID)));
      }
      S := DBName(IntToHex(GetTickCount, 8));

      FDatabase.DriverName := 'INTRBASE';
      FDatabase.DatabaseName := S;
      FDatabase.Params.Values['SERVER NAME'] := Param('db');
      FDatabase.Params.Values['USER NAME'] := Param('user', 'SYSDBA');
      FDatabase.Params.Values['PASSWORD'] := Param('password', 'masterkey');
    finally
      Conn.Free
    end;

    FDatabase.Temporary := True;
    FDatabase.TransIsolation := tiReadCommitted;
    FDatabase.LoginPrompt := False;

    // TODO:   FDBase.AutoCommit := True;
    FDatabase.Connected := True;

    FDBQueries := TPropSet.Create(FDatabase);
    ConfigureQueries;
    Params.Add(FDBQueries);
  except
    FreeAndNil(FDatabase);
    raise
  end
end;

procedure TMyskaBaseIBStore.DoDisconnect;
begin
  FreeAndNil(FDatabase);
end;
{$ENDIF}

begin
{$IFDEF PROFILER_ON}
  {$IFDEF IB_DRIVER_ZEOS}Profiler.Log('IB Driver: Zeos');{$ENDIF}
  {$IFDEF IB_DRIVER_IBX}Profiler.Log('IB Driver: IBX');{$ENDIF}
  {$IFDEF IB_DRIVER_BDE}Profiler.Log('IB Driver: BDE');{$ENDIF}
{$ENDIF}
{$IFDEF IB_DRIVER_IBX}
  DisableMonitoring
{$ENDIF}
end.
