{ VERSION: 1.1 }
{ 1.1 / 12.7.2005
  - podporuje so_Closed
  1.0 / 15.5.2005
}
unit uMyskaODBCDB;

interface
uses
  Classes, DB, uMyskaDB, uMyskaSQL, mDataBas;

type
  TODBCStore = class(TMyskaSQLStore)
  private
    FTableNames: TStrings;
    procedure ClearMetadataCache;
  protected
    FDBase: TmDataBase;
  protected
    function DoExec(const Query: string; Options: TSelectOptions): TDataSet; override;
    procedure DoConnect; override;
    procedure DoDisconnect; override;
  protected
    function CmdTableExists(const Params: Variant): TDataSet;
  protected
    procedure ConfigureConnection(Params: TStrings); virtual; abstract;
  end;

implementation
uses
  SysUtils, Variants, uPropSet, mQuery, StrUtils, mTable, DBCommon;

type
  TRWQuery = class(TmTable)
  protected
    procedure InternalOpen; override;
  end;

procedure TRWQuery.InternalOpen;
var
  OldSelect: string;
  iSQL,uSQL,dSQL: TStrings;
begin
  if InsertSQL.Text='' then begin
    OldSelect := SQL.Text;
    try
      iSQL := TStringList.Create;
      uSQL := TStringList.Create;
      dSQL := TStringList.Create;
      try
         MakeSQL(TableName, IndexFieldNames, MasterFieldNames,iSQL,uSQL,dSQL);
         InsertSQL.assign( iSQL);
         ModifySQL.assign( uSQL);
         DeleteSQL.assign( dSQL);
      finally
         iSQL.free;
         uSQL.free;
         dSQL.free;
      end;
    finally
      if OldSelect <> '' then
        SQL.Text := OldSelect
    end
  end;
  inherited InternalOpen;
end;


function TODBCStore.DoExec(const Query: string; Options: TSelectOptions): TDataSet;
var
  Exec: TmQuery;
  ROQuery: TmQuery;
  RWQuery: TRWQuery;
begin
  Result := nil;
  if so_Command in Options then begin
    Exec := TmQuery.Create(nil);
    try
      Exec.DataBase := FDBase;
      Exec.ParamCheck := False;
      Exec.SQL.Text := Query;
      if AnsiStartsText('CREATE ', Query) or
         AnsiStartsText('DROP ', Query) then
        ClearMetadataCache;
      Exec.ExecSQL
    finally
      Exec.Free
    end;
  end else begin
    if [so_ReadOnly, so_ReadOnce] * Options = [so_ReadOnly, so_ReadOnce] then begin
      ROQuery := TmQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.DataBase := FDBase;
      ROQuery.ParamCheck := False;
      ROQuery.RequestLive := False;
      ROQuery.UniDirectional := True;
      ROQuery.SQL.Text := Query
    end else
    if so_ReadOnly in Options then begin
      ROQuery := TmQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.DataBase := FDBase;
      ROQuery.ParamCheck := False;
      ROQuery.RequestLive := False;
      ROQuery.SQL.Text := Query
    end else begin
      RWQuery := TRWQuery.Create(Self { TODO: });
      Result := RWQuery;
      RWQuery.DataBase := FDBase;
      RWQuery.ParamCheck := False;
      RWQuery.RequestLive := True;
      RWQuery.TableName := GetTableNameFromSQL(Query);
      RWQuery.SQL.Text := Query
    end;
    try
      if not (so_Closed in Options) then
        Result.Active := True
    except
      Result.Free;
      raise
    end
  end
end;

procedure TODBCStore.DoConnect;
var
  Conn: TStrings;
begin
  FDBase := TmDataBase.Create(nil);
  try
    Conn := TStringList.Create;
    try
      Conn.Text := StringReplace(Connection, ';' , #13#10, [rfReplaceAll]);

      if (Conn.Count = 1) and (Pos('=', Conn.Text) = 0) then
        Conn.Text := 'db=' + Conn.Text;

      ConfigureConnection(Conn);

//      FDBase.Open(Conn.Values['user'], Conn.Values['password']);
    finally
      Conn.Free
    end;

//    FDBase.AutoCommit := True;
    FDBase.TransIsolation := TxnReadCommitted;
    FDBase.OdbcCursors := ocUSE_ODBC;
    FDBase.Connect;

    FDBQueries := TPropSet.Create(FDBase);
    ConfigureQueries;
    Params.Add(FDBQueries);
  except
    FreeAndNil(FDBase);
    raise
  end
end;

procedure TODBCStore.ClearMetadataCache;
begin
  FreeAndNil(FTableNames)
end;

procedure TODBCStore.DoDisconnect;
begin
  FreeAndNil(FTableNames);
  FreeAndNil(FDBase)
end;

function TODBCStore.CmdTableExists(const Params: Variant): TDataSet;
var
  I: Integer;
begin
  if not Assigned(FTableNames) then begin
    FTableNames := TStringList.Create;
    FDBase.GetTableNames(FTableNames);
    TStringList(FTableNames).Sorted := True
  end;
  I := FTableNames.IndexOf(Params[0]);
  if I < 0 then
    Result := nil
  else
    Result := Wrap(FTableNames[I])
end;

end.
