{ VERSION: 1.8 }
{ 1.7 / 1.10.2005
  - opraveno: funkce SqlPos() nepeskakovala literly
  1.6 / 25.8.2005
  - implementuje TMyskaStore.OpenCache()
  1.5 / 16.5.2005
  - zveejnna funkce SqlPos() kter dl tot jako Pos() akort e peskakuje
    etzcov literly
  - nov funkce SqlNormalize() vyhod nadbyten mezery a oddlovae dk se zachovnm
    etzcovch literl
  1.4 / 15.5.2005
  - implementuje zkladn verzi stihae vcepkazovch pkaz ( viz. DoParseSQL )
  1.3 / 21.4.2005
  - podporuje TMyskaStore.ResetGenID()
  1.2 / 2.4.2005
  - podporuje ukladani Params[] do databaze
  1.1 / 10.3.2005
  - podporuje TMyskaStore.CreateRegion()
  - nova interni metoda TMyskaStore.CreateTable() pro zalozeni tabulky pokud
    neexistuje
}
unit uMyskaSQL;

interface
uses
  DB, Classes, uMyskaDB;

type
  TEventHandler = procedure (ChangeKnown: Boolean) of object;
  TMyskaSQLStore = class(TMyskaStore)
  protected
    procedure DoParseSQL(const Query: string; Statements: TStrings); override;
    procedure DoPrepareApps; override;
    procedure DoPrepareSessions; override;
    procedure DoOpenSession(const AppID: string; UserID: Cardinal); override;
    procedure DoCloseSession(const AppID: string); override;
    procedure DoAuthenticateSession(UserID: Cardinal); override;
    procedure DoLoadUsers; override;
    procedure DoModifyUser(User: TMyskaUser; IsNew: Boolean); override;
    procedure DoCleanUp; override;
    procedure DoPrepareLocks; override;
    function DoLock(const LockName: string): TObject; override;
    procedure DoAdd(Sender: TMyskaApp); override;
    function DoGenID(const GeneratorName: string): Cardinal; override;
    procedure DoResetGenID(const GeneratorName: string); override;
    procedure ConfigureQueries; override;
    procedure DoPrepareRegions; override;
    function DoPrepareRegion(const RegionName: string; var Hint: Integer): string; override;
    function DoCreateRegion(const RegionName: string; Hint: Integer): TMyskaRegion; override;
    procedure DoPreparePropertyBlobs; override;
    procedure DoLoadPropertyBlob(const ID: string; Stream: TStream); override;
    procedure DoSavePropertyBlob(const ID: string; Stream: TStream); override;
  protected
    procedure CreateTable(const TableName: string; const TableDeclaration: string); overload;
    procedure CreateTable(const TableName: string; const TableDeclaration: string; const Args: array of const); overload;
  protected
    function Wrap(const FieldValue: Integer): TDataSet; overload;
    function Wrap(const FieldValue: string): TDataSet; overload;
  protected
    function ConnectRegionEvent(ID: Integer; Handler: TEventHandler): TObject; virtual;
    procedure SignalRegionEvent(ID: Integer); virtual;
  public
    function OpenCache(AnOwner: TComponent; const Fields: array of TFieldDefinition): TDataSet; override;
  end;

function SqlPos(const SubStr: string; const S: string): Integer;
function SqlNormalize(const Text: string): string;

implementation
uses
  DBClient, SysUtils, SimpleTimers, SimpleTimer;

procedure TMyskaSQLStore.CreateTable(const TableName: string; const TableDeclaration: string);
begin
  CreateTable(TableName, TableDeclaration, [])
end;

procedure TMyskaSQLStore.CreateTable(const TableName: string; const TableDeclaration: string; const Args: array of const);
begin
  // TODO: nemelo by se to nejak zamknout ?
  while not Exists('{db.table.exists(' + TableName + ')}') do
    try
      Exec('CREATE TABLE ' + TableName + '(' + TableDeclaration + ')', [so_Command], Args)
    except
    end
end;

procedure TMyskaSQLStore.DoPrepareApps;
begin
  inherited;
  CreateTable(
    '{mys.table.name(APPS)}',
      'APPID {db.type.string(30)} NOT NULL PRIMARY KEY,' +
      'VERSION {db.type.generator}'
  )
end;

procedure TMyskaSQLStore.DoPrepareSessions;
begin
  inherited;
  CreateTable(
    '{mys.table.name(SESSIONS)}',
      'APPID {db.type.string(30)} NOT NULL,' +
      'USERID {db.type.generator} NOT NULL,' +
      'SESSIONID {db.type.generator} NOT NULL, PRIMARY KEY (APPID, SESSIONID)'
  )
end;

procedure TMyskaSQLStore.DoOpenSession(const AppID: string; UserID: Cardinal);
begin
  inherited;
  Exec('INSERT INTO {mys.table.name(SESSIONS)}(APPID, USERID, SESSIONID) VALUES (' +
       '''%s'', %d, %d)', [so_Command], [AppID, UserID, SessionID])
end;

procedure TMyskaSQLStore.DoCloseSession(const AppID: string);
begin
  inherited;
  Exec('DELETE FROM {mys.table.name(SESSIONS)} WHERE APPID=''%s'' AND SESSIONID=%d',
       [so_Command], [AppID, SessionID])
end;

procedure TMyskaSQLStore.DoAuthenticateSession(UserID: Cardinal);
begin
  inherited;
  Exec('UPDATE {mys.table.name(SESSIONS)} SET USERID=%1:d WHERE SESSIONID=%0:d',
       [so_Command],
       [SessionID, UserID])
end;

procedure TMyskaSQLStore.DoPreparePropertyBlobs;
begin
  inherited;
  CreateTable(
    '{mys.table.name(PROPS)}',
      'PROPID {db.type.string(30)} NOT NULL PRIMARY KEY,' +
      'PROPDATA {db.type.blob}'
  )
end;

procedure TMyskaSQLStore.DoLoadPropertyBlob(const ID: string; Stream: TStream);
var
  Lck: TObject;
  D: TDataSet;
  Src: TStream;
begin
  Lck := Lock('mys.props');
  try
    D := Exec('SELECT * FROM {mys.table.name(PROPS)} WHERE PROPID=''%s''', [so_ReadOnly, so_ReadOnce], [ID]);
    try
      Stream.Size := 0;
      if not D.Eof then begin
        Src := D.CreateBlobStream(D.FieldByName('PROPDATA'), bmRead);
        try
          Stream.CopyFrom(Src, 0);
        finally
          Src.Free
        end;
      end
    finally
      D.Free
    end
  finally
    Lck.Free
  end
end;

procedure TMyskaSQLStore.DoSavePropertyBlob(const ID: string; Stream: TStream);
var
  Lck: TObject;
  D: TDataSet;
  Dst: TStream;
begin
  Lck := Lock('mys.props');
  try
    if (not Assigned(Stream)) or (Stream.Size = 0) then
      Exec('DELETE FROM {mys.table.name(PROPS)} WHERE PROPID=''%s''', [so_Command], [ID])
    else begin
      D := Exec('SELECT * FROM {mys.table.name(PROPS)} WHERE PROPID=''%s''', [], [ID]);
      try
        if D.Eof then begin
          D.Append;
          D['PROPID'] := ID
        end else
          D.Edit;
        Dst := D.CreateBlobStream(D.FieldByName('PROPDATA'), bmWrite);
        try
          Dst.CopyFrom(Stream, 0)
        finally
          Dst.Free
        end;
        D.Post
      finally
        D.Free
      end
    end;
  finally
    Lck.Free
  end
end;

procedure TMyskaSQLStore.DoLoadUsers;
var
  D: TDataSet;
begin
  inherited;
  CreateTable(
    '{mys.table.name(USERS)}',
      'USERID {db.type.generator} NOT NULL PRIMARY KEY,' +
      'USERNAME {db.type.string(20)} NOT NULL,' +
      'USERPWD {db.type.string(33)} NOT NULL,' +
      'USERFLAGS INTEGER NOT NULL'
  );

  D := Exec('SELECT * FROM {mys.table.name(USERS)}', [so_ReadOnly, so_ReadOnce]);
  try
    while not D.Eof do begin
      Users.AddRecord(
        D.FieldValues['USERID'],
        D.FieldValues['USERNAME'],
        D.FieldValues['USERPWD'],
        D.FieldValues['USERFLAGS']
      );
      D.Next
    end
  finally
    D.Free
  end
end;

procedure TMyskaSQLStore.DoModifyUser(User: TMyskaUser; IsNew: Boolean);
var
  Pass: Integer;
begin
  for Pass := 1 to 2 do begin
    if IsNew then begin
      try
        Exec('INSERT INTO {mys.table.name(USERS)} (USERID, USERNAME, USERPWD, USERFLAGS) VALUES (' +
             '%d, ''%s'', ''%s'', %d)', [so_Command], [User.ID, User.Name, User.PasswordHash, User.Flags]);
        exit
      except
      end
    end else begin
      try
        Exec('UPDATE {mys.table.name(USERS)} SET USERNAME=''%1:s'', USERPWD=''%2:s'', USERFLAGS=%3:d WHERE USERID=%0:d',
             [so_Command], [User.ID, User.Name, User.PasswordHash, User.Flags]);
        exit
      except
      end
    end;
    IsNew := not IsNew
  end
end;

procedure TMyskaSQLStore.DoCleanUp;
begin
  inherited;
  if Exists('{db.table.exists({mys.table.name(LOCKS)})}') then
    Exec('DELETE FROM {mys.table.name(LOCKS)} WHERE SESSIONID <> %d', [so_Command], [SessionID]);
  if Exists('{db.table.exists({mys.table.name(SESSIONS)})}') then
    Exec('DELETE FROM {mys.table.name(SESSIONS)} WHERE SESSIONID <> %d', [so_Command], [SessionID]);
end;

const
  MaxLockName = 50;           // TUNE:
  MaxRegionName = 50;         // TUNE:

procedure TMyskaSQLStore.DoPrepareLocks;
begin
  inherited;
  CreateTable(
    '{mys.table.name(LOCKS)}',
      'LOCKNAME {db.type.string(%d)} NOT NULL PRIMARY KEY,' +
      'SESSIONID {db.type.generator}',
      [MaxLockName]
  )
end;

procedure TMyskaSQLStore.DoPrepareRegions;
begin
  inherited;
  CreateTable(
    '{mys.table.name(REGIONS)}',
      'REGIONNAME {db.type.string(%d)} NOT NULL PRIMARY KEY,' +
      'REGIONID {db.type.generator},' +
      'REGIONAGE {db.type.generator}',
      [MaxRegionName]
  )
end;

function TMyskaSQLStore.DoPrepareRegion(const RegionName: string; var Hint: Integer): string;
var
  Lck: TObject;
begin
  Result := UpperCase(RegionName);
  if Length(Result) > MaxRegionName then
    SetLength(Result, MaxRegionName);

  Lck := Lock('mys.regions');
  try
    Hint := Select('SELECT REGIONID FROM {mys.table.name(REGIONS)} WHERE REGIONNAME = ''%s''', [Result], -1);
    if Hint < 0 then begin
      Hint := GenID('MYS.REGION');
      Exec('INSERT INTO {mys.table.name(REGIONS)}(REGIONNAME,REGIONID,REGIONAGE) VALUES (''%s'', %d, 0)',
           [so_Command], [Result, Hint])
    end;
  finally
    Lck.Free
  end;
end;

type
  TRegion = class(TMyskaRegion)
  private
    FEvent: TObject;
    FOpened: Boolean;
    FChanged: Boolean;
    procedure Close(Lck: TObject; DoWrite: Boolean);
    procedure ConnectEvent;
    procedure DisconnectEvent;
    procedure Check(ChangeKnown: Boolean);
  private
    FStore: TMyskaSQLStore;
    FRegionAge: Integer;
    FRegionID: Integer;
    FLockName: string;
  protected
    procedure SetOnChange(Handler: TNotifyEvent); override;
  public
    destructor Destroy; override;
    function Open(Mode: TOpenMode): TObject; override;
    function Changed: Boolean; override;
    property OnChange: TNotifyEvent read FOnChange write SetOnChange;
  end;

destructor TRegion.Destroy;
begin
  OnChange := nil;
  inherited
end;

procedure TRegion.ConnectEvent;
begin
  FEvent := FStore.ConnectRegionEvent(FRegionID, Check)
end;

procedure TRegion.DisconnectEvent;
begin
  FreeAndNil(FEvent)
end;

procedure TRegion.Check(ChangeKnown: Boolean);
begin
  if not FChanged then begin
    if ChangeKnown then
      FChanged := True
    else
      // check
      Changed;
    if FChanged then
      if Assigned(FOnChange) then
        FOnChange(Self)
  end
end;

procedure TRegion.SetOnChange(Handler: TNotifyEvent);
begin
  if Assigned(FOnChange) then begin
    FOnChange := Handler;
    if not Assigned(Handler) then
      DisconnectEvent
  end else begin
    FOnChange := Handler;
    if Assigned(Handler) then begin
      ConnectEvent;
      // TODO: detect change now
    end
  end
end;

function TRegion.Changed: Boolean;
var
  Lck: TObject;
begin
  Result := FChanged;
  if FOpened or Result then exit;

  Lck := Open(om_Read);
  try
    Result := FChanged
  finally
    Lck.Free
  end;
  if Result then
    // zachovame priznak ( pri zavreni se vymazal )
    FChanged := True
end;

type
  TRegionCloser = class
  public
    FRegion: TRegion;
    FLock: TObject;
    FWriteMode: Boolean;
    destructor Destroy; override;
  end;

destructor TRegionCloser.Destroy;
begin
  inherited;
  FRegion.Close(FLock, FWriteMode)
end;

function TRegion.Open(Mode: TOpenMode): TObject;
var
  Lck: TObject;
  Age: Integer;
  Cls: TRegionCloser;
begin
  if FLockName = '' then
    FLockName := Format('MYS.REGION%d', [FRegionID]);

  Lck := FStore.Lock(FLockName);
  try
    Age := FStore.Select('SELECT REGIONAGE FROM {mys.table.name(REGIONS)} WHERE REGIONID=%d', [FRegionID]);
    FChanged := Age <> FRegionAge;
    FRegionAge := Age;

    Cls := TRegionCloser.Create;
    Cls.FRegion := Self;
    Cls.FLock := Lck;
    Cls.FWriteMode := Mode in [om_ReadWrite, om_Write];
    Result := Cls
  except
    Lck.Free;
    raise
  end
end;

procedure TRegion.Close(Lck: TObject; DoWrite: Boolean);
begin
  try
   if DoWrite then begin
      Inc(FRegionAge);
      FStore.Exec('UPDATE {mys.table.name(REGIONS)} SET REGIONAGE=%1:d WHERE REGIONID=%0:d', [so_Command], [FRegionID, FRegionAge])
    end
  finally
    Lck.Free;
    FChanged := False
  end;
  if DoWrite then
    FStore.SignalRegionEvent(FRegionID)
end;

function TMyskaSQLStore.DoCreateRegion(const RegionName: string; Hint: Integer): TMyskaRegion;
var
  Reg: TRegion;
begin
  Reg := TRegion.Create(Self { TODO: });
  Result := Reg;
  Reg.FStore := Self;
  Reg.FRegionID := Hint;
  Reg.FRegionAge := -1
end;

type
  TRegionEvent = class
  private
    procedure OnTimer(Sender: TObject);
  public
    Handler: TEventHandler;
    Poller: TSimpleTimer;
    constructor Create;
    destructor Destroy; override;
  end;

procedure TRegionEvent.OnTimer(Sender: TObject);
begin
  try
    if Assigned(Handler) then
      Handler(False)
  finally
    Poller.Enabled := True
  end
end;

constructor TRegionEvent.Create;
begin
  inherited Create;
  Poller := TSimpleTimer.Create;
  Poller.Interval := 2*1000 { TUNE: };
  Poller.OnTimer := OnTimer;
  Poller.AutoDisable := True;
  Poller.Enabled := True
end;

destructor TRegionEvent.Destroy;
begin
  Poller.Free;
  inherited
end;

function TMyskaSQLStore.ConnectRegionEvent(ID: Integer; Handler: TEventHandler): TObject;
var
  Ev: TRegionEvent;
begin
  Ev := TRegionEvent.Create;
  Ev.Handler := Handler;
  Result := Ev
end;

procedure TMyskaSQLStore.SignalRegionEvent(ID: Integer);
begin
end;

type
  TLock = class
  public
    Store: TMyskaStore;
    Command: string;
    destructor Destroy; override;
  end;

destructor TLock.Destroy;
begin
  if Assigned(Store) then
    Store.Exec(Command, [so_Command]);
  inherited
end;

function TMyskaSQLStore.DoLock(const LockName: string): TObject;
var
  S: string;
  Lck: TLock;
  Ok: Boolean;
  Timer: TTimer;
begin
  // TODO: timeout
  Result := nil;

  S := UpperCase(LockName);
  if Length(S) > MaxLockName then
    SetLength(S, MaxLockName);

  TimerStart(Timer);
  Ok := False;
  while not Ok do
  try
    if TimerValue(Timer) > 2*1000 { TUNE: } then begin
      try
        // TODO: kdyz ceka na zamek vic aplikaci a umrel jenom ten kdo to puvodne zamknul,
        // tak to nezabere
        if ExclusiveAccess then
          DoCleanUp;
      except
      end;
      TimerReStart(Timer)
    end;
    Exec('INSERT INTO {mys.table.name(LOCKS)}(LOCKNAME, SESSIONID) VALUES (''%s'', %d)', [so_Command], [S, SessionID]);
    Lck := TLock.Create;
    Lck.Store := Self;
    Lck.Command := 'DELETE FROM {mys.table.name(LOCKS)} WHERE LOCKNAME=''' + S + '''';
    Result := Lck;
    Ok := True
  except
    // TODO: wait a little
    Sleep(100)
  end
end;

procedure TMyskaSQLStore.DoAdd(Sender: TMyskaApp);
begin
  inherited;
  with Sender.DBQueries do begin
    Values['app.sessions'] := '{mys.app.sessions({app.id})}'
  end
end;

function TMyskaSQLStore.DoGenID(const GeneratorName: string): Cardinal;
begin
  Result := Select('{db.generator.increment}', [GeneratorName])
end;

procedure TMyskaSQLStore.DoResetGenID(const GeneratorName: string);
begin
  Exec('{db.generator.setValue}', [so_Command], [GeneratorName, 0])
end;

function TMyskaSQLStore.OpenCache(AnOwner: TComponent; const Fields: array of TFieldDefinition): TDataSet;
var
  Data: TClientDataSet;
  I: Integer;
begin
  Data := TClientDataSet.Create(nil);
  try
    Result := Data;
    for I := Low(Fields) to High(Fields) do
      Data.FieldDefs.Add(Fields[I].Name, Fields[I].DataType, Fields[I].Size);
    Data.CreateDataSet;
    Data.Open
  except
    Data.Free;
    raise
  end
end;

const
  WrapFieldName = 'HODNOTA';     // TUNE:

function TMyskaSQLStore.Wrap(const FieldValue: Integer): TDataSet;
begin
  Result := OpenCache([FieldDefinition(WrapFieldName, ftInteger)]);
  Result.InsertRecord([FieldValue]);
  Result.First
end;

function TMyskaSQLStore.Wrap(const FieldValue: string): TDataSet;
begin
  Result := OpenCache([FieldDefinition(WrapFieldName, ftString, Length(FieldValue) + 1)]);
  Result.InsertRecord([FieldValue]);
  Result.First
end;

procedure TMyskaSQLStore.ConfigureQueries;
begin
  inherited;
  with FDBQueries do begin
    ArrayValues['mys.app.getversion', 1] := 'SELECT VERSION FROM {mys.table.name(APPS)} WHERE {db.operator.stricomp(APPID,''%s'')}';
    ArrayValues['mys.app.setversion', 2] := 'UPDATE {mys.table.name(APPS)} SET VERSION=%1:d WHERE {db.operator.stricomp(APPID,''%0:s'')}';
    ArrayValues['mys.app.insert', 2] := 'INSERT INTO {mys.table.name(APPS)}(APPID, VERSION) VALUES ({db.operator.upper}(''%s''), %d)';

    ArrayValues['mys.app.sessions', 1] := 'SELECT * FROM {mys.table.name(SESSIONS)} WHERE {db.operator.stricomp(APPID,''%s'')}';

    ArrayValues['app.table.name', 1] := '%s';
    ArrayValues['app.table.name', 2] := '%1:s';
    ArrayValues['app.generator.name', 1] := '%s'
  end
end;

function SqlPos(const SubStr: string; const S: string): Integer;
var
  InQuotes: Boolean;
  Quote: Char;
  I: Integer;
begin
  InQuotes := False;
  for I := 1 to Length(S) - Length(SubStr) + 1 do
    case S[I] of
    '''', '"':
      begin
        if not InQuotes then begin
          InQuotes := True;
          Quote := S[I]
        end else
        if S[I] = Quote then
          InQuotes := False;
      end;
    else
      if (not InQuotes) and (StrLComp(@S[I], @SubStr[1], Length(SubStr)) = 0) then begin
        Result := I;
        exit
      end
    end;

  Result := 0
end;

function SqlNormalize(const Text: string): string;
var
  I: Integer;
begin
  Result := Text;
  while True do begin
    I := SqlPos(#13, Result);
    if I<>0 then begin
      Result[I] := ' ';
      continue
    end;
    I := SqlPos(#10, Result);
    if I<>0 then begin
      Delete(Result, I, 1);
      continue
    end;
    I := SqlPos(#9, Result);
    if I<>0 then begin
      Result[I] := ' ';
      continue
    end;
    I := SqlPos('  ', Result);
    if I<>0 then begin
      Delete(Result, I, 1);
      continue
    end;
    break
  end;
  Result := Trim(Result);
end;

procedure TMyskaSQLStore.DoParseSQL(const Query: string; Statements: TStrings);

  function CommentLess(const command:string):string;
  var
    i,j:integer;
    list:TStrings;
    line:string;
  begin
    list:=TStringList.Create;
    list.Text:=command;

    i:=0;
    while i<list.Count do begin
      line:=list[i];
      j:=SqlPos('//',line);
      if j<>0 then list[i]:=Copy(line,1,j-1);
      inc(i)
    end;
    Result:=list.Text;
    list.Free;

    while true do begin
      i:=SqlPos('/*',Result);
      if i=0 then break;
      j:=Pos('*/',Copy(Result,i+2,maxint));
      if j=0 then break; // TODO: syntax error

      Delete(Result,i,2+2+j-1)
    end;
  end;

const
  setTerm='SET TERM ';
var
  I: Integer;
  Token: string;
  Command: string;
  Separator: string;
begin
  if (Pos(';', Query) = 0) { urcite tam neni vic prikazu } and
     (Pos('/', Query) = 0) { urcite tam neni komentar } then begin
    Statements.Clear;
    Statements.Add(Query)
  end else begin
    Statements.Clear;
    Separator := ';';
    command:=CommentLess(Query);
    while command<>'' do begin
      i:=SqlPos(separator,command);
      if i=0 then begin
        token:=command;
        command:=''
      end else begin
        token:=Copy(command,1,i-1);
        Delete(command,1,i+length(separator)-1)
      end;
      token:=Trim(token);
      if token='' then continue;

      if UpperCase(Copy(token,1,length(setTerm)))=setTerm then begin
        separator:=Trim(Copy(token,length(setTerm)+1,maxint));
        continue
      end;

      Statements.Add(Token)
    end
  end
end;

end.
