{ VERSION: 1.2 }
{ 1.2 / 2.10.2005
  - implementuje funkce GenNewID, DoInitRecord na zklad meta-informac
  1.1 / 18.7.2005
}
unit uMyskaTables;

interface
uses
  Classes, DB, uMyskaDB, uPropSet, uMyskaMetaTables;

type
  TDataSetFields = class(TComponent)
  private
    FItems: TList;
  protected
    procedure AssignTo(Dest: TPersistent); override;
  public
    procedure Assign(Source: TPersistent); override;
    destructor Destroy; override;
  end;

  TRecordBuffer = class(TComponent)
  private
    FItems: TPropSet;      // skutecne hodnoty
    FNulls: TStringList;
    FRemoved: TStringList;
    procedure CreateBuffers;
    procedure FreeBuffers;
    function GetValue(const Key: string): Variant;
    procedure SetValue(const Key: string; const Value: Variant);
    function GetName(Index: Integer): string;
    function BuildNames: TStrings;
  protected
    procedure AssignTo(Dest: TPersistent); override;
    function Code: string;
  public
    procedure SaveToStream(Stream: TStream);
    procedure LoadFromStream(Stream: TStream);
    procedure Remove(const Key: string);
    function Equals(Source: TRecordBuffer): Boolean;
    procedure Assign(Source: TPersistent); override;
    destructor Destroy; override;
    function Empty: Boolean;
    function Count: Integer;
    property Names[Index: Integer]: string read GetName;
    property Values[const Key: string]: Variant read GetValue write SetValue; default;
  end;

  TQueryIndex = (
    qi_Open
  );
  TTableProvider = class(TComponent)
  private
    FDescriptor: TTableDescriptor;
    FLastKey: Variant;
    FQuery: array [TQueryIndex] of string;
    function GetActive: Boolean;
    procedure CatchOnNewRecord(Sender: TDataSet);
    procedure CatchBeforePost(Sender: TDataSet);
    function GetDescriptor: TTableDescriptor;
    function GetQuery(Index: TQueryIndex): string;
    procedure SetQuery(Index: TQueryIndex; const Value: string);
  protected
    FCurrentFilter: Variant;
    FReadOnly: Boolean;
    FKeyField: string;
    FMasterFields: string;
    FTableName: string;
    FDataSet: TDataSet;
    FApplication: TMyskaApp;
    function ReadOnlyOptions: TSelectOptions;
    function DoOpen(const Filter: Variant): TDataSet; virtual;
    procedure DoInitRecord(DataSet: TDataSet); virtual;
    procedure DoDelete; virtual;
    function GenNewID: Variant; virtual;
    function GetName: string; virtual;
    function GetTableName: string; virtual;
    property Query[Index: TQueryIndex]: string read GetQuery write SetQuery;
  public
    procedure SetFactory(AnApp: TMyskaApp; const ATableName: string = ''; const AMasterFields: string = '');
  public
    destructor Destroy; override;
    procedure Open; overload;
    procedure Open(const Filter: Variant); overload;
    procedure Close;
    procedure Refresh;
    function Locate(const Key: Variant): Boolean;
    procedure Delete;
    property LastKeyValue: Variant read FLastKey;
  public
    property Name: string read GetName;
    property DataSet: TDataSet read FDataSet;
    property KeyField: string read FKeyField;
    property ReadOnly: Boolean read FReadOnly write FReadOnly;
    property Active: Boolean read GetActive;
  public
    property Descriptor: TTableDescriptor read GetDescriptor;
  end;

  TAppWriter = class
  private
    FApp: TMyskaApp;
    FStream: TStream;
  protected
    procedure Write(const Value: string); overload;
    procedure Write(const Value: Integer); overload;
  public
    constructor Create(AStream: TStream; AnApp: TMyskaApp);
    procedure Write(const ID: string; Buffer: TRecordBuffer); overload;
  end;

  TAppReader = class
  private
    FApp: TMyskaApp;
    FStream: TStream;
  protected
    function ReadInteger: Integer;
    function ReadString: string;
  public
    constructor Create(AStream: TStream; AnApp: TMyskaApp);
    function Read(var ID: string; Buffer: TRecordBuffer): Boolean; overload;
  end;

function SafeFileName(const FileName: string): string;

procedure SetTitle(Data: TDataSet; const Titles: string);
procedure SetReadOnly(Data: TDataSet; const Fields: string);
procedure SetHidden(Data: TDataSet; const Fields: string);

function ReadOnlyOptions(ReadOnly: Boolean): TSelectOptions;

function DataSetFields(ADescriptor: TTableDescriptor; CanCreate: Boolean = False): TDataSetFields;

function Contains(const FieldName: string; const FieldNames: string): Boolean;

implementation
uses
  SysUtils, Variants, DBConsts;

function SafeFileName(const FileName: string): string;

  procedure FuckOff(const Chars: string); overload;
  var
    I: Integer;
  begin
    for I := 1 to Length(Chars) do
      Result := StringReplace(Result, Chars[I], '', [rfReplaceAll])
  end;

var
  F: string;
begin
  Result := FileName;
  FuckOff('&');

  while (Result <> '') and (Result[Length(Result)] = '.') do
    SetLength(Result, Length(Result) - 1);

  while Pos('  ', Result) <> 0 do
    Result := StringReplace(Result, '  ', ' ', [rfReplaceAll]);

  Result := Trim(Result);
  FuckOff(':\/?*' { TUNE: })
end;

// --------------------------------------------------------------------------

procedure SetTitle(Data: TDataSet; const Titles: string);
var
  L: TStrings;
  I: Integer;
  F: TField;
  FieldName: string;
  FieldCaption: string;
begin
  L := TStringList.Create;
  try
    L.Text := StringReplace(Titles, ';', #13#10, [rfReplaceAll]);
    for I := 0 to L.Count - 1 do begin
      FieldName := L.Names[I];
      if FieldName = '' then begin
        FieldName := L.Strings[I];
        FieldCaption := FieldName
      end else
        FieldCaption := L.ValueFromIndex[I];
      F := Data.FindField(FieldName);
      if Assigned(F) then
        F.DisplayLabel := FieldCaption
    end
  finally
    L.Free
  end
end;

procedure SetReadOnly(Data: TDataSet; const Fields: string);
var
  L: TStrings;
  I: Integer;
  F: TField;
begin
  L := TStringList.Create;
  try
    L.Text := StringReplace(Fields, ';', #13#10, [rfReplaceAll]);
    for I := 0 to L.Count - 1 do begin
      F := Data.FindField(L[I]);
      if Assigned(F) then
        F.ReadOnly := True
    end
  finally
    L.Free
  end
end;

procedure SetHidden(Data: TDataSet; const Fields: string);
var
  L: TStrings;
  I: Integer;
  F: TField;
begin
  L := TStringList.Create;
  try
    L.Text := StringReplace(Fields, ';', #13#10, [rfReplaceAll]);
    for I := 0 to L.Count - 1 do begin
      F := Data.FindField(L[I]);
      if Assigned(F) then
        F.Visible := False;
    end
  finally
    L.Free
  end
end;

// --------------------------------------------------------------------------

function DataSetFields(ADescriptor: TTableDescriptor; CanCreate: Boolean): TDataSetFields;
var
  I: Integer;
begin
  Result := nil;
  if Assigned(ADescriptor) then begin
    for I := 0 to ADescriptor.ComponentCount - 1 do
      if ADescriptor.Components[I] is TDataSetFields then begin
        Result := TDataSetFields(ADescriptor.Components[I]);
        break
      end;

    if (not Assigned(Result)) and CanCreate then
      Result := TDataSetFields.Create(ADescriptor);
  end
end;

destructor TDataSetFields.Destroy;
begin
  Assign(nil);
  inherited
end;

procedure TDataSetFields.Assign(Source: TPersistent);

  function Clone(Source: TField): TField;
  begin
    Result := TFieldClass(Source.ClassType).Create(nil);
    Result.FieldName := Source.FieldName;
    Result.Size := Source.Size;
  end;

  procedure CreateFrom(Source: TDataSet); overload;
  var
    I: Integer;
  begin
    FItems := TList.Create;
    for I := 0 to Source.FieldCount - 1 do
      FItems.Add(Clone(Source.Fields[I]))
  end;

var
  I: Integer;
begin
  if not Assigned(Source) then begin
    if Assigned(FItems) then
      for I := 0 to FItems.Count - 1 do
        TObject(FItems[I]).Free;
      FreeAndNil(FItems)
  end else
  if Source is TDataSet then begin
    Assign(nil);
    CreateFrom(TDataSet(Source))
  end else
    inherited
end;

procedure TDataSetFields.AssignTo(Dest: TPersistent);

  procedure SaveTo(Dest: TDataSet); overload;

    procedure Clone(Source: TField);
    var
      F: TField;
    begin
      F := TFieldClass(Source.ClassType).Create(Dest);
      F.FieldName := Source.FieldName;
      F.Size := Source.Size;
      F.DataSet := Dest
    end;

  var
    I: Integer;
  begin
    if not Assigned(FItems) then exit;

    for I := 0 to FItems.Count - 1 do
      Clone(TField(FItems[I]))
  end;

begin
  if Dest is TDataSet then
    SaveTo(TDataSet(Dest))
  else
    inherited
end;

// ------------------------------------------------------------------------

destructor TRecordBuffer.Destroy;
begin
  Assign(nil);
  inherited
end;

procedure TRecordBuffer.SaveToStream(Stream: TStream);

  procedure Write(Value: Integer); overload;
  begin
    Stream.WriteBuffer(Value, SizeOf(Value))
  end;

  procedure Write(const Value: string); overload;
  begin
    Write(Length(Value));
    if Value <> '' then
      Stream.WriteBuffer(Value[1], Length(Value))
  end;

begin
  if not Assigned(FItems) then
    CreateBuffers;
  FItems.SaveToStream(Stream);
  Write(Trim(FNulls.Text));
  Write(Trim(FRemoved.Text));
end;

procedure TRecordBuffer.LoadFromStream(Stream: TStream);

  function ReadInteger: Integer;
  begin
    Stream.ReadBuffer(Result, SizeOf(Result))
  end;

  function ReadString: string;
  begin
    SetLength(Result, ReadInteger);
    if Result <> '' then
      Stream.ReadBuffer(Result[1], Length(Result))
  end;

begin
  FreeBuffers;
  CreateBuffers;
  FItems.LoadFromStream(Stream);
  FNulls.Text := ReadString;
  FNulls.Sorted := True;
  FRemoved.Sorted := False;
  FRemoved.Text := ReadString;
  FRemoved.Sorted := True
end;

procedure TRecordBuffer.CreateBuffers;
begin
  FItems := TPropSet.Create(nil);
  FNulls := TStringList.Create;
  FRemoved := TStringList.Create;
  FRemoved.Sorted := True
end;

procedure TRecordBuffer.FreeBuffers;
begin
  FreeAndNil(FItems);
  FreeAndNil(FNulls);
  FreeAndNil(FRemoved)
end;

procedure TRecordBuffer.Assign(Source: TPersistent);

  procedure CreateFrom(Source: TDataSet); overload;
  var
    I: Integer;
    F: TField;
  begin
    CreateBuffers;
    for I := 0 to Source.FieldCount - 1 do begin
      F := Source.Fields[I];
      if F.ReadOnly then continue;
      if F.IsNull then
        FNulls.Add(UpperCase(F.FieldName))
      else begin
        FItems[UpperCase(F.FieldName)] := F.Value;
        if FItems[F.FieldName] <> F.Value then
          Assert(False)
      end
    end;
    FNulls.Sorted := True
  end;

  procedure CreateFrom(Source: TRecordBuffer); overload;
  var
    Buf: TStream;
  begin
    if Source.Empty then exit;
    CreateBuffers;
    FNulls.Text := Source.FNulls.Text;
    FNulls.Sorted := True;
    Buf := TMemoryStream.Create;
    try
      Source.FItems.SaveToStream(Buf);
      Buf.Position := 0;
      FItems.LoadFromStream(Buf);
    finally
      Buf.Free
    end;
    FRemoved.Sorted := False;
    FRemoved.Text := Source.FRemoved.Text;
    FRemoved.Sorted := True
  end;

begin
  if not Assigned(Source) then begin
    FreeBuffers;
  end else
  if Source is TDataSet then begin
    Assign(nil);
    CreateFrom(TDataSet(Source))
  end else
  if Source is TRecordBuffer then begin
    Assign(nil);
    CreateFrom(TRecordBuffer(Source))
  end else
    inherited
end;

procedure TRecordBuffer.AssignTo(Dest: TPersistent);

  procedure SaveTo(Dest: TDataSet); overload;
  var
    I: Integer;
    V: Variant;
    VT: TVarType;
    F: TField;

    function GetField(const FieldName: string): TField;
    begin
      Result := nil;
      if FRemoved.IndexOf(FieldName) >= 0 then exit;
      Result := Dest.FindField(FieldName);
      if Assigned(Result) and Result.ReadOnly then
        Result := nil
    end;

  begin
    if Empty then exit;

    for I := 0 to FNulls.Count - 1 do begin
      F := GetField(FNulls[I]);
      if not Assigned(F) then continue;
      F.Clear
    end;

    for I := 0 to FItems.Count - 1 do begin
      F := GetField(FItems.Names[I]);
      if not Assigned(F) then continue;

      if F.FieldKind = fkLookup then begin
        if FItems.ValueFromIndex[I] = '' then
          F.DataSet.FieldByName(F.KeyFields).Clear
        else
        if F.LookupDataSet.Locate(F.LookupResultField, FItems.ValueFromIndex[I], []) then
          F.DataSet[F.KeyFields] := F.LookupDataSet[F.LookupKeyFields]
        else
          DatabaseErrorFmt(SFieldValueError, [F.DisplayName]);
      end else begin
        V := FItems.ValueFromIndex[I];
        VT := VarType(V);
        if (VT = varNull) or ((not (F.DataType in [ftString, ftMemo, ftFmtMemo])) and (((VT = varString) or (VT = varOleStr)) and (Trim(string(V)) = ''))) then
          F.Clear
        else
          F.Value := V
      end
    end;
  end;

begin
  if Dest is TDataSet then
    SaveTo(TDataSet(Dest))
  else
    inherited
end;

function TRecordBuffer.Empty: Boolean;
begin
  Result := (not Assigned(FItems)) or ((FItems.Count = 0) and (FNulls.Count = 0))
end;

function TRecordBuffer.Equals(Source: TRecordBuffer): Boolean;
begin
  if Assigned(Source) and (not Source.Empty) then begin
    Result := not Empty;
    if Result then
      Result := Code = Source.Code
  end else
    Result := Empty
end;

function TRecordBuffer.Code: string;
const
  Separator = #1;
begin
  if Assigned(FItems) then
    // TODO: Code je presne priblizne kvuli blokacim FRemoved ktere muzou byt
    // ruzne
    Result := FItems.AsText + Separator + FNulls.Text + Separator + FRemoved.Text
  else
    Result := ''
end;

procedure TRecordBuffer.Remove(const Key: string);
var
  I: Integer;
begin
  if Empty then exit;

  I := FNulls.IndexOf(Key);
  if I >= 0 then begin
    FNulls.Delete(I);
    exit
  end;

  I := FItems.IndexOf(Key);
  if I >= 0 then begin
    FItems.ValueFromIndex[I] := Unassigned;
    I := FRemoved.IndexOf(Key);
    if I < 0 then
      FRemoved.Add(UpperCase(Key));
    exit
  end
end;

function TRecordBuffer.GetValue(const Key: string): Variant;
var
  I: Integer;
begin
  Result := Unassigned;
  if Empty then exit;
  if FRemoved.IndexOf(Key) >= 0 then exit;
  if FNulls.IndexOf(Key) >= 0 then begin
    Result := Null;
    exit
  end;
  Result := FItems[Key]
end;

procedure TRecordBuffer.SetValue(const Key: string; const Value: Variant);
var
  I: Integer;
begin
  if not Assigned(FItems) then
    CreateBuffers;

  I := FRemoved.IndexOf(Key);
  if I >= 0 then
    FRemoved.Delete(I);

  if VarIsNull(Value) then begin
    I := FNulls.IndexOf(Key);
    if I >= 0 then exit;
    FItems[Key] := Unassigned;
    FNulls.Add(Key);
    exit
  end;

  FItems[Key] := Value
end;

function TRecordBuffer.GetName(Index: Integer): string;
var
  N: TStrings;
begin
  N := BuildNames;
  try
    Result := N[Index]
  finally
    N.Free
  end
end;

function TRecordBuffer.Count: Integer;
var
  N: TStrings;
begin
  N := BuildNames;
  try
    Result := N.Count
  finally
    N.Free
  end
end;

function TRecordBuffer.BuildNames: TStrings;
var
  N: TStringList;
  I, J: Integer;
begin
  N := TStringList.Create;
  if Empty then exit;
  N.Duplicates := dupIgnore;
  N.AddStrings(FNulls);
  N.Sorted := True;
  for I := 0 to FItems.Count - 1 do
    N.Add(FItems.Names[I]);
  for I := 0 to FRemoved.Count - 1 do begin
    J := N.IndexOf(FRemoved[I]);
    if J >= 0 then
      N.Delete(J);
  end;
  Result := N
end;

// ------------------------------------------------------------------------

function TTableProvider.Locate(const Key: Variant): Boolean;
begin
  Result := False;
  if not Active then exit;
  if VarIsEmpty(Key) or VarIsNull(Key) then begin
    Result := True;
    exit
  end;
  if KeyField = '' then exit;
  Result := FDataSet.Locate(KeyField, Key, [])
end;

destructor TTableProvider.Destroy;
begin
  Close;
  inherited
end;

procedure TTableProvider.Open;
begin
  Open(Unassigned)
end;

procedure TTableProvider.CatchOnNewRecord(Sender: TDataSet);
begin
  DoInitRecord(Sender)
end;

procedure TTableProvider.CatchBeforePost(Sender: TDataSet);
var
  F: TField;
  OldReadOnly: Boolean;
begin
  F := Sender.FindField(KeyField);
  if Sender.State = dsInsert then begin
    if Assigned(F) and F.IsNull then begin
      OldReadOnly := F.ReadOnly;
      F.ReadOnly := False;
      F.Value := GenNewID;
      F.ReadOnly := OldReadOnly
    end
  end;
  if Assigned(F) then
    FLastKey := F.Value
  else
    FLastKey := Unassigned
end;

procedure TTableProvider.Open(const Filter: Variant);
begin
  Close;
  FDataSet := DoOpen(Filter);
  if Assigned(FDataSet) then begin
    SetReadOnly(FDataSet, KeyField);
    if (not ReadOnly) and (not FDataSet.CanModify) then
      FReadOnly := True;
    FDataSet.OnNewRecord := CatchOnNewRecord;
    FDataSet.BeforePost := CatchBeforePost;
  end;
  FCurrentFilter := Filter
end;

function ReadOnlyOptions(ReadOnly: Boolean): TSelectOptions;
begin
  if ReadOnly then
    Result := [so_ReadOnly]
  else
    Result := []
end;

function TTableProvider.ReadOnlyOptions: TSelectOptions;
begin
  if ReadOnly then
    Result := [so_ReadOnly]
  else
    Result := []
end;

procedure TTableProvider.Refresh;
begin
  Close;
  Open(FCurrentFilter)
end;

procedure TTableProvider.Close;
begin
  FreeAndNil(FDataSet)
end;

function Contains(const FieldName: string; const FieldNames: string): Boolean;
begin
  Result := Pos(UpperCase(';' + FieldName + ';'), UpperCase(';' + FieldNames + ';')) <> 0
end;

function TTableProvider.DoOpen(const Filter: Variant): TDataSet;
var
  I: Integer;
  F: TField;
  FD: TFieldDescriptor;
  LD: TTableDescriptor;
  FieldStructure: TDataSetFields;
begin
  Result := nil;

  if (Query[qi_Open] <> '') and Assigned(Descriptor) then begin
    Result := FApplication.Exec(Query[qi_Open], [so_Closed] + ReadOnlyOptions, [Filter]);

    FieldStructure := DataSetFields(Descriptor, False);
    if not Assigned(FieldStructure) then begin
      try
        Result.Active := True
      except
        Result.Free;
        raise
      end;
      FieldStructure := DataSetFields(Descriptor, True);
      FieldStructure.Assign(Result);
      Result.Active := False;
    end;

    Result.Assign(FieldStructure);

    for I := 0 to Descriptor.Fields.Count - 1 do begin
      FD := Descriptor.Fields[I];
      F := Result.FindField(FD.FieldName);
      if Assigned(F) then begin
        F.DisplayLabel := FD.DisplayLabel;
        if (FD.LookupTable <> '') and not Contains(FD.FieldName, FMasterFields) then
          LD := Descriptor.Collection.Find(FD.LookupTable)
        else
          LD := nil;
        if Assigned(LD) then begin
          F.Visible := False;
          
          F := TStringField.Create(Result);
          F.FieldName := '_' + FD.FieldName; { TUNE: }
          F.DisplayLabel := FD.DisplayLabel;
          F.FieldKind := fkLookup;
          F.KeyFields := FD.FieldName;
          F.LookupDataSet := LD.LookupCache;
          F.LookupKeyFields := LD.KeyField;
          F.LookupResultField := LD.ValueField;
          F.LookupCache := True;
          F.Size := F.LookupDataSet.FieldByName(F.LookupResultField).Size;
          F.DataSet := Result;
        end;
      end;
    end;

    for I := 0 to Result.FieldCount - 1 do begin
      F := Result.Fields[I];
      if F.Lookup then
        FD := Descriptor.Fields.Find(F.KeyFields)
      else
        FD := Descriptor.Fields.Find(F.FieldName);
      if not Assigned(FD) then
        F.Visible := False;
    end;

    try
      Result.Active := True
    except
      Result.Free;
      raise
    end;

    SetReadOnly(Result, KeyField);
    SetHidden(Result, FMasterFields);
    SetReadOnly(Result, FMasterFields);
  end;
end;

function TTableProvider.GetActive: Boolean;
begin
  Result := Assigned(FDataSet) and FDataSet.Active
end;

procedure TTableProvider.Delete;
begin
  if not ReadOnly then
    DoDelete
end;

function TTableProvider.GetName: string;
begin
  Result := ClassName
end;

procedure TTableProvider.DoInitRecord(DataSet: TDataSet);
var
  I: Integer;
begin
  if not Assigned(Descriptor) then exit;
  for I := 0 to Descriptor.Fields.Count - 1 do
    with Descriptor.Fields[I] do
      if not VarIsEmpty(DefaultValue) then
        DataSet[FieldName] := DefaultValue
end;

function TTableProvider.GenNewID: Variant;
var
  FD: TFieldDescriptor;
begin
  if Assigned(Descriptor) then
    FD := Descriptor.Fields.Find(KeyField)
  else
    FD := nil;
  if Assigned(FD) and (FD.Generator <> '') then
    Result := FApplication.GenID(FD.Generator)
  else
    Result := Unassigned
end;

procedure TTableProvider.DoDelete;
begin
end;

function TTableProvider.GetDescriptor: TTableDescriptor;
begin
  Result := FDescriptor;
end;

function TTableProvider.GetQuery(Index: TQueryIndex): string;
begin
  Result := FQuery[Index]
end;

procedure TTableProvider.SetQuery(Index: TQueryIndex; const Value: string);
begin
  FQuery[Index] := Value
end;

procedure TTableProvider.SetFactory(AnApp: TMyskaApp; const ATableName: string; const AMasterFields: string);
begin
  if ATableName <> '' then
    FTableName := ATableName;
  if AMasterFields <> '' then
    FMasterFields := AMasterFields;
  FApplication := AnApp;
  if not Assigned(FDescriptor) then begin
    // TODO: use Application name as prefix
    FDescriptor := DatabaseDescriptor(AnApp).Tables.Find(GetTableName);
    if Assigned(FDescriptor) then
      FKeyField := FDescriptor.KeyField
  end
end;

function TTableProvider.GetTableName: string;
begin
  Result := FTableName
end;

// ---------------------------------------------------------------------------

const
  SignatureFormat='MysApp;%s'#13#10;

constructor TAppWriter.Create(AStream: TStream; AnApp: TMyskaApp);
begin
  inherited Create;
  FStream := AStream;
  FApp := AnApp;
  Write(Format(SignatureFormat, [FApp.AppID]));
  Write(FApp.AppVersion);
end;

procedure TAppWriter.Write(const Value: string);
var
  Len: Byte;
begin
  Len := Length(Value);
  FStream.WriteBuffer(Len, SizeOf(Len));
  if Len <> 0 then
    FStream.WriteBuffer(Value[1], Len)
end;

procedure TAppWriter.Write(const Value: Integer);
begin
  FStream.WriteBuffer(Value, SizeOf(Value));
end;

procedure TAppWriter.Write(const ID: string; Buffer: TRecordBuffer);
begin
  Write(ID);
  Buffer.SaveToStream(FStream)
end;

// ---------------------------------------------------------------------------

procedure Check(Condition: Boolean; const Msg: string = '');
begin
  if not Condition then
    raise EReadError.Create(Msg);
end;

constructor TAppReader.Create(AStream: TStream; AnApp: TMyskaApp);
begin
  inherited Create;
  FStream := AStream;
  FApp := AnApp;
  Check(ReadString = Format(SignatureFormat, [FApp.AppID]));
  Check(ReadInteger = FApp.AppVersion);
end;

function TAppReader.ReadInteger: Integer;
begin
  Check(FStream.Read(Result, SizeOf(Result)) = SizeOf(Result))
end;

function TAppReader.ReadString: string;
var
  Len: Byte;
begin
  Check(FStream.Read(Len, SizeOf(Len)) = SizeOf(Len));
  SetLength(Result, Len);
  if Len <> 0 then
    Check(FStream.Read(Result[1], Len) = Len)
end;

function TAppReader.Read(var ID: string; Buffer: TRecordBuffer): Boolean;
begin
  Result := FStream.Position < FStream.Size;
  if not Result then exit;
  ID := ReadString;
  Buffer.LoadFromStream(FStream)
end;

end.
