{ VERSION: 1.6 }
(*
  1.6 / 12.7.2005
  - podporuje so_Closed
  1.5 / 14.5.2005
  - opraveno: ve verzi zeos-u z 12.5.2005 funkce CmdTableExists() nereagovala
    na zmny v databzi ( viz. GetMetadata.ClearCache )
  1.4 / 9.5.2005
  - implementuje TMyskaStore.BeginUpdate jako vypnut AutoCommit
  1.3 / 3.4.2005
  - podporuje viceprikazove dotazy ( viz DoParseSQL )
  1.2 / 11.3.2005
  - opraveno: nefungovalo Exec('CREATE PROCEDURE..') protoze nebylo mozne pouzit 'SET TERM'
  1.1 / 8.3.2005
  - novy dotaz TMyskaApp.Exec('{app.sessions}') vrati seznam aktualne prihlasenych
    uzivatelu
*)
unit uMyskaZeosDB;

interface
uses
  Classes, SysUtils, uMyskaDB, uMyskaSQL, DB, ZConnection, ZDbcIntfs, ZDataSet;

type
  TZeosStore = class(TMyskaSQLStore)
  private
    FOldAutoCommit: Boolean;
  protected
    FDBase: TZConnection;
  protected
    procedure DoParseSQL(const Query: string; Statements: TStrings); override;
    function DoExec(const Query: string; Options: TSelectOptions): TDataSet; override;
    procedure DoConnect; override;
    procedure DoDisconnect; override;
    procedure DoBeginUpdate; override;
    procedure DoEndUpdate; override;
  protected
    function Wrap(Data: IZResultSet): TDataSet; overload;
    function CmdTableExists(const Params: Variant): TDataSet;
  protected
    procedure ConfigureConnection(Params: TStrings); virtual; abstract;
  public
    property Handle: TZConnection read FDBase;
  end;

implementation
uses
  ZAbstractRODataSet, ZSqlProcessor, ZScriptParser, uPropSet, StrUtils;

procedure TZeosStore.DoParseSQL(const Query: string; Statements: TStrings);
var
  Parser: TZSqlProcessor;
  I: Integer;
begin
  if (Pos(';', Query) = 0) { urcite tam neni vic prikazu } and
     (Pos('/', Query) = 0) { urcite tam neni komentar } then begin
    Statements.Clear;
    Statements.Add(Query)
  end else begin
    Parser := TZSqlProcessor.Create(nil);
    try
      Parser.Script.Text := Query;
      Parser.Connection := FDBase;
      Parser.DelimiterType := dtSetTerm;
      Parser.Parse;
      Statements.Clear;
      for I := 0 to Parser.StatementCount - 1 do
        Statements.Add(Parser.Statements[I])
    finally
      Parser.Free
    end
  end
end;

type
  TDataSetAccess = class(TDataSet)
  end;
function TZeosStore.DoExec(const Query: string; Options: TSelectOptions): TDataSet;
var
  Exec: TZReadOnlyQuery;
  RWQuery: TZQuery;
  ROQuery: TZReadOnlyQuery;
begin
  Result := nil;
  if so_Command in Options then begin
    Exec := TZReadOnlyQuery.Create(nil);
    try
      Exec.Connection := FDBase;
      Exec.ParamCheck := False;
      Exec.SQL.Text := Query;
      if AnsiStartsText('CREATE ', Query) or
         AnsiStartsText('DROP ', Query) then
        FDBase.DbcConnection.GetMetadata.ClearCache;
      Exec.ExecSQL;
    finally
      Exec.Free
    end
  end else begin
    if [so_ReadOnly, so_ReadOnce] * Options = [so_ReadOnly, so_ReadOnce] then begin
      ROQuery := TZReadOnlyQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.Connection := FDBase;
      ROQuery.ParamCheck := False;
      // TODO: proc to nefunguje na IB ? TDataSetAccess(ROQuery).SetUniDirectional(True);
      ROQuery.SQL.Text := Query
    end else
    if so_ReadOnly in Options then begin
      ROQuery := TZReadOnlyQuery.Create(Self { TODO: });
      Result := ROQuery;
      ROQuery.Connection := FDBase;
      ROQuery.ParamCheck := False;
      ROQuery.SQL.Text := Query
    end else begin
      RWQuery := TZQuery.Create(Self { TODO: });
      Result := RWQuery;
      RWQuery.Connection := FDBase;
      RWQuery.ParamCheck := False;
      RWQuery.RequestLive := True;
      RWQuery.SQL.Text := Query
    end;
    try
      if not (so_Closed in Options) then
        Result.Active := True
    except
      Result.Free;
      raise
    end
  end
end;

procedure TZeosStore.DoConnect;
var
  Conn: TStrings;
begin
  FDBase := TZConnection.Create(nil);
  try
    Conn := TStringList.Create;
    try
      Conn.Text := StringReplace(Connection, ';' , #13#10, [rfReplaceAll]);

      if (Conn.Count = 1) and (Pos('=', Conn.Text) = 0) then
        Conn.Text := 'db=' + Conn.Text;

      ConfigureConnection(Conn);
    finally
      Conn.Free
    end;

    FDBase.Properties.Values['defaults'] := 'false';
    FDBase.TransactIsolationLevel := tiReadCommitted;
//    FDBase.AutoCommit := True;
    FDBase.Connect;

    FDBQueries := TPropSet.Create(FDBase);
    ConfigureQueries;
    Params.Add(FDBQueries);
  except
    FreeAndNil(FDBase);
    raise
  end
end;

procedure TZeosStore.DoDisconnect;
begin
  FreeAndNil(FDBase)
end;

procedure TZeosStore.DoBeginUpdate;
begin
  FOldAutoCommit := FDBase.AutoCommit;
  FDBase.AutoCommit := False
end;

procedure TZeosStore.DoEndUpdate;
begin
  if FOldAutoCommit then
    FDBase.Commit;
  FDBase.AutoCommit := FOldAutoCommit;
end;

type
  TResultDataSet = class(TZAbstractRODataSet)
  protected
    function CreateResultSet(SQL: string; MaxRows: Integer):
      IZResultSet; override;
  end;

function TResultDataSet.CreateResultSet(SQL: string; MaxRows: Integer): IZResultSet;
begin
  if ResultSet = nil then
    Result := inherited CreateResultSet(SQL, MaxRows)
  else
    Result := ResultSet as IZResultSet
end;

function TZeosStore.Wrap(Data: IZResultSet): TDataSet;
var
  D: TResultDataSet;
begin
  D := TResultDataSet.Create(nil);
  Result := D;
  D.Connection := FDBase;
  D.ResultSet := Data;
  D.Open
end;

function TZeosStore.CmdTableExists(const Params: Variant): TDataSet;
var
  Data: IZResultSet;
begin
  Data := FDBase.DbcConnection.GetMetadata.GetTables('', '', UpperCase(Params[0]), nil);
  Result := Wrap(Data)
end;

end.
