{ VERSION: 1.4 }
{ 1.4 / 9.5.2005
  - nov funkce DecodeVarType(S): TVarType efektivnj
    ne VarType(DecodeVariant(S))
  1.3 / 2.4.2005
  - hodnoty typu string muzou byt uvozene jak v '' tak ""
  - textovy zapis jmeno(index)=hodnota zapisuje do ArrayValues[]
  1.2 / 31.3.2005
  - podporuje jednorozmerne VarArray()
}
unit uPropSet;

interface
uses
  Classes;

// TODO: pouzij TPropSetUpdateMode i pro textovy format
// TODO: aplikuj umAdd spravne i na ArrayValues[] (tj. kdyz je v pameti nebo ve
//   streamu ulozene Array[]

type
  TPropSetUpdateMode = (
    umClear,                  // clear previous content before importing
    umAdd,                    // preserve old values
    umOverwrite               // overwrite existing values
  );
  TPropSet = class(TComponent)
  private
    FModified: Boolean;
    FLinks: TList;
    FNames: TStrings;
    FValues: TStrings;
    function GetName(Index: Integer): string;
    function GetValue(const ID: string): Variant;
    function GetValueFromIndex(Index: Integer): Variant;
    procedure SetValueFromIndex(Index: Integer; const Value: Variant);
    procedure SetValue(const ID: string; const Value: Variant);
    function GetArrayValue(const ID: string; FieldIndex: Cardinal): Variant;
    procedure SetArrayValue(const ID: string; FieldIndex: Cardinal; const Value: Variant);
    function GetArrayValueFromIndex(Index: Integer; FieldIndex: Cardinal): Variant;
    procedure SetArrayValueFromIndex(Index: Integer; FieldIndex: Cardinal; const Value: Variant);
    function GetArraySize(const ID: string): Cardinal;
    function GetAsText: string;
    procedure InternalSetAsText(const Value: string; Append: Boolean);
    procedure SetAsText(const Value: string);
  public
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Add(Data: TPropSet);
    procedure Remove(Data: TPropSet);
  public
    procedure SaveToStream(Stream: TStream);
    procedure LoadFromStream(Stream: TStream; UpdateMode: TPropSetUpdateMode = umClear);
    procedure AddText(const Text: string);
    property AsText: string read GetAsText write SetAsText;
  protected
    property Modified: Boolean read FModified write FModified;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    function Count: Integer;
// TODO:    function Get(const ID: string; const DefaultValue: Variant): Variant;
    function IndexOf(const ID: string): Integer;
    property Names[Index: Integer]: string read GetName;
    property Values[const ID: string]: Variant read GetValue write SetValue; default;
    property ValueFromIndex[Index: Integer]: Variant read GetValueFromIndex write SetValueFromIndex;
    property ArrayValues[const ID: string; FieldIndex: Cardinal]: Variant read GetArrayValue write SetArrayValue;
    property ArraySize[const ID: string]: Cardinal read GetArraySize;
    property ArrayValueFromIndex[Index: Integer; FieldIndex: Cardinal]: Variant read GetArrayValueFromIndex write SetArrayValueFromIndex;
  end;

  TPropSetBook = class(TComponent)
  private
    FModified: Boolean;
    FNames: TStrings;
    function GetName(Index: Integer): string;
    function GetValue(const ID: string): TPropSet;
    function GetModified: Boolean;
    function GetAsText: string;
    procedure SetAsText(const Value: string);
  public
    constructor Create(AnOwner: TComponent);
    destructor Destroy; override;
  public
    procedure SaveToTextStream(Stream: TStream);
    procedure LoadFromTextStream(Stream: TStream);
    property AsText: string read GetAsText write SetAsText;
  public
    procedure Clear;
    function Count: Integer;
    property Names[Index: Integer]: string read GetName;
    property Values[const ID: string]: TPropSet read GetValue; default;
    property Modified: Boolean read GetModified;
  end;

function DecodeVariant(const S: string): Variant; overload;
function DecodeVariant(const S: string; var Pos: Integer): Variant; overload;
function DecodeVarType(const S: string): TVarType; overload;
function DecodeVarType(const S: string; const Pos: Integer): TVarType; overload;
function EncodeVariant(const V: Variant; Streamable: Boolean = False): string;

implementation
uses
  Variants, DateUtils, SysUtils;

{ TPropSet -------------------------------------------------------------------}

type
  TArrayHeader = packed record
    VT: TVarType;
    LowBound: SmallInt;
    HighBound: SmallInt;
  end;

function DecodeVariant(const S: string): Variant;
var
  Pos: Integer;
begin
  Pos := 1;
  Result := DecodeVariant(S, Pos)
end;

function DecodeVariant(const S: string; var Pos: Integer): Variant;
var
  MaxPos: Integer;
  VT: TVarType;

  function DecodeInteger: Integer;
  begin
    Move(S[Pos + SizeOf(TVarType)], Result, SizeOf(Result));
    Inc(Pos, SizeOf(TVarType) + SizeOf(Result))
  end;

  function DecodeDouble: Double;
  begin
    Move(S[Pos + SizeOf(TVarType)], Result, SizeOf(Result));
    Inc(Pos, SizeOf(TVarType) + SizeOf(Result))
  end;

  function DecodeDate: TDateTime;
  begin
    Result := DecodeDouble
  end;

  function DecodeString: string;
  var
    Size: Integer;
  begin
    if (VT and varByRef) <> 0 then begin
      Move(S[Pos + SizeOf(TVarType)], Size, SizeOf(Size));
      SetLength(Result, Size);
      if Result <> '' then
        Move(S[Pos + SizeOf(TVarType) + SizeOf(Size)], Result[1], Size);
      Inc(Pos, SizeOf(TVarType) + SizeOf(Size) + Size)
    end else begin
      Result := Copy(S, Pos + SizeOf(TVarType), MaxInt);
      Pos := MaxPos + 1
    end
  end;

  function DecodeBoolean: Boolean;
  begin
    Move(S[Pos + SizeOf(TVarType)], Result, SizeOf(Result));
    Inc(Pos, SizeOf(TVarType) + SizeOf(Result))
  end;

  function DecodeArray: Variant;
  var
    H: TArrayHeader;
    I: Integer;
  begin
    Move(S[Pos], H, SizeOf(H));
    Inc(Pos, SizeOf(H));
    Result := VarArrayCreate([H.LowBound, H.HighBound], VT and varTypeMask);
    for I := H.LowBound to H.HighBound do
      Result[I] := DecodeVariant(S, Pos)
  end;

begin
  MaxPos := Length(S);
  if Pos > MaxPos then
    Result := Unassigned
  else begin
    Move(S[Pos], VT, SizeOf(VT));

    if (VT and varArray) <> 0 then
      Result := DecodeArray
    else
      case VT and varTypeMask of
      varInteger:
        Result := DecodeInteger;

      varDouble:
        Result := DecodeDouble;

      varDate:
        Result := DecodeDate;

      varString:
        Result := DecodeString;

      varBoolean:
        Result := DecodeBoolean;

      else
        // TODO:
        Result := Unassigned;
        Inc(Pos, SizeOf(VT))
      end;
  end
end;

function DecodeVarType(const S: string; const Pos: Integer): TVarType;
begin
  if (Pos <= 0) or ((Pos - 1) + SizeOf(Result) > Length(S)) then
    Result := varEmpty
  else
    Move(S[Pos], Result, SizeOf(Result));
end;

function DecodeVarType(const S: string): TVarType;
begin
  if S = '' then
    Result := varEmpty
  else
    Move(S[1], Result, SizeOf(Result));
end;

function EncodeVariant(const V: Variant; Streamable: Boolean): string;
var
  VT: TVarType;

  function Encode(VT: TVarType): string; overload;
  begin
    if Streamable then begin
      SetLength(Result, SizeOf(VT));
      Move(VT, Result[1], SizeOf(VT))
    end else
      Result := ''
  end;

  function Encode(VT: TVarType; Value: Integer): string; overload;
  begin
    SetLength(Result, SizeOf(VT) + SizeOf(Value));
    Move(VT, Result[1], SizeOf(VT));
    Move(Value, Result[1 + SizeOf(VT)], SizeOf(Value))
  end;

  function Encode(VT: TVarType; Value: Double): string; overload;
  begin
    SetLength(Result, SizeOf(VT) + SizeOf(Value));
    Move(VT, Result[1], SizeOf(VT));
    Move(Value, Result[1 + SizeOf(VT)], SizeOf(Value))
  end;

{  function Encode(VT: TVarType; Value: TDateTime): string; overload;
  begin
    SetLength(Result, SizeOf(VT) + SizeOf(Value));
    Move(VT, Result[1], SizeOf(VT));
    Move(Value, Result[1 + SizeOf(VT)], SizeOf(Value))
  end;
 }
  function Encode(VT: TVarType; Value: Boolean): string; overload;
  begin
    SetLength(Result, SizeOf(VT) + SizeOf(Value));
    Move(VT, Result[1], SizeOf(VT));
    Move(Value, Result[1 + SizeOf(VT)], SizeOf(Value))
  end;

  function Encode(VT: TVarType; const Value: string): string; overload;
  var
    Size: Integer;
  begin
    if Streamable then begin
      Size := Length(Value);
      SetLength(Result, SizeOf(VT) + SizeOf(Size) + Size);
      VT := VT or varByRef;    // tohle je nejaky bit podle ktereho se rozlisi format s delkou
      Move(VT, Result[1], SizeOf(VT));
      Move(Size, Result[1 + SizeOf(VT)], SizeOf(Size));
      if Value <> '' then
        Move(Value[1], Result[1 + SizeOf(VT) + SizeOf(Size)], Size)
    end else begin
      SetLength(Result, SizeOf(VT) + Length(Value));
      Move(VT, Result[1], SizeOf(VT));
      if Value <> '' then
        Move(Value[1], Result[1 + SizeOf(VT)], Length(Value))
    end
  end;

  function Encode(VT: TVarType; const V: Variant): string; overload;
  var
    H: TArrayHeader;
    I: Integer;
  begin
    H.VT := VT;
    H.LowBound := VarArrayLowBound(V, 1);
    H.HighBound := VarArrayHighBound(V, 1);
    SetLength(Result, SizeOf(H));
    Move(H, Result[1], SizeOf(H));
    for I := H.LowBound to H.HighBound do
      Result := Result + EncodeVariant(V[I], True)
  end;

begin
  VT := VarType(V);
  if (VT and varArray) <> 0 then
    Result := Encode((VT and VarTypeMask) or varArray, V)
  else
    case VT and VarTypeMask of
    varEmpty,
    varNull:
      Result := Encode(varEmpty);

    varSmallInt,
    varInteger,
    varShortInt,
    varByte,
    varWord,
    varLongWord:
      Result := Encode(varInteger, Integer(V));

    varSingle,
    varDouble:
      Result := Encode(varDouble, Double(V));

    varDate:
      Result := Encode(varDate, TDateTime(V));

    varString,
    varOleStr:
      Result := Encode(varString, string(V));

    varBoolean:
      Result := Encode(varBoolean, Boolean(V));

    else
      // TODO:
      Result := Encode(varEmpty)
    end
end;

const
  AnySlot = MaxInt;

function VariantCodeHasSlot(const Code: string; Index: Integer = AnySlot): Boolean;
var
  H: TArrayHeader;
begin
  Result := Length(Code) >= SizeOf(H);
  if Result then begin
    Move(Code[1], H, SizeOf(H));
    Result := ((H.VT and varArray) <> 0) and ((Index = AnySlot) or ((Index >= H.LowBound) and (Index <= H.HighBound)))
  end
end;

function TPropSet.GetName(Index: Integer): string;
var
  P: TPropSet;
  I: Integer;
begin
  Result := '';
  if Index >= 0 then begin
    if Index < FNames.Count then
      Result := FNames[Index]
    else begin
      Dec(Index, FNames.Count);
      for I := 0 to FLinks.Count - 1 do begin
        P := TPropSet(FLinks[I]);
        if Index < P.Count then begin
          Result := P.Names[Index];
          exit
        end;
        Dec(Index, P.Count)
      end
    end
  end
end;

function TPropSet.GetValue(const ID: string): Variant;
begin
  Result := ValueFromIndex[IndexOf(ID)]
end;

function TPropSet.GetArrayValue(const ID: string; FieldIndex: Cardinal): Variant;
begin
  Result := ArrayValueFromIndex[IndexOf(ID), FieldIndex]
end;

function TPropSet.GetValueFromIndex(Index: Integer): Variant;
var
  I: Integer;
  P: TPropSet;
begin
  Result := Unassigned;
  if Index >= 0 then begin
    if Index < FNames.Count then
      Result := DecodeVariant(FValues[Integer(FNames.Objects[Index])])
    else begin
      Dec(Index, FNames.Count);
      for I := 0 to FLinks.Count - 1 do begin
        P := TPropSet(FLinks[I]);
        if Index < P.Count then begin
          Result := P.ValueFromIndex[Index];
          exit
        end;
        Dec(Index, P.Count)
      end
    end
  end
end;

function TPropSet.GetArrayValueFromIndex(Index: Integer; FieldIndex: Cardinal): Variant;
var
  I: Integer;
  P: TPropSet;
  S: string;
begin
  Result := Unassigned;
  if Index >= 0 then begin
    if Index < FNames.Count then begin
      S := FValues[Integer(FNames.Objects[Index])];
      if VariantCodeHasSlot(S, FieldIndex) then
        Result := DecodeVariant(S)[FieldIndex]
      else
      if (not VariantCodeHasSlot(S)) and (FieldIndex = 0) then
        Result := DecodeVariant(S)
    end else begin
      Dec(Index, FNames.Count);
      for I := 0 to FLinks.Count - 1 do begin
        P := TPropSet(FLinks[I]);
        if Index < P.Count then begin
          Result := P.ArrayValueFromIndex[Index, FieldIndex];
          exit
        end;
        Dec(Index, P.Count)
      end
    end
  end
end;

function TPropSet.GetArraySize(const ID: string): Cardinal;
var
  I: Integer;
  P: TPropSet;
  Index: Integer;
  V: Variant;
begin
  Index := IndexOf(ID);
  Result := 0;
  if Index >= 0 then begin
    if Index < FNames.Count then begin
      // TODO: zjisti to bez dekodovani
      V := DecodeVariant(FValues[Integer(FNames.Objects[Index])]);
      if VarIsArray(V) then
        Result := VarArrayHighBound(V, 1) - VarArrayLowBound(V, 1) + 1
      else
        // TODO: nemelo by to vracet 0 pro unassigned ?
        Result := 1
    end else begin
      Dec(Index, FNames.Count);
      for I := 0 to FLinks.Count - 1 do begin
        P := TPropSet(FLinks[I]);
        if Index < P.Count then begin
          Result := P.GetArraySize(ID);
          exit
        end;
        Dec(Index, P.Count)
      end
    end
  end
end;

procedure TPropSet.SetValueFromIndex(Index: Integer; const Value: Variant);
var
  I: Integer;
  P: TPropSet;
  S: string;
begin
  if Index >= 0 then begin
    if Index < FNames.Count then begin
      S := EncodeVariant(Value);
      I := Integer(FNames.Objects[Index]);
      if S <> FValues[I] then begin
        FValues[I] := S;
        FModified := True
      end
    end else begin
      Dec(Index, FNames.Count);
      for I := 0 to FLinks.Count - 1 do begin
        P := TPropSet(FLinks[I]);
        if Index < P.Count then begin
          P.ValueFromIndex[Index] := Value;
          exit
        end;
        Dec(Index, P.Count)
      end
    end
  end
end;

procedure TPropSet.SetArrayValueFromIndex(Index: Integer; FieldIndex: Cardinal; const Value: Variant);
var
  I: Integer;
  P: TPropSet;
  S: string;
  A: Variant;
begin
  if Index >= 0 then begin
    if Index < FNames.Count then begin
      I := Integer(FNames.Objects[Index]);
      S := FValues[I];
      if VariantCodeHasSlot(S, FieldIndex) then begin
        // normalni zapis do pole
        A := DecodeVariant(S);
        A[FieldIndex] := Value;

        // TODO: smrskni pole kdyz se zapisujou na konec nic-y

        ValueFromIndex[Index] := A
      end else
      if (not VariantCodeHasSlot(S)) and (FieldIndex = 0) then begin
        // jedno polozkove pole jako primo hodnota 1. prvku
        ValueFromIndex[Index] := Value
      end else
      if VariantCodeHasSlot(S) then begin
        // zvetseni rozmeru pole
        A := DecodeVariant(S);
        VarArrayRedim(A, FieldIndex);
        A[FieldIndex] := Value;
        ValueFromIndex[Index] := A
      end else begin
        // konverze hodnoty na pole
        A := VarArrayCreate([0, FieldIndex], varVariant);
        A[0] := DecodeVariant(S);
        A[FieldIndex] := Value;
        ValueFromIndex[Index] := A
      end
    end else begin
      Dec(Index, FNames.Count);
      for I := 0 to FLinks.Count - 1 do begin
        P := TPropSet(FLinks[I]);
        if Index < P.Count then begin
          P.ArrayValueFromIndex[Index, FieldIndex] := Value;
          exit
        end;
        Dec(Index, P.Count)
      end
    end
  end
end;

function TPropSet.IndexOf(const ID: string): Integer;
var
  I, J: Integer;
begin
  for I := FLinks.Count - 1 downto 0 do begin
    Result := TPropSet(FLinks[I]).IndexOf(ID);
    if Result >= 0 then begin
      Inc(Result, FNames.Count);
      for J := 0 to I - 1 do
        Inc(Result, TPropSet(FLinks[J]).Count);
      exit
    end;
  end;
  Result := FNames.IndexOf(ID)
end;

procedure TPropSet.SetValue(const ID: string; const Value: Variant);
var
  S: string;
  I: Integer;
begin
  I := IndexOf(ID);
  if I < 0 then begin
    S := EncodeVariant(Value);
    if S <> '' then begin
      // TODO: nemelo by se to zapsat do nejakeho FLinks[] ?
      FNames.AddObject(ID, TObject(FValues.Count));
      FValues.Add(S);
      FModified := True
    end
  end else
    ValueFromIndex[I] := Value
end;

procedure TPropSet.SetArrayValue(const ID: string; FieldIndex: Cardinal; const Value: Variant);
var
  I: Integer;
  A: Variant;
begin
  I := IndexOf(ID);
  if I < 0 then begin
    if FieldIndex = 0 then
      Values[ID] := Value
    else begin
      A := VarArrayCreate([0, FieldIndex], varVariant);
      A[FieldIndex] := Value;
      Values[ID] := A
    end
  end else
    ArrayValueFromIndex[I, FieldIndex] := Value
end;

constructor TPropSet.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  FNames := TStringList.Create;
  TStringList(FNames).Sorted := True;
  FValues := TStringList.Create;
  FLinks := TList.Create;
end;

function TPropSet.Count: Integer;
var
  I: Integer;
begin
  Result := FNames.Count;
  for I := FLinks.Count - 1 downto 0 do
    Inc(Result, TPropSet(FLinks[I]).Count)
end;

procedure TPropSet.Add(Data: TPropSet);
var
  I: Integer;
begin
  I := FLinks.IndexOf(Data);
  if I < 0 then begin
    FLinks.Add(Data);
    Data.FreeNotification(Self);
  end
end;

procedure TPropSet.Remove(Data: TPropSet);
var
  I: Integer;
begin
  I := FLinks.IndexOf(Data);
  if I >= 0 then begin
    Data.RemoveFreeNotification(Self);
    FLinks.Remove(Data)
  end
end;

procedure TPropSet.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (Operation = opRemove) and (AComponent is TPropSet) then Remove(TPropSet(AComponent));
  inherited
end;

destructor TPropSet.Destroy;
begin
  if Assigned(FLinks) then
    while FLinks.Count <> 0 do
      Remove(TPropSet(FLinks[0]));

  FreeAndNil(FNames);
  FreeAndNil(FValues);
  FreeAndNil(FLinks);
  inherited
end;

procedure TPropSet.Clear;
begin
  if FNames.Count <> 0 then begin
    FNames.Clear;
    FValues.Clear;
    FModified := True
  end
end;

procedure TPropSet.SaveToStream(Stream: TStream);

  procedure Save(const Value: string); overload;
  var
    N: Integer;
  begin
    N := Length(Value);
    Stream.WriteBuffer(N, SizeOf(N));
    if N <> 0 then
      Stream.WriteBuffer(Value[1], N)
  end;

  procedure Save(Value: Integer); overload;
  begin
    Stream.WriteBuffer(Value, SizeOf(Value))
  end;

var
  I: Integer;
begin
  Save(FNames.Count);
  for I := 0 to FNames.Count - 1 do begin
    Save(FNames[I]);
    Save(FValues[Integer(FNames.Objects[I])])
  end;
end;

procedure TPropSet.LoadFromStream(Stream: TStream; UpdateMode: TPropSetUpdateMode);
var
  N: Integer;
  I: Integer;

  function Load: string;
  var
    N: Integer;
  begin
    Stream.ReadBuffer(N, SizeOf(N));
    SetLength(Result, N);
    if N <> 0 then
      Stream.ReadBuffer(Result[1], N)
  end;

var
  Key: string;
  Value: string;
  Pos: Integer;
begin
  if UpdateMode = umClear then
    Clear
  else
  if FNames.Count = 0 then
    UpdateMode := umClear;

  I := Stream.Position;
  if Stream.Read(N, SizeOf(N)) = SizeOf(N) then begin
    for I := N - 1 downto 0 do
      if UpdateMode = umClear then begin
        FNames.AddObject(Load, TObject(FValues.Count));
        FValues.Add(Load)
      end else begin
        Key := Load;
        Value := Load;
        Pos := FNames.IndexOf(Key);
        if Pos < 0 then begin
          FNames.AddObject(Key, TObject(FValues.Count));
          FValues.Add(Value)
        end else
        if UpdateMode = umOverwrite then
          FValues[Integer(FNames.Objects[Pos])] := Value
      end
  end else
    Stream.Position := I
end;

const
  NullStr = 'Nil'; // TUNE:

function TPropSet.GetAsText: string;

  function Encode(const Value: Variant; NonEmpty: Boolean = False): string;
  var
    OldSeparator: Char;
    Dt: TDateTime;

    function EncodeStr(const Value: string): string;
    var
      I: Integer;
    begin
      Result := '';
      for I := 1 to Length(Value) do
        case Value[I] of
        #13:
          Result := Result + '\r';
        #10:
          Result := Result + '\n';
        #9:
          Result := Result + '\t';
        '"', '''', '\':
          Result := Result + '\' + Value[I];
        else
          Result := Result + Value[I]
        end
    end;

    function EncodeArray: string;
    var
      LowBound: Integer;
      HighBound: Integer;
      I: Integer;
    begin
      LowBound := VarArrayLowBound(Value, 1);
      HighBound := VarArrayHighBound(Value, 1);
      Result := '(';
      for I := LowBound to HighBound do begin
        if I > LowBound then
          Result := Result + ',';
        Result := Result + Encode(Value[I], True)
      end;
      Result := Result + ')'
    end;

  var
    OldDateSeparator: Char;
    OldTimeSeparator: Char;
    VT: TVarType;
  begin
    VT := VarType(Value);
    if (VT and varArray) <> 0 then
      Result := EncodeArray
    else
      case VT and varTypeMask of
      varEmpty,
      varNull:
        if NonEmpty then
          Result := NullStr
        else
          Result := '';

      varSmallInt,
      varInteger,
      varShortInt,
      varByte,
      varWord,
      varLongWord:
        Result := IntToStr(Value);

      varSingle,
      varDouble:
        begin
          OldSeparator := DecimalSeparator;
          DecimalSeparator := '.';
          // TODO: a co znaminko ?
          Result := FormatFloat('0.0###', Value);
          DecimalSeparator := OldSeparator
        end;

      varDate:
        begin
          OldDateSeparator := DateSeparator;
          DateSeparator := '/';
          OldTimeSeparator := TimeSeparator;
          TimeSeparator := ':';
          Dt := Value;
          if DateOf(Dt) = 0 then
            Result := FormatDateTime('hh:nn:ss:zzz', Dt)
          else
          if TimeOf(Dt) = 0 then
            Result := FormatDateTime('yyyy/mm/dd', Dt)
          else
            Result := FormatDateTime('yyyy/mm/dd hh:nn:ss:zzz', Dt);
          DateSeparator := OldDateSeparator;
          TimeSeparator := OldTimeSeparator;
        end;

      varString,
      varOleStr:
        Result := '"' + EncodeStr(string(Value)) + '"';

      varBoolean:
        if Boolean(Value) then
          Result := DefaultTrueBoolStr
        else
          Result := DefaultFalseBoolStr;

      else
        // TODO:
        if NonEmpty then
          Result := NullStr
        else
          Result := ''
      end
  end;

var
  Lines: TStrings;
  I: Integer;
begin
  Lines := TStringList.Create;
  try
    for I := 0 to FNames.Count - 1 do
      Lines.Add(FNames[I] + '=' + Encode(ValueFromIndex[I]));
    Result := Lines.Text
  finally
    Lines.Free
  end
end;

procedure TPropSet.SetAsText(const Value: string);
begin
  InternalSetAsText(Value, False)
end;

procedure TPropSet.AddText(const Text: string);
begin
  InternalSetAsText(Text, True)
end;

procedure TPropSet.InternalSetAsText(const Value: string; Append: Boolean);

  function Decode(const Text: string): Variant;

    procedure Split(const Text: string; Separator: Char; var A: Integer; var B: Integer; var C: Integer; var D: Integer); overload;
    var
      Items: TStrings;

      function Item(Index: Integer): Integer;
      begin
        if Index < Items.Count then
          Result := StrToIntDef(Items[Index], 0)
        else
          Result := 0
      end;

    begin
      Items := TStringList.Create;
      try
        Items.Text := StringReplace(Trim(Text), Separator, #13#10, [rfReplaceAll]);
        A := Item(0);
        B := Item(1);
        C := Item(2);
        D := Item(3)
      finally
        Items.Free
      end
    end;

    procedure Split(const Text: string; Separator: Char; var A: Integer; var B: Integer; var C: Integer); overload;
    var
      Dummy: Integer;
    begin
      Split(Text, Separator, A, B, C, Dummy)
    end;

    function DecodeStr(const Text: string): string;
    var
      I: Integer;
    begin
      if Pos('\', Text) = 0 then
        Result := Text
      else begin
        Result := '';
        I := 1;
        while I <= Length(Text) do
          if Text[I] = '\' then begin
            Inc(I);
            case Text[I] of
            'r':
              begin
                Result := Result + #13;
                Inc(I)
              end;

            'n':
              begin
                Result := Result + #10;
                Inc(I)
              end;

            't':
              begin
                Result := Result + #9;
                Inc(I)
              end;

            '"', '''', '\':
              begin
                Result := Result + Text[I];
                Inc(I)
              end;

            // TODO: dalsi znaky

            else
              Result := Result + '\';
            end
          end else begin
            Result := Result + Text[I];
            Inc(I)
          end
      end
    end;

    function DecodeArray(const Text: string): Variant;
    const
      Escapables = ['"', '''', '\'];
    var
      Items: TStrings;
      Level: Integer;
      InQuotes: Boolean;
      I: Integer;
      Token: string;
      Quote: Char;
    begin
      Items := TStringList.Create;
      try
        Level := 0;
        InQuotes := False;
        Token := '';
        I := 1;
        while I <= Length(Text) do begin
          case Text[I] of
          '(':
            begin
              if not InQuotes then
                Inc(Level);
              Token := Token + Text[I]
            end;

          ')':
            begin
              if not InQuotes then
                Dec(Level);
              Token := Token + Text[I]
            end;

          ',':
            if (not InQuotes) and (Level = 0) then begin
              Items.Add(Trim(Token));
              Token := ''
            end else
              Token := Token + Text[I];

          '"', '''':
            if not InQuotes then begin
              InQuotes := True;
              Quote := Text[I];
              Token := Token + Text[I]
            end else
            if Text[I] = Quote then begin
              InQuotes := False;
              Token := Token + Text[I]
            end else
              Token := Token + Text[I];

          '\':
            if InQuotes and (I + 1 <= Length(Text)) and (Text[I + 1] in Escapables) then begin
              Token := Token + Text[I] + Text[I + 1];
              Inc(I)
            end else
              Token := Token + Text[I];

          else
            Token := Token + Text[I]
          end;
          Inc(I)
        end;

        // TODO: nejak to nasignalizuj
        Assert((not InQuotes) and (Level = 0));

        Items.Add(Trim(Token));

        Result := VarArrayCreate([0, Items.Count - 1], varVariant);
        for I := 0 to Items.Count - 1 do
          Result[I] := Decode(Items[I]);

      finally
        Items.Free
      end
    end;

    function IsNumber(const S: string): Boolean;
    var
      I, L: Integer;
    begin
      if (S <> '') and (S[1] in ['+', '-']) then
        L := 2
      else
        L := 1;
      for I := L to Length(S) do
        if not (S[I] in ['0'..'9']) then begin
          Result := False;
          exit
        end;
      Result := Length(S) >= L
    end;

  var
    OldSeparator: Char;
    I: Integer;
    Year, Month, Day: Integer;
    Hour, Minute, Second, MSec: Integer;
  begin
    if (Text = '') or SameText(Text, NullStr) then
      Result := Unassigned
    else
    if Text[1] in ['"', ''''] then
      Result := DecodeStr(Copy(Text, 2, Length(Text) - 2))
    else
    if Text[1] = '(' then
      Result := DecodeArray(Copy(Text, 2, Length(Text) - 2))
    else
    if SameText(Text, DefaultFalseBoolStr) then
      Result := False
    else
    if SameText(Text, DefaultTrueBoolStr) then
      Result := True
    else
    if Pos('.', Text) <> 0 then begin
      OldSeparator := DecimalSeparator;
      DecimalSeparator := '.';
      Result := StrToFloatDef(Text, 0);
      DecimalSeparator := OldSeparator
    end else
    if Pos('/', Text) <> 0 then begin
      I := Pos(':', Text);
      if I = 0 then begin
        Split(Text, '/', Year, Month, Day);
        Result := EncodeDate(Year, Month, Day)
      end else begin
        I := Pos(' ', Text);
        Split(Copy(Text, 1, I - 1), '/', Year, Month, Day);
        // TODO: jak se to dela s tou MSec ?
        Split(Copy(Text, I + 1, MaxInt), ':', Hour, Minute, Second, MSec);
        Result := EncodeDateTime(Year, Month, Day, Hour, Minute, Second, MSec)
      end
    end else
    if Pos(':', Text) <> 0 then begin
      Split(Text, ':', Hour, Minute, Second, MSec);
      Result := EncodeTime(Hour, Minute, Second, MSec)
    end else
    if IsNumber(Text) then
      Result := StrToIntDef(Text, 0)
    else
      Result := DecodeStr(Text)
  end;

  procedure SetSlot(const ID: string; const Value: Variant);
  var
    I: Integer;
  begin
    I := Pos('(', ID);
    if I = 0 then
      Values[ID] := Value
    else begin
      ArrayValues[TrimRight(Copy(ID, 1, I - 1)), StrToIntDef(Trim(Copy(ID, I + 1, Length(ID) - I - 1)), 0)] := Value
    end
  end;

var
  Lines: TStrings;
  I, J: Integer;
  Line: string;
begin
  if not Append then
    Clear;

  Lines := TStringList.Create;
  try
    Lines.Text := Value;
    for I := 0 to Lines.Count - 1 do begin
      Line := Trim(Lines[I]);
      if (Line = '') or (Line[1] = ';') then continue;
      J := Pos('=', Line);
      if J = 0 then
        SetSlot(Line, True)
      else
        SetSlot(TrimRight(Copy(Line, 1, J - 1)), Decode(TrimLeft(Copy(Line, J + 1, MaxInt))))
    end
  finally
    Lines.Free
  end
end;

{ TPropSetBook ---------------------------------------------------------------}

function TPropSetBook.Count: Integer;
begin
  Result := FNames.Count
end;

constructor TPropSetBook.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  FNames := TStringList.Create;
  TStringList(FNames).Sorted := True
end;

procedure TPropSetBook.Clear;
var
  I: Integer;
begin
  if Assigned(FNames) then begin
    for I := Count - 1 downto 0 do
      FNames.Objects[I].Free;
    FNames.Clear
  end
end;

destructor TPropSetBook.Destroy;
begin
  Clear;
  FNames.Free;
  inherited;
end;

function TPropSetBook.GetName(Index: Integer): string;
begin
  Result := FNames[Index]
end;

function TPropSetBook.GetValue(const ID: string): TPropSet;
var
  I: Integer;
begin
  I := FNames.IndexOf(ID);
  if I < 0 then begin
    Result := TPropSet.Create(nil);
    FNames.AddObject(ID, Result)
  end else
    Result := TPropSet(FNames.Objects[I])
end;

procedure TPropSetBook.SaveToTextStream(Stream: TStream);
var
  I: Integer;

  procedure WriteLn(const Text: string);

    procedure Write(const Text: string);
    begin
      if Text <> '' then
        Stream.WriteBuffer(Text[1], Length(Text))
    end;

  begin
    Write(Text);
    Write(#13#10)
  end;

var
  Key: string;
begin
  for I := 0 to Count - 1 do begin
    Key := FNames[I];
    if not ((I = 0) and (Key = '')) then
      WriteLn('[' + Key + ']');
    WriteLn(TPropSet(FNames.Objects[I]).AsText)
  end
end;

procedure TPropSetBook.LoadFromTextStream(Stream: TStream);
var
  Lines: TStrings;
  A, B: Integer;
  CurrentSection: string;
  NextSection: string;

  function FindSection(Index: Integer; var Key: string): Integer;
  begin
    Key := '';
    while Index < Lines.Count do
      if Copy(TrimLeft(Lines[Index]), 1, 1) = '[' then begin
        Key := Trim(Lines[Index]);
        Key := Copy(Key, 2, Length(Key) - 2);
        break
      end else
        Inc(Index);
    Result := Index
  end;

  procedure ProcessSection(const Key: string; Index: Integer; Count: Integer);
  var
    S: string;
    Item: TPropSet;
  begin
    if Count <= 0 then exit;
    S := '';
    while Count > 0 do begin
      S := S + Lines[Index] + #13#10;
      Inc(Index);
      Dec(Count)
    end;
    Item := Values[Key];
    Item.AsText := S;
    Item.Modified := False
  end;

begin
  Clear;
  Lines := TStringList.Create;
  try
    Lines.LoadFromStream(Stream);
    CurrentSection := '';
    A := 0;
    while A < Lines.Count do begin
      B := FindSection(A, NextSection);
      ProcessSection(CurrentSection, A, B - A);
      CurrentSection := NextSection;
      A := B + 1;
    end;
    FModified := False
  finally
    Lines.Free
  end
end;

function TPropSetBook.GetAsText: string;
var
  Stream: TStringStream;
begin
  Stream := TStringStream.Create('');
  try
    SaveToTextStream(Stream);
    Result := Stream.DataString
  finally
    Stream.Free
  end
end;

procedure TPropSetBook.SetAsText(const Value: string);
var
  Stream: TStringStream;
begin
  Stream := TStringStream.Create(Value);
  try
    LoadFromTextStream(Stream)
  finally
    Stream.Free
  end
end;

function TPropSetBook.GetModified: Boolean;
var
  I: Integer;
begin
  Result := FModified;
  if not Result then
    for I := Count - 1 downto 0 do
      if TPropSet(FNames.Objects[I]).Modified then begin
        Result := True;
        exit
      end
end;

end.
