{ VERSION: 1.2 }
{ 1.2 / 29.5.2005
  - pomocn funkce ObjectResourceToAnsiText(), ObjectTextToAnsiText()
  1.1 / 11.5.2005
  - pomocn funkce SaveToStream(TStream, TComponent) a SaveToTextStream(TStream, TComponent)
  - nov funkce pro uloen komponenty do INI souboru do zadan sekce ( viz. SaveToIni() )
  - nov funkce pro konverzi INI zpisu komponenty do standartnho tvaru pouitelnho ve streamovai
    ( viz. ObjectIniToBinary(), ObjectIniToText() )
  - INI ten nutn potebuje pstup k plnmu znn ( jednoduch uvozovky kolem hodnoty ).
    na WinXP standartn WinAPI funkce ( TIniFile ) ty uvozovky seere. Pro ten tedy pouvej
    TMemIniFile !!!
  1.0 / 21.4.2005
  - funkce ObjectBinaryToAnsiText() funguje stejn jako funkce
    Classes.ObjectBinaryToText() ale v Delphi7 ukld text v kdovn Windows
    a ne v kdovn UTF8. Text je potom itelnj, protoe jsou znaky s diakritikou
    normln vidt
}
unit uStreamUtils;

interface
uses
  Classes, IniFiles;

procedure ObjectBinaryToIni(Input: TStream; Output: TCustomIniFile; const Section: string);
procedure ObjectIniToText(Input: TCustomIniFile; const Section: string; Output: TStream; const RootClassName: string);
procedure ObjectIniToBinary(Input: TCustomIniFile; const Section: string; Output: TStream; const RootClassName: string);
procedure ObjectBinaryToAnsiText(Input: TStream; Output: TStream);
procedure ObjectResourceToAnsiText(Input: TStream; Output: TStream);
procedure ObjectTextToAnsiText(InputOutput: TStream);

procedure SaveToStream(Stream: TStream; Component: TComponent; Root: TComponent = nil);
procedure SaveToTextStream(Stream: TStream; Component: TComponent; Root: TComponent = nil);
procedure SaveToIni(Ini: TCustomIniFile; const Section: string; Component: TComponent; Root: TComponent = nil);

implementation
uses
  TypInfo, StrUtils, SysUtils, RTLConsts;

procedure SaveToStream(Stream: TStream; Component: TComponent; Root: TComponent);
var
  W: TWriter;
begin
  W := TWriter.Create(Stream, 1024 { TUNE: });
  try
    W.Root := Root;
    W.WriteSignature;
    W.WriteComponent(Component);
  finally
    W.Free
  end;
end;

procedure SaveToTextStream(Stream: TStream; Component: TComponent; Root: TComponent);
var
  Tmp: TStream;
begin
  Tmp := TMemoryStream.Create;
  try
    SaveToStream(Tmp, Component, Root);
    Tmp.Position := 0;
    ObjectBinaryToAnsiText(Tmp, Stream);
  finally
    Tmp.Free
  end
end;

procedure SaveToIni(Ini: TCustomIniFile; const Section: string; Component: TComponent; Root: TComponent = nil);
var
  Tmp: TStream;
begin
  Tmp := TMemoryStream.Create;
  try
    SaveToStream(Tmp, Component, Root);
    Tmp.Position := 0;
    ObjectBinaryToIni(Tmp, Ini, Section)
  finally
    Tmp.Free
  end
end;

const
  Quote = '''';

type
  TIniWriter = class
  private
    Level: Integer;
    Nums: array of Integer;
    Ini: TCustomIniFile;
    Section: string;
    InList: Boolean;
    Buf: string;
    Sep: string;
    BufEmpty: Boolean;
    PropName: string;
    procedure AddChunk(const Value: string);
    function EnQuote(const Value: string): string;
  public
    constructor Create(AnIni: TCustomIniFile; const ASection: string);
    procedure Open(const ASection: string; AutoInc: Boolean = True);
    procedure Close;
    procedure WriteString(const PropertyName: string; const Value: string);
    procedure WriteIdent(const PropertyName: string; const Value: string);
    procedure WriteInteger(const PropertyName: string; const Value: Integer);
    procedure OpenList(const PropertyName: string; const Head: string; const Separator: string);
    procedure AddString(const Value: string);
    procedure AddIdent(const Value: string);
    procedure AddInteger(const Value: Integer);
    procedure CloseList(const Tail: string);
  end;

function TIniWriter.EnQuote(const Value: string): string;
begin
  // TODO: zakoduj nezobrazitelne znaky ve Value
  Result := Quote + Value + Quote
end;

procedure TIniWriter.OpenList(const PropertyName: string; const Head: string; const Separator: string);
begin
  // TODO: at se to da vnorovat
  PropName := PropertyName;
  Buf := Head;
  BufEmpty := True;
  Sep := Separator;
  InList := True
end;

procedure TIniWriter.CloseList(const Tail: string);
begin
  Buf := Buf + Tail;
  Ini.WriteString(Section, PropName, Buf);
  InList := False
end;

procedure TIniWriter.AddChunk(const Value: string);
begin
  if not BufEmpty then
    Buf := Buf + Sep;
  BufEmpty := False;
  Buf := Buf + Value
end;

procedure TIniWriter.AddString(const Value: string);
begin
  AddChunk(EnQuote(Value))
end;

procedure TIniWriter.AddIdent(const Value: string);
begin
  AddChunk(Value)
end;

procedure TIniWriter.AddInteger(const Value: Integer);
begin
  AddChunk(IntToStr(Value))
end;

procedure TIniWriter.WriteString(const PropertyName: string; const Value: string);
begin
  if PropertyName = '' then
    AddString(Value)
  else
    Ini.WriteString(Section, PropertyName, EnQuote(Value));
end;

procedure TIniWriter.WriteIdent(const PropertyName: string; const Value: string);
begin
  if PropertyName = '' then
    AddIdent(Value)
  else
    Ini.WriteString(Section, PropertyName, Value);
end;

procedure TIniWriter.WriteInteger(const PropertyName: string; const Value: Integer);
begin
  if PropertyName = '' then
    AddInteger(Value)
  else
    Ini.WriteInteger(Section, PropertyName, Value);
end;

procedure TIniWriter.Open(const ASection: string; AutoInc: Boolean);
begin
  Section := Section + DotSep + ASection;
  if AutoInc then begin
    Nums[Level] := Nums[Level] + 1;
    Section := Section + '(' + IntToStr(Nums[Level]) + ')'
  end;
  Inc(Level);
  if Length(Nums) <= Level then
    SetLength(Nums, Level + 1);
  Nums[Level] := 0;
end;

procedure TIniWriter.Close;
var
  I: Integer;
begin
  for I := Length(Section) downto 1 do
    if Section[I] = DotSep then begin
      SetLength(Section, I - 1);
      Dec(Level);
      break
    end
end;

constructor TIniWriter.Create(AnIni: TCustomIniFile; const ASection: string);
var
  List: TStrings;
  I: Integer;
begin
  inherited Create;
  Ini := AnIni;
  Section := ASection;
  if Section <> '' then begin
    List := TStringList.Create;
    try
      Ini.ReadSections(List);
      for I := 0 to List.Count - 1 do begin
        if SameText(List[I], Section) or AnsiStartsText(Section + DotSep, List[I]) then
          Ini.EraseSection(List[I]);
      end
    finally
      List.Free
    end
  end;
  Level := 0;
  SetLength(Nums, 1);
  Nums[0] := 0
end;

const
  CollectionTag = '<>';

procedure ObjectBinaryToIni(Input: TStream; Output: TCustomIniFile; const Section: string);
var
  SaveSeparator: Char;
  Reader: TReader;
  Writer: TIniWriter;
  ObjectName, PropName: string;

  procedure ConvertValue(const PropertyName: string); forward;

  procedure ConvertHeader(Ignore: Boolean);
  var
    ClassName: string;
    Flags: TFilerFlags;
    Position: Integer;
  begin
    Reader.ReadPrefix(Flags, Position);
    ClassName := Reader.ReadStr;
    ObjectName := Reader.ReadStr;
    { TODO:
    if ffInherited in Flags then
      WriteStr('inherited ')
    else if ffInline in Flags then
      WriteStr('inline ')
    else
      WriteStr('object ');
    }
    if not Ignore then begin
      if ffChildPos in Flags then
        Writer.Open(ClassName + '[' + IntToStr(Position) + ']')
      else
        Writer.Open(ClassName);

      if ObjectName <> '' then
        Writer.WriteIdent('Name', ObjectName);
    end;

    if ObjectName = '' then
      ObjectName := ClassName;  // save for error reporting
  end;

(* TODO:
  procedure ConvertBinary;
  const
    BytesPerLine = 32;
  var
    MultiLine: Boolean;
    I: Integer;
    Count: Longint;
    Buffer: array[0..BytesPerLine - 1] of Char;
    Text: array[0..BytesPerLine * 2 - 1] of Char;
  begin
    Reader.ReadValue;
    WriteStr('{');
    Inc(NestingLevel);
    Reader.Read(Count, SizeOf(Count));
    MultiLine := Count >= BytesPerLine;
    while Count > 0 do
    begin
      if MultiLine then NewLine;
      if Count >= 32 then I := 32 else I := Count;
      Reader.Read(Buffer, I);
      BinToHex(Buffer, Text, I);
      Writer.Write(Text, I * 2);
      Dec(Count, I);
    end;
    Dec(NestingLevel);
    WriteStr('}');
  end;
*)

  procedure ConvertProperty; forward;

  procedure ConvertValue(const PropertyName: string);
  const
    LineLength = 64;
  var
    I, J, K, L: Integer;
    S: string;
    W: WideString;
    LineBreak: Boolean;
  begin
    case Reader.NextValue of
      vaList:
        begin
          Reader.ReadValue;
          Writer.OpenList(PropertyName, '(', ',');
          while not Reader.EndOfList do
          begin
            ConvertValue('');
          end;
          Reader.ReadListEnd;
          Writer.CloseList(')');
        end;
      vaInt8, vaInt16, vaInt32:
        Writer.WriteInteger(PropertyName, Reader.ReadInteger);
(* TODO:
      vaExtended:
        WriteStr(FloatToStrF(Reader.ReadFloat, ffFixed, 16, 18));
      vaSingle:
        WriteStr(FloatToStr(Reader.ReadSingle) + 's');
      vaCurrency:
        WriteStr(FloatToStr(Reader.ReadCurrency * 10000) + 'c');
      vaDate:
        WriteStr(FloatToStr(Reader.ReadDate) + 'd');
*)
      vaWString, vaUTF8String:
        begin
          Writer.WriteString(PropertyName, Reader.ReadWideString);
        end;
      vaString, vaLString:
        begin
          Writer.WriteString(PropertyName, Reader.ReadString);
        end;
      vaIdent, vaFalse, vaTrue, vaNil, vaNull:
        Writer.WriteIdent(PropertyName, Reader.ReadIdent);
(* TODO:
      vaBinary:
        ConvertBinary;
*)
      vaSet:
        begin
          Reader.ReadValue;
          Writer.OpenList(PropertyName, '[', ',');
          I := 0;
          while True do
          begin
            S := Reader.ReadStr;
            if S = '' then Break;
            Writer.AddIdent(S);
            Inc(I);
          end;
          Writer.CloseList(']');
        end;
      vaCollection:
        begin
          Writer.WriteIdent(PropertyName, CollectionTag);
          I := 1;
          Reader.ReadValue;
          while not Reader.EndOfList do
          begin
            if Reader.NextValue in [vaInt8, vaInt16, vaInt32] then
              Writer.Open(PropertyName + '[' + IntToStr(Reader.ReadInteger) + ']<' + IntToStr(I) + '>', False)
            else
              Writer.Open(PropertyName + '<' + IntToStr(I) + '>', False);
            Reader.CheckValue(vaList);
            while not Reader.EndOfList do ConvertProperty;
            Reader.ReadListEnd;
            Writer.Close;
            Inc(I)
          end;
          Reader.ReadListEnd;
        end;
(* TODO:
      vaInt64:
        WriteStr(IntToStr(Reader.ReadInt64));
*)
    else
      raise EReadError.CreateResFmt(@sPropertyException,
        [ObjectName, DotSep, PropName, IntToStr(Ord(Reader.NextValue))]);
    end;
  end;

  procedure ConvertProperty;
  begin
    PropName := Reader.ReadStr;  // save for error reporting
    ConvertValue(PropName);
  end;

  procedure ConvertObject(Ignore: Boolean);
  begin
    ConvertHeader(Ignore);
    while not Reader.EndOfList do ConvertProperty;
    Reader.ReadListEnd;
    while not Reader.EndOfList do ConvertObject(False);
    Reader.ReadListEnd;
    if not Ignore then
      Writer.Close
  end;

begin
  Reader := TReader.Create(Input, 4096);
  SaveSeparator := DecimalSeparator;
  DecimalSeparator := '.';
  try
    Writer := TIniWriter.Create(Output, Section);
    try
      Reader.ReadSignature;
      ConvertObject(True);
    finally
      Writer.Free;
    end;
  finally
    DecimalSeparator := SaveSeparator;
    Reader.Free;
  end;
end;

function SortNums(List: TStringList; Index1, Index2: Integer): Integer;

  function Key(const S: string): Integer;
  var
    I: Integer;
  begin
    I := Pos('(', S);
    if I = 0 then
      Result := 0
    else
      Result := StrToInt(Copy(S, I + 1, Length(S) - (I + 1)))
  end;

begin
  Result := Key(List[Index1]) - Key(List[Index2])
end;

function SortItems(List: TStringList; Index1, Index2: Integer): Integer;

  function Key(const S: string): Integer;
  var
    I: Integer;
  begin
    I := Pos('<', S);
    if I = 0 then
      Result := 0
    else
      Result := StrToInt(Copy(S, I + 1, Length(S) - (I + 1)))
  end;

begin
  Result := Key(List[Index1]) - Key(List[Index2])
end;

procedure ObjectIniToText(Input: TCustomIniFile; const Section: string; Output: TStream; const RootClassName: string);
var
  NestingLevel: Integer;

  procedure Write(const Msg: string);
  begin
    if Msg <> '' then
      Output.WriteBuffer(Msg[1], Length(Msg))
  end;

  procedure WriteIndent;
  var
    I: Integer;
  begin
    for I := 1 to NestingLevel do Write('  ');
  end;

  procedure WriteLn; overload;
  begin
    Write(#13#10)
  end;

  procedure WriteLn(const Msg: string); overload;
  begin
    Write(Msg);
    WriteLn
  end;

  procedure ReadObject(const Section: string; const ObjectClass: string; const ClassName: string; HasName: Boolean); forward;

  procedure ReadCollection(const Section: string);
  var
    List: TStrings;
    Keys: TStringList;
    I, J: Integer;
    Prefix: string;
    Name: string;
  begin
    List := TStringList.Create;
    Keys := TStringList.Create;
    Keys.Sorted := True;
    try
      Input.ReadSections(List);
      Prefix := Section + '<';
      for I := 0 to List.Count - 1 do begin
        Name := List[I];
        if AnsiStartsText(Prefix, Name) then begin
          Delete(Name, 1, Length(Prefix));
          J := Pos(DotSep, Name);
          if J <> 0 then
            SetLength(Name, J - 1);
          if Keys.IndexOf(Name) < 0 then
            Keys.Add('<' + Name)
        end;
      end;

      Keys.Sorted := False;
      Keys.CustomSort(SortItems);

      for I := 0 to Keys.Count - 1 do begin
        Name := Keys[I];
        J := Pos('<', Name);
        SetLength(Name, J - 1);
        if Name = '' then
          Name := 'item'
        else
          Name := 'item ' + Name;
        ReadObject(Section + Keys[I], Name, '', False)
      end;

    finally
      Keys.Free;
      List.Free
    end
  end;

  procedure ReadObject(const Section: string; const ObjectClass: string; const ClassName: string; HasName: Boolean);
  var
    List: TStrings;
    Keys: TStringList;
    I, J: Integer;
    Name: string;
    Prefix: string;
  begin
    List := TStringList.Create;
    try
      WriteIndent;
      Input.ReadSection(Section, List);
      if HasName then
        Name := Input.ReadString(Section, 'Name', '')
      else
        Name := '';
        
      if Name = '' then begin
        Write(ObjectClass);
        if ClassName <> '' then
          Write(' ' + ClassName);
        WriteLn
      end else begin
        WriteLn(ObjectClass + ' ' + Name + ': ' + ClassName);
        I := List.IndexOf('Name');
        if I >= 0 then
          List.Delete(I)
      end;

      Inc(NestingLevel);

      for I := 0 to List.Count - 1 do begin
        WriteIndent;
        Name := Input.ReadString(Section, List[I], '');
        if Name = CollectionTag then begin
          WriteLn(List[I] + ' = <');
          Inc(NestingLevel);
          ReadCollection(Section + DotSep + List[I]);
          Dec(NestingLevel);
          WriteIndent;
          WriteLn('>')
        end else
          WriteLn(List[I] + ' = ' + Name);
      end;

      Keys := TStringList.Create;
      Keys.Sorted := True;
      try
        Input.ReadSections(List);
        Prefix := Section + DotSep;
        for I := 0 to List.Count - 1 do begin
          Name := List[I];
          if AnsiStartsText(Prefix, Name) then begin
            Delete(Name, 1, Length(Prefix));
            J := Pos(DotSep, Name);
            if J <> 0 then
              SetLength(Name, J - 1);
            if (Name[Length(Name)] <> '>') and { not a collection item }
               (Keys.IndexOf(Name) < 0) then
              Keys.Add(Name)
          end;
        end;

        Keys.Sorted := False;
        Keys.CustomSort(SortNums);

        for I := 0 to Keys.Count - 1 do begin
          Name := Keys[I];
          J := Pos('(', Name);
          if J <> 0 then
            SetLength(Name, J - 1);
          ReadObject(Prefix + Keys[I], 'object', Name, True)
        end;

      finally
        Keys.Free
      end;

      Dec(NestingLevel);

      WriteIndent;
      WriteLn('end')
    finally
      List.Free
    end
  end;

begin
  NestingLevel := 0;
  ReadObject(Section, 'object', RootClassName, True);
end;

procedure ObjectIniToBinary(Input: TCustomIniFile; const Section: string; Output: TStream; const RootClassName: string);
var
  Tmp: TStream;
begin
  Tmp := TMemoryStream.Create;
  try
    ObjectIniToText(Input, Section, Tmp, RootClassName);
    Tmp.Position := 0;
    ObjectTextToBinary(Tmp, Output)
  finally
    Tmp.Free
  end
end;

{$IFDEF VER150}
procedure ObjectUtf8TextToAnsiText(Stream: TStream; Position: Integer);
var
  Code: string;
  I, J: Integer;
  N: Integer;
  CharCode: Integer;
  WS: WideString;
  SS: string;
  LastChar: Char;

  procedure Write(Ch: Char); overload;
  begin
    Stream.WriteBuffer(Ch, SizeOf(Ch));
    LastChar := Ch
  end;

  procedure Write(const S: string); overload;
  begin
    if S <> '' then begin
      Stream.WriteBuffer(S[1], Length(S));
      LastChar := S[Length(S)]
    end
  end;

  procedure Write(const S: string; Start: Integer; Count: Integer); overload;
  begin
    if Count > 0 then begin
      Stream.WriteBuffer(S[Start], Count);
      LastChar := S[Start + Count - 1]
    end
  end;

  procedure WriteStr(const S: string);
  var
    I: Integer;
    Opened: Boolean;
  begin
    if S = '' then
      Write('''' + '''')
    else begin
      Opened := False;
      for I := 1 to Length(S) do
        case S[I] of
        '''':
          begin
            if Opened then begin
              Write('''');
              Opened := False
            end;
            Write('#' + IntToStr(Ord(S[I])))
          end;
        else
          if not Opened then begin
            if LastChar = '''' then
              Stream.Position := Stream.Position - 1
            else
              Write('''');
            Opened := True
          end;
          Write(S[I])
        end;
      if Opened then begin
        Write('''');
//        Opened := False
      end
    end
  end;

begin
  N := Stream.Size - Position;
  if N <= 0 then exit;
  SetLength(Code, N + 1);
  Code[N + 1] := #0;
  Stream.Position := Position;
  Stream.ReadBuffer(Code[1], N);
  Stream.Position := Position;

  LastChar := #0;
  SetLength(WS, 1);

  I := 1;
  while I <= N do
    case Code[I] of
    '#':
      begin
        CharCode := 0;
        J := I + 1;
        while Code[J] in ['0'..'9'] do begin
          CharCode := CharCode * 10 + (Ord(Code[J]) - Ord('0'));
          Inc(J)
        end;
        // J je nova hodnota pro I
        WS[1] := WideChar(SmallInt(CharCode));
        SS := WS;
        CharCode := Ord(SS[1]);

        if (CharCode >= Ord(' ')) and (CharCode <> Ord('''')) then begin
          if LastChar = '''' then
            Stream.Position := Stream.Position - 1
          else
            Write('''');

          Write(Char(CharCode));
          Write('''')
        end else
          Write('#' + IntToStr(Ord(CharCode)));
        I := J
      end;

    '''':
      begin
        J := I + 1;
        SS := '';
        while True do begin
          if Code[J] = '''' then begin
            Inc(J);
            if Code[J] <> '''' then
              break;
            SS := SS + ''''
          end else
            SS := SS + Code[J];
          Inc(J)
        end;
        // J je nova hodnota pro I

        // timhle zarucime ze nasledujici interpretace # bude umet spravne
        // navazat, protoze se eliminuje kodovani ''''
        WriteStr(SS);
        // Write(Code, I, J - I);

        I := J
      end;

    else
      Write(Code[I]);
      Inc(I)
    end;

  Stream.Size := Stream.Position
end;
{$ELSE}
procedure ObjectUtf8TextToAnsiText(Stream: TStream; Position: Integer);
begin
end;
{$ENDIF}

procedure ObjectBinaryToAnsiText(Input: TStream; Output: TStream);
var
  Position: Integer;
begin
  Position := Output.Position;
  ObjectBinaryToText(Input, Output);
  ObjectUtf8TextToAnsiText(Output, Position);
end;

procedure ObjectResourceToAnsiText(Input: TStream; Output: TStream);
var
  Position: Integer;
begin
  Position := Output.Position;
  ObjectResourceToText(Input, Output);
  ObjectUtf8TextToAnsiText(Output, Position);
end;

procedure ObjectTextToAnsiText(InputOutput: TStream);
begin
  ObjectUtf8TextToAnsiText(InputOutput, 0)
end;

end.
