unit cgiMain;

interface

uses
  Windows, SysUtils, Classes, HTTPApp, HttpProt, IniFiles;

type
  TRouter = class(TWebModule)
    procedure RouterDefActionAction(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
    procedure WebModuleCreate(Sender: TObject);
    procedure WebModuleBeforeDispatch(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  private
    Fields: TStrings;
  public
    { Public declarations }
  end;

var
  Router: TRouter;

implementation

{$R *.dfm}

function GetConfigFile: TCustomIniFile;
var
  Buffer: array[0..MAX_PATH] of Char;
  S: string;
begin
  SetString(S, Buffer, GetModuleFileName(HInstance,
    @Buffer, SizeOf(Buffer)));
  S := ChangeFileExt(S, '.ini');
  Result := TIniFile.Create(S)
end;

procedure TRouter.RouterDefActionAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  Http: THttpCli;
  URL: string;
  Item: string;
  Buf: TStream;
  Cfg: TCustomIniFile;

  function EncodeFields(Fields: TStrings): string;
  var
    I: Integer;
  begin
    Result := '';
    if (not Assigned(Fields)) or (Fields.Count = 0) then exit;
    for I := 0 to Fields.Count - 1 do begin
      if I > 0 then
        Result := Result + '&';
      Result := Result + Fields.Names[I] + '=' + HTTPEncode(Fields.ValueFromIndex[I])
    end;
    if Result <> '' then
      Result := '?' + Result
  end;

var
  Server: string;  
begin
  Cfg := GetConfigFile;
  try
    URL := Request.PathInfo;
    Delete(URL, 1, Length(Request.ScriptName));

    Item := ChangeFileExt(ExtractFileName(StringReplace(Request.ScriptName, '/', '\', [rfReplaceAll])), '');

    Server := Cfg.ReadString('Default', 'Server', 'http://127.0.0.1:8081/todolist');

    if (URL = '') or (URL = '/') then
      URL := Cfg.ReadString('Default', 'URL', '/todo/index.htm');

    if (URL <> '') and (URL[1] <> '/') then
      URL := '/' + URL;

    Server := Cfg.ReadString('Server', Item, Server);

    if (Server <> '') and (Server[Length(Server)] <> '/') then
      Server := Server + '/';

    URL := Server + Item + URL;

    Http := THttpCli.Create(Self);
    try
      Http.Agent := 'Todo';

      Buf := TMemoryStream.Create;
      try
        Http.RcvdStream := Buf;

        case Request.MethodType of
        mtGet:
          begin
            Http.URL := URL + EncodeFields(Fields);
            Http.Get;
          end;
        mtPut:
          begin
            Http.URL := URL;   // TODO: fields + content
            Http.Put;
          end;
        mtPost:
          begin
            Http.URL := URL;   // TODO: fields
            Http.Post;
          end;
        mtHead:
          begin
            Http.URL := URL;   // TODO: ?
            Http.Head;
          end;
        end;
        Buf.Position := 0;
        Response.Expires := Now;
        Response.SetCustomHeader('Pragma', 'no-cache');
        Response.SetCustomHeader('Cache-control', 'no-cache');
        Response.ContentStream := Buf;
      except
        Buf.Free;
        raise
      end
    finally
      Http.Free
    end
  finally
    Cfg.Free
  end
end;

procedure TRouter.WebModuleCreate(Sender: TObject);
begin
//  Http.LocalAddr := '127.0.0.1'
end;

procedure TRouter.WebModuleBeforeDispatch(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
begin
 case Request.MethodType of
 mtPost:
   Fields := Request.ContentFields;
 mtGet:
   Fields := Request.QueryFields;
 else
   Fields := nil
 end;
end;

end.
