{ VERSION: 1.2 }
{ 1.2 / 3.6.2006
  - AppVersion = 4
  - opraveno: ListPendingTasks() nefungovalo pro databzi Access, protoe v
    dotazu chyblo AS pro pejmenovn pol
  1.1 / 18.10.2005
}
unit uAppTodo;

interface
uses
  DB, Classes, uMyskaDB, uMyskaTables, uMyskaApp;

type
  TTodoApp = class(TMyskaApp)
  private
    FUserID: Cardinal;
    FUserName: string;
    function GetGroupAge(GroupID: Cardinal): Integer;
    procedure SetGroupAge(GroupID: Cardinal; Age: Integer);
    function GetGroupNumSeed(GroupID: Cardinal): Integer;
    procedure SetGroupNumSeed(GroupID: Cardinal; Seed: Integer);
  protected
    procedure SetID; override;
  public
    function CreateProviderUkolniky(AnOwner: TComponent; SystemFields: Boolean = False): TTableProvider;
    procedure DeleteUkolnik(GroupID: Cardinal; PreserveID: Boolean);
    property GroupAge[GroupID: Cardinal]: Integer read GetGroupAge write SetGroupAge;
    property GroupNumSeed[GroupID: Cardinal]: Integer read GetGroupNumSeed write SetGroupNumSeed;
    function Login(const UserName: string; const Password: string): Boolean;
    procedure Logout;
    function ListPendingTasks: TDataSet;
    property UserID: Cardinal read FUserID;
    property UserName: string read FUserName;
  end;

implementation
uses
  Variants, SysUtils;

procedure TTodoApp.SetID;
begin
  AppID := 'TODO';
  AppVersion := 4
end;

type
  TTaskGroupData = class(TTableProvider)
  protected
    FStore: TTodoApp;
    function GetName: string; override;
    procedure DoDelete; override;
  public
    constructor Create(AnOwner: TComponent; AStore: TTodoApp; AllFields: Boolean);
  end;

constructor TTaskGroupData.Create(AnOwner: TComponent; AStore: TTodoApp; AllFields: Boolean);
begin
  inherited Create(AnOwner);
  FStore := AStore;
  SetFactory(AStore, 'TaskGroup');
  if AllFields then
    Query[qi_Open] := 'select * from TaskGroup'
  else
    Query[qi_Open] := 'select GroupID, Description from TaskGroup';
end;

procedure TTaskGroupData.DoDelete;
begin
  FStore.DeleteUkolnik(DataSet[KeyField], False)
end;

function TTaskGroupData.GetName: string;
begin
  Result := 'kolnk'
end;

procedure TTodoApp.DeleteUkolnik(GroupID: Cardinal; PreserveID: Boolean);
begin
  Store.Exec('delete from Task where GroupID=%d', [so_Command], [GroupID]);
  Store.Exec('delete from TaskNote where GroupID=%d', [so_Command], [GroupID]);
  if not PreserveID then
    Store.Exec('delete from TaskGroup where GroupID=%d', [so_Command], [GroupID]);
end;

function TTodoApp.CreateProviderUkolniky(AnOwner: TComponent; SystemFields: Boolean): TTableProvider;
begin
  Result := TTaskGroupData.Create(AnOwner, Self, SystemFields);
end;

function TTodoApp.ListPendingTasks: TDataSet;
begin
  Result := Store.Exec(
    'select' +
    '  t.TaskID as TaskID,' +
    '  g.Description as Description,' +
    '  t.Num as Num,' +
    '  t.Notes as Notes,' +
    '  t.Activity as Activity,' +
    '  t.Priority as Priority,' +
    '  t.Category as Category,' +
    '  t.Created as Created,' +
    '  t.Status as Status,' +
    '  t.SolverID as SolverID,' +
    '  (select count(NoteID) from TaskNote n where n.TaskID = t.TaskID) as NotesCount ' +
    'from Task t,TaskGroup g ' +
    'where' +
    '  (t.GroupID = g.GroupID) and' +
    '  (not (t.Activity in (6,7,8))) and' +
    '  (t.SolverID is not null) and (t.SolverID <> 0)',
    [so_ReadOnly]
  );
end;

function TTodoApp.GetGroupAge(GroupID: Cardinal): Integer;
var
  Value: Variant;
begin
  Value := Store.Select('select Modified from TaskGroup where GroupID=%d', [GroupID]);
  if VarIsEmpty(Value) or VarIsNull(Value) then
    Result := -1
  else
    Result := DateTimeToFileDate(Value)
end;

procedure TTodoApp.SetGroupAge(GroupID: Cardinal; Age: Integer);
var
  D: TDataSet;
  F: TField;
begin
  D := Store.Exec('select * from TaskGroup where GroupID=%d', [], [GroupID]);
  try
    F := D.FieldByName('Modified');
    D.Edit;
    if Age = -1 then
      F.Clear
    else
      F.AsDateTime := FileDateToDateTime(Age);
    D.Post
  finally
    D.Free
  end
end;

function TTodoApp.GetGroupNumSeed(GroupID: Cardinal): Integer;
var
  Value: Variant;
begin
  Value := Store.Select('select NumSeed from TaskGroup where GroupID=%d', [GroupID]);
  if VarIsEmpty(Value) or VarIsNull(Value) then
    Result := 0
  else
    Result := Value
end;

procedure TTodoApp.SetGroupNumSeed(GroupID: Cardinal; Seed: Integer);
begin
  Store.Exec('update TaskGroup set NumSeed=%1:d where GroupID=%0:d', [so_Command], [GroupID, Seed])
end;

function TTodoApp.Login(const UserName: string; const Password: string): Boolean;
begin
  Result := Store.Users.Login(UserName, Password, FUserID);
  if Result then
    FUserName := Store.Users.Find(FUserID).Name
end;

procedure TTodoApp.Logout;
begin
  FUserID := 0;
  FUserName := ''
end;

end.
