{ VERSION: 1.1 }
{ 1.1 / 8.10.2005
  1.0 / 2.2.2005
}
unit uTodoLists;

interface
uses
  uTodoList, Classes;

type
  TTodoBook = class;
  TTodoBookEntry = class
  private
    FBook: TTodoBook;
    FFileName: string;
    FProjectName: string;
    FEnabled: Boolean;
    FIsShared: Boolean;
    procedure SetEnabled(Value: Boolean);
    procedure SetFileName(const Value: string);
  protected
    property Book: TTodoBook read FBook;
  public
    constructor Create(ABook: TTodoBook);
    property FileName: string read FFileName write SetFileName;
    property ProjectName: string read FProjectName;
    property IsShared: Boolean read FIsShared;
    property Enabled: Boolean read FEnabled write SetEnabled;
  end;
  TTodoBook = class
  private
    FList: TList;
    function GetCount: Integer;
    function GetItem(Index: Integer): TTodoBookEntry;
    function GetText: string;
    procedure SetText(const Value: string);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Delete(Item: TTodoBookEntry);
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TTodoBookEntry read GetItem; default;
  public
    function FindProject(const ProjectName: string): TTodoBookEntry;
    function IndexOf(Item: TTodoBookEntry): Integer; overload;
  public
    function FileExists(const FileName: string): Boolean; virtual;
    function FileAge(const FileName: string): Integer; virtual;
  public
    // TStrings compatibility
    function IndexOf(const FileName: string): Integer; overload;
    procedure Move(CurIndex, NewIndex: Integer);
    procedure Insert(Index: Integer; const FileName: string);
    function Add(const FileName: string): Integer;
    property Text: string read GetText write SetText;
  end;

implementation
uses
  Windows, SysUtils;

constructor TTodoBook.Create;
begin
  inherited Create;
  FList := TList.Create
end;

function TTodoBook.GetCount: Integer;
begin
  Result := FList.Count
end;

function TTodoBook.GetItem(Index: Integer): TTodoBookEntry;
begin
  Result := TTodoBookEntry(FList[Index])
end;

procedure TTodoBook.Clear;
var
  I: Integer;
begin
  for I := Count - 1 downto 0 do
    Items[I].Free;
  FList.Clear
end;

procedure TTodoBook.Delete(Item: TTodoBookEntry);
var
  I: Integer;
begin
  I := FList.IndexOf(Item);
  if I >= 0 then begin
    FList.Delete(I);
    Item.Free
  end
end;

destructor TTodoBook.Destroy;
begin
  Clear;
  FList.Free;
  inherited;
end;

function TTodoBook.FindProject(const ProjectName: string): TTodoBookEntry;
var
  I: Integer;
begin
  for I := Count - 1 downto 0 do begin
    Result := Items[I];
    if Result.Enabled and SameText(Result.ProjectName, ProjectName) then
      exit
  end;
  Result := nil
end;

function TTodoBook.IndexOf(const FileName: string): Integer;
var
  I: Integer;
begin
  for I := Count - 1 downto 0 do
    if SameText(Items[I].FileName, FileName) then begin
      Result := I;
      exit
    end;
  Result := -1
end;

function TTodoBook.IndexOf(Item: TTodoBookEntry): Integer;
begin
  Result := FList.IndexOf(Item)
end;

procedure TTodoBook.Move(CurIndex, NewIndex: Integer);
begin
  FList.Move(CurIndex, NewIndex);
end;

procedure TTodoBook.Insert(Index: Integer; const FileName: string);
var
  Item: TTodoBookEntry;
begin
  Item := TTodoBookEntry.Create(Self);
  Item.FileName := FileName;
  FList.Insert(Index, Item);
end;

function TTodoBook.Add(const FileName: string): Integer;
var
  Item: TTodoBookEntry;
begin
  Item := TTodoBookEntry.Create(Self);
  Item.FileName := FileName;
  Result := FList.Add(Item)
end;

function TTodoBook.GetText: string;
var
  I: Integer;
begin
  Result := '';
  for I := 0 to Count - 1 do begin
    if Result <> '' then
      Result := Result + #13#10;
    Result := Result + Items[I].FileName
  end
end;

procedure TTodoBook.SetText(const Value: string);
var
  L: TStrings;
  I: Integer;
begin
  Clear;
  L := TStringList.Create;
  try
    L.Text := Value;
    for I := 0 to L.Count - 1 do
      Add(L[I])
  finally
    L.Free
  end
end;

function TTodoBook.FileExists(const FileName: string): Boolean;
begin
  Result := SysUtils.FileExists(FileName)
end;

function TTodoBook.FileAge(const FileName: string): Integer;
begin
  Result := SysUtils.FileAge(FileName)
end;

//--------------------------------------------------------------

constructor TTodoBookEntry.Create(ABook: TTodoBook);
begin
  inherited Create;
  FBook := ABook
end;

procedure TTodoBookEntry.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
    FEnabled := Book.FileExists(FileName) or Book.FileExists(FileName + '.todolist')
end;

procedure TTodoBookEntry.SetFileName(const Value: string);
begin
  FFileName := Value;
  FIsShared := (Copy(FileName, 1, 2) = '\\') or (GetDriveType(PChar(ExtractFileDrive(FileName))) = DRIVE_REMOTE);
  FProjectName := ChangeFileExt(ExtractFileName(FileName), '');
  Enabled := True        // trigger fileExists() detector
end;

end.
