{ VERSION: 1.3 }
{ 1.3 / 4.6.2006
  - podporuje strnky pro projekt typu 'singleton', tj. neexistuje dn platn
    HomeLinkFormat 
  1.2 / 3.6.2006
  - podporuje nov koncov stav 'featureDuplicate'
  - pedpokld azen aktulnch kol kter dv na prvn msto ty co nejsou ekajc
    a oddluje je opticky od ekajcch
}

unit uWebUI;

interface
uses
  Windows, Classes, SysUtils, DB, uTodoList, uTodoLists, IniFilesD5, CfgArray;

type
  TPageStyle = set of (
    psFixedFontSize,
    psProjectList,
    psBlankProject,
    psUsers
  );
  ERedirect = class(Exception)
  end;
  TPageGenerator = class(TComponent)
  private
    FStyle: TPageStyle;
    FCurrentProject: string;
    FTodo: TTodoList;
    FTodos: TDataSet;
    FTodoNotes: TDataSet;
    FUserName: string;
    procedure SetUserName(const Value: string);
  protected
    TodoList: TDataNavigator;
    CompleteList: TDataNavigator;
    HistoryList: TDataNavigator;
  private
    procedure _SetCurrentProject(const Value: string);
  protected
    procedure SetCurrentProject(const Value: string; const User: string; const Password: string);
    function SelectProject(const Value: string; const User: string; const Password: string): string; virtual; abstract;
  public
    function RootDefaultPage: string; virtual; abstract;
    function CurrentFile: string; virtual; abstract;
    function DefaultPage: string; virtual; abstract;
    function FontSize: Integer; virtual; abstract;
    function ConfigFiles: TCfgArrayReader; virtual; abstract;
    function MRU: TTodoBook; virtual; abstract;
  protected
    CategoryEnum,
    StatusEnum,
    ActivityEnum,
    PriorityEnum: TEnum;
    CategoryDescriptions,
    StatusDescriptions,
    ActivityDescriptions,
    PriorityDescriptions: string;
    NullCategory,
    NullStatus,
    NullActivity,
    NullPriority: string;
  private
    function GetNewRecordCategory: string;
    procedure SetNewRecordCategory(const Value: string);
    function GetNewRecordModule: string;
    procedure SetNewRecordModule(const Value: string);
    function GetNewRecordPriority: string;
    procedure SetNewRecordPriority(const Value: string);
  private
    property NewRecordCategory: string read GetNewRecordCategory write SetNewRecordCategory;
    property NewRecordModule: string read GetNewRecordModule write SetNewRecordModule;
    property NewRecordPriority: string read GetNewRecordPriority write SetNewRecordPriority;
  public
    constructor Create(AnOwner: TComponent); override;
    function GetPage(PathItems: TStrings; const Path: string; QueryFields: TStrings; const User: string = ''; const Password: string = ''): string;
  private
    FLinkBase: string;
    FHomeLinkFormat: string;
    ProjectID: string;
    function HomeDefaultPage: string;
    function FormatHomeDefaultPage(const Fmt: string): string;
    function TodoLinker(var Link: string): string;
  public
    fldTodos_ID: string;
    fldTodos_Text: string;
    fldTodoNotes_ID: string;
    fldTodoNotes_Text: string;
  private
    function FieldNameToIDName(const FieldName: string): string;
    function GetUserName(Data: TDataSet; const FieldName: string; const FormatString: string = '%s'): string;
    procedure AssignUser(Field: TField; const UserID: string);
  protected
    procedure GetUserNames(List: TStrings; var Indexed: Boolean); virtual;
    function TranslateUserName(Field: TField): string; virtual;
    function ProjectServer: Boolean; virtual;
    function ListPendingTasks(AnOwner: TComponent): TDataNavigator; virtual;
  public
    property Todos: TDataSet read FTodos write FTodos;
    property TodoNotes: TDataSet read FTodoNotes write FTodoNotes;
    property Todo: TTodoList read FTodo write FTodo;
    property CurrentProject: string read FCurrentProject write _SetCurrentProject;
    property Style: TPageStyle read FStyle write FStyle;
    property HomeLinkFormat: string read FHomeLinkFormat write FHomeLinkFormat;
    property UserName: string read FUserName write SetUserName;
  end;

type
  TRefLinker = function (var Link: string): string of object;

function WebName(const Name: string): string;
function UnWebName(const Name: string): string;
function FileText(const FileName: string): string;
function WikiPage(const Text: string): string;
function HTMLText(const AText: string; Formated: Boolean = False; EnableTags: Boolean = False; const TodoLinker: TRefLinker = nil): string;
function ShortHTMLText(const AText: string; Formated: Boolean = False; EnableTags: Boolean = False; ForceDots: Boolean = False; const TodoLinker: TRefLinker = nil): string;

implementation
uses
  Graphics, GraphUtil, Variants, HTTPApp, DateUtils, fileUtils, DXResource;

function FileText(const FileName: string): string;
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone);
  try
    SetLength(Result, Stream.Size);
    if Result <> '' then
      Stream.ReadBuffer(Result[1], Length(Result))
  finally
    Stream.Free
  end
end;

function WebName(const Name: string): string;
begin
  Result := HTTPEncode(Name)
end;

function UnWebName(const Name: string): string;
begin
  Result := HTTPDecode(Name)
end;

function WikiPage(const Text: string): string;
var
  Lines: TStrings;
  I: Integer;
  Level: Integer;
  S: string;
  N: Integer;

  function SetLevel(var S: string; NewLevel: Integer): Boolean;
  begin
    if Level = NewLevel then begin
      Result := False;
      exit;
    end;
    while Level > NewLevel do begin
      S := '</ul>' + S;
      Dec(Level)
    end;
    while Level < NewLevel do begin
      S := '<ul>' + S;
      Inc(Level)
    end;
    Result := True
  end;

  function CountOfChar(Ch: Char; const Text: string; Start: Integer = 1): Integer;
  var
    I: Integer;
  begin
    Result := 0;
    for I := Start to Length(Text) do
      if Text[I] = Ch then
        Inc(Result)
      else
        break
  end;

label
  noList, listItem;
begin
  // TODO: support other wiki formaters
  if (Pos('*', Text) = 0) and (Pos('-', Text) = 0) then begin
    Result := Text;
    exit
  end;
  Lines := TStringList.Create;
  try
    Lines.Text := Text;

    Level := 0;
    for I := 0 to Lines.Count - 1 do begin
      S := Lines[I];
      if S <> '' then begin
        if S[1] = '*' then begin
          N := CountOfChar('*', S);
          Delete(S, 1, N);
        listItem:
          S := '<li>' + Trim(S){ TODO: + '</li>'};
          SetLevel(S, N);
          Lines[I] := S
        end else begin
          N := CountOfChar(' ', S);
          if Copy(S, 1 + N, 1) = '-' then begin
            Inc(N, CountOfChar('-', S, 1 + N));
            Delete(S, 1, N);
            S := TrimLeft(S);
            goto listItem
          end else
            goto noList
        end
      end else begin
        noList:
        if SetLevel(S, 0) then
          Lines[I] := S
      end;
    end;

    Result := TrimRight(Lines.Text)
  finally
    Lines.Free
  end
end;

function HTMLText(const AText: string; Formated: Boolean; EnableTags: Boolean; const TodoLinker: TRefLinker): string;

  function MakeHREF(const Source: string): string;
  const
    HttpPrefix = 'http://';
    HttpStuffPrefix = '<a href="';
    HttpStuffPostfix = '" target=_new>';
    HttpStuffTerminator = '</a>';
    TodoPrefix = 'todo:';
  var
    I: Integer;
    L: Integer;
    J: Integer;
    Stuff: Integer;
    Link: string;
    LinkName: string;

    procedure DoInsert(const Text: string; Position: Integer);
    begin
      Insert(Text, Result, Position + Stuff);
      Inc(Stuff, Length(Text));
    end;

    procedure DoDelete(Position: Integer; Count: Integer);
    begin
      Delete(Result, Position + Stuff, Count);
      Dec(Stuff, Count);
    end;

  begin
    Result := Source;
    Stuff := 0;
    I := 1;
    while I <= Length(Source) do begin
      if StrLComp(@Source[I], HttpPrefix, Length(HttpPrefix)) = 0 then begin
        if (I = 1) or ((I > 1) and (Source[I - 1] in [' ', #13, #10, '>'])) then begin
          L := 0;
          J := I;
          while (J <= Length(Source)) and (not (Source[J] in [' ', '"', #13, #10])) do begin
            Inc(J);
            Inc(L)
          end;

          DoInsert(HttpStuffPrefix, I);
          DoInsert(HttpStuffPostfix, J);
          DoInsert(Copy(Source, I, L), J);
          DoInsert(HttpStuffTerminator, J);

          I := J;
        end else
          Inc(I)
      end else
      if Assigned(TodoLinker) and (StrLComp(@Source[I], TodoPrefix, Length(TodoPrefix)) = 0) then begin
        if (I = 1) or ((I > 1) and (Source[I - 1] in [' ', #13, #10, '>'])) then begin
          L := 0;
          J := I;
          while (J <= Length(Source)) and (not (Source[J] in [' ', '"', #13, #10])) do begin
            Inc(J);
            Inc(L)
          end;

          LinkName := Copy(Copy(Source, I, L), Length(TodoPrefix) + 1, MaxInt);
          Link := TodoLinker(LinkName);
          DoInsert(Format('<a href="%1:s">%0:s</a>', [LinkName, Link]), I);
          DoDelete(I, L);

          I := J;
        end else
          Inc(I)
      end else
        Inc(I)
    end
  end;

const
  lt = #1;
  gt = #2;

  procedure ReplaceTags(var Text: string; const Tags: array of string);
  var
    I: Integer;
    Tag: string;
  begin
    for I := Low(Tags) to High(Tags) do begin
      Tag := Tags[I];
      Text := StringReplace(Text, '<' + Tag + '>', lt + Tag + gt, [rfReplaceAll, rfIgnoreCase]);
      Text := StringReplace(Text, '</' + Tag + '>', lt + '/' + Tag + gt, [rfReplaceAll, rfIgnoreCase]);
    end
  end;

var
  I: Integer;
  Text: string;
begin
  Text := AText;

  if EnableTags then begin
    if not Formated then
      Text := WikiPage(Text);
    ReplaceTags(Text, ['b', 'i', 'li', 'ol', 'ul']);
  end;

  Result := '';
  for I := 1 to Length(Text) do
    case Text[I] of
    #13:
      ;
    #10:
      if Formated then
        Result := Result + #13#10
      else
        Result := Result + '<br>';
    '<':
      Result := Result + '&lt;';
    '>':
      Result := Result + '&gt;';
    '&':
      Result := Result + '&amp;';
    ' ':
      if Formated then
        Result := Result + ' '
      else
        Result := Result + ' '; // TODO: &nbsp;';
    else
      Result := Result + Text[I]
    end;

  if EnableTags then begin
    Result := StringReplace(StringReplace(Result, lt, '<', [rfReplaceAll]), gt, '>', [rfReplaceAll]);
    Result := MakeHREF(Result)
  end;
end;

function ShortHTMLText(const AText: string; Formated: Boolean; EnableTags: Boolean; ForceDots: Boolean; const TodoLinker: TRefLinker): string;
var
  I: Integer;
  S: string;
  Dots: Integer;
begin
  Dots := 0;
  S := Trim(AText);
  I := Pos(#13#10, S);
  if (I = 0) and ForceDots then
    I := Length(S) + 1;
  if I <> 0 then begin
    S := Copy(S, 1, I - 1);
    I := Length(S);
    while (I >= 1) and (S[I] = '.') do begin
      Dec(I);
      Inc(Dots)
    end;
    Dots := 3 - Dots;
  end;
  if Dots < 0 then
    Dots := 0;
  Result := HTMLText(S, Formated, EnableTags, TodoLinker) + StringOfChar('.', Dots)
end;

//---------------------------------

function TPageGenerator.HomeDefaultPage: string;
begin
  if psProjectList in Style then
    Result := DefaultPage
  else if HomeLinkFormat = '' then
    Result := ''
  else
    Result := Format(HomeLinkFormat, [ProjectID])
end;

function TPageGenerator.FormatHomeDefaultPage(const Fmt: string): string;
var
  Page: string;
begin
  Page := HomeDefaultPage;
  if Page = '' then
    Result := ''
  else
    Result := Format(Fmt, [Page])
end;

procedure TPageGenerator._SetCurrentProject(const Value: string);
begin
  SetCurrentProject(Value, '', '')
end;

procedure TPageGenerator.SetCurrentProject(const Value: string; const User: string; const Password: string);
begin
  FCurrentProject := SelectProject(Value, User, Password)
end;

function TPageGenerator.GetNewRecordCategory: string;
begin
  Result := Todo.GetParam('Category', NullCategory)
end;

procedure TPageGenerator.SetNewRecordCategory(const Value: string);
begin
  Todo.SetParam('Category', Value);
end;

function TPageGenerator.GetNewRecordModule: string;
begin
  Result := Todo.GetParam('Module', '')
end;

procedure TPageGenerator.SetNewRecordModule(const Value: string);
begin
  Todo.SetParam('Module', Value);
end;

function TPageGenerator.GetNewRecordPriority: string;
begin
  Result := Todo.GetParam('Priority', NullPriority)
end;

procedure TPageGenerator.SetNewRecordPriority(const Value: string);
begin
  Todo.SetParam('Priority', Value);
end;

constructor TPageGenerator.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  Style := [psFixedFontSize, psProjectList];
  fldTodos_ID := 'ID';
  fldTodos_Text := 'Note';
  fldTodoNotes_ID := 'ID';
  fldTodoNotes_Text := 'Note';
end;

function TPageGenerator.FieldNameToIDName(const FieldName: string): string;
begin
  if SameText(FieldName, fldTodos_ID) then
    Result := 'ID'
  else
  if SameText(FieldName, fldTodos_Text) then
    Result := 'Note'
  else
    Result := FieldName
end;

procedure TPageGenerator.SetUserName(const Value: string);
begin
  FUserName := Value;
  if Value = '' then
    Style := Style - [psUsers]
  else
    Style := Style + [psUsers]
end;

procedure TPageGenerator.GetUserNames(List: TStrings; var Indexed: Boolean);
begin
  Indexed := False
end;

function TPageGenerator.TranslateUserName(Field: TField): string;
begin
  Result := Field.AsString
end;

function TPageGenerator.ProjectServer: Boolean;
begin
  Result := False
end;

function TPageGenerator.ListPendingTasks(AnOwner: TComponent): TDataNavigator;
begin
  Result := nil
end;

procedure TPageGenerator.AssignUser(Field: TField; const UserID: string);
var
  List: TStringList;
  Indexed: Boolean;
  ID: Integer;
begin
  if (UserID = '') or (UserID = '0') then
    Field.Clear
  else begin
    List := TStringList.Create;
    try
      GetUserNames(List, Indexed);
      if Indexed then begin
        ID := StrToIntDef(UserID, 0);
        if List.IndexOfObject(TObject(ID)) >= 0 then
          Field.AsInteger := ID
        else
          Field.Clear
      end else begin
        List.Sorted := True;
        if List.IndexOf(UserID) >= 0 then
          Field.AsString := UserID
        else
          Field.Clear
      end
    finally
      List.Free
    end
  end
end;

function TPageGenerator.GetUserName(Data: TDataSet; const FieldName: string; const FormatString: string): string;
var
  F: TField;
begin
  Result := '';
  if Assigned(Data) then begin
    F := Data.FindField(FieldName);
    if Assigned(F) then begin
      if not F.IsNull then
        Result := TranslateUserName(F);
      if Result <> '' then
        Result := Format(FormatString, [Result])
    end;
  end
end;

function TPageGenerator.TodoLinker(var Link: string): string;
var
  Project: string;
  Task: string;
  I: Integer;
begin
  I := Pos('/', Link);
  if I = 0 then begin
    Project := '';
    Task := Link
  end else begin
    Project := WebName(Copy(Link, 1, I - 1));
    Task := Copy(Link, I + 1, MaxInt)
  end;
  if Project = '' then begin
    Result := FLinkBase + Task + '/index.htm';
  end else begin
    if Task = '' then
      Result := FLinkBase + '../../' + Project + '/todo/index.htm'
    else
      Result := FLinkBase + '../../' + Project + '/todo/' + Task + '/index.htm'
  end;
  Link := 'todo:' + Link
end;

function TPageGenerator.GetPage(PathItems: TStrings; const Path: string; QueryFields: TStrings; const User: string; const Password: string): string;
const
  HR = '<hr width=100% noshade size=1>';
var
  Buf: string;

  procedure Write(const Msg: string); overload;
  begin
    Buf := Buf + Msg
  end;

  procedure Write(const Fmt: string; const Args: array of const); overload;
  begin
    Write(Format(Fmt, Args))
  end;

  function Item(Index: Integer): string;
  begin
    if (Index < 0) or (Index >= PathItems.Count) then
      Result := ''
    else
      Result := PathItems[Index]
  end;

  procedure Redirect(const URL: string); overload;
  begin
    raise ERedirect.Create(URL);
    {
    Buf := '';
    Write('<html><head><META HTTP-EQUIV=Refresh CONTENT="0; URL=%s"></head>', [URL]);
    Write('<body><a href="%s">redirect</a>', [URL])
    }
  end;

{  procedure Redirect(const Fmt: string; const Args: array of const); overload;
  begin
    Redirect(Format(Fmt, Args))
  end; }

  function SmartDateToStr(D: TDateTime): string;
  begin
    { TODO:
    if SameDate(D, Date) then
      Result := 'dnes'
    else
    if SameDate(D, Date - 1) then
      Result := 'vera'
    else
    if SameDate(D, Date - 2) then
      Result := 'pedevrem'
    else
    }
      Result := DateToStr(D)
  end;

  function FieldToDate(F: TField): string; overload;
  begin
    if (F = nil) or F.IsNull then
      Result := ''
    else
      Result := SmartDateToStr(F.AsDateTime)
  end;

  function SmartDateTimeToStr(D: TDateTime): string;
  begin
//    Result := DateTimeToStr(D);
    Result := FormatDateTime('d.m.yyyy hh:nn', D);
    Result := StringReplace(Result, ' ', '&nbsp;', [rfReplaceAll])
  end;

  function FieldToDate(const FieldName: string; Data: TDataSet = nil): string; overload;
  begin
    if Data = nil then Data := Todos;
    Result := FieldToDate(Data.FindField(FieldName))
  end;

  function FieldToDateTime(F: TField): string; overload;
  begin
    if (F = nil) or F.IsNull then
      Result := ''
    else
      Result := SmartDateTimeToStr(F.AsDateTime)
  end;

  function FieldToDateTime(const FieldName: string; Data: TDataSet = nil): string; overload;
  begin
    if Data = nil then Data := Todos;
    Result := FieldToDateTime(Data.FindField(FieldName))
  end;

  function FileAge(const FileName: string): Integer;
  begin
    if SameText(FileName, CurrentFile + '.todolist') and Assigned(Todo) and Todo.Modified then
      Result := FileAgeNow
    else
      Result := MRU.FileAge(FileName)
  end;

  function FileToDate(const FileName: string): string;
  var
    Age: Integer;
  begin
    Age := FileAge(FileName);
    if Age = -1 then
      Result := ''
    else
      Result := SmartDateToStr(FileDateToDateTime(Age))
  end;

  function FileToDateTime(const FileName: string): string;
  var
    Age: Integer;
  begin
    Age := FileAge(FileName);
    if Age = -1 then
      Result := ''
    else
      Result := SmartDateTimeToStr(FileDateToDateTime(Age))
  end;

  const
    DateTimeSortKeyLength = 14;
  function FileToDateTimeSortKey(const FileName: string): string;
  var
    Age: Integer;
  begin
    Age := FileAge(FileName);
    if Age = -1 then
      Result := StringOfChar('0', DateTimeSortKeyLength)
    else
      Result := FormatDateTime('yyyymmddhhnnss', FileDateToDateTime(Age))
  end;

  function DateDiff(A, B: TDateTime): string; overload;
  var
    N: Integer;
  begin
    N := DaysBetween(A, B);
    { TODO: ?
    case N of
    0: Result := 'hned';
    1: Result := '1 den';
    2,3,4: Result := Format('%d dny', [N]);
    else
      Result := Format('%d dn', [N])
    end;
    }
    Result := IntToStr(N)
  end;

  function DateDiff(const FieldA: string; const FieldB: string): string; overload;
  begin
    Result := DateDiff(Todos.FieldByName(FieldA).AsDateTime, Todos.FieldByName(FieldB).AsDateTime)
  end;

  const
    Yes = #1;
    No = '';

  procedure GenToolBar(const Names: array of string; const URLs: array of string);
  var
    I: Integer;
    State: (sDisabled, sActive, sLink);

    procedure GenButtonNew(const LinkPrefix: string);

      function Img(const Category: string): string;
      begin
        case State of
        sDisabled:
          Result := Format('<img src="/images/disabled.%0:s.gif" border=0 width=16 height=16 hspace=2>', [Category]);
        sActive:
          Result := Format('<img src="/images/%0:s.gif" border=0 width=16 height=16 hspace=2>', [Category]);
        sLink:
          Result := Format('<a href="%1:s0/%0:s.htm"><img src="/images/%0:s.gif" border=0 width=16 height=16 hspace=2></a>', [Category, LinkPrefix]);
        end
      end;

    begin
{
      case State of
      sDisabled:
        Write('<font color=#C0C0C0>Nov</font>%s%s%s%s', [Img('bug'), Img('task'), Img('feature'), Img('question')]);
      sActive:
        Write('Nov%s%s%s%s', [Img('bug'), Img('task'), Img('feature'), Img('question')]);
      sLink:
        Write('Nov%s%s%s%s', [Img('bug'), Img('task'), Img('feature'), Img('question')]);
      end
}
      Write('%s%s%s%s', [Img('bug'), Img('task'), Img('feature'), Img('question')]);
    end;

  begin
    for I := Low(Names) to High(Names) do begin
      if I > Low(Names) then
        Write(' | ');

      if (I > High(URLs)) or (URLs[I] = '') then
        State := sDisabled
      else
      if URLs[I] = '.' then
        State := sActive
      else
        State := sLink;

      if Names[I] = '{new}' then
        GenButtonNew(Copy(URLs[I], 1, Length(URLs[I]) - 1))
      else begin
        case State of
        sDisabled:
          Write('<font color=#C0C0C0>%s</font>', [HTMLText(Names[I])]);
        sActive:
          Write(HTMLText(Names[I]));
        sLink:
          Write('<a href="%s">%s</a>', [URLs[I], HTMLText(Names[I])]);
        end
      end
    end;
    if (High(Names) - Low(Names) + 1) = 0 then
      Write('&nbsp;');
    Write(HR)
  end;

  procedure _GenNavigator(const Names: array of string; const URLs: array of string);
  var
    I: Integer;
    BackURL: string;

    procedure Put(const Name: string; const URL: string; Disabled: Boolean = False);
    begin
      if URL = '' then begin
        if Disabled then
          Write('<font color=#C0C0C0>%s</font>', [HTMLText(Name)])
        else
          Write(HTMLText(Name))
      end else
        Write('<a href="%s">%s</a>', [URL, HTMLText(Name)])
    end;

  var
    Max: Integer;
  begin
    Max := High(Names);
    while (Max >= Low(Names)) and (Names[Max] = '') do
      Dec(Max);

    BackURL := '';
    for I := Max downto Low(URLs) do
      if URLs[I] <> '' then begin
        BackURL := URLs[I];
        break
      end;

    Put('<<', BackURL, True);
    Write(' | ');
    for I := Low(Names) to Max do begin
      if I > Low(Names) then
        Write(' \ ');
      Put(Names[I], URLs[I]);
    end;
    if UserName = '' then
      Write(HR)
    else
      Write(
        '<table width=100%% border=0 cellspacing=0 cellpadding=0><tr><td width=100%%>%s</td><td align=right>%s</td></tr></table>',
// TUNE:        [HR, StringReplace(Format(' Uivatel: %s', [UserName]), ' ', '&nbsp;', [rfReplaceAll])]
        [HR, StringReplace(Format('[ %s ]', [UserName]), ' ', '&nbsp;', [rfReplaceAll])]
      )
  end;

  procedure GenNavigator(const Names: array of string; const URLs: array of string; const TaskRoot: string);
  var
    NewNames: array of string;
    NewURLs: array of string;

    function Max(A, B: Integer): Integer;
    begin
      if A > B then
        Result := A
      else
        Result := B
    end;

  var
    I: Integer;
  begin
    FLinkBase := TaskRoot;
    if psProjectList in Style then
      _GenNavigator(Names, URLs)
    else begin
      SetLength(NewNames, Max(Length(Names) - 1, 0));
      SetLength(NewURls, Length(NewNames));
      if Length(NewNames) > 0 then begin
        NewNames[0] := Names[1];
        NewURLs[0] := URLs[0]
      end;
      for I := 1 to Length(NewNames) - 1 do begin
        NewNames[I] := Names[I + 1];
        NewURLs[I] := URLs[I + 1]
      end;
      _GenNavigator(NewNames, NewURLs)
    end
  end;

  function IsEnum(Field: TField; Enum: TEnum; const DefaultValue: string; const EnumValue: string): Boolean;
  var
    Value: string;
  begin
    if Field.IsNull then
      Value := DefaultValue
    else
      Value := Enum.IDToName(Field.AsInteger, DefaultValue);
    Result := SameText(Value, EnumValue)
  end;

  procedure GenEnum(Field: TField; Enum: TEnum; const Descriptions: string; const Selection: string; const DefaultValue: string; const Name: string);
  var
    Value: string;
    Labels: TStrings;
    I: Integer;
    ID: string;
    List: TStringList;
  begin
    if (not Assigned(Field)) or Field.IsNull then
      Value := DefaultValue
    else
      Value := Enum.IDToName(Field.AsInteger, DefaultValue);

    Labels := TStringList.Create;
    try
      Labels.Text := StringReplace(Descriptions, ',', #13#10, [rfReplaceAll]);

      if Selection <> '' then begin
        List := TStringList.Create;
        try
          List.CommaText := Selection;
          List.Sorted := True;

          { intersect descriptions }
          I := 0;
          while I < Labels.Count do
            if List.IndexOf(Labels.Names[I]) >= 0 then
              Inc(I)
            else
              Labels.Delete(I);

          { intersect selected value }
          if List.IndexOf(Value) < 0 then
            Value := DefaultValue
        finally
          List.Free
        end;
      end;

      if Name = '' then
        Write(HTMLText(Labels.Values[Value]))
      else begin
        if SameText(Selection, Value) then begin
          // jednohodnotove moznosti
          Write('<input type=hidden name=%s value=%d>', [Name, Enum.NameToID(Value, 0)]);
          Write(HTMLText(Labels.Values[Value]))
        end else begin
          Write('<select name=%s>', [Name]);
          for I := 0 to Labels.Count - 1 do begin
            ID := Labels.Names[I];
            if SameText(ID, Value) then
              Write('<option value=%d selected>', [Enum.NameToID(ID, 0)])
            else
              Write('<option value=%d>', [Enum.NameToID(ID, 0)]);
            Write(HTMLText(Labels.Values[ID]));
            Write('</option>')
          end;
          Write('</select>')
        end
      end
    finally
      Labels.Free
    end
  end;

  procedure GenUsersEnum(Field: TField; const Name: string);
  var
    List: TStringList;
    Value: string;
    Indexed: Boolean;
    I: Integer;

    function ListID(Index: Integer): string;
    begin
      if Indexed then
        Result := IntToStr(Integer(List.Objects[Index]))
      else
        Result := List[Index]
    end;

  begin
    if (not Assigned(Field)) or Field.IsNull then
      Value := ''
    else
      Value := TranslateUserName(Field);

    List := TStringList.Create;
    try
      GetUserNames(List, Indexed);
      List.Sort;
      List.Insert(0, '');

      if Name = '' then
        Write(HTMLText(Value))
      else begin
        Write('<select name=%s>', [Name]);
        for I := 0 to List.Count - 1 do begin
          if SameText(List[I], Value) then
            Write('<option value="%s" selected>', [ListID(I)])
          else
            Write('<option value="%s">', [ListID(I)]);
          Write(HTMLText(List[I]));
          Write('</option>')
        end;
        Write('</select>')
      end
    finally
      List.Free
    end;
  end;

  function CurrentCategory(Data: TDataSet = nil): string;
  var
    F: TField;
  begin
    if Data = nil then
      Data := Todos;
    F := Data.FieldByName('Category');
    if F.IsNull then
      Result := NullCategory
    else
      Result := CategoryEnum.IDToName(F.AsInteger, NullCategory)
  end;

  function CurrentActivity(Data: TDataSet = nil): string;
  var
    F: TField;
  begin
    if Data = nil then
      Data := Todos;
    F := Data.FieldByName('Activity');
    if F.IsNull then
      Result := NullActivity
    else
      Result := ActivityEnum.IDToName(F.AsInteger, NullActivity)
  end;

  function Select(Value: Boolean; const IfTrue: string; const IfFalse: string = ''): string;
  begin
    if Value then
      Result := ifTrue
    else
      Result := ifFalse
  end;

  procedure GenEditor(Data: TDataSet; const Operation: string; const DisplayFields: string; const EditFields: string;
    const ActivityValues: string; const DefaultActivity: string;
    const StatusValues: string; const DefaultStatus: string;
    const CategoryValues: string = ''; const DefaultCategory: string = '');

    function DataField(const FieldName: string): TField;
    begin
      if Assigned(Data) then
        Result := Data.FieldByName(FieldName)
      else
        Result := nil
    end;

    function DataFieldValue(const FieldName: string): Variant;
    var
      F: TField;
    begin
      F := DataField(FieldName);
      if Assigned(F) then
        Result := F.Value
      else
      if SameText(FieldName, 'Category') then
        Result := NewRecordCategory
      else
      if SameText(FieldName, 'Module') then
        Result := NewRecordModule
      else
      if SameText(FieldName, 'Priority') then
        Result := NewRecordPriority
      else
        Result := Unassigned
    end;

    function DataDefault(const IfData: string; const IfNoData: string): string;
    begin
      if Assigned(Data) then
        Result := IfData
      else
        Result := IfNoData
    end;

    function FieldVisible(const FieldName: string): Boolean;
    begin
      Result := Pos(',' + LowerCase(FieldName) + ',', ',' + LowerCase(DisplayFields) + ',') <> 0
    end;

    function FieldReadOnly(const FieldName: string): Boolean;
    begin
      Result := Pos(',' + LowerCase(FieldName) + ',', ',' + LowerCase(EditFields) + ',') = 0
    end;

    function EnumName(const FieldName: string): string;
    begin
      if FieldReadOnly(FieldName) then
        Result := ''
      else
        Result := FieldName
    end;

  begin
    if not Assigned(Data) then begin
      GenNavigator(['Projekty', CurrentProject, 'koly', Operation],
                   [FormatHomeDefaultPage('../../../%s'), '', Format('../%s', [DefaultPage]), ''],
                   '../'
                  );
    end else
    case Todo.TodoRecordActivity of
    aActive:
      GenNavigator(['Projekty', CurrentProject, 'koly', Format('%d', [Integer(DataFieldValue(fldTodos_ID))]), Operation],
                   [FormatHomeDefaultPage('../../../%s'), '', Format('../%s', [DefaultPage]), DefaultPage, ''],
                   '../'
                  );
    aComplete:
      GenNavigator(['Projekty', CurrentProject, 'koly', 'Vyeen', Format('%d', [Integer(DataFieldValue(fldTodos_ID))]), Operation],
                   [FormatHomeDefaultPage('../../../%s'), '', '', '../resolved.htm', DefaultPage, ''],
                   '../'
                  );
    aClosed:
      GenNavigator(['Projekty', CurrentProject, 'koly', 'Historie', Format('%d', [Integer(DataFieldValue(fldTodos_ID))]), Operation],
                   [FormatHomeDefaultPage('../../../%s'), '', '', '../history.htm', DefaultPage, ''],
                   '../'
                  );
    else
      Assert(False)
    end;

    if EditFields <> '' then
      Write('<form action="../update.exe" method=get>');  // TODO: method=post

    Write('<table border=0>');

    if FieldVisible('ID') then
      Write('<tr><td align=right>ID&nbsp;:</td><td>%d</td></tr>', [Integer(DataFieldValue(fldTodos_ID))]);

    if FieldVisible('Created') then
      Write('<tr><td align=right>Vytvoen&nbsp;:</td><td>%s</td></tr>', [FieldToDate(DataField('Created'))]);

    if FieldVisible('Category') then begin
      Write('<tr><td align=right>Kategorie&nbsp;:</td><td>');
      GenEnum(DataField('Category'), CategoryEnum, CategoryDescriptions,
        CategoryValues,
        Select(DefaultCategory <> '', DefaultCategory, DataDefault(NullCategory, NewRecordCategory)), EnumName('Category'));
      Write('</td></tr>');
    end;

    if FieldVisible('Activity') then begin
      Write('<tr><td align=right>Aktivita&nbsp;:</td><td>');
      GenEnum(DataField('Activity'), ActivityEnum, ActivityDescriptions,
        ActivityValues,
        DefaultActivity, EnumName('Activity'));
      Write('</td></tr>');
    end;

    if FieldVisible('Status') then begin
      Write('<tr><td align=right>Stav&nbsp;:</td><td>');
      GenEnum(DataField('Status'), StatusEnum, StatusDescriptions,
        StatusValues,
        DefaultStatus, EnumName('Status'));
      Write('</td></tr>');
    end;

    if FieldVisible('Priority') then begin
      Write('<tr><td align=right>Priorita&nbsp;:</td><td>');
      GenEnum(DataField('Priority'), PriorityEnum, PriorityDescriptions,
        '',
        DataDefault(NullPriority, NewRecordPriority), EnumName('Priority'));
      Write('</td></tr>');
    end;

    if FieldVisible('SolverID') and (psUsers in Style) then begin
      Write('<tr><td align=right>eitel&nbsp;:</td><td>');
      if FieldReadOnly('SolverID') then
        Write(HTMLText(GetUserName(Data, 'SolverID')))
      else
        GenUsersEnum(DataField('SolverID'), 'SolverID');
      Write('<td></tr>')
    end;

    if FieldVisible('Module') then begin
      Write('<tr><td align=right>Modul&nbsp;:</td><td>');
      if FieldReadOnly('Module') then
        Write(HTMLText(string(DataFieldValue('Module'))))
      else
        Write('<input type=text name=Module value="%s">', [string(DataFieldValue('Module'))]);
      Write('</td></tr>')
    end;

    if FieldVisible('Note') then begin
      Write('<tr><td align=right valign=top>Text&nbsp;:%s</td><td>', [GetUserName(Data, 'ReporterID', '<br>[ %s ]&nbsp;&nbsp;')]);
      if FieldReadOnly('Note') then
        Write(HTMLText(string(DataFieldValue(fldTodos_Text)), True, True, TodoLinker))
      else
        Write('<textarea name=Note rows=5 cols=100>%s</textarea>', [HTMLText(string(DataFieldValue(fldTodos_Text)), True, False)]);
      Write('</td></tr>')
    end;

    if Assigned(Data) then begin
      if FieldVisible('Notes.Last') and (not FieldReadOnly('Notes.Last')) and (not Todo.TodoNotesNavigator.IsEmpty) then begin
        Todo.TodoNotesNavigator.Last;
        Write('<tr><td align=right valign=top>Posledn pipomnka&nbsp;:%s</td><td>', [GetUserName(TodoNotes, 'ReporterID', '<br>[ %s ]&nbsp;&nbsp;')]);
        Write('<textarea name=Notes.Last rows=5 cols=100>%s</textarea>', [HTMLText(string(TodoNotes.FieldValues[fldTodoNotes_Text]), True, False)]);
        Write('</td></tr>');
      end;

      if FieldVisible('Notes.New') and (not FieldReadOnly('Notes.New')) then begin
        Write('<tr><td align=right valign=top>Nov pipomnka&nbsp;:</td><td>');
        Write('<textarea name=Notes.New rows=5 cols=100></textarea>');
        Write('</td></tr>');
      end;
    end;

    Write('</table>');

    if EditFields <> '' then begin
      Write('<center><input type=submit></center>');
      if FieldVisible('ID') then
        Write('<input type=hidden name=ID value=%d>', [Integer(DataFieldValue(fldTodos_ID))]);
      Write('</form>');
    end
  end;

var
  I: Integer;
  NewID: Integer;
  List: TStringList;
  StatusValues: string;
  DefaultStatus: string;
  ColorIndex: Integer;

  function CurrentStatus(Data: TDataSet = nil): string;
  var
    F: TField;
  begin
    if Data = nil then
      Data := Todos;
    F := Data.FieldByName('Status');
    if F.IsNull then
      Result := NullStatus
    else
      Result := StatusEnum.IDToName(F.AsInteger, NullStatus)
  end;

  function CurrentPriority(Data: TDataSet = nil): string;
  var
    F: TField;
  begin
    if Data = nil then
      Data := Todos;
    F := Data.FieldByName('Priority');
    if F.IsNull then
      Result := NullPriority
    else
      Result := PriorityEnum.IDToName(F.AsInteger, NullPriority)
  end;

  function CurrentModule: string;
  var
    F: TField;
  begin
    F := Todos.FieldByName('Module');
    if F.IsNull then
      Result := ''
    else
      Result := F.AsString
  end;

  function FileImage(IsShared: Boolean): string;
  begin
    if IsShared then
      Result := 'shared'
    else
      Result := 'personal'
  end;

  function PriorityImage(Data: TDataSet = nil): string;

    function IsPriority(const Values: array of string; var Index: Integer): Boolean;
    var
      I: Integer;
      S: string;
    begin
      S := CurrentPriority(Data);
      Result := False;
      for I := Low(Values) to High(Values) do
        if SameText(S, Values[I]) then begin
          Index := (I - Low(Values)) + 1;
          Result := True;
          exit
        end
    end;

  var
    N: Integer;
  begin
    if IsPriority(['lowFix', 'mediumFix', 'highFix'], N) then
      Result := Format('scheduled.now.%d', [N])
    else
    if IsPriority(['lowWhenIdle', 'mediumWhenIdle', 'highWhenIdle'], N) then
      Result := Format('scheduled.whenidle.%d', [N])
    else
    if IsPriority(['dontFix'], N) then
      Result := 'scheduled.never'
    else
      Result := ''
  end;

  function PriorityColor(Data: TDataSet = nil): string;

    function C(Never, Now, WhenIdle: Integer): TColor;

      function N(I: Integer): Integer;
      begin
        case I of
        3: Result := 255;
        2: Result := 171;
        1: Result := 86
        else
          Result := 0
        end
      end;

      function M(N: Single): Integer;
      begin
        if N > 255 then
          Result := 255
        else
          Result := Round(N)
      end;

      function Mult(C: TColor; N: Integer): TColor;
      begin
        C := ColorToRGB(C);
        Result := RGB(M(GetRValue(C) * (N/3)), M(GetGValue(C) * (N/3)), M(GetBValue(C) * (N/3)))
      end;

      function Add(const Colors: array of TColor): TColor;
      var
        R, G, B: Integer;
        I: Integer;
      begin
        R := 0;
        G := 0;
        B := 0;
        for I := Low(Colors) to High(Colors) do begin
          Inc(R, GetRValue(Colors[I]));
          Inc(G, GetGValue(Colors[I]));
          Inc(B, GetBValue(Colors[I]))
        end;
        Result := RGB(M(R), M(G), M(B))
      end;

    begin
      Result := Add([Mult(clRed, Never), Mult(clLime, Now), Mult(clYellow, WhenIdle)])
    end;

  var
    S: string;
    Color: TColor;
  begin
    S := PriorityImage(Data);
    if SameText(S, 'scheduled.never') then
      Color := C(3, 0, 0)
    else
    if SameText(S, 'scheduled.now.1') then
      Color := C(0, 1, 2)
    else
    if SameText(S, 'scheduled.now.2') then
      Color := C(0, 2, 1)
    else
    if SameText(S, 'scheduled.now.3') then
      Color := C(0, 3, 0)
    else
    if SameText(S, 'scheduled.whenidle.1') then
      Color := C(2, 0, 1)
    else
    if SameText(S, 'scheduled.whenidle.2') then
      Color := C(1, 0, 2)
    else
    if SameText(S, 'scheduled.whenidle.3') then
      Color := C(0, 0, 3)
    else
      Color := C(0, 0, 0);

    Color := GetHighLightColor(Color, -69);

    Result := Format('#%.2x%.2x%.2x', [GetRValue(Color), GetGValue(Color), GetBValue(Color)])
  end;

  function ActivityImage(Data: TDataSet = nil): string;

    function IsStatus(const Values: array of string): Boolean;
    var
      I: Integer;
      S: string;
    begin
      S := CurrentStatus(Data);
      Result := False;
      for I := Low(Values) to High(Values) do
        if SameText(S, Values[I]) then begin
          Result := True;
          exit
        end
    end;

  var
    Activity: string;
  begin
    Activity := CurrentActivity(Data);
    if SameText(Activity, 'notStarted') then
      Result := 'waiting'
    else
    if SameText(Activity, 'scheduledDevelopment') then
      Result := 'scheduled.development'
    else
    if SameText(Activity, 'scheduledTesting') then
      Result := 'scheduled.testing'
    else
    if SameText(Activity, 'developing')  then
      Result := 'inprogress.development'
    else
    if SameText(Activity, 'testing') then
      Result := 'inprogress.testing'
    else
    if SameText(Activity, 'complete') or SameText(Activity, 'closed') then begin
      Result := 'closed';

      if IsStatus(['bugFixed', 'taskFixed', 'featureImplemented', 'questionResponded']) then
        Result := Result + '.done'
      else
      if IsStatus(['bugNoReproduce', 'questionWaitingForInfo']) then
        Result := Result + '.moredata'
      else
      if IsStatus(['bugDuplicate', 'bugNotBug', 'taskDuplicate', 'taskAlreadyImplemented', 'featureAlreadyImplemented', 'featureDuplicate']) then
        Result := Result + '.duplicate'
      else
      if IsStatus(['bugWontFix', 'taskWontFix', 'featureWontImplement', 'questionWontRespond', 'questionSPAM']) then
        Result := Result + '.canceled'
      else
      if IsStatus(['bugPostponed']) then
        Result := 'waiting';  // TODO:
    end else
      Result := ''
  end;

  function SecurityStatus: string;
  begin
    if Assigned(Todo) and (Todo.Password <> '') then
      Result := ' (crypted)' // TUNE:
    else
      Result := ''
  end;

  function IDRef: string;
  begin
    Result := '#' + IntToStr(Todos.FieldValues[fldTodos_ID])
  end;

var
  PreviousActivity: TActivity;
  ShowPage: (pClosed, pResolved, pTodos, pItem);
  IsShared: Boolean;
  S: string;
  CurrentFileName: string;
  DisplayCurrentFile: Boolean;
  Nav: TDataNavigator;
  ActivityGroup: (agUnknown, agActive, agWaiting);
begin
  Buf := '';
  FLinkBase := '';

  Write('<html><head><meta http-equiv="Content-Type" content="text/html; charset=windows-1250">');
  Write('<style type="text/css">');
  Write(' table.note {border: 1px solid #000000}');
  Write(' hr {height: 1px; width: 100%; color: #000000}');
  Write(' body, td, input, select, textarea {font-family: sans-serif');
  if psFixedFontSize in Style then
    Write('; font-size: %dpt}', [FontSize]);
  Write('}');
//  Write(' td {font-size: %dpt}', [FontSize]);
  Write(' a:link { color: #0000FF; font-style: normal; text-decoration: none; }');
  Write(' a:link:visited { color: #0000A0; font-style: normal; text-decoration: none; }');
//  Write(' a:link:hover { color: #0000FF; background-color: #C0C0C0; font-style: normal; text-decoration: none; }');
//  Write(' a:link:active{ color: #C05000; font-style: normal; text-decoration: none;}');
  (*
  Write(' a:link { color: #FF8000; font-style: normal; text-decoration: none; }');
  Write(' a:link:visited { color: #C05000; font-style: normal; text-decoration: none; border-top: 1px dotted; border-bottom: 1px dotted;}');
  Write(' a:link:hover { color: #FFFFFF; background-color: #FF8000; font-style: normal; text-decoration: none;}');
  Write(' a:link:active{ color: #C05000; font-style: normal; text-decoration: none;}');
  *)
//  Write(' textarea:focus, input:focus, select:focus {background-color: #ffffee}');
  Write('</style>');
  Write('</head><body bgcolor=#FFFFFF text=#000000>');

  DisplayCurrentFile := True;

  if Path = 'about.htm' then begin
    DisplayCurrentFile := False;
    Write('<a href="javascript:history.back(1)">&lt;&lt;</a> | Npovda \ O Aplikaci<hr>');
    Write(HTMLText(FileText(ConfigFiles.ExpandFileName('about.txt')), False, True))
  end else
  if Path = DefaultPage then begin
    // list of projects
    DisplayCurrentFile := False;

    GenNavigator(['Projekty'], [''], '');
    if ProjectServer then
      GenToolBar(['Konkrtn koly'], ['pending.htm'])
    else
      GenToolBar([], []);

    Write('<table border=0>');

    List := TStringList.Create;
    for I := 0 to MRU.Count - 1 do
      if MRU[I].Enabled then begin
        IsShared := MRU[I].IsShared;
        List.AddObject(FileToDateTimeSortKey(MRU[I].FileName + '.todolist') + Chr(Ord(IsShared) + Ord('0')) + MRU[I].ProjectName, TObject(I));
      end;
    List.Sort;
    for I := List.Count - 1 downto 0 do
      Write('<tr><td><a href="%0:s/todo/%2:s"><img src="/images/book.%3:s.gif" border=0 width=16 height=16 hspace=2>%1:s</a></td><td align=right>%4:s</td></tr>', [WebName(Copy(List[I], 1 + DateTimeSortKeyLength + 1, MaxInt)), Copy(List[I], 1 + DateTimeSortKeyLength + 1, MaxInt), DefaultPage, FileImage(Boolean(Ord(List[I][1 + DateTimeSortKeyLength]) - Ord('0'))), FileToDateTime(MRU[Integer(List.Objects[I])].FileName + '.todolist')]);
    List.Free;

    Write('</table>');
  end else
  if (Path = 'pending.htm') and ProjectServer then begin
    // list of pending tasks
    DisplayCurrentFile := False;
    CurrentProject := '';

    GenNavigator(['Projekty', 'Pehled'], [DefaultPage, ''], '');
    GenToolBar(['Konkrtn koly'], ['.']);

    Write('<table border=0><tr><td>ID</td><td>eitel</td><td>Text</td></tr>');

    Nav := ListPendingTasks(nil);
    if Assigned(Nav) then
    try
      Nav.First;
      while not Nav.Eof do begin
        Write('<tr bgcolor=%s>', [PriorityColor(Nav.DataSet)]);

        Write('<td align=right valign=top nowrap><a href="%6:s/todo/%0:d/%1:s">%5:s/%0:d<img src="/images/%2:s.gif" border=0 width=16 height=16 hspace=2><img src="/images/%3:s.gif" border=0 width=16 height=16 hspace=2>' +
              '<img src="/images/%4:s.gif" border=0 width=16 height=16 hspace=2></a></td>',
              [Integer(Nav.DataSet.FieldValues[fldTodos_ID]), DefaultPage, CurrentCategory(Nav.DataSet), PriorityImage(Nav.DataSet), ActivityImage(Nav.DataSet), Nav.DataSet.FieldValues['Description'], WebName(Nav.DataSet.FieldValues['Description'])]);
        Write('<td valign=top>%s</td>', [HTMLText(GetUserName(Nav.DataSet, 'SolverID'))]);
//        Write('<td valign=top>%s</td>', [string(Todos.FieldValues['Module'])]);
        Write('<td valign=top>%s</td>', [ShortHTMLText(string(Nav.DataSet.FieldValues[fldTodos_Text]), True { TODO: this cuts-off line breaker }, True, Nav.DataSet.FieldValues['NotesCount'] > 0 {, TODO: TodoLinker })]);
        Write('</tr>');

        Nav.Next
      end
    finally
      Nav.Free
    end;
{    List := TStringList.Create;
    for I := 0 to MRU.Count - 1 do
      if MRU[I].Enabled then begin
        IsShared := MRU[I].IsShared;
        List.AddObject(FileToDateTimeSortKey(MRU[I].FileName + '.todolist') + Chr(Ord(IsShared) + Ord('0')) + MRU[I].ProjectName, TObject(I));
      end;
    List.Sort;
    for I := List.Count - 1 downto 0 do
      Write('<tr><td><a href="%0:s/todo/%2:s"><img src="/images/book.%3:s.gif" border=0 width=16 height=16 hspace=2>%1:s</a></td><td align=right>%4:s</td></tr>', [WebName(Copy(List[I], 1 + DateTimeSortKeyLength + 1, MaxInt)), Copy(List[I], 1 + DateTimeSortKeyLength + 1, MaxInt), DefaultPage, FileImage(Boolean(Ord(List[I][1 + DateTimeSortKeyLength]) - Ord('0'))), FileToDateTime(MRU[Integer(List.Objects[I])].FileName + '.todolist')]);
    List.Free; }

    Write('</table>');
  end else begin
    // CurrentProject := UnWebName(Item(0));
    ProjectID := UnWebName(Item(0));
    SetCurrentProject(ProjectID, User, Password);
    if (CurrentProject = '') and (psBlankProject in Style) and (Item(1) = 'todo') and (Item(2) = DefaultPage) then begin
      { virtual blank project }
      GenNavigator(['Projekty', ProjectID, 'koly'],
                   [FormatHomeDefaultPage('../../%s'), '', ''],
                   ''
                  );
    end else
    if (CurrentProject <> '') and (Item(1) = 'todo') then begin
      if Item(2) = DefaultPage then begin
        // list of todos

        GenNavigator(['Projekty', CurrentProject, 'koly'],
                     [FormatHomeDefaultPage('../../%s'), '', ''],
                     ''
                    );

        GenToolBar(['{new}', 'Aktuln', 'Vyeen', 'Historie'],
                   [Select(not Todo.ReadOnly, Yes {'0/index.htm'}, No), '.', Select(not CompleteList.IsEmpty, 'resolved.htm'), Select(not HistoryList.IsEmpty, 'history.htm')]);

        Write('<table border=0><tr><td>ID</td>%s<td>Modul</td><td>Text</td></tr>', [Select(psUsers in Style, '<td>eitel</td>')]);
        TodoList.First;
        ActivityGroup := agUnknown;
        ColorIndex := 0;
        while not TodoList.Eof do begin
          { TODO:
          case ColorIndex of
          0: Write('<tr>');
          1: Write('<tr bgcolor=#E0E0E0>')
          end;
          }
          if ActivityImage = 'waiting' then begin
            if ActivityGroup = agActive then begin
              if psUsers in Style then
                Write('<tr><td colspan=5>&nbsp;</td></tr>')
              else
                Write('<tr><td colspan=4>&nbsp;</td></tr>')
            end;
            ActivityGroup := agWaiting
          end else begin
            ActivityGroup := agActive
          end;

          Write('<tr bgcolor=%s>', [PriorityColor]);

          ColorIndex := 1 - ColorIndex;
          Write('<a name=%0:d></a><td align=right valign=top><a href="%0:d/%s">%0:d<img src="/images/%2:s.gif" border=0 width=16 height=16 hspace=2><img src="/images/%3:s.gif" border=0 width=16 height=16 hspace=2>' +
                '<img src="/images/%4:s.gif" border=0 width=16 height=16 hspace=2></a></td>',
                [Integer(Todos.FieldValues[fldTodos_ID]), DefaultPage, CurrentCategory, PriorityImage, ActivityImage]);
          if psUsers in Style then
            Write('<td valign=top>%s</td>', [HTMLText(GetUserName(Todos, 'SolverID'))]);
          Write('<td valign=top>%s</td>', [string(Todos.FieldValues['Module'])]);
          Write('<td valign=top>%s</td>', [ShortHTMLText(string(Todos.FieldValues[fldTodos_Text]), True { TODO: this cuts-off line breaker }, True, not Todo.TodoNotesNavigator.IsEmpty, TodoLinker)]);
          Write('</tr>');
          TodoList.Next;
        end;
        Write('</table>')
      end else
      if Item(2) = 'resolved.htm' then begin
        GenNavigator(['Projekty', CurrentProject, 'koly', 'Vyeen'],
                     [FormatHomeDefaultPage('../../%s'), '', DefaultPage, ''],
                     ''
                    );

        GenToolBar(['{new}', 'Aktuln', 'Vyeen', 'Historie'],
                   [No, DefaultPage, '.', Select(not HistoryList.IsEmpty, 'history.htm')]);

{        GenToolBar(['Nov', 'Historie'],
                   ['0/index.htm', 'history.htm']);
}
        Write('<table border=0><tr><td>ID</td><td>Vyeeno</td>%s<td>Text</td></tr>', [Select(psUsers in Style, '<td>eitel</td>')]);
        CompleteList.First;
        ColorIndex := 0;
        while not CompleteList.Eof do begin
          case ColorIndex of
          0: Write('<tr>');
          1: Write('<tr bgcolor=#E0E0E0>')
          end;
          Write('<a name=%0:d></a>', [Integer(Todos.FieldValues[fldTodos_ID])]);
          ColorIndex := 1 - ColorIndex;
          Write('<td align=right valign=top><a href="%0:d/%s">%0:d<img src="/images/%2:s.gif" border=0 width=16 height=16 hspace=2><img src="/images/%3:s.gif" border=0 width=16 height=16 hspace=2></a></td>', [Integer(Todos.FieldValues[fldTodos_ID]), DefaultPage, CurrentCategory, ActivityImage]);
          Write('<td valign=top align=right>%s</td>', [FieldToDate('Resolved')]);
          if psUsers in Style then
            Write('<td valign=top>%s</td>', [HTMLText(GetUserName(Todos, 'SolverID'))]);
//          Write('<td valign=top>%s</td>', [string(Data.FieldValues['Module'])]);
          Write('<td valign=top>%s</td>', [ShortHTMLText(string(Todos.FieldValues[fldTodos_Text]), True { TODO: this cuts-off line breaker }, True, not Todo.TodoNotesNavigator.IsEmpty, TodoLinker)]);
          Write('</tr>');
          CompleteList.Next;
        end;
        Write('</table>')
      end else
      if Item(2) = 'history.htm' then begin
        GenNavigator(['Projekty', CurrentProject, 'koly', 'Historie'],
                     [FormatHomeDefaultPage('../../%s'), '', DefaultPage, ''],
                     ''
                    );

        GenToolBar(['{new}', 'Aktuln', 'Vyeen', 'Historie'],
                   [No, DefaultPage, Select(not CompleteList.IsEmpty, 'resolved.htm'), '.']);

{        GenToolBar(['Nov', 'Historie'],
                   ['0/index.htm', 'history.htm']);
}
        Write('<table border=0><tr><td>ID</td><td>Uzaveno</td><td>Trvn<td>Text</td></tr>');
        HistoryList.First;
        ColorIndex := 0;
        while not HistoryList.Eof do begin
          case ColorIndex of
          0: Write('<tr>');
          1: Write('<tr bgcolor=#E0E0E0>')
          end;
          Write('<a name=%0:d></a>', [Integer(Todos.FieldValues[fldTodos_ID])]);
          ColorIndex := 1 - ColorIndex;
          Write('<td align=right valign=top><a href="%0:d/%s">%0:d<img src="/images/%2:s.gif" border=0 width=16 height=16 hspace=2><img src="/images/%3:s.gif" border=0 width=16 height=16 hspace=2></a></td>', [Integer(Todos.FieldValues[fldTodos_ID]), DefaultPage, CurrentCategory, ActivityImage]);
          Write('<td valign=top align=right>%s</td>', [FieldToDate('Closed')]);
          Write('<td valign=top align=right>%s</td>', [DateDiff('Closed', 'Created')]);
//          Write('<td valign=top>%s</td>', [string(Data.FieldValues['Module'])]);
          Write('<td valign=top>%s</td>', [ShortHTMLText(string(Todos.FieldValues[fldTodos_Text]), True { TODO: this cuts-off line breaker }, True, not Todo.TodoNotesNavigator.IsEmpty, TodoLinker)]);
          Write('</tr>');
          HistoryList.Next;
        end;
        Write('</table>')
      end else
      if Item(2) = 'update.exe' then begin
        // record insert/update

        if QueryFields.IndexOfName('ID') < 0 then begin
          if not Todo.ReadOnly then begin

            with Todos do begin
              Append;
              for I := 0 to FieldCount - 1 do begin
                S := FieldNameToIDName(Fields[I].FieldName);
                if QueryFields.IndexOfName(S) >= 0 then begin
                  if SameText(S, 'SolverID') then
                    AssignUser(Fields[I], QueryFields.Values[S])
                  else
                    Fields[I].AsString := QueryFields.Values[S];
                end
              end;

              FieldByName('Created').AsDateTime := Now;

              Todo.Post(Todos);
            end;

            NewID := Todos.FieldValues[fldTodos_ID];

            NewRecordPriority := CurrentPriority;
            NewRecordCategory := CurrentCategory;
            NewRecordModule := CurrentModule;

            Redirect(Format('%d/%s', [NewID, DefaultPage]))
          end else
            Redirect(DefaultPage)
        end else begin
          NewID := StrToIntDef(QueryFields.Values['ID'], 0);
          if Todos.Locate(fldTodos_ID, NewID, []) then begin
            PreviousActivity := Todo.TodoRecordActivity;

            if not Todo.ReadOnly then begin
              with Todos do begin
                Edit;
                for I := 0 to FieldCount - 1 do begin
                  S := FieldNameToIDName(Fields[I].FieldName);
                  if QueryFields.IndexOfName(S) >= 0 then begin
                    if SameText(S, 'SolverID') then
                      AssignUser(Fields[I], QueryFields.Values[S])
                    else
                      Fields[I].AsString := QueryFields.Values[S];
                  end
                end;

                case Todo.TodoRecordActivity of
                aClosed:
                  begin
                    FieldByName('Closed').AsDateTime := Now;
                    if (PreviousActivity = aActive) or FieldByName('Resolved').IsNull then
                      // resolve and close
                      FieldByName('Resolved').AsDateTime := Now;
                  end;

                aComplete:
                  begin
                    if PreviousActivity <> aComplete then
                      FieldByName('Resolved').AsDateTime := Now;
                  end;
                end;

                Todo.Post(Todos);
              end;

              if QueryFields.IndexOfName('Notes.Last') >= 0 then begin
                Todo.TodoNotesNavigator.Last;
                S := Trim(QueryFields.Values['Notes.Last']);
                if S = '' then
                  TodoNotes.Delete
                else with TodoNotes do begin
                  Edit;
                  FieldByName(fldTodoNotes_Text).AsString := S;
                  Todo.Post(TodoNotes);
                end
              end;

              S := Trim(QueryFields.Values['Notes.New']);
              if S <> '' then
              with TodoNotes do begin
                Append;

                FieldByName('Created').AsDateTime := Now;
                FieldByName(fldTodoNotes_Text).AsString := S;

                Todo.Post(TodoNotes);
              end;

            end;

            if Todo.TodoRecordActivity = aDeleted then begin
              if not Todo.ReadOnly then begin
                Todos.Delete;
              end;

              case PreviousActivity of
              aComplete:
                Redirect('resolved.htm');
              aClosed:
                Redirect('history.htm');
              else
                Redirect(DefaultPage);
              end;
            end else begin
              ShowPage := pItem;

              if PreviousActivity <> Todo.TodoRecordActivity then
              case PreviousActivity of
              aActive:
                // protoze moh jit jenom na odkladaci zalozku, vratime se na hlavni seznam
                ShowPage := pTodos;

              aComplete:
                case Todo.TodoRecordActivity of
                aActive:
                  // reaktivace - nechame fokus na zaznamu
                  ;
                aClosed:
                  // vratime se na seznam
                  ShowPage := pResolved;
                end;

              aClosed:
                case Todo.TodoRecordActivity of
                aActive:
                  // znovuotevreni - nechame fokus na zaznamu
                  ;
                aComplete:
                  // nejaka reaktivace o krok zpatky - nechame fokus na zaznamu
                  ;
                end;
              end;

              case ShowPage of
              pClosed:
                Redirect('history.htm');
              pResolved:
                Redirect('resolved.htm');
              pTodos:
                Redirect(DefaultPage);
              pItem:
                Redirect(Format('%d/%s', [NewID, DefaultPage]));
              else
                Assert(False)
              end;
            end
          end else
            Redirect(DefaultPage)
        end
      end else
      if Item(2) = '0' then begin
        // record insert
        if Item(3) = 'bug.htm' then
          GenEditor(nil, 'Nov',
                    'Category,Priority,Module,Note',
                    'Category,Priority,Module,Note',
                    '',
                    NullActivity,
                    '',
                    NullStatus,
                    'bug',
                    'bug'
                    )
        else
        if Item(3) = 'task.htm' then
          GenEditor(nil, 'Nov',
                    'Category,Priority,Module,Note',
                    'Category,Priority,Module,Note',
                    '',
                    NullActivity,
                    '',
                    NullStatus,
                    'task',
                    'task'
                    )
        else
        if Item(3) = 'feature.htm' then
          GenEditor(nil, 'Nov',
                    'Category,Priority,Module,Note',
                    'Category,Priority,Module,Note',
                    '',
                    NullActivity,
                    '',
                    NullStatus,
                    'feature',
                    'feature'
                    )
        else
        if Item(3) = 'question.htm' then
          GenEditor(nil, 'Nov',
                    'Category,Priority,Module,Note',
                    'Category,Priority,Module,Note',
                    '',
                    NullActivity,
                    '',
                    NullStatus,
                    'question',
                    'question'
                    )
        else
          // DefaultPage
          GenEditor(nil, 'Nov',
                    'Category,Priority,Module,Note',
                    'Category,Priority,Module,Note',
                    '',
                    NullActivity,
                    '',
                    NullStatus
                    )
      end else begin
        if Todos.Locate(fldTodos_ID, StrToIntDef(Item(2), 0), []) then begin
          if Item(3) = 'edit.htm' then begin
            // operace muze menit stavy 'category', 'activity'
            GenEditor(Todos, 'Zmnit',
                      'ID,Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'notStarted,scheduledDevelopment,developing,scheduledTesting,testing',
                      'notStarted',
                      'active',
                      'active'
                      )
          end else
          if Item(3) = 'reopen.htm' then begin
            // nastavi 'status'='active', 'activity'='not started'
            GenEditor(Todos, 'Znovuotevt',
                      'ID,Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'notStarted,scheduledDevelopment,developing,scheduledTesting,testing',
                      'notStarted',
                      'active',
                      'active'
                      )
          end else
          if Item(3) = 'resolve.htm' then begin
            // operace muze menit stavy 'status' v mezich 'resolved', nastavi 'activity'='complete'
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'bug') then begin
              StatusValues := 'bugFixed,bugNoReproduce,bugDuplicate,bugPostponed,bugWontFix,bugNotBug';
              DefaultStatus := 'bugFixed'
            end else
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'task') then begin
              StatusValues := 'taskFixed,taskDuplicate,taskWontFix,taskAlreadyImplemented';
              DefaultStatus := 'taskFixed'
            end else
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'feature') then begin
              StatusValues := 'featureImplemented,featureWontImplement,featureAlreadyImplemented,featureDuplicate';
              DefaultStatus := 'featureImplemented'
            end else
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'question') then begin
              StatusValues := 'questionResponded,questionWontRespond,questionSPAM,questionWaitingForInfo';
              DefaultStatus := 'questionResponded'
            end;

            GenEditor(Todos, 'Vyeit',
                      'ID,Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'Activity,Status,SolverID,Module,Note,Notes.Last,Notes.New',
                      'complete',
                      'complete',
                      StatusValues,
                      DefaultStatus
                      )
          end else
          if Item(3) = 'close.htm' then begin
            // operace nastavi 'activity' = 'closed'
            GenEditor(Todos, 'Uzavt',
                      'ID,Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'Activity,Module,Note,Notes.Last,Notes.New',
                      'closed',
                      'closed',
                      '',
                      NullStatus
                      )

          end else
          if Item(3) = 'resolveclose.htm' then begin
            // operace muze menit stavy 'status' v mezich 'resolved', nastavi 'activity'='complete'
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'bug') then begin
              StatusValues := 'bugFixed,bugNoReproduce,bugDuplicate,bugPostponed,bugWontFix,bugNotBug';
              DefaultStatus := 'bugFixed'
            end else
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'task') then begin
              StatusValues := 'taskFixed,taskDuplicate,taskWontFix,taskAlreadyImplemented';
              DefaultStatus := 'taskFixed'
            end else
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'feature') then begin
              StatusValues := 'featureImplemented,featureWontImplement,featureAlreadyImplemented,featureDuplicate';
              DefaultStatus := 'featureImplemented'
            end else
            if IsEnum(Todos.FieldByName('Category'), CategoryEnum, NullCategory, 'question') then begin
              StatusValues := 'questionResponded,questionWontRespond,questionSPAM,questionWaitingForInfo';
              DefaultStatus := 'questionResponded'
            end;

            GenEditor(Todos, 'Vyeit a uzavt',
                      'ID,Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'Activity,Status,SolverID,Module,Note,Notes.Last,Notes.New',
                      'closed',
                      'closed',
                      StatusValues,
                      DefaultStatus
                      )
          end else
          if Item(3) = 'reactivate.htm' then begin
            // nastavi 'status'='active', 'activity'='not started'
            // nastavi 'status'='active', 'activity'='not started'
            GenEditor(Todos, 'Znovuotevt',
                      'ID,Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'Category,Activity,Status,Priority,SolverID,Module,Note,Notes.Last,Notes.New',
                      'notStarted,scheduledDevelopment,developing,scheduledTesting,testing',
                      'notStarted',
                      'active',
                      'active'
                      )
          end else
          if Item(3) = 'delete.htm' then begin
            GenEditor(Todos, 'Smazat',
                      'ID,Category,Activity,Status,Priority,SolverID,Module,Note',
                      'Activity',
                      'deleted',
                      'deleted',
                      '',
                      NullStatus
                      )
          end else
          if Item(3) = DefaultPage then begin
            case Todo.TodoRecordActivity of
            aActive:
              begin
                GenNavigator(['Projekty', CurrentProject, 'koly', Format('%d', [Integer(Todos.FieldValues[fldTodos_ID])])],
                             [FormatHomeDefaultPage('../../../%s'), '', Format('../%s%s', [DefaultPage, IDRef]), ''], '../');

                if not Todo.ReadOnly then
                GenToolBar(['{new}', 'Znovuotevt', 'Zmnit', 'Vyeit', 'Uzavt', 'Smazat'],
                           ['../' + Yes, '', 'edit.htm', 'resolve.htm', 'resolveclose.htm', 'delete.htm']);
              end;

            aComplete:
              begin
                GenNavigator(['Projekty', CurrentProject, 'koly', 'Vyeen', Format('%d', [Integer(Todos.FieldValues[fldTodos_ID])])],
                             [FormatHomeDefaultPage('../../../%s'), '', Format('../%s', [DefaultPage]), Format('../resolved.htm%s', [IDRef]), ''], '../');
                if not Todo.ReadOnly then
                GenToolBar(['{new}', 'Znovuotevt', 'Zmnit', 'Vyeit', 'Uzavt', 'Smazat'],
                           [No, 'reactivate.htm', 'resolve.htm', '', 'close.htm', 'delete.htm'])
              end;

            aClosed:
              begin
                GenNavigator(['Projekty', CurrentProject, 'koly', 'Historie', Format('%d', [Integer(Todos.FieldValues[fldTodos_ID])])],
                             [FormatHomeDefaultPage('../../../%s'), '', Format('../%s', [DefaultPage]), Format('../history.htm%s', [IDRef]), ''], '../');
                if not Todo.ReadOnly then
                GenToolBar(['{new}', 'Znovuotevt', 'Zmnit', 'Vyeit', 'Uzavt', 'Smazat'],
                           [No, 'reopen.htm', '', '', '', 'delete.htm'])
              end;
            else
              Assert(False)
            end;

            if Todo.ReadOnly then
              GenToolBar(['{new}', 'Znovuotevt', 'Zmnit', 'Vyeit', 'Uzavt', 'Smazat'], []);

            Write('<table>');
            Write('<tr><td align=right>ID&nbsp;:</td><td>%d</td></tr>', [Integer(Todos.FieldValues[fldTodos_ID])]);
            Write('<tr><td align=right>Vytvoen&nbsp;:</td><td>%s</td></tr>', [FieldToDate('Created')]);

            case Todo.TodoRecordActivity of
            aClosed:
              begin
                Write('<tr><td align=right>Vyeen&nbsp;:</td><td>%s</td></tr>', [FieldToDate('Resolved')]);
                Write('<tr><td align=right>Uzaven&nbsp;:</td><td>%s</td></tr>', [FieldToDate('Closed')]);
              end;

            aComplete:
              begin
                Write('<tr><td align=right>Vyeen&nbsp;:</td><td>%s</td></tr>', [FieldToDate('Resolved')]);
              end;
            end;

            Write('<tr><td align=right>Kategorie&nbsp;:</td><td>');
            Write('<img src="/images/%s.gif" width=16 height=16 hspace=2>', [CurrentCategory]);
            GenEnum(Todos.FieldByName('Category'), CategoryEnum, CategoryDescriptions,
              '',
              NullCategory, '');
            Write('</td></tr>');

            Write('<tr><td align=right>Aktivita&nbsp;:</td><td>');
            Write('<img src="/images/%s.gif" width=16 height=16 hspace=2>', [ActivityImage]);
            GenEnum(Todos.FieldByName('Activity'), ActivityEnum, ActivityDescriptions,
              '',
              NullActivity, '');
            Write('</td></tr>');

            Write('<tr><td align=right>Stav&nbsp;:</td><td>');
            GenEnum(Todos.FieldByName('Status'), StatusEnum, StatusDescriptions,
              '',
              NullStatus, '');
            Write('</td></tr>');

            if Todo.TodoRecordActivity = aActive then begin
              Write('<tr><td align=right>Priorita&nbsp;:</td><td>');
              Write('<img src="/images/%s.gif" width=16 height=16 hspace=2>', [PriorityImage]);
              GenEnum(Todos.FieldByName('Priority'), PriorityEnum, PriorityDescriptions,
                '',
                NullPriority, '');
              Write('</td></tr>');
            end;

            if psUsers in Style then
              Write('<tr><td align=right>eitel&nbsp;:</td><td>%s</td></tr>', [GetUserName(Todos, 'SolverID')]);

            Write('<tr><td align=right>Modul&nbsp;:</td><td>%s</td></tr>', [string(Todos.FieldValues['Module'])]);
            Write('<tr><td align=right valign=top>Text&nbsp;:%1:s</td><td valign=top>%0:s</td></tr>', [HTMLText(string(Todos.FieldValues[fldTodos_Text]), False, True, TodoLinker), GetUserName(Todos, 'ReporterID', '<br> [ %s ]&nbsp;&nbsp;')]);

            if not Todo.TodoNotesNavigator.IsEmpty then begin
              Todo.TodoNotesNavigator.First;
              while not Todo.TodoNotesNavigator.Eof do begin
                Write('<tr><td valign=top align=right>%s&nbsp;:%s</td>', [FieldToDateTime('Created', TodoNotes), GetUserName(TodoNotes, 'ReporterID', '<br>[ %s ]&nbsp;&nbsp;')]);
                Write('<td valign=top>%s</td></tr>', [HTMLText(string(TodoNotes.FieldValues[fldTodoNotes_Text]), False, True, TodoLinker)]);
                Todo.TodoNotesNavigator.Next
              end;
            end;

            Write('</table>');

          end
        end
      end;
    end else
      Redirect(RootDefaultPage)
  end;

  if DisplayCurrentFile then begin
    CurrentFileName := CurrentFile;
    if CurrentFileName <> '' then
      CurrentFileName := CurrentFileName + '.todolist'
  end else
    CurrentFileName := '';

//  Write('<p align=right><font size=1 color=#C0C0C0>%s</font></p>', [DateTimeToStr(Now)]);
  Write('<table border=0 width=100%%><tr><td>&nbsp;</td><td>&nbsp;</td></tr><tr><td><font color=#C0C0C0>%s%s</font></td><td align=right><font color=#C0C0C0>%s</font></td></tr></table>',
        [CurrentFileName, Select(CurrentFileName <> '', SecurityStatus), DateTimeToStr(Now)]);
  Write('</body></html>');
  Result := Buf
end;

end.
