{ VERSION: 1.1 }
{ 1.1 / 3.6.2006
  - nov konfiguran parametr Options.WindowsAuthenticationDomain umouje
    nastavit kter server se pouije pro ovovn hesel
  - azen aktulnch kol dv na prvn msto ty co nejsou ekajc ( viz.
    TodoListSortKey() )
}
unit wConfigDB;

interface

uses
  fileSys, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, CoolTrayIcon, Menus, uWebUI, uWebServer, CfgArray,
  uTodoLists, uTodoList, IniFilesD5, DB, uMyskaDB, uMyskaApp, uAppTodo, uMyskaTables,
  ComCtrls, StdCtrls, ExtCtrls, Buttons, ActnList;

type
  TConfigForm = class(TForm)
    Tray: TCoolTrayIcon;
    TrayPopupMenu: TPopupMenu;
    MenuQuit: TMenuItem;
    N1: TMenuItem;
    MenuConfig: TMenuItem;
    MenuBrowse: TMenuItem;
    Book: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    BtnOk: TButton;
    PageProjects: TTabSheet;
    MRUView: TListView;
    OpenDialog: TOpenDialog;
    PageUsers: TTabSheet;
    Panel3: TPanel;
    BtnInsert: TSpeedButton;
    BtnEdit: TSpeedButton;
    BtnDelete: TSpeedButton;
    BtnExport: TSpeedButton;
    BtnImport: TSpeedButton;
    BtnPrint: TSpeedButton;
    Actions: TActionList;
    ActionInsert: TAction;
    ActionEdit: TAction;
    ActionDelete: TAction;
    ActionImport: TAction;
    ActionExport: TAction;
    SaveDialog: TSaveDialog;
    procedure TrayStartup(Sender: TObject; var ShowMainForm: Boolean);
    procedure MenuQuitClick(Sender: TObject);
    procedure MenuConfigClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MenuBrowseClick(Sender: TObject);
    procedure TrayDblClick(Sender: TObject);
    procedure BtnOkClick(Sender: TObject);
    procedure MRUViewEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure ActionInsertExecute(Sender: TObject);
    procedure ActionsUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure ActionDeleteExecute(Sender: TObject);
    procedure ActionEditExecute(Sender: TObject);
    procedure ActionImportExecute(Sender: TObject);
    procedure ActionExportExecute(Sender: TObject);
    procedure MRUViewResize(Sender: TObject);
  private
    DoClose: Boolean;
    WebServer: TWebServer;
    Pager: TPageGenerator;
    FConfigFiles: TCfgArrayReader;
    MRU: TTodoBook;
    Config: TCustomIniFile;
    TodoBase: TTodoApp;
    DataBase: TMyskaStore;
    function GetConnected: Boolean;
    procedure SetConnected(Value: Boolean);
    procedure StartUp;
    function ConfigFiles: TCfgArrayReader;
  private
    procedure LoadMRU;
    procedure ReLoadMRU(Entry: TTodoBookEntry; const ProjectName: string);
    procedure InsertMRU(ID: Cardinal; const ProjectName: string; Edit: Boolean = False);
  private
    procedure Quit;
    property Connected: Boolean read GetConnected write SetConnected;
  private
    CategoryEnum,
    StatusEnum,
    ActivityEnum,
    PriorityEnum: TEnum;
    CategoryDescriptions,
    StatusDescriptions,
    ActivityDescriptions,
    PriorityDescriptions: string;
    NullCategory,
    NullStatus,
    NullActivity,
    NullPriority: string;
  private
    TodoList: TDataNavigator;
    CompleteList: TDataNavigator;
    HistoryList: TDataNavigator;

    function HistoryListSortKey(Sender: TObject; Data: TDataSet): string;
    function CompleteListSortKey(Sender: TObject; Data: TDataSet): string;
    function TodoListSortKey(Sender: TObject; Data: TDataSet): string;
    function PendingListSortKey(Sender: TObject; Data: TDataSet): string;

    procedure BindFields(Sender: TDataSet);
  private
    Todos: TDataSet;
    TodoNotes: TDataSet;

    procedure RecordChanged(Sender: TDataSet);
    procedure DeleteRecord(Sender: TDataSet);
    procedure NewRecord(Sender: TDataSet; Updating: Boolean = False);
  private
    FCurrentProject: string;
    CurrentFile: string;
    Todo: TTodoList;
    function ValidProjectName(const Value: string): Boolean;
    function ProjectFile(const Value: string): string;
    procedure SetCurrentProject(const Value: string; const User: string; const Password: string);
    procedure CloseFile;
    procedure OpenFile(const FileName: string);
    property CurrentProject: string read FCurrentProject;
  private
    procedure SetClient(const Host: string; const User: string; const Password: string);
  end;

var
  ConfigForm: TConfigForm;

implementation
uses
  StrUtils, PackFileSys, MiscUtils, SystemUtils, uMyskaIBStore, uMyskaJetStore, wEditUsers;

{$R *.dfm}

function ProjectName(const FileName: string): string;
begin
  Result := ChangeFileExt(ExtractFileName(FileName), '');
end;

function FileNameToID(const FileName: string): Cardinal;
var
  Drive: string;
begin
  Drive := ExtractFileName(ExtractFileDir(FileName));
  Result := StrToIntDef(Drive, 0)
end;

procedure TConfigForm.TrayStartup(Sender: TObject;
  var ShowMainForm: Boolean);
begin
  ShowMainForm := False;
  SetWindowLong(Application.Handle, GWL_EXSTYLE, GetWindowLong(Application.Handle, GWL_EXSTYLE) and (not WS_EX_APPWINDOW) or WS_EX_TOOLWINDOW)
end;

procedure TConfigForm.Quit;
begin
  DoClose := True;
  Close
end;

procedure TConfigForm.MenuQuitClick(Sender: TObject);
begin
  Quit
end;

procedure TConfigForm.MenuConfigClick(Sender: TObject);
begin
  Show;
  Application.BringToFront;
  BringToFront
end;

procedure TConfigForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not DoClose then begin
    Hide;
    Action := caNone
  end
end;

function TConfigForm.GetConnected: Boolean;
begin
  Result := Assigned(WebServer)
end;

type
  TGenerator = class(TPageGenerator)
  private
    function UI: TConfigForm;
  protected
    function SelectProject(const Value: string; const User: string; const Password: string): string; override;
    function TranslateUserName(Field: TField): string; override;
    procedure GetUserNames(List: TStrings; var Indexed: Boolean); override;
    function ProjectServer: Boolean; override;
    function ListPendingTasks(AnOwner: TComponent): TDataNavigator; override;
  protected
    function RootDefaultPage: string; override;
    function CurrentFile: string; override;
    function DefaultPage: string; override;
    function FontSize: Integer; override;
    function ConfigFiles: TCfgArrayReader; override;
    function MRU: TTodoBook; override;
  end;

function TGenerator.UI: TConfigForm;
begin
  Result := ConfigForm
end;

function TGenerator.SelectProject(const Value: string; const User: string; const Password: string): string;
begin
  UI.SetCurrentProject(Value, User, Password);
  Result := UI.CurrentProject;
  Self.Todos := UI.Todos;
  Self.TodoNotes := UI.TodoNotes;
  Self.Todo := UI.Todo;
  Self.CategoryEnum := UI.CategoryEnum;
  Self.StatusEnum := UI.StatusEnum;
  Self.ActivityEnum := UI.ActivityEnum;
  Self.PriorityEnum := UI.PriorityEnum;
  Self.CategoryDescriptions := UI.CategoryDescriptions;
  Self.StatusDescriptions := UI.StatusDescriptions;
  Self.ActivityDescriptions := UI.ActivityDescriptions;
  Self.PriorityDescriptions := UI.PriorityDescriptions;
  Self.NullCategory := UI.NullCategory;
  Self.NullStatus := UI.NullStatus;
  Self.NullActivity := UI.NullActivity;
  Self.NullPriority := UI.NullPriority;
  Self.TodoList := UI.TodoList;
  Self.CompleteList := UI.CompleteList;
  Self.HistoryList := UI.HistoryList;
end;

function TGenerator.TranslateUserName(Field: TField): string;
var
  User: TMyskaUser;
begin
  User := UI.TodoBase.Users.Find(Field.AsInteger);
  if Assigned(User) then
    Result := User.Name
  else
    Result := ''
end;

procedure TGenerator.GetUserNames(List: TStrings; var Indexed: Boolean);
var
  I: Integer;
  User: TMyskaUser;
begin
  for I := 0 to UI.TodoBase.Users.Count - 1 do begin
    User := UI.TodoBase.Users[I];
    List.AddObject(User.Name, TObject(User.ID))
  end;
  Indexed := True
end;

function TGenerator.ProjectServer: Boolean;
begin
  Result := True
end;

type
  TOwningNavigator = class(TDataNavigator)
  public
    destructor Destroy; override;
  end;

destructor TOwningNavigator.Destroy;
var
  D: TDataSet;
begin
  D := DataSet;
  DataSet := nil;
  D.Free;
  inherited
end;

function TGenerator.ListPendingTasks(AnOwner: TComponent): TDataNavigator;
begin
  Result := TOwningNavigator.Create(AnOwner);
  try
    Result.DataSet := UI.TodoBase.ListPendingTasks;
    Result.KeyField := 'TaskID';
    Result.OnGetSortKey := UI.PendingListSortKey;
    while not Result.DataSet.Eof do begin
      Result.Add;
      Result.DataSet.Next
    end;
    Result.Sort
  except
    Result.Free;
    raise
  end
end;

function TGenerator.RootDefaultPage: string;
begin
  Result := '/todolist/index.htm'
end;

function TGenerator.CurrentFile: string;
begin
  Result := UI.CurrentFile;
  if Result <> '' then
    Result := UI.DataBase.Connection + '?' + IntToStr(FileNameToID(Result))
end;

function TGenerator.DefaultPage: string;
begin
  Result := 'index.htm'
end;

function TGenerator.FontSize: Integer;
begin
  Result := 0
end;

function TGenerator.ConfigFiles: TCfgArrayReader;
begin
  Result := UI.ConfigFiles
end;

function TGenerator.MRU: TTodoBook;
begin
  Result := UI.MRU
end;

//---------------------------------------------------------------------------

function TConfigForm.ValidProjectName(const Value: string): Boolean;
begin
  Result := (Value = '') or (ProjectFile(Value) <> '')
end;

function TConfigForm.ProjectFile(const Value: string): string;
var
  Item: TTodoBookEntry;
begin
  Result := '';
  if Value = '' then exit;
  if SameText(Value, CurrentProject) then begin
    Result := CurrentFile;
    exit
  end;
  Item := MRU.FindProject(Value);
  if Assigned(Item) then
    Result := Item.FileName
end;

procedure TConfigForm.SetCurrentProject(const Value: string; const User: string; const Password: string);
var
  Item: TTodoBookEntry;
begin
  if not SameText(FCurrentProject, Value) then begin
    CloseFile;
    Item := MRU.FindProject(Value);
    if Assigned(Item) then
      OpenFile(Item.FileName)
  end
end;

procedure TConfigForm.CloseFile;
begin
  if Assigned(Todos) and Todos.Active and (CurrentFile <> '') then begin
    // TODO: except
    Todos := nil;
    TodoNotes := nil;

    FreeAndNil(TodoList);
    FreeAndNil(HistoryList);
    FreeAndNil(CompleteList);

    FreeAndNil(Todo);
  end;
  CurrentFile := '';
  FCurrentProject := '';
end;

procedure TConfigForm.RecordChanged(Sender: TDataSet);

  procedure Migrate(const Sources: array of TDataNavigator; Destination: TDataNavigator);
  var
    I: Integer;
    Removed: Boolean;
  begin
    Removed := False;
    for I := Low(Sources) to High(Sources) do
      if Sources[I].Remove then begin
        Removed := True;
        break
      end;
    if Removed then begin
      Destination.Add;
      Destination.Sort
    end else
      Destination.Changed
  end;

begin
  if Sender = Todos then
  case Todo.TodoRecordActivity of
  aActive:
    Migrate([HistoryList, CompleteList], TodoList);
  aComplete:
    Migrate([HistoryList, TodoList], CompleteList);
  aClosed:
    Migrate([TodoList, CompleteList], HistoryList);
  else
    ; // TODO: to je pred deletem
  end
end;

procedure TConfigForm.DeleteRecord(Sender: TDataSet);
begin
  if Sender = Todos then begin
    TodoList.Remove;
    HistoryList.Remove;
    CompleteList.Remove;
  end
end;

procedure TConfigForm.NewRecord(Sender: TDataSet; Updating: Boolean);

  procedure Add(Destination: TDataNavigator);
  begin
    Destination.Add;
    if not Updating then
      Destination.Sort
  end;

begin
  if Sender = Todos then
  case Todo.TodoRecordActivity of
  aActive:
    Add(TodoList);
  aComplete:
    Add(CompleteList);
  aClosed:
    Add(HistoryList);
  else
    Assert(False)
  end
end;

function TConfigForm.HistoryListSortKey(Sender: TObject; Data: TDataSet): string;
begin
  Result := FormatDateTime('yyyymmddhhnnss', Data.FieldByName('Closed').AsDateTime)
end;

function TConfigForm.CompleteListSortKey(Sender: TObject; Data: TDataSet): string;
begin
  Result := FormatDateTime('yyyymmddhhnnss', Data.FieldByName('Resolved').AsDateTime)
end;

function TConfigForm.TodoListSortKey(Sender: TObject; Data: TDataSet): string;
var
  F: TField;
  Value: string;
  Prio: Integer;
  Activ: Integer;
  Catg: Integer;

  function Enum(const Value: string; const Values: array of string; Base: Integer; DefaultValue: Integer): Integer;
  var
    I: Integer;
  begin
    for I := Low(Values) to High(Values) do
      if SameText(Value, Values[I]) then begin
        Result := (I - Low(Values)) + Base;
        exit
      end;
    Result := DefaultValue
  end;

begin
  F := Data.FieldByName('Priority');
  if F.IsNull then
    Value := NullPriority
  else
    Value := PriorityEnum.IDToName(F.AsInteger, NullPriority);

  Prio := Enum(Value, ['highFix', 'mediumFix', 'lowFix', 'highWhenIdle', 'mediumWhenIdle', 'lowWhenIdle', 'dontFix'], 1, 1 { TODO: ?});

  F := Data.FieldByName('Category');
  if F.IsNull then
    Value := NullCategory
  else
    Value := CategoryEnum.IDToName(F.AsInteger, NullCategory);

  // TUNE: order defines category priorities
  Catg := Enum(Value, ['bug' ,'question', 'task', 'feature', 'event'], 1, 1 { TODO: });

  F := Data.FieldByName('Activity');
  if F.IsNull then
    Value := NullActivity
  else
    Value := ActivityEnum.IDToName(F.AsInteger, NullActivity);

  Activ := Enum(Value, ['testing', 'developing', 'scheduledTesting', 'scheduledDevelopment', 'notStarted'], 1, 1 { TODO: });

  Result := IntToStr(Activ * 100 + Prio * 10 + Catg);
  Result := StringOfChar('0', 5 - Length(Result)) + Result;

  Result := Result + FormatDateTime('yyyymmddhhnnss', Data.FieldByName('Created').AsDateTime)
end;

function TConfigForm.PendingListSortKey(Sender: TObject; Data: TDataSet): string;
var
  F: TField;
  Value: string;
  Prio: Integer;
  Activ: Integer;
  Catg: Integer;

  function Enum(const Value: string; const Values: array of string; Base: Integer; DefaultValue: Integer): Integer;
  var
    I: Integer;
  begin
    for I := Low(Values) to High(Values) do
      if SameText(Value, Values[I]) then begin
        Result := (I - Low(Values)) + Base;
        exit
      end;
    Result := DefaultValue
  end;

begin
  F := Data.FieldByName('Priority');
  if F.IsNull then
    Value := NullPriority
  else
    Value := PriorityEnum.IDToName(F.AsInteger, NullPriority);

  Prio := Enum(Value, ['highFix', 'mediumFix', 'lowFix', 'highWhenIdle', 'mediumWhenIdle', 'lowWhenIdle', 'dontFix'], 1, 1 { TODO: ?});

  F := Data.FieldByName('Category');
  if F.IsNull then
    Value := NullCategory
  else
    Value := CategoryEnum.IDToName(F.AsInteger, NullCategory);

  // TUNE: order defines category priorities
  Catg := Enum(Value, ['bug' ,'question', 'task', 'feature', 'event'], 1, 1 { TODO: });

  F := Data.FieldByName('Activity');
  if F.IsNull then
    Value := NullActivity
  else
    Value := ActivityEnum.IDToName(F.AsInteger, NullActivity);

  Activ := Enum(Value, ['testing', 'scheduledTesting', 'developing', 'scheduledDevelopment', 'notStarted'], 1, 1 { TODO: });

  Result := IntToStr(Activ * 100 + Prio * 10 + Catg);
  Result := StringOfChar('0', 5 - Length(Result)) + Result;

  Result := Result + FormatDateTime('yyyymmddhhnnss', Data.FieldByName('Created').AsDateTime)
end;

procedure TConfigForm.BindFields(Sender: TDataSet);
begin
  if Sender.Tag = di_Todos then begin
    HistoryList.BindFields;
    CompleteList.BindFields;
    TodoList.BindFields;
  end
end;

type
  TDBTodoList = class(TTodoList)
  private
    GroupID: Cardinal;
    FNewNum: Integer;
    FLastNum: Integer;
    FSolidCache: Boolean;
    function GenNum: Integer;
  protected
    TodoBase: TTodoApp;
  protected
    function TodosKeyField: string; override;
    function TodoNotesKeyField: string; override;
    function TodoNotesMasterField: string; override;
    function TodosMemoField: string; override;
    function TodoNotesMemoField: string; override;
    procedure UpdateData(Sender: TDataSet; State: TDataState); override;
    function GenID(Sender: TDataSet): Integer; override;
    procedure Changed; override;
    function SolidCache: Boolean; override;
  public
    constructor Create(AnOwner: TComponent; ATodoBase: TTodoApp);
    function ValidateUser(const User: string; const Password: string): string; override;
    function Connect(const AFileName: string; CreateNew: Boolean = False; ForceReadOnly: Boolean = False): Boolean; override;
    procedure LoadData; override;
  end;

function TDBTodoList.TodosKeyField: string;
begin
  Result := 'TaskID'
end;

function TDBTodoList.TodoNotesKeyField: string;
begin
  Result := 'NoteID'
end;

function TDBTodoList.TodoNotesMasterField: string;
begin
  Result := 'TaskID'
end;

function TDBTodoList.TodosMemoField: string;
begin
  Result := 'Notes'
end;

function TDBTodoList.TodoNotesMemoField: string;
begin
  Result := 'Notes'
end;

constructor TDBTodoList.Create(AnOwner: TComponent; ATodoBase: TTodoApp);
begin
  TodoBase := ATodoBase;
  if (TodoBase.Store is TMyskaJetStore) then
    FSolidCache := False
  else
    FSolidCache := True;
  inherited Create(AnOwner);
end;

function TDBTodoList.SolidCache: Boolean;
begin
  Result := FSolidCache
end;

function TDBTodoList.Connect(const AFileName: string; CreateNew: Boolean; ForceReadOnly: Boolean): Boolean;
begin
  Close;

  if SameText(ExtractFileExt(AFileName), '.todolist') then begin
    FFileName := ChangeFileExt(AFileName, '');
  end else begin
    FFileName := AFileName;
  end;

  GroupID := FileNameToID(FFileName);

  FTodos := TodoBase.Exec('select * from Task where GroupID=%d', [], [GroupID]);
  FTodos.Tag := di_Todos;

  FTodoNotes := TodoBase.Exec('select * from TaskNote where GroupID=%d', [], [GroupID]);
  FTodoNotes.Tag := di_TodoNotes;

  FLastNum := TodoBase.GroupNumSeed[GroupID];

  Result := True
end;

function TDBTodoList.GenNum: Integer;
var
  N: Integer;
  F: TField;
begin
  Result := FLastNum;
  if Result = 0 then begin
    // naskocime na posledni maximum
    F := Todos.FieldByName('Num');
    Todos.First;
    while not Todos.Eof do begin
      N := F.AsInteger;
      if N > Result then
        Result := N;
      Todos.Next
    end;
    Inc(Result)
  end else begin
//    while Todos.Locate('Num', Result, []) do
    while TodoBase.Exists('select Num from Task where GroupID=%d and Num=%d', [GroupID, Result]) do
      Inc(Result);
  end
end;

function TDBTodoList.GenID(Sender: TDataSet): Integer;
begin
  case Sender.Tag of
  di_Todos:
    begin
      Result := TodoBase.GenID('\Task');
      FNewNum := GenNum
    end;

  di_TodoNotes:
    Result := TodoBase.GenID('\TaskNote');
  else
    Result := 0
  end
end;

procedure TDBTodoList.UpdateData(Sender: TDataSet; State: TDataState);
var
  F: TField;
begin
  if (State = dInsert) and (Sender.Tag in [di_Todos, di_TodoNotes]) then begin
    Sender.FieldValues['GroupID'] := GroupID;
    if TodoBase.UserID <> 0 then
      Sender.FieldValues['ReporterID'] := TodoBase.UserID;
    if Sender.Tag = di_Todos then begin
      F := Sender.FieldByName('Num');
      if F.IsNull then
        F.AsInteger := FNewNum
      else
        FNewNum := F.AsInteger;
      FLastNum := FNewNum;
      TodoBase.GroupNumSeed[GroupID] := FNewNum
    end
  end
end;

procedure TDBTodoList.Changed;
begin
  TodoBase.GroupAge[GroupID] := DateTimeToFileDate(Now);
  inherited;
  Modified := False
end;

procedure TDBTodoList.LoadData;

  procedure RegisterRecords(Source: TDataSet);
  begin
    if not Assigned(OnNewRecord) then exit;
    Source.First;
    while not Source.Eof do begin
      OnNewRecord(Source, True);
      Source.Next
    end;
  end;

begin
  RegisterRecords(Todos);
  RegisterRecords(TodoNotes);
  DataLoaded
end;

function TDBTodoList.ValidateUser(const User: string; const Password: string): string;
begin
  Result := TodoBase.UserName
end;

procedure TConfigForm.OpenFile(const FileName: string);
var
  I: Integer;
  Ok: Boolean;
begin
  CloseFile;

  Todo := TDBTodoList.Create(nil, TodoBase);
  try
// TODO:    Todo.OnChange := ScheduleAutoSave;
    Ok := Todo.Connect(FileName);
    if not Ok then begin
      FreeAndNil(Todo);
      exit
    end;

    CurrentFile := Todo.FileName;
    FCurrentProject := ProjectName(CurrentFile);

    Todos := Todo.Todos;

    TodoList := TDataNavigator.Create(Self);
    TodoList.DataSet := Todos;
    TodoList.KeyField := 'TaskID';
    TodoList.OnGetSortKey := TodoListSortKey;

    HistoryList := TDataNavigator.Create(Self);
    HistoryList.DataSet := Todos;
    HistoryList.KeyField := 'TaskID';
    HistoryList.Ascending := False;
    HistoryList.OnGetSortKey := HistoryListSortKey;

    CompleteList := TDataNavigator.Create(Self);
    CompleteList.DataSet := Todos;
    CompleteList.KeyField := 'TaskID';
    CompleteList.Ascending := True;
    CompleteList.OnGetSortKey := CompleteListSortKey;

    TodoNotes := Todo.TodoNotes;

    Todo.OnNewRecord := NewRecord;
    Todo.OnRecordChanged := RecordChanged;
    Todo.OnDeleteRecord := DeleteRecord;
    Todo.OnBindFields := BindFields;

    Todo.ActivityEnum := ActivityEnum;
    Todo.StatusEnum := StatusEnum;
    Todo.CategoryEnum := CategoryEnum;
    Todo.PriorityEnum := PriorityEnum;

    Todo.LoadData;

    TodoList.Sort;
    HistoryList.Sort;
    CompleteList.Sort;
  except
    Todos := nil;
    TodoNotes := nil;
    FreeAndNil(Todo);
    raise
  end;
end;

//---------------------------------------------------------------------------

type
  TDBTodoBook = class(TTodoBook)
  public
    function FileExists(const FileName: string): Boolean; override;
    function FileAge(const FileName: string): Integer; override;
  protected
    TodoBase: TTodoApp;
  public
    constructor Create(ATodoBase: TTodoApp);
  end;

constructor TDBTodoBook.Create(ATodoBase: TTodoApp);
begin
  TodoBase := ATodoBase;
  inherited Create
end;

function TDBTodoBook.FileExists(const FileName: string): Boolean;
begin
  Result := True
end;

function TDBTodoBook.FileAge(const FileName: string): Integer;
begin
  Result := TodoBase.GroupAge[FileNameToID(FileName)]
end;

procedure TConfigForm.LoadMRU;
var
  T: TTableProvider;
  Item: TListItem;
begin
  MRU.Clear;
  MRUView.Clear;
  T := TodoBase.CreateProviderUkolniky(nil, True);
  try
    T.ReadOnly := True;
    T.Open;
    while not T.DataSet.Eof do begin
      MRU.Add(Format('\\%d\%s.todolist', [Cardinal(T.DataSet['GroupID']), T.DataSet['Description']]));
      Item := MRUView.Items.Add;
      Item.Caption := T.DataSet['Description'];
      Item.SubItems.Add(IntToStr(T.DataSet['GroupID']));
      T.DataSet.Next
    end
  finally
    T.Free
  end;
  MRUView.OnResize(MRUView)
end;

procedure TConfigForm.ReLoadMRU(Entry: TTodoBookEntry; const ProjectName: string);
begin
  TodoBase.Exec('update TaskGroup set Description=''%1:s'' where GroupID=%0:d', [so_Command], [FileNameToID(Entry.FileName), ProjectName]);
  Entry.FileName := Format('\\%d\%s.todolist', [FileNameToID(Entry.FileName), ProjectName]);
end;

procedure TConfigForm.InsertMRU(ID: Cardinal; const ProjectName: string; Edit: Boolean);
var
  Item: TListItem;
begin
  MRU.Add(Format('\\%d\%s.todolist', [ID, ProjectName]));
  Item := MRUView.Items.Add;
  Item.Caption := ProjectName;
  Item.SubItems.Add(IntToStr(ID));
  MRUView.ItemIndex := Item.Index;
  MRUView.OnResize(MRUView);
  if Edit then
    Item.EditCaption
end;

procedure TConfigForm.SetClient(const Host: string; const User: string; const Password: string);
begin
  if Host = '' then begin
    TodoBase.Logout;
    Pager.UserName := TodoBase.UserName
  end else
  if TodoBase.Login(User, Password) then
    Pager.UserName := TodoBase.UserName
  else
    raise EBadPassword.Create('');
end;

procedure TConfigForm.SetConnected(Value: Boolean);

  procedure SetMetaData(App: TMyskaApp; const AFolder: string);
  var
    Folder: string;
  begin
    if AFolder = '' then exit;
    Folder := AFolder;
    if Folder[Length(Folder)] <> '\' then
      Folder := Folder + '\';
    App.Params['app.files'] := Folder + App.AppID + '\'
  end;

  procedure DoDisconnect;
  begin
    FreeAndNil(MRU);
    FreeAndNil(WebServer);
    FreeAndNil(Pager);
    FreeAndNil(TodoBase);
    FreeAndNil(DataBase);
    FreeAndNil(CategoryEnum);
    FreeAndNil(StatusEnum);
    FreeAndNil(ActivityEnum);
    FreeAndNil(PriorityEnum);
  end;

  procedure BuildEnum(var Enum: TEnum; var Descriptions: string; var Null: string; const TableName: string; const NullValue: string; const Exclude: string = '');
  var
    D: TDataSet;
    EnumDef: string;
    ID: string;
  begin
    D := TodoBase.Exec('select * from {app.table.name(' + TableName + ')} order by Num', [so_ReadOnly]);
    try
      EnumDef := '';
      Descriptions := '';
      while not D.Eof do begin
        if EnumDef <> '' then begin
          EnumDef := EnumDef + ',';
          Descriptions := Descriptions + ','
        end;
        ID := string(D['NumCode']);
        if Pos(',' + UpperCase(ID) + ',', ',' + UpperCase(Exclude) + ',') = 0 then begin
          EnumDef := EnumDef + Format('%s=%d', [ID, integer(D['Num'])]);
          Descriptions := Descriptions + Format('%s=%s', [ID, string(D['Description'])]);
        end;
        D.Next
      end;
    finally
      D.Free
    end;
    Enum := TEnum.Create(EnumDef);
    Null := NullValue
  end;

begin
  if Connected = Value then exit;
  if Value then begin
    try
      WebServer := TWebServer.Create(Self);
      DataBase := CreateStore(Self, Config.ReadString('Options', 'DBName', ChangeFileExt(ParamStr(0), '.mdb')));
      DataBase.Users.WindowsAuthentication := StrToBool(Config.ReadString('Options', 'WindowsAuthentication', '0'));
      DataBase.Users.WindowsAuthenticationDomain := Config.ReadString('Options', 'WindowsAuthenticationDomain', '');
      TodoBase := TTodoApp.Create(DataBase);
      SetMetaData(TodoBase, ConfigFiles.RootPath + 'dotazy');
      TodoBase.Connected := True;
      MRU := TDBTodoBook.Create(TodoBase);
      LoadMRU;
      wEditUsers.CreateEditor(Self, PageUsers, DataBase).Visible := True;
      BuildEnum(CategoryEnum, CategoryDescriptions, NullCategory, 'CategoryEnum', 'bug', 'event');
      BuildEnum(StatusEnum, StatusDescriptions, NullStatus, 'StatusEnum', 'active');
      BuildEnum(ActivityEnum, ActivityDescriptions, NullActivity, 'ActivityEnum', 'notStarted');
      BuildEnum(PriorityEnum, PriorityDescriptions, NullPriority, 'PriorityEnum', 'highWhenIdle');
      Pager := TGenerator.Create(Self);
      Pager.fldTodos_ID := 'Num';
      Pager.fldTodos_Text := 'Notes';
      Pager.fldTodoNotes_ID := 'NoteID';
      Pager.fldTodoNotes_Text := 'Notes';
      WebServer.AutoPort := False;
      WebServer.Port := Config.ReadInteger('Options', 'Port', WebServer.Port);
      WebServer.DefaultPage := 'index.htm';
      WebServer.WebRootFolder := 'todolist';
      WebServer.DocDir := ConfigFiles.RootPath;
      WebServer.ValidProjectName := ValidProjectName;
      WebServer.OnSetClient := SetClient;
      WebServer.Pager := Pager;
      WebServer.Start;
      Caption := Format('TodoDB - %s', [DataBase.Connection]);
      Tray.Hint := Caption
    except
      DoDisconnect;
      raise
    end
  end else
    DoDisconnect
end;

procedure TConfigForm.FormCreate(Sender: TObject);
begin
  NotifyWhenIdle(StartUp)
end;

procedure TConfigForm.StartUp;
begin
  Config := TIniFile.Create(ChangeFileExt(ParamStr(0), '.ini'));
  try
    Connected := True;
    Tray.IconVisible := True
  except
    NotifyWhenIdle(Quit);
    raise
  end
end;

procedure TConfigForm.FormDestroy(Sender: TObject);
begin
  CloseFile;
  Connected := False;
  FreeAndNil(FConfigFiles);
  FreeAndNil(Config);
end;

function TConfigForm.ConfigFiles: TCfgArrayReader;
begin
  Result := FConfigFiles;
  if not Assigned(Result) then begin
    FConfigFiles := TCfgReader.Create;
    Result := FConfigFiles
  end
end;

procedure TConfigForm.MenuBrowseClick(Sender: TObject);
begin
  Exec(Format('http://127.0.0.1:%d/todolist/index.htm', [WebServer.Port]), '', '', [ef_Interactive])
end;

procedure TConfigForm.TrayDblClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to Tray.PopupMenu.Items.Count - 1 do
    with Tray.PopupMenu do
      if Items[I].Default then begin
        Items[I].OnClick(Items[I]);
        break
      end
end;

procedure TConfigForm.BtnOkClick(Sender: TObject);
begin
  Close
end;

procedure TConfigForm.MRUViewEdited(Sender: TObject; Item: TListItem;
  var S: String);

  function Normalize(const S: string): string;

    procedure FuckOff(const Chars: string);
    var
      I: Integer;
    begin
      for I := 1 to Length(Chars) do
        Result := StringReplace(Result, Chars[I], '', [rfReplaceAll])
    end;

  begin
    Result := Trim(S);
    FuckOff('\/"''')
  end;

var
  I: Integer;
  OldEntry: TTodoBookEntry;
  NewEntry: TTodoBookEntry;
begin
  S := Normalize(S);
  OldEntry := MRU.FindProject(Item.Caption);
  if S = '' then begin
    S := OldEntry.ProjectName;
    exit
  end;
  NewEntry := MRU.FindProject(S);
  if Assigned(NewEntry) then begin
    if OldEntry = NewEntry then begin
      if OldEntry.ProjectName <> S then
        ReLoadMRU(OldEntry, S)
    end else begin
      // dolo by k duplicitnmu nzvu
      S := OldEntry.ProjectName
    end
  end else begin
    ReLoadMRU(OldEntry, S)
  end
end;

procedure ImportFile(Dst: TDBTodoList; Src: TTodoList);
var
  I: Integer;
  SrcF: TField;
  DstF: TField;

  function TodosFieldName(const FieldName: string): string;
  begin
    if SameText(FieldName, 'ID') then
      Result := 'Num'
    else
    if SameText(FieldName, 'Note') then
      Result := 'Notes'
    else
      Result := FieldName
  end;

  function TodoNotesFieldName(const FieldName: string): string;
  begin
    if SameText(FieldName, 'Note') then
      Result := 'Notes'
    else
      Result := FieldName
  end;

var
  OldCaption: string;
  UserName: string;
  User: TMyskaUser;
begin
  OldCaption := ConfigForm.Caption;
  try
    Src.Todos.First;
    while not Src.Todos.Eof do begin
      Dst.Todos.Append;
      for I := 0 to Src.Todos.FieldCount - 1 do begin
        SrcF := Src.Todos.Fields[I];
        DstF := Dst.Todos.FindField(TodosFieldName(SrcF.FieldName));
        if Assigned(DstF) then
          if not SrcF.IsNull then begin
            if AnsiMatchText(DstF.FieldName, ['ReporterID', 'SolverID']) then begin
              UserName := Trim(SrcF.AsString);
              if UserName <> '' then begin
                User := Dst.TodoBase.Store.Users.Find(UserName);
                if not Assigned(User) then
                  User := Dst.TodoBase.Users.Add(UserName, '', False);
                DstF.Value := User.ID
              end
            end else
              DstF.Value := SrcF.Value
          end
      end;
      Dst.Post(Dst.Todos);

      Src.TodoNotesNavigator.First;
      while not Src.TodoNotesNavigator.Eof do begin
        Dst.TodoNotes.Append;
        for I := 0 to Src.TodoNotes.FieldCount - 1 do begin
          SrcF := Src.TodoNotes.Fields[I];
          DstF := Dst.TodoNotes.FindField(TodoNotesFieldName(SrcF.FieldName));
          if Assigned(DstF) then
            if not SrcF.IsNull then
              DstF.Value := SrcF.Value
        end;
        Dst.Post(Dst.TodoNotes);

        Src.TodoNotesNavigator.Next
      end;

      ConfigForm.Caption := Format('%d %%', [Round((Src.Todos.RecNo / Src.Todos.RecordCount) * 100)]);
      Src.Todos.Next
    end;

    Dst.TodoBase.GroupAge[Dst.GroupID] := SysUtils.FileAge(Src.FullFileName);
    Dst.FLastNum := 0;
    Dst.TodoBase.GroupNumSeed[Dst.GroupID] := 0;
  finally
    ConfigForm.Caption := OldCaption
  end
end;

procedure ExportFile(Src: TDBTodoList; Dst: TTodoList);
var
  I: Integer;
  SrcF: TField;
  DstF: TField;

  function TodosFieldName(const FieldName: string): string;
  begin
    if SameText(FieldName, 'Num') then
      Result := 'ID'
    else
    if SameText(FieldName, 'Notes') then
      Result := 'Note'
    else
      Result := FieldName
  end;

  function TodoNotesFieldName(const FieldName: string): string;
  begin
    if SameText(FieldName, 'Notes') then
      Result := 'Note'
    else
      Result := FieldName
  end;

var
  User: TMyskaUser;
begin
  Src.Todos.First;
  while not Src.Todos.Eof do begin
    Dst.Todos.Append;
    for I := 0 to Src.Todos.FieldCount - 1 do begin
      SrcF := Src.Todos.Fields[I];
      DstF := Dst.Todos.FindField(TodosFieldName(SrcF.FieldName));
      if Assigned(DstF) then
        if not SrcF.IsNull then begin
          if AnsiMatchText(SrcF.FieldName, ['ReporterID', 'SolverID']) then begin
            User := Src.TodoBase.Store.Users.Find(SrcF.AsInteger);
            if Assigned(User) then
              DstF.Value := User.Name
          end else
            DstF.Value := SrcF.Value
        end
    end;
    Dst.Post(Dst.Todos);

    Src.TodoNotesNavigator.First;
    while not Src.TodoNotesNavigator.Eof do begin
      Dst.TodoNotes.Append;
      for I := 0 to Src.TodoNotes.FieldCount - 1 do begin
        SrcF := Src.TodoNotes.Fields[I];
        DstF := Dst.TodoNotes.FindField(TodoNotesFieldName(SrcF.FieldName));
        if Assigned(DstF) then
          if not SrcF.IsNull then
            DstF.Value := SrcF.Value
      end;
      Dst.Post(Dst.TodoNotes);

      Src.TodoNotesNavigator.Next
    end;

//    ConfigForm.Caption := Format('%d %%', [Round((Src.Todos.RecNo / Src.Todos.RecordCount) * 100)]);
    Src.Todos.Next
  end;

  Dst.SetParam('Users', Src.TodoBase.Store.Users.CommaText);
{
  Dst.TodoBase.GroupAge[Dst.GroupID] := SysUtils.FileAge(Src.FullFileName);
  Dst.FLastNum := 0;
  Dst.TodoBase.GroupNumSeed[Dst.GroupID] := 0;
}  
end;

procedure TConfigForm.ActionInsertExecute(Sender: TObject);
var
  ProjectName: string;
  No: Integer;
  GroupID: Cardinal;

  function Name: string;
  begin
    if No = 0 then
      Result := ProjectName
    else
      Result := ProjectName + IntToStr(No)
  end;

begin
  Wait(True);
  try
    ProjectName := 'Nov kolnk';
    No := 0;
    while MRU.FindProject(Name) <> nil do
      Inc(No);
    GroupID := TodoBase.GenID('\TaskGroup');
    TodoBase.Exec('insert into TaskGroup(GroupID, Description) values (%d,''%s'')', [so_Command], [GroupID, Name]);
    InsertMRU(GroupID, Name, True)
  finally
    Wait(False)
  end
end;

procedure TConfigForm.ActionsUpdate(Action: TBasicAction;
  var Handled: Boolean);
var
  Selected: Boolean;
begin
  Selected := MRUView.ItemIndex >= 0;
  ActionEdit.Enabled := Selected;
  ActionImport.Enabled := Selected;
  ActionExport.Enabled := Selected;
  ActionDelete.Enabled := Selected;
  Handled := True
end;

procedure TConfigForm.ActionDeleteExecute(Sender: TObject);
var
  ProjectName: string;
  Entry: TTodoBookEntry;
  GroupID: Cardinal;
begin
  ProjectName := MRUView.Items[MRUView.ItemIndex].Caption;
  if Application.MessageBox(
      PChar(Format('Chce odstranit kolnk "%s" vetn vech zznam ?', [ProjectName])),
      PChar(Caption), mb_yesno or mb_iconquestion or mb_defbutton2) = IDYES then begin
    Wait(True);
    try
      CloseFile;
      Entry := MRU.FindProject(ProjectName);
      GroupID := FileNameToID(Entry.FileName);
      MRU.Delete(Entry);
      MRUView.Items.Delete(MRUView.ItemIndex);
      MRUView.OnResize(MRUView);
      TodoBase.DeleteUkolnik(GroupID, False);
    finally
      Wait(False)
    end
  end
end;

procedure TConfigForm.ActionEditExecute(Sender: TObject);
begin
  MRUView.ItemFocused.EditCaption
end;

procedure TConfigForm.ActionImportExecute(Sender: TObject);
var
  Entry: TTodoBookEntry;
  GroupID: Cardinal;
  Data: TTodoList;
begin
  Entry := MRU.FindProject(MRUView.Items[MRUView.ItemIndex].Caption);
  OpenDialog.FileName := Entry.ProjectName;
  if OpenDialog.Execute then begin
    if Application.MessageBox(
        PChar(Format('Ped importem budou z kolnku "%s" odstranny vechny zznamy', [Entry.ProjectName])),
        PChar(Caption), mb_okcancel or mb_iconwarning or mb_defbutton2) = IDOK then begin
      Wait(True);
      try
        CloseFile;
        GroupID := FileNameToID(Entry.FileName);
        TodoBase.DeleteUkolnik(GroupID, True);

        Data := TTodoList.Create(nil);
        try
          Data.Connect(OpenDialog.FileName, False, True);

          Data.ActivityEnum := ActivityEnum;
          Data.StatusEnum := StatusEnum;
          Data.CategoryEnum := CategoryEnum;
          Data.PriorityEnum := PriorityEnum;

          Data.LoadData;

          TodoBase.GroupNumSeed[GroupID] := 0;
          OpenFile(Entry.FileName);
          ImportFile(TDBTodoList(Todo), Data);
        finally
          Data.Free
        end;
      finally
        Wait(False)
      end
    end
  end
end;

procedure TConfigForm.ActionExportExecute(Sender: TObject);
var
  Entry: TTodoBookEntry;
  Data: TTodoList;
  GroupID: Cardinal;
  Age: Integer;
begin
  Entry := MRU.FindProject(MRUView.Items[MRUView.ItemIndex].Caption);
  GroupID := FileNameToID(Entry.FileName);
  SaveDialog.FileName := Entry.ProjectName;
  if SaveDialog.Execute then begin
    Wait(True);
    try
      Data := TTodoList.Create(nil);
      try
        Data.Connect(SaveDialog.FileName, True);

        Data.ActivityEnum := ActivityEnum;
        Data.StatusEnum := StatusEnum;
        Data.CategoryEnum := CategoryEnum;
        Data.PriorityEnum := PriorityEnum;

        Data.LoadData;

        OpenFile(Entry.FileName);
        ExportFile(TDBTodoList(Todo), Data);
      finally
        Data.Free
      end;
      Age := TodoBase.GroupAge[GroupID];
      if Age <> -1 then
        OSFileSystem.FileSetDate(SaveDialog.FileName, Age)
    finally
      Wait(False)
    end
  end;
end;

procedure TConfigForm.MRUViewResize(Sender: TObject);
begin
  with MRUView do Columns[0].Width := ClientWidth - Columns[1].Width
end;

end.
